#!/bin/sh

#############################################################################
#  Licensed Materials - Property of IBM
#  5724-M24
#  Copyright IBM Corporation 2009, 2011. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#############################################################################

echo ""
echo "Start Business Space Data Migration Script"
echo ""

#######################################################
#                                                     #
# Subroutine: Usage                                   #
#                                                     #
#######################################################
usage () {
  echo  ""
  echo  "migrateBSpaceData -host <host name> -port <SOAP port>"
  echo  "                  -user <server user id> -password  <server password>"
  echo 	"                  -server <server name> -node  <node name>"
  echo  "                  -cluster <cluster name>"
  echo 	"                  -dbcopy -dbupgrade"
  echo  ""
  echo  "where: " 
  echo  ""
  echo  "-host <host name>:"
  echo  "     The host name of Business Space server you connect to"
  echo  ""
  echo  "-port <SOAP port>:"
  echo  "     The SOAP port of Business Space server you connect to"
  echo  ""
  echo  "-user <server user id>:"
  echo  "     Administrative user id of Business Space server"
  echo  ""
  echo  "-password  <server password>:"
  echo  "     Password for administrative user id of Business Space server"
  echo  "" 
  echo  "-server <server name>:"
  echo  "     Name of server"
  echo  ""
  echo  "-node  <node name>:"
  echo  "	    Name of node"
  echo  ""
  echo  "-cluster <cluster name>:"
  echo  "     Name of cluster"
  echo  ""
  echo  "-dbcopy:"
  echo  "    Copy Business Space data from v7.0.x to v7.5"
  echo  ""
  echo  "-dbupgrade:"
  echo  "    Upgrade Business Space data from v7.0.x to v7.5"
  exit
 }   
  

#######################################################
#                                                     #
# Read command line options                           #
#                                                     #
#######################################################

if [ $# -ne 12 ]; then
  if [ $# -ne 10 ]; then
  	if [ $# -ne 9 ]; then
    	usage
    fi
  fi
fi
  
while [ $# -gt 0 ]; do
  case $1 in
    -host) host=$2; shift; shift;;
    -port) port=$2; shift; shift;;
    -user) user=$2; shift; shift;;
    -password) password=$2; shift; shift;;
	-server) server=$2; shift; shift;;
	-node) node=$2; shift; shift;;
	-cluster) cluster=$2; shift; shift;;
	-dbcopy) type="dbcopy"; shift;;
	-dbupgrade) type="dbupgrade"; shift;;
    -h) usage;;
    -help) usage;;
    *) echo "Invalid parameter: $1"; usage;;
  esac;
done
    
#echo "host=$host"
#echo "port=$port"
#echo "user=$user"
#echo "password=$password"

#######################################################
#                                                     #
# Set WAS environment                                 #
#                                                     #
#######################################################
CUR_DIR=`pwd`
SH_R_DIR=`dirname ${0}`
cd "$SH_R_DIR/../../"
WASHOME=`pwd`
export WASHOME
. "$WASHOME/bin/setupCmdLine.sh"


#######################################################
#                                                     #
# call migrateBSpaceData.jy                           #
#                                                     #
#######################################################

#echo "call migrateBSpaceData.jy"
echo "Starting Business Space data migration, please check log file $WASHOME/logs/BusinessSpaceDataMigration.log to make sure migration is complete without any exception."

if [ -n "$server" ] ; then
"$WASHOME"/bin/wsadmin.sh -connType SOAP -host $host -port $port -user $user -password $password -lang jython -f "$WASHOME"/BusinessSpace/scripts/migrateBSpaceData.jy $server $node 2> "$WASHOME"/logs/BusinessSpaceDataMigration.log
fi

if [ -n "$cluster" ] ; then
"$WASHOME"/bin/wsadmin.sh -connType SOAP -host $host -port $port -user $user -password $password -lang jython -f "$WASHOME"/BusinessSpace/scripts/migrateBSpaceData.jy $cluster 2> "$WASHOME"/logs/BusinessSpaceDataMigration.log
fi

if [ -n "$type" ] ; then
"$WASHOME"/bin/wsadmin.sh -connType SOAP -host $host -port $port -user $user -password $password -lang jython -f "$WASHOME"/BusinessSpace/scripts/migrateBSpaceData.jy $type $user $port 2> "$WASHOME"/logs/BusinessSpaceDataMigration.log
fi

echo "Business Space data migration is complete! Please also check the log and trace files under profile folder to make sure migration is complete without any exception." > $WASHOME/logs/BusinessSpaceDataMigration.log
