dojo.provide("com.ibm.pb.control.RenderingController");

dojo.require("dojo.io.script");
dojo.require("com.ibm.pb.iwidget.iWidgetBase");
dojo.require("com.ibm.pb.iwidget.control");
dojo.require("com.ibm.pb.layout.LayoutParser");

// not an iWidget, but uses the iWidget base for convenient provision of member variables
dojo.declare("com.ibm.pb.control.RenderingController", com.ibm.pb.iwidget.iWidgetBase, {	
	MINIMIZE_CLASS: "ibm-minimized",
	MAXIMIZE_CLASS: "ibm-maximized",
	HIDDEN_CONTAINER_NAME: "ibmHiddenWidgets",
	
	PRERENDER_BUFFER_ID: "ibmPreRenderBuffer",
	MAXIMIZE_BUFFER_ID: "ibmMaximizeBuffer",
	MAXIMIZE_PLACEHOLDER_ID: "ibmMaximizePlaceHolder",
	
	STYLE_METADATA: "colorPalette",
	INITIAL_STYLE_ID: "initial",
	PAGE_STYLE_METADATA: "pageStyle",
	PAGE_STYLE_ID: "ibmPageStyleLink",
	
	widgetAccessor: null,
	navStateModel: null,
	maximizeBuffer: null,
	preRenderBuffer: null,
	layoutContainers: null,
	activeElement: null,
	layoutParser: com.ibm.pb.layout.LayoutParser,
	
	modeWidgets: {},

	constructor: function(){
		var lgr = this.LOGGER, mname = "constructor";
		lgr.entering(mname, arguments);
		this.navStateModel = com.ibm.mashups.enabler.model.state.NavigationStateModelFactory.getNavigationStateModel();
		this.navModel = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel();
		this.evtSvc.subscribeEvent(this.iWidgetConstants.CHANGE_WIDGETWINDOWSTATE, this, this.onWidgetWindowStateChange);
		this.evtSvc.subscribeEvent("com.ibm.mashups.builder.preRenderProcess", this, this.onPreRenderProcess);
		this.evtSvc.subscribeEvent("com.ibm.mashups.builder.layoutRefresh", this, this.onLayoutRefresh);
		this.configService = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
		this.layoutContainers = dojo.byId("layoutContainers");
        // TODO: Why are the widgets not parsed when outside lotusFrame?  Issue with lotusMain usage is if client does not include it in their theme.
        this.preRenderBuffer = dojo.create("div", {id: this.PRERENDER_BUFFER_ID, style:{position: "absolute", top: "-9999px"} }, dojo.byId("lotusMain"), "last");
		this.layoutContainersDefaultY = parseInt(dojo.coords(this.layoutContainers).y);
		this.doCheckNewPage();
		lgr.exiting(mname, arguments);
	},
	boostrapPreRenderProcess: function(){
		// summary: used at bootstrap to process window states before rendering layout
		var mname="boostrapPreRenderProcess", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		try {
			/* START test for non-content node */
				var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
				var nm = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel();
				var page = nm.find(pageId).start();
				if(page && page.getNavigationElementType() == com.ibm.mashups.enabler.navigation.Constants.NAVIGATION_LABEL) {
					var defaultLabelContent = this.configService.getValue("labelDefaultRuntimeContent");
					if(defaultLabelContent) {
						if(dojo.isFunction(defaultLabelContent)) {
							defaultLabelContent = defaultLabelContent(pageId);
						}
						this.layoutContainers.innerHTML = defaultLabelContent;
						this.layoutContainers.className = "ibmLayoutContainers";
						lgr.exiting(mname, arguments);
						return;
					}
			  	} /* END test for non-content node */
			  	
		  		/* Normal content page handling */
				this._setActiveElement();
				
		        this.processControls(this.layoutContainers,true);
		        dojo.removeClass(this.layoutContainers, "ibmLayoutContainersHidden");
		        //page.primeServerRuntimeContent(this.layoutContainers);	// skip priming for now
		        if(this.activeElement){ 
			        this._setActiveElementFocus();
		        }	  			
		} catch (e) {
			console.log(e)
		}

		lgr.exiting(mname, arguments);
	},
	onPreRenderProcess: function(iEvent){
		// summary: used during page switches to process window states before rendering layout
		var mname="onPreRenderProcess", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		//this._setActiveElement(); has errors and halts execution in IE
		
		this.preRenderBuffer.innerHTML = iEvent.markup;
		
		var pageId = this.runtimeModel.getCurrentPage().getID();
		var page = this.navModel.find(pageId).start();
		
		// ************************************************
		// * THIS ACTIVATES PRIMING                       *
		// * YOU CAN DEACTIVATE IT IN THE CONFIGURATION   *
		if(ibmConfig["com.ibm.mashups.priming.enabled"] && 
			page.getNavigationElementType() == com.ibm.mashups.enabler.navigation.Constants.NAVIGATION_PAGE &&
			!iEvent.id && 
			this.runtimeModel.getCurrentPage().getPageMode() == com.ibm.mashups.enabler.model.state.PageModeAccessor.VIEW){
			// Priming is enabled, the page is a regular navigation node and we are doing a full layout refresh and we are in view mode
			console.info("activating Optimus Prime, savior of performance and killer of the Requesticons!");
			page.primeServerRuntimeContent(this.preRenderBuffer);			
		}
		// * END PRIMING ACTIVATION                        *
		// *************************************************
		
		this.processControls(this.preRenderBuffer,!iEvent.id);
		
		if(this.activeElement){ 
			this._setActiveElementFocus();
		}
		

		var transitionName = page.getMetaData('transition');
		var transitionFn = null;
		if (ibmCfg && ibmCfg.themeConfig && ibmCfg.themeConfig.transitions){
			if (transitionName){
				transitionFn = ibmCfg.themeConfig.transitions[transitionName];
			} else {
				transitionFn = ibmCfg.themeConfig.transitions[ibmCfg.themeConfig.transitions.fallback];
			}
		}
        this.evtSvc.broadcastEvent("com.ibm.mashups.builder.preRenderComplete", { "id": iEvent.id, "markupDivId":  this.PRERENDER_BUFFER_ID, "fn": transitionFn} ); 
		
        this.doChangeStyle();
		
		this.doCheckNewPage();
		
		lgr.exiting(mname, arguments);
	},
	processControls: function(node,addMaxBuffer){
		// summary: after markup has been set in pre-render buffer,
		//			the controls are processed
		// node: the node that contains the layout controls, 
		//		 all OOB cases it is the pre-render buffer
		// addMaxBuffer: true if the maximize buffer should be
		//		 		 created inside the given node
		//				 will be false when refreshing only a single
		//				 container or control
		var mname="processControls", lgr = this.LOGGER;
		lgr.entering(mname, arguments);

		var controls = new Array();
		var widgetAccessor = null;
        
        var pid = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
        var layoutAccessor = com.ibm.mashups.enabler.model.state.AccessorFactory.getLayoutAccessor(this.navStateModel, pid);
        var navstateTemplateUrl = layoutAccessor.getTemplateURL();

		// if there is no Navstate template URL 
		// or if it is different from the Instance template URL
		// then don't apply the widget size, they are invalid
        var honorNavStateWidgetSizes = false;
        if (navstateTemplateUrl) {
            // get the page instance layout template url
            var navNode = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel().find(pid).start(true);
            var content = navNode.getContent().start(true);
            var instanceTemplateUrl = content.getMetaData("com.ibm.portal.layout.template.ref");
            honorNavStateWidgetSizes = (navstateTemplateUrl == instanceTemplateUrl);
        }
		
		// clean the maximizeBuffer since we're processing fresh controls
		// reuse it when we refresh a single control (change skin, etc)
		if(this.maximizeBuffer && addMaxBuffer) {
			dojo.destroy(this.maximizeBuffer);
		}
		// create a new maximizeBuffer, hide it completely, and put it at the end of the container node we're processing
		// that way anything that's inside the maximizeBuffer will be included in any processing by other components
		// such as a live text invocation against the node we're processing
		if(addMaxBuffer) {
			this.maximizeBuffer = dojo.create("div", {id: this.MAXIMIZE_BUFFER_ID, style:{position: "absolute", top: "-9999px"} }, dojo.byId("lotusMain"), "last");
		}
		
		// restore the layoutContainers height if set by maximize
		if (this.layoutContainers.style.height.length > 0) {
			this.layoutContainers.style.height = "";
		}
		
		// disconnect the onresize event if it is connected
		if (this.onBrowserResizeHandler) {
			dojo.disconnect(this.onBrowserResizeHandler);
		}
		
		// gather window states of the controls
		dojo.forEach(
		    dojo.query(".component-control", node),
		    function(control){
				var cid = this.layoutParser.extractID(control);
				var id = cid; // we can assume that window ID and control ID are always the same, even in live preview
				widgetAccessor = com.ibm.mashups.enabler.model.state.AccessorFactory.getWidgetAccessor(this.navStateModel, id);
				controls.push({node: control, id: id, state: widgetAccessor.getWindowState(), size: widgetAccessor.getSize()});
		    }, 
			this);

		// process widget size
		if (honorNavStateWidgetSizes) {
	        dojo.forEach(
	                controls,
	                function(control){
	                    if(control.size){
	                        this._processWidgetSize(control.node, control.size);
	                    }
	                }, 
	                this);
		}

		// process minimized controls
		dojo.forEach(
		    controls,
		    function(control){
				if(control.state == "minimize"){
					this._processMinimize(control.node);
				}
		    }, 
			this);
		
		// process a maximized control
		dojo.forEach(
		    controls,
		    function(control){
				if(control.state == "maximize"){
					this._processMaximize(control.node, node);
				}
		    }, 
			this);
		
		// extend the sandboxed widget height to 100%
		dojo.forEach(
			dojo.query(".iw-iWidget[sandbox='true']", node),
			function(sandboxed){
				dojo.addClass(sandboxed,"extendediwiWidget");
			}
		);
		// #17713 memory leak for each DIV:component-control if not applied
		// remove the nodes that ref to the dom node that will be removed shortly
		for (var i=0, l=controls.length; i<l; i++) {
			controls[i].node = null;
			controls[i] = null;
		}
		controls.length = 0;
		
		lgr.exiting(mname, arguments);
	},
	addModeWidgets: function(mode, widgetIds) {
		if(!this.modeWidgets[mode]) {
			this.modeWidgets[mode] = [];
		}
		this.modeWidgets[mode].push.apply(this.modeWidgets[mode], widgetIds);
	},
	changePageMode: function(mode, cb) {
		var page = this.runtimeModel.getCurrentPage(),
			currentMode = page.getPageMode(), 
			me = this;
		if(mode && mode != currentMode) {
			var editModeLoadedAsynchronously = ( typeof ibmCfg.themeConfig.editModeJSURL !== "undefined" );
			var changeModeWork = function() {
				var widgetIds = me.modeWidgets[mode];
				var cmCbk = function() {
					me.evtSvc.broadcastEvent("com.ibm.mashups.builder.changePageMode", mode);
					page.setPageMode(mode);
					if(cb) {
						cb();
					}
				}
				
				if ( editModeLoadedAsynchronously && !me.editModeInitialized ) { 
						com.ibm.themes.PageBuilder2.initEditMode();
						me.editModeInitialized = true;
					}
				
				if(widgetIds && !widgetIds.loaded && widgetIds.length > 0) {
					
					var count = widgetIds.length,
						decrFunc = function() {
							if(--count <= 0) {
								widgetIds.loaded = true;
								cmCbk();
							}
						};
					dojo.forEach(widgetIds, function(widgetId){
						this.onLoad(widgetId, decrFunc);
					}, com.ibm.pb.iwidget.control);
				}
				else {
					cmCbk();
				}
			}
			
			if ( editModeLoadedAsynchronously && !this.editModeInitialized ) {
				dojo.io.script.get({
					url: ibmCfg.themeConfig.editModeJSURL,
					checkString: "com.ibm.pb.PB2EditModeLoadedFlag",
					load: changeModeWork
				});
			} else {
				changeModeWork();
			}
		}
	},
	onLayoutRefresh: function(iEvent){
		// summary: restore any widgets so that the layout can refresh normally
		//			if this is not done, the page area will show the layout twice after the user restores the widget
		//			layout is refreshed (for example) after adding content or changing the layout of a CSA page
		// iEvent: stores these values: [containerId]
		if(this.widgetAccessor && this.widgetAccessor.getWindowState() == this.widgetAccessor.MAX) {
			this.evtSvc.publishEvent(this.iWidgetConstants.CHANGE_WIDGETWINDOWSTATE, {id: this.widgetAccessor.wid, newWindowState: this.widgetAccessor.NORMAL, oldWindowState: this.widgetAccessor.MAX});
		}
	},
	onWidgetWindowStateChange: function(iEvent){
		// summary: process widget window state change (maximize, minimize, restore)
		// iEvent: stores these values: [id, newWindowState, oldWindowState]
		var mname="onWidgetWindowStateChange", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		this.widgetAccessor = com.ibm.mashups.enabler.model.state.AccessorFactory.getWidgetAccessor(this.navStateModel, iEvent.id);
		this.currentWidget = this.layoutParser.findById(iEvent.controlId);

		switch(iEvent.newWindowState){
			case this.widgetAccessor.MIN:
			  this.doMinimize(iEvent);
			  break;
			case this.widgetAccessor.MAX:
			  this.doMaximize(iEvent);
			  break;
			default:
			  this.doRestore(iEvent);
		}
		
		lgr.exiting(mname, arguments);
	},
	doMinimize: function(widgetInfo){
		var mname="doMinimize", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		this.widgetAccessor.setWindowState(this.widgetAccessor.MIN);
		if(widgetInfo.oldWindowState == this.widgetAccessor.MAX){
			this._removeMaximize(this.currentWidget);
		}
		this._processMinimize(this.currentWidget);
		
		this._updateNavStateModel(widgetInfo);
	
		lgr.exiting(mname, arguments);
	},
	doMaximize: function(widgetInfo){
		var mname="doMaximize", lgr = this.LOGGER;
		lgr.entering(mname, arguments);

		this.widgetAccessor.setWindowState(this.widgetAccessor.MAX);
		
		if(widgetInfo.oldWindowState == this.widgetAccessor.MIN){
			this._removeMinimize(this.currentWidget);
		}
		
		// need the _setActiveElement/_setActiveElementFocus pair because _processMaximize
		// manipulates the DOM (and potentially the focus to fix an IE bug) so that
		// the focus is the same after this call
		this._setActiveElement();
		this._processMaximize(this.currentWidget, this.layoutContainers);
		if(this.activeElement){ 
			this._setActiveElementFocus();
		}
		
		var lc = this._getLayoutControl(this.layoutParser.extractID(this.currentWidget));
		var css_style = lc.getMetadata("css-style");
		
		if(this.isFreeform() || css_style) {
			this._processMaximizeFreeform(this.currentWidget);
		}
		
		this._updateNavStateModel(widgetInfo);
		
		lgr.exiting(mname, arguments);
	},
	doRestore: function(widgetInfo){
		var mname="doRestore", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		this.widgetAccessor.setWindowState(this.widgetAccessor.NORMAL);

		if(widgetInfo.oldWindowState == this.widgetAccessor.MIN){
			this._removeMinimize(this.currentWidget);
		}else if(widgetInfo.oldWindowState == this.widgetAccessor.MAX){
			this._removeMaximize(this.currentWidget);			
		}
		
		var lc = this._getLayoutControl(this.layoutParser.extractID(this.currentWidget));
		var css_style_backup = lc.getMetadata("css-style-backup");
		
		if(this.isFreeform() || css_style_backup) {
			this._processRestoreFreeform(this.currentWidget);
		}
		
		this._updateNavStateModel(widgetInfo);
		
		lgr.exiting(mname, arguments);
	},
	doCheckNewPage: function(){
		// summary: go into edit mode automatically when switching to blank pages
		var mname="doCheckNewPage", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		if(ibmCfg.themeConfig.autoEditNewPages && dojo.query(".component-control",this.layoutContainers).length == 0){
			lgr.trace(mname, "Going into edit mode on a blank page");
			this.changePageMode("edit");
		}
		lgr.exiting(mname, arguments);
	},
	doChangeStyle: function(id){
		// summary: changes the styles on page switched
		// id: id of the page
		var mname="doChangeStyle", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		var nm = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel();
		var pid = id || com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		var pageRef = nm.find(pid);
		pageRef.setFinishedCallback(dojo.hitch(this, function(page){
			this._processPageStyle(page.getMetaData(this.PAGE_STYLE_METADATA));
			var contentRef = page.getContent();
			contentRef.setFinishedCallback(dojo.hitch(this, function(content){
				this._findAggregatedColorPalette(nm, content, page);
			}));
			contentRef.start(false);
		}));
		pageRef.start(false);
		
		lgr.exiting(mname, arguments);
	},
	_processPageStyle: function(url){
		// summary: changes the assigned page style
		// url: url to page stylesheet
		var mname="_processPageStyle", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		var link = dojo.byId(this.PAGE_STYLE_ID),
			head = document.getElementsByTagName("head")[0];
		if(link){
			head.removeChild(link);
			delete link;
		}
		if(url){
			var newLink = dojo.create("link",{type:"text/css",id:this.PAGE_STYLE_ID,rel:"stylesheet",href:url,media:"screen"});
			head.appendChild(newLink);
		}
		
		lgr.exiting(mname, arguments);
	},
	_processColorPalette: function(url){
		// summary: changes the color palette style from the shelf on the page
		// url: url to color palette stylesheet
		var mname="_processColorPalette", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		dojo.require("com.ibm.pb.customize.enabler.ChangeStyleController");
		var csc = com.ibm.pb.customize.enabler.ChangeStyleController;
		var title = this.INITIAL_STYLE_ID;
		if(url){
			title = url.substring(url.lastIndexOf("/")+1);
		}
		dojo.hitch(csc, "_switchStylesheet", title, url)();
		ibmCfg.portalConfig.aggregatedStyle = url;
		dojo.publish("com.ibm.pb.customize.pageStyleChanged",[url]);  // alert the customize shelf of the style change
		
		lgr.exiting(mname, arguments);
	},
	_findAggregatedColorPalette: function(navModel, content, page){
		// summary: climbs the navigation model to find the aggregated color palette page style
		// navModel: the navigation model
		// content: the content of the current page on which the color palette is set, or a label
		// page: the page associated with the content, or a label
		var style = content.getMetaData(this.STYLE_METADATA);
		if(style) {
			this._processColorPalette(style);
		} else {
			var parentDef = navModel.getParent(page);
			parentDef.setFinishedCallback( dojo.hitch(this, function(parent){
				if(!parent){
					this._processColorPalette(null);
				} else if(parent.getContent){
					var contentDef = parent.getContent();
					contentDef.setFinishedCallback( dojo.hitch(this,function(content){
						this._findAggregatedColorPalette(navModel, content, parent);
					}));
					contentDef.start(false);
				} else {  // parent is not a page so does not have a corresponding content node
					this._findAggregatedColorPalette(navModel, parent, parent);
				}
			}));
			parentDef.start(false);
		}
	},
	_processWidgetSize: function(node, size){
		if (size.w) {
			dojo.style(node, "width", size.w);
		}
		if (size.h) {
			dojo.style(node, "height", size.h);
		}
	},
	_processMinimize: function(currentWidget){
		// summary: minimize a widget
		// currentWidget: widget to minimize
		var mname="_processMinimize", lgr = this.LOGGER;
		lgr.entering(mname, arguments);

		var widgetID = this.layoutParser.extractID(currentWidget);
		var minimizeBufferDiv = dojo.byId(widgetID+"_minimizebuffer");
		if (!minimizeBufferDiv) {
    		minimizeBufferDiv = dojo.create("div", {id: widgetID+"_minimizebuffer", style:{display: "none"} }, currentWidget, "first");
    	}
		dojo.style(minimizeBufferDiv, "height", currentWidget.style.height);
		dojo.addClass(currentWidget, this.MINIMIZE_CLASS);
		dojo.style(currentWidget, "height", null);

		lgr.exiting(mname, arguments);
	},
	_processMaximize: function(currentWidget, container){
		// summary: make a widget maximized and 
		//			store other controls in maximized buffer
		// currentWidget: widget to maximize
		// container: the container to place the maximized widget
		var mname="_processMaximize", lgr = this.LOGGER;
		lgr.entering(mname, arguments);

		var containerHeight = dojo.contentBox(container).h;
		// find the maximize buffer in the container
		this.maximizeBuffer = dojo.byId(this.MAXIMIZE_BUFFER_ID);
		
		dojo.addClass(currentWidget, this.MAXIMIZE_CLASS);
		dojo.create("div", {id: this.MAXIMIZE_PLACEHOLDER_ID, style:{display: "none"} }, currentWidget, "after");
		if(dojo.isIE){
			// IE has a bug where moving an ancestor node of the current focused node
			// causes the focus to mess up on things like text input fields such that
			// even though they appear focused, the user cannot type in them.
			// The fix is to focus another node before manipulating the DOM of the ancestor.
			// In both doMaximize and boostrapPreRenderProcess we save the current focus
			// and then set it back afterwards.
			container.focus();
		}
		// #19446 we also need to consider the resizer
		var nodes = dojo.query("> DIV", container);
		for(var i=0,j=nodes.length; i<j; i++) {
			var node = nodes[i];
			if(dojo.hasClass(node, "component-container") || dojo.hasClass(node, "component-container-resizer")) {
				this.maximizeBuffer.appendChild(node);
			}
		}
		dojo.place(currentWidget, container, "last");	
		
		// # 18798
		this._maximizeWidget(currentWidget, true);
		this.onBrowserResizeHandler = dojo.connect(window, "onresize", this, "_onBrowserResize");
		
		lgr.exiting(mname, arguments);
	},
	
	_onBrowserResize: function() {
		//console.log("_onBrowserResize");
		var maxWidget = dojo.query("."+this.MAXIMIZE_CLASS, this.layoutContainers)[0];
		this._maximizeWidget(maxWidget, false);
	},
	
	_maximizeWidget: function(currentWidget, keepOldWidgetHeight) {
		var containerY = dojo.coords(this.layoutContainers).y;
		if (containerY < 0) {
			containerY = this.layoutContainersDefaultY;
		}
		var windowH = this._getWindowHeight();
		var containerHeight = windowH - containerY;
		this.layoutContainers.style.height = containerHeight + "px";
		
		var controlHeight = dojo.contentBox(currentWidget).h;
		var widgetBody = dojo.query(".lotusWidgetBody", currentWidget)[0];
		var bodyBox = dojo.contentBox(widgetBody);
		var bodyHeight = bodyBox.h;
		if (keepOldWidgetHeight) {
			if (bodyHeight > 0) {
				this.maxWidgetStore = {controlHeight: controlHeight+"px", bodyHeight: bodyHeight+"px"};
				
			}
		}
		var maxControlHeight = containerHeight - 25;
		var maxBodyHeight;
		if ((bodyHeight > 0) || (!this.maxWidgetStore)) {
			maxBodyHeight = maxControlHeight - (controlHeight - bodyHeight);
		}
		else {
			maxBodyHeight = maxControlHeight - (parseInt(this.maxWidgetStore["controlHeight"]) - parseInt(this.maxWidgetStore["bodyHeight"]));
		}
		
		currentWidget.style.height = maxControlHeight + "px";
		widgetBody.style.height = maxBodyHeight + "px";
		
		var changes = {newHeight: maxBodyHeight,newWidth:bodyBox.w};
		var widgetID = this.layoutParser.extractID(currentWidget);
		this.evtSvc.publishEvent(com.ibm.mashups.iwidget.Constants.RESIZE_WIDGET + "." + widgetID, changes);
	},
	
	_getWindowHeight: function() {
		if (dojo.isIE) {
			return document.documentElement.offsetHeight || document.body.offsetHeight;
		} else {
			return window.innerHeight;
		}
	},
	
	_removeMinimize: function(widget){
		var mname="_removeMinimize", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		var widgetID = this.layoutParser.extractID(widget);
		var minimizeBufferDiv = dojo.byId(widgetID+"_minimizebuffer");
		if (minimizeBufferDiv) {
			dojo.style(widget, "height", minimizeBufferDiv.style.height);
			dojo.style(minimizeBufferDiv, "height", null);
		}
		dojo.removeClass(widget, this.MINIMIZE_CLASS);
		
		lgr.exiting(mname, arguments);
	},
	_removeMaximize: function(widget){
		var mname="_removeMaximize", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		dojo.removeClass(widget, this.MAXIMIZE_CLASS);
		var placeHolder = dojo.byId(this.MAXIMIZE_PLACEHOLDER_ID);
		if(placeHolder){
			placeHolder.parentNode.replaceChild(widget, placeHolder);
			var nodes = dojo.query("> DIV", dojo.byId(this.MAXIMIZE_BUFFER_ID));
			for (var i = 0, j = nodes.length; i < j; i++) {
				var node = nodes[i];
				if (dojo.hasClass(node, "component-container") || dojo.hasClass(node, "component-container-resizer")) {
					this.layoutContainers.appendChild(node);
					if (dojo.hasClass(node, "component-container")) {
						dojo.publish("/com/ibm/mashups/livetext/livetextchanged", [node, true]);
					}
				}
			}
		}
		
		// #18798
		this.layoutContainers.style.height = "";
		widget.style.height = this.maxWidgetStore["controlHeight"];
		var widgetBody = dojo.query(".lotusWidgetBody", widget)[0];
		widgetBody.style.height = this.maxWidgetStore["bodyHeight"];
		dojo.disconnect(this.onBrowserResizeHandler);
		
		lgr.exiting(mname, arguments);
	},
	_setActiveElement: function(){
		if(document.activeElement){ 
			this.activeElement = document.activeElement; 
		}
	},
	_setActiveElementFocus: function(){
		if (this.activeElement.nodeName == "IFRAME") {
			return;  // do not try to set focus on an iframe, avoid "Permission denied" errors in IE
		}
		this.activeElement.focus(); 
		this.activeElement = null;
	},
	_updateNavStateModel: function(widgetInfo){
		var mname="_updateNavStateModel", lgr = this.LOGGER;
		lgr.entering(mname, arguments);	

		this.evtSvc.publishEvent(com.ibm.mashups.iwidget.Constants.WIDGET_WINDOWSTATECHANGED, {id: widgetInfo.id, oldWindowState: widgetInfo.oldWindowState, newWindowState: widgetInfo.newWindowState});			
		this.navStateModel.commit().start();

		lgr.exiting(mname, arguments);
	},
	
	////////////////// For freeform /////////////////////////////////////
	isFreeform: function() {
		return dojo.query(".ibmFreeform", this.layoutContainers).length > 0 ? true : false;
	},
	
	_getLayoutControl: function(id) {
		var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		var navModel = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel();
		var pageNode = navModel.find(pageId).start();
		var layoutModel = navModel.getLayoutModel(pageNode);
		return layoutModel.find(id).start();
	},
	
	_processMaximizeFreeform: function(currentWidget) {
		var mname="_processMaximizeFreeform", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		// backup the original style metadata
		var widgetID = this.layoutParser.extractID(currentWidget);
		var lc = this._getLayoutControl(widgetID);
		var css_style = lc.getMetadata("css-style");
		// no wrapper-css-style in the latest build
		lc.removeMetadata("css-style");
		lc.setMetadata("css-style-backup", css_style);
		
		// remove the absolute positions
		currentWidget.style.position = "";
		currentWidget.style.left = "";
		currentWidget.style.top = "";
		currentWidget.style.height = "";
		currentWidget.style.width = "";
		
		lgr.exiting(mname, arguments);
	},
	
	_processRestoreFreeform: function(currentWidget) {
		var mname="_processRestoreFreeform", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		// restore the original style metadata
		var widgetID = this.layoutParser.extractID(currentWidget);
		var lc = this._getLayoutControl(widgetID);
		var css_style = lc.getMetadata("css-style-backup");
		lc.removeMetadata("css-style-backup");
		lc.setMetadata("css-style", css_style);
		if (css_style) {
			// restore the original style to dom node
			if(this.isFreeform())
				dojo.style(currentWidget, "position", "absolute");
			var left = this._extractStyle(css_style, "left");
			if (left) 
				dojo.style(currentWidget, "left", left);
			var top = this._extractStyle(css_style, "top");
			if (top) 
				dojo.style(currentWidget, "top", top);
			var height = this._extractStyle(css_style, "height");
			if (height) 
				dojo.style(currentWidget, "height", height);
			var width = this._extractStyle(css_style, "width");
			if (width) 
				dojo.style(currentWidget, "width", width);
		}
		lgr.exiting(mname, arguments);
	},
	
	_extractStyle: function(styleString, styleName) {
		var start = styleString.indexOf(styleName);
		if(start > -1) {
			var end = styleString.indexOf(";", start);
			var style2 = styleString.substring(start+styleName.length+1, end);	// eg. height: 123px;
			return dojo.trim(style2);
		} else {
			return null;
		}
	}
	////////////////// For freeform /////////////////////////////////////
});

(function(){
	var controller = null;
	com.ibm.pb.control.RenderingController.getController = function() {
		if(!controller) {
			controller = new com.ibm.pb.control.RenderingController();
		}
		return controller;
	};
})();
