dojo.provide("com.ibm.pb.themes.commonInit");

dojo.require("com.ibm.pb.control.RenderingController");
dojo.require("com.ibm.pb.control.NavigationController");
dojo.require("com.ibm.pb.control.DNDController");
dojo.require("com.ibm.pb.iwidget.PageModeWidgetHandler");
dojo.require("com.ibm.pb.data.NavigationPrimer");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.bundles", "SharedMessages");

com.ibm.pb.themes.commonInit = function(initArgs) {
    if (!initArgs) {
        initArgs = {};
    }
    
    var lgr = com.ibm.mashups.enabler.logging.Logger.getLogger("com.ibm.pb.themes.commonInit"), mname = "commonInit", sharedMessages = dojo.i18n.getLocalization("com.ibm.bundles", "SharedMessages");
    lgr.entering(mname);
	
	ibmCfg.portalConfig.isRTL = ibmCfg.themeConfig.isRTL;
	if (ibmCfg.themeConfig.isRTL){
		dojo.addClass(dojo.body(), "mumrtl");
		dojo.addClass(dojo.body(), "lotus_rtl");
	}
    
    /* Prime the navigation model if enabled */
    if (ibmConfig.primeNavigation && initArgs.navPrimingContainers) {
        lgr.trace(mname, "Priming navigation model");
        var np = new com.ibm.pb.data.NavigationPrimer();
        dojo.forEach(initArgs.navPrimingContainers, np.parse, np);
        np.finalize();
        ibmCfg.themeConfig.navigationPrimed = true;
    }
    
    var evtSvc = com.ibm.mashups.iwidget.services.ServiceManager.getService("eventService");
    
    lgr.trace(mname, "Initializing navigation model and HiddenMetaDataLoadingStrategy");
    /* Initialize the theme's HiddenMetaDataLoadingStrategy on the navigation model */
    var nm = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel(), hiddenMdStrategy = new com.ibm.mashups.enabler.strategy.HiddenMetaDataLoadingStrategy(ibmCfg.themeConfig.navHiddenMetadata);
    nm.addStrategy(hiddenMdStrategy);
    
    // TODO, re-enable in the performance APAR
//    lgr.trace(mname, "Initializing page load ahead strategy");
//    var pageLoadAheadStrategy = new com.ibm.mashups.enabler.strategy.PageLoadAheadStrategy(true, false, false);
//    nm.addStrategy(pageLoadAheadStrategy);
    
    lgr.trace(mname, "Initializing navigation node to content node meta data sync strategy");
    var syncStrategy = new com.ibm.mashups.enabler.strategy.SyncMetaDataStrategy();
    nm.addStrategy(syncStrategy);
    
    var currentPageId = null;
    var testPage = null;
    var redirect = false;
    var statehash = null;

    // 1, get page ID from cookie, which is passed when login
    // IT IS IMPORTANT THAT THIS CODE RUNS BEFORE THE NAVIGATION STATE MODEL IS FETCHED.
    if(dojo.cookie("url_pid")) {
        statehash = dojo.cookie("url_pid");
        dojo.cookie("url_pid", null, {expires: -1, path: "/"});
        window.location.hash = "#"+encodeURIComponent(statehash);
        
        /*
        // 2, try to get page from navigation model
        testPage = nm.find(currentPageId).start();  // 404 if not found
        if(!testPage) {
            // 3, try to get page from shared navigation model
            if (currentPageId !== ibmPortalConfig.currentPageOID) {
                var snm = com.ibm.mashups.enabler.navigation.Factory.getSharedNavigationModel();
                var sharedPage = snm.find(currentPageId).start(true);   // 404 if not found
                if (sharedPage && !snm.isAccepted(sharedPage)) {
                    var sharedPageRoot = nm.findDefaultAcceptParent().start(true);
                    if (sharedPageRoot) {
                        // auto accept the page
                        nm.insert(sharedPage, sharedPageRoot);
                        nm.commit().start(true);
                        testPage = sharedPage;
                    }
                }
            }
        }
        if (!testPage)
            redirect = true;
        */
    }
    
    /* Initalize the builder API RuntimeModel using the SPIs using the current context */
    lgr.trace(mname, "Initializing builder RuntimeModel");
    var navStateModel = com.ibm.mashups.enabler.model.state.NavigationStateModelFactory.getNavigationStateModel();
    var pageAccessor = com.ibm.mashups.enabler.model.state.AccessorFactory.getPageAccessor(navStateModel);
    var spaceAccessor = com.ibm.mashups.enabler.model.state.AccessorFactory.getSpaceAccessor(navStateModel);
    
    // 2, get the pid from the nav state
    // if statehash is definied, then the server or login form is telling us via the url_pid
    // cookie where we should be directed.  we also want to execute this code block
    // if we need to go to the last page visited on login
    var goToLastPageVisited = com.ibm.mm.builder.utils.navigationUtil.isLastPageWhenLogin();
    if (statehash || goToLastPageVisited) {
        currentPageId = pageAccessor.getPageID();
        if (!currentPageId) {
            currentPageId = ibmPortalConfig.currentPageOID;
            lgr.trace(mname, "Could not retrieve selected page from navigation state, retrieving from bootstrap config: ${0}", [currentPageId]);
            if ("-1" !== currentPageId) {
                pageAccessor.setPageID(currentPageId);
            }
        }
        if ("-1" !== currentPageId) {
            testPage = nm.find(currentPageId).start();
        }
    }
    
    // 3, try to get the page from shared navigation
    // this will take effect after you login, in this case cookie:url_pid is not exist
    if(!testPage && currentPageId && goToLastPageVisited) {
        if (currentPageId !== ibmPortalConfig.currentPageOID) {
            redirect = true; // the currentPageId that was retrieved by now could not be navigated to, so we definitely are redirected now
            var snm = com.ibm.mashups.enabler.navigation.Factory.getSharedNavigationModel();
            var sharedPage = snm.find(currentPageId).start(true);   // 404 if not found
            if (sharedPage && !snm.isAccepted(sharedPage)) {
                var sharedPageRoot = nm.findDefaultAcceptParent().start(true);
                if (sharedPageRoot) {
                    // auto accept the page
                    nm.insert(sharedPage, sharedPageRoot);
                    nm.commit().start(true);
                    testPage = sharedPage;
                }
            }
        }
    }
    
    // 4, home space - bspace case/overridden navstate preprocess
    if (!testPage) {
        testPage = com.ibm.mm.builder.utils.navigationUtil.gotoHomeSpace();
        if (testPage) {
            if ((currentPageId) && ("-1" != currentPageId)) {
                // if a currentPageId has been set above it means that we could not navigate to it. therefore we will now be redirected
                redirect = true;
            }
            currentPageId = testPage.getID();
            pageAccessor.setPageID(currentPageId);
        }
    }
    
    // 5, need to check whether the page or the containing space is hidden
    // this check is ommitted if the current user is the space owner (#17153)
    var spaceModel = com.ibm.mashups.enabler.space.Factory.getSpaceModel();
// 20070   var _sID = spaceAccessor.getSpaceID();
// 20070	if ((testPage) && (!_sID)) { // maybe the navState isn't setup correctly and this page really is part of a space
// 20070    	var spaceNode = spaceModel.findSpaceByNavigation(testPage).start(true);
// 20070   	if (spaceNode) {
// 20070       	_sID = spaceNode.getID();
// 20070   	}
// 20070	}
// 20070
// 20070    if(_sID) {
// 20070        var space = spaceModel.find(_sID).start();
// 20070       if(space) { // we are in space instead of in 'My Mashup Apps'
// 20070           if (!space.hasRole(com.ibm.mashups.enabler.ac.RoleType.MANAGER)) {
// 20070               if(testPage.isHidden() || space.isHidden()) {
// 20070                   testPage = null;
// 20070                   spaceAccessor.setSpaceID(nm.getRoot().start().getID());
// 20070               }
// 20070           }
// 20070       }
// 20070   }
    
    // 7, fall back to first child of navigation root page or root page itself
    if (!testPage) {
        var root = nm.getRoot().start(true);
        testPage = nm.getChildren(root).next() || root;
        if ((currentPageId) && ("-1" != currentPageId)) {
            // if a currentPageId has been set above it means that we could not navigate to it. therefore we will now be redirected
            redirect = true;
        }
    }

    // 7.5, Special case where we check whether we have selected a label even though there are child pages. If this is the case we select the first child
    if (testPage) {
        if (testPage.getNavigationElementType() === com.ibm.mashups.enabler.navigation.Constants.NAVIGATION_LABEL) {
            testPage = nm.getChildren(testPage).next() || testPage;
        }
    }

    // 8, set status in the end
	if (testPage) {
		currentPageId = testPage.getID();
	}
    // get a new page accessor if the current page is part of a space
    var currentSpaceId = spaceAccessor.getSpaceID();
    // now we need to fix the Navigational State since the navigational state doesn't know whether the page which was set is a page inside of a space
    var spaceNode = spaceModel.findSpaceByNavigation(testPage).start(true);
    if (spaceNode) {
        if (spaceNode.getID() != currentSpaceId) {
            currentSpaceId = spaceNode.getID();
            spaceAccessor.setSpaceID(currentSpaceId);
        }
        pageAccessor = com.ibm.mashups.enabler.model.state.AccessorFactory.getPageAccessor(navStateModel, currentSpaceId);
    }
    else {
        spaceAccessor.setSpaceID(); // unset
        pageAccessor = com.ibm.mashups.enabler.model.state.AccessorFactory.getPageAccessor(navStateModel);
    }
    pageAccessor.setPageID(currentPageId);
	navStateModel.commit({addToHistory: true}).start();
    
    var runtimeModel = com.ibm.mashups.builder.model.Factory.getRuntimeModel();
    var currentPage = runtimeModel.getCurrentPage();
    currentPage.setID(currentPageId);
    currentPage.setPageMode("view");
    runtimeModel.getCurrentTheme().setID(ibmCfg.themeConfig.themeUniqueName);
	
	/* Initialize the theme's HiddenMetaDataLoadingStrategy on the layout model, needed to see container and control names */
	var strategyService = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.StrategyService.SERVICE_NAME);
	var s = new com.ibm.mashups.enabler.strategy.HiddenMetaDataLoadingStrategy(ibmCfg.themeConfig.layoutHiddenMetadata);
	strategyService.setStrategy("com.ibm.mashups.enabler.layout.LayoutModel", s);
    
    /* Set the browser window title to match the current page */
    lgr.trace(mname, "Setting window title with current page title");
    nm.find(currentPageId).setFinishedCallback(function(res, status) {
        if (status != 200) {
            lgr.severe(mname, sharedMessages.nav_renderingError_0);
            dojo.publish("/portal/status/theme", ["error", sharedMessages.nav_renderingError_0, sharedMessages["nav_renderingError_0.useraction"]]);
            if (status == "404" || status == 404 || status == 0) {
                // this page no longer exists
                window.location = window.location.protocol + "//" + window.location.host + ibmCfg.portalConfig ? ibmCfg.portalConfig.portalURI : ibmConfig["com.ibm.mashups.contextroot"];
            }
        }
        else if (initArgs.setWindowTitle) {
            var title = "";
            var currentPageNode = res;
			if(currentPageId == nm.getRoot().start().getID()) {	// Root naviagtion
				title = ibmCfg.enablerConfig["com.ibm.mashups.productname"];
			} else {
				var localizedContext = com.ibm.mashups.enabler.context.Factory.getLocalizedContext();
            	var currentPageTitle = localizedContext.getTitle(currentPageNode);
            	if (dojo.isString(ibmCfg.themeConfig.portletOverridePageTitle) && ibmCfg.themeConfig.portletOverridePageTitle != currentPageTitle) {
                	title = ibmCfg.themeConfig.portletOverridePageTitle.replace(/&amp;/g, "&").replace(/&quot;/g, '"').replace(/&gt;/g, ">").replace(/&lt;/g, "<");
            	}
            	else {
                	title = currentPageTitle;
            	}
			}
            com.ibm.mashups.enabler.context.Factory.getPageContext().setBrowserTitle(title);
        }
    }).start();
    
    if (initArgs.useNavigationController) {
        /* Initialize the navigation controller */
        lgr.trace(mname, "Initializing navigation controller");
        ibmCfg.controllers.navigation = com.ibm.pb.control.NavigationController.getController();
    }
    
    if (initArgs.useRenderingController) {
        /* Initialize the rendering controller */
        lgr.trace(mname, "Initializing rendering controller");
        ibmCfg.controllers.rendering = com.ibm.pb.control.RenderingController.getController();
        ibmCfg.controllers.rendering.boostrapPreRenderProcess();
    }
    
    if (initArgs.useDNDController) {
        lgr.trace(mname, "Initializing page layout drag-and-drop controller");
        /* Initialize the drag-and-drop controller for page layout DND actions */
        ibmCfg.controllers.dnd = com.ibm.pb.control.DNDController.getController();
    }
    
    if (initArgs.serverRuntimeContentPrimingNode) {
    
    }
    
    if (initArgs.customInit) {
        initArgs.customInit();
    }

    if (initArgs.initLiveTextService) {
        lgr.trace(mname, "Initializing live text service");
        var liveTextInitArgs = initArgs.liveTextInitArgs || {}, livetextService = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.livetext.ServiceModel.SERVICE_NAME);
        if (initArgs.lazyLoadModeWidgets) {
            var pageModeWidgetHandler = new com.ibm.pb.iwidget.PageModeWidgetHandler({
                renderingController: ibmCfg.controllers.rendering,
                matchingModes: ["edit"]
            });
            /* The pageModeWidgetHandler picks up widgets that are flagged to work in certain modes and prepares them to be lazy loaded until that mode is entered */
            liveTextInitArgs.preProcessCallbackFunc = dojo.hitch(pageModeWidgetHandler, "preProcessCallbackFunc");
            liveTextInitArgs.postProcessCallbackFunc = dojo.hitch(pageModeWidgetHandler, function() {
				pageModeWidgetHandler.postProcessCallbackFunc(arguments);
				// #14071 display message if the page is redirected
				if (redirect) {
					var msgTitle = com.ibm.mm.builder.coreWidgetsStrings.I_NAVIGATION_REDIRECT_TITLE ? com.ibm.mm.builder.coreWidgetsStrings.I_NAVIGATION_REDIRECT_TITLE : "You have been redirected";
					var msgBody = com.ibm.mm.builder.coreWidgetsStrings.I_NAVIGATION_REDIRECT_TITLE_MESSAGE ? com.ibm.mm.builder.coreWidgetsStrings.I_NAVIGATION_REDIRECT_TITLE_MESSAGE : "You can't visit the page, because it has been deleted or you do not have authority.";
					var statusMessage = new com.ibm.widgets.StatusMessage("info", msgTitle, msgBody);
					dojo.publish("/message/status",[{message: statusMessage, uid: "ibmStatusBox"}]);
				}
			});
        }
        /* Initialize the live text service, which will immediately parse the page for semantic content */
        livetextService.init(liveTextInitArgs);
    }

	if(typeof(stproxy) != "undefined" && !stproxy.isLoggedIn && 
		typeof(ibmCfg.portalConfig) != "undefined" && typeof(ibmCfg.portalConfig.currentUser) != "undefined" && ibmCfg.portalConfig.currentUser.length > 0) {
		lgr.trace(mname, "Logging " + ibmCfg.portalConfig.currentUser + " onto Sametime");
		stproxy.login.loginByToken(ibmCfg.portalConfig.currentUser, stproxy.awareness.AVAILABLE, stproxy.i18nStrings.statusAvailable, null, function() {
				lgr.severe(mname, "Sametime login for " + ibmCfg.portalConfig.currentUser + " failed.");
		});
	}
}
