/* Copyright IBM Corp. 2011  All Rights Reserved.                    */
dojo.provide("com.ibm.mm.widget.dataViewer._WB_VER_.v210.ColumnManager");

dojo.declare("com.ibm.mm.widget.dataViewer._WB_VER_.v210.ColumnManager", null, {
	
	settingMap: null,
	
	history: null,
	
	maxHistoryVolumn: 5,
	
	constructor: function(args) {
		this.settingMap = {};
		this.history = [];
		
		this.reload(args);
	},
	
	reload: function(args) {
		if (args) {
			if (dojo.isString(args)) {
				args = dojo.fromJson(args);
			}
			dojo.mixin(this, args);
		}
	},
	
	/**
	 * add column setting
	 * @param {Object} source {feedURL | [source , name]}
	 * @param {Object} columnSetting
	 */
	add: function(source, columnSetting) {
		var id = this._generateSourceId(source);
		if (id) {
			this.settingMap[id] = columnSetting;
			
			var index = dojo.indexOf(this.history, id);
			if (index > -1) {
				this.history.splice(index, 1);
			}
			this.history.push(id);
			
			if (this.history.length > this.maxHistoryVolumn) {
				// remove the oldest setting
				delete this.settingMap[this.history.shift()];
			}
		}
	},
	
	get: function(source) {
		return this.settingMap[this._generateSourceId(source)];
	},
	
	toJson: function() {
		var obj = {};
		obj.settingMap = this.settingMap;
		obj.history = this.history;
		return dojo.toJson(obj);
	},
	
	fromJson: function(jsonString) {
		if (!jsonString || !dojo.isString(jsonString)) {
			this.settingMap = {};
			this.history = [];
		} else {
			try {
				var obj = dojo.fromJson(jsonString);
				dojo.mixin(this, obj);
			} catch (e) {
				this.settingMap = {};
				this.history = [];
			}
		}
	},
	
	_generateSourceId: function(source) {
		if (source) {
			if (source.sourceId) {
				return source.sourceId;
			}
			if (source.feedURL) {
				var url = source.feedURL;
				var queryIndex = source.feedURL.indexOf("?");
				if (queryIndex > -1) {
					url = url.substring(0, queryIndex);
				}
				return url;
			} else if (source.source && source.name) {
				return source.source + "#" + name;
			}
		}
		return "";
	}
});
