/** Licensed Materials - Property of IBM, 5724-U69, (C) Copyright IBM Corp. 2009, 2010 - All Rights reserved.  **/
dojo.provide("com.ibm.themes.PageBuilder2.theme.edit");
if(!dojo._hasResource["com.ibm.pb.decorations.DirectionalMoveControls"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.decorations.DirectionalMoveControls"] = true;
dojo.provide("com.ibm.pb.decorations.DirectionalMoveControls");






/**
 * Provides controls to move a node between containers or between siblings in its own container
 */
dojo.declare("com.ibm.pb.decorations.DirectionalMoveControls", [com.ibm.mashups.builder.model.ContextMenuLoader, com.ibm.pb.logging.Loggable], {
	
	layoutParser: com.ibm.pb.layout.LayoutParser,
	
	directions: ["up", "down", "left", "right"],
	contentsQuery: ">",
	containerQuery: "div",
		
	constructor: function(args) {
		if(args) {
			dojo.safeMixin(this, args);
		}
	},
	getItems: function(/*String*/ contextMenuId, /*String*/ targetResourceId, /*String*/ targetResourceType, /*Object*/ someObject) {
		var mname = "getItems", lgr = this.LOGGER, me = this;
		lgr.entering(mname, arguments);
		dfd = new com.ibm.mm.builder.DeferredImpl(this, function(deferred, sync, params) {
			var callBack = deferred.getFinishedCallback(), callBackParams = deferred.getFinishedCallbackParameters(),
				ret = this.getControls(targetResourceId);
			if(callBack) {
				callBack(ret, 200, callBackParams);
			}
			return ret;
		});
		lgr.exiting(mname, arguments);
		return dfd;
	},
	getControls: function(node) {
		var ret = [], ordinal = 1;
		node = this.layoutParser.findById(node);
		if(node) {
			var nodeContainer = this.getContainer(node);
			var isFreeformLayoutContainer = dojo.hasClass(nodeContainer, "ibmFreeform");
			dojo.forEach(this.directions, function(dir){
				if(isFreeformLayoutContainer){
					var container = this.getContainerByDirection(node, dir);
					if(container){
						ret.push(dojo.mixin(this._getItem(dir), {
						ordinal: ordinal++,
						actionCallBackFunction: this.getMoveAction(node, container, sibling, dir)
					}));
					}
				}else{
					var sibling = this.getSiblingByDirection(node, dir), container = null;
				if(!sibling) {
					container = this.getContainerByDirection(node, dir);
				}
				if(sibling || container) {
					ret.push(dojo.mixin(this._getItem(dir), {
						ordinal: ordinal++,
						actionCallBackFunction: this.getMoveAction(node, sibling ? nodeContainer : container, sibling, dir)
					}));
				}
				}
			}, this);
		}
		return ret;
	},
	_getItem: function(dir) {
		return {
			id: "DirectionalMove:" + dir,
			localized: new com.ibm.pb.localized.DojoLocalized({
				bundlePackage: "com.ibm.bundles",
				bundleName: "Theme",
				bundleKey: "move_" + dir
			}),
			enabled: true
		};
	},
	nextElementSibling: function(node) {
		var next = node && node.nextSibling;
		if(!next) {
			return null;
		}
		else if(next.nodeType != 1) {
			return this.nextElementSibling(next);
		}
		else {
			return next;
		}
	},
	getPostMoveAction: function(node, parentNode, nextSibling) {
		return function(){};
	},
	onMove: function(node, refNode, position) {
		this.onChange();
	},
	moveNode: function(node, refNode, position) {
		dojo.place(node, refNode, position);
		this.onMove(node, refNode, position);
	},
	getMoveAction: function(childNode, parentNode, siblingNode, dir) {
		var mname = "getMoveAction", lgr = this.LOGGER, pos = "last", func, postMove, me = this, refNode;
		if(siblingNode && siblingNode.parentNode == parentNode) {
			// if we're here, we're just swapping node positions, we don't use dir explicitly
			// because it has already been calculated by examining nodes in the DOM
			pos = this._isBefore(childNode, siblingNode) ? "after" : "before";
			postMove = this.getPostMoveAction(childNode, parentNode, pos == "after" ? this.nextElementSibling(siblingNode) : siblingNode);
			lgr.trace(mname, "Get action to move ${0} ${1} ${2}", childNode, pos, siblingNode);
			refNode = siblingNode;
		}
		else {
			lgr.trace(mname, "Get action to move ${0} to ${1} of ${2}", childNode, pos, parentNode);
			postMove = this.getPostMoveAction(childNode, parentNode);
			refNode = parentNode;
		}
		return function(){
			me.moveNode(childNode, refNode, pos);
			postMove();
		};
	},
	_isBefore: function(node, sibling) {
		var idx = this._childIdx(node),
			sidx = this._childIdx(sibling);
		return idx < sidx;
	},
	_childIdx: function(node) {
		return dojo.indexOf(node.parentNode.childNodes, node);
	},
	isLTR: function() {
		if(this._isLTR !== undefined) {
			this._isLTR = (dojo.body().dir || dojo.doc.documentElement.dir || "ltr").toLowerCase() == "ltr";
		}
		return this._isLTR;
	},
	getSiblingByDirection: function(node, direction) {
		var mname = "getSiblingByDirection", lgr = this.LOGGER;
		var siblings = this.getContents(this.getContainer(node)).filter(function(c){
			return c != node;
		});
		var nodes = com.ibm.pb.screen.position.findByDirection(node, siblings, direction);
		lgr.trace(mname, "Found siblings %o in direction %o from %o", nodes, direction, node); 
		return nodes[0];
	},
	getContainerByDirection: function(node, direction) {
		var mname = "getContainerByDirection", lgr = this.LOGGER;
		var container = this.getContainer(node);
		var containers = this.getContainers().filter(function(c){
			return c != container;
		});
		var nodes = com.ibm.pb.screen.position.findByDirection(node, containers, direction);
		lgr.trace(mname, "Found containers %o in direction %o from %o", nodes, direction, node);
		return nodes[0];
	},
	getContents: function(node) {
		return dojo.query(this.contentsQuery, node);
	},
	getContainers: function() {
		return dojo.query(this.containerQuery);
	},
	getContainer: function(node) {
		return node.parentNode;
	}
});

}

if(!dojo._hasResource["com.ibm.pb.decorations.LayoutDirectionalMoveControls"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.decorations.LayoutDirectionalMoveControls"] = true;
dojo.provide("com.ibm.pb.decorations.LayoutDirectionalMoveControls");



/**
 * Provides controls to move a node between containers or between siblings in its own container
 */
dojo.declare("com.ibm.pb.decorations.LayoutDirectionalMoveControls", com.ibm.pb.decorations.DirectionalMoveControls, {
	
	builderFactory: com.ibm.mashups.builder.model.Factory,
	
	contentsQuery: "> .component-control",
	containerQuery: ".component-container:not(.hiddenWidgetsContainer)",
	
	constructor: function() {
		this.evtSvc = com.ibm.mashups.iwidget.services.ServiceManager.getService("eventService");
		this.evtSvc.subscribeEvent("com.ibm.mashups.builder.widgetMoved", this, "onChange");
		this.nm = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel();
	},
	
	getControls: function(node) {
		// if view mode, no move action available
		if(this.builderFactory.getRuntimeModel().getCurrentPage().getPageMode() == "view") { 
			return []; 
		} else {
			var navStateModel = com.ibm.mashups.enabler.model.state.NavigationStateModelFactory.getNavigationStateModel();
			var widgetAccessor = com.ibm.mashups.enabler.model.state.AccessorFactory.getWidgetAccessor(navStateModel,node);
			var state = widgetAccessor.getWindowState();
			// if in maximized mode, no move action available
			if (widgetAccessor.MAX == state){
				return []; 
			} else {
				return this.inherited(arguments);
			}
		}
	},
	
	getPostMoveAction: function(node, parentNode, nextSibling) {
		var nodeId = this.layoutParser.extractID(node),
			parentId = this.layoutParser.extractID(parentNode),
			nextId = (nextSibling && this.layoutParser.extractID(nextSibling)) || null,
			pageId, currentNavNode, lm, lmNode, lmParentNode, lmNextNode;
		pageId = this.builderFactory.getRuntimeModel().getCurrentPage().getID();
		currentNavNode = this.nm.find(pageId).start();
		lm = this.nm.getLayoutModel(currentNavNode);
		lmNode = lm.find(nodeId).start();
		lmParentNode = lm.find(parentId).start();
		if(nextId) {
			lmNextNode = lm.find(nextId).start();
		}
		return function(){
			if(lm.confirmInsert(lmNode, lmParentNode, lmNextNode)) {
				lm.insert(lmNode, lmParentNode, lmNextNode);
				com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(true);
			}
		};
	},
	
	onMove: function(node, refNode, position) {
		this.evtSvc.broadcastEvent("com.ibm.mashups.builder.widgetMoved", this.layoutParser.extractID(node));
	}
});

}

if(!dojo._hasResource["com.ibm.data.ProxyHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.ProxyHelper"] = true;
dojo.provide( "com.ibm.data.ProxyHelper" );

dojo.require( "com.ibm.utilities" );

dojo.declare( "com.ibm.data.ProxyHelper",
			null,
			{
				// proxyURI: String
				//		URI to the proxy servlet.
				proxyURI: "",
				
				urlThroughProxy: function(/*String*/ url) {
					return com.ibm.utilities.urlToProxyUrl(url ? url : "", this.proxyURI);
				}
			}
);

}

if(!dojo._hasResource["com.ibm.data.CatalogMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.CatalogMixin"] = true;
// Datastores written for use in the content search widget must inherit from this class to make the mapItem and prepareQuery function available to the widget
// These functions are used to unify access to datastores, so the widget can  query and access items the same way for every source

dojo.provide("com.ibm.data.CatalogMixin");
dojo.declare("com.ibm.data.CatalogMixin", 
		null,
		{	
			searchable: true,
			
			mapItem: function(/*DatstoreItem*/item){
				// summary: Returns a map containing parameters for the given datastore item
				//		The keys must be: label, description, tags, rating, url, id, thumbnail
				//			- label, description, url, id and thumbnail are strings
				//			- tags is an array of strings
				//			- rating is a string that contains a number, note that decimal values are allowed
				//		Having all content search datastores create item maps with the same keys allows the widget to access information uniformly
				// 		If a datastore item does not have the same attributes for getValue as seen below, this function must be overridden
				//		If a datastore item does not have an appropriate parameter to  map to a key, that map entry can be omitted
				//		An example for a datastore item with a title, summary, rating and identifier:
				//			mapItem: function(/*DatastoreItem*/item){
				//				// does not include the "tags" and "url" keys
				//				var map = {};
				//				map["label"] = this.getEPResolvedValue(item, "title", "untitled");
				//				map["description"] = this.getEPResolvedValue(item, "summary", null);
				//				map["rating"] = this.getEPResolvedValue(item, "rating", "0.0");
				//				map["id"] = this.getEPResolvedValue(item, "identifier", null);
				//				map["thumbnail"] = this.getEPResolvedValue(item, "thumbnail", null);
				//				return map;
				//			}
				var map = {};
				map["label"] = this.getEPResolvedValue(item, "label", "untitled");
				map["description"] = this.getEPResolvedValue(item, "description", null);
				map["tags"] = this.getEPResolvedValue(item, "tags", null);
				map["rating"] = this.getEPResolvedValue(item, "rating", "0.0");
				map["url"] = this.getEPResolvedValue(item, "url", null);
				map["id"] = this.getEPResolvedValue(item, "id", null);
				map["thumbnail"] = this.getEPResolvedValue(item, "thumbnail", null);
				return map;
			},

			prepareQuery: function(/*Object*/keywordArgs) {
				// summary: User overridable function to modify the keywordArgs object used to fetch results from the back end
				//		Called after the keywordArgs.query property has been normalized to an object map but before any other processing has taken place to build the URL
				//		After this function is called, anything in the keywordArgs.query map is appended to the fetch URL in the form (?|&)<key1>=<value1> &<key2>=<value2> &<key3>=<value3> ...
				//		Can be used to extract other parts of the keywordArgs map to add properties to the query object that are dependent on back end implementation
				//		Properties of the query object that are not part of the fetch URL should be deleted
				//		The keywordArgs object is defined in dojo.data.api.Read.fetch and query will be an object containing the key "keywords" with an array of strings as a value
				//		An example for a feed URL containing "searchTerms", "startIndex" and "count" parameters:
				//			prepareQuery: function(/*Object*/keywordArgs) {
				//				if(keywordArgs.query.keywords) {
				//					// searchTerms is a comma separated list of search parameters
				//					keywordArgs.query.searchTerms = keywordArgs.query.keywords.join(",");
				//					delete keywordArgs.query.keywords;
				//				}
				//				if(keywordArgs.count) {
				//					keywordArgs.query.count = keywordArgs.count;
				//					if(keywordArgs.start == null) {
				//						keywordArgs.start = 0;
				//					}
				//					keywordArgs.query.startIndex = keywordArgs.start;
				//				}
				//			}
				return keywordArgs;
			},
			
			validateSelf: function(){
				// summary: validates this datastore, if it does not validate the source will not be used
				// returns true for validation and false otherwise
				return true;
			},
			
			getSourceLabel: function(){
				return false;
			}
});

}

if(!dojo._hasResource["com.ibm.data.JsonStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.JsonStore"] = true;
// A json store that can be used for the customize shelf
// dojo.data.ItemFileReadstore already is a JSON store, this store extends it and overwrites the _fetchItems function to do an AND instead of an OR search for the given keywords
// it also adds support for passing in parameters for an nls bundle and keywords

dojo.provide("com.ibm.data.JsonStore");







dojo.declare("com.ibm.data.JsonStore", 
		[dojo.data.ItemFileReadStore, com.ibm.data.ProxyHelper, com.ibm.data.CatalogMixin],
		{	
			nlsBundle: null,
		
			_getItemsFromLoadedData: function(/* Object */ dataObject){
				this.inherited("_getItemsFromLoadedData",arguments);
				
				this.shelfBundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
				var localizationPackageName = dataObject.localizationPackageName;
				var localizationBundleName = dataObject.localizationBundleName;
				if(localizationPackageName && localizationBundleName){
					this.nlsBundle = dojo.i18n.getLocalization(localizationPackageName, localizationBundleName);
					dojo.forEach(this._arrayOfAllItems,
						dojo.hitch(this, function(item) {
							if (item.label) {
								item.label[0] = (this.nlsBundle && this.nlsBundle[item.label[0]]) ? this.nlsBundle[item.label[0]] : item.label[0];
							}
							else {
								item.label = [];
							}
							if (!item.label[0]) {
								item.label[0] = this.shelfBundle.add_content_untitled;
							}
							if (item.description) {
								item.description[0] = (this.nlsBundle && this.nlsBundle[item.description[0]]) ? this.nlsBundle[item.description[0]] : item.description[0];
							}
							if (item.help) {
								item.help[0] = (this.nlsBundle && this.nlsBundle[item.help[0]]) ? this.nlsBundle[item.help[0]] : item.help[0];
							}
						})
					);
				}
			},
			
			mapItem: function(/*DatstoreItem*/item){
				var map = {};
			
				map.label = this.getValue(item, "label", null);
				map.description = this.getValue(item, "description", null);
				map.url = this.getValue(item, "url", null);
				map.id = this.getValue(item, "id", null);
				map.thumbnail = this.getValue(item, "thumbnail", null);
				map.help = this.getValue(item, "help", null);
				map.takeoverFcn = this.getValue(item,"takeover",null);
				
				return map;
			},
		
			_fetchItems: function(/*Object*/ keywordArgs, /*Function*/ findCallback, /*Function*/ errorCallback){
				var self = this;
				var filter = function(requestArgs, arrayOfItems){
					
					var items = [];
					var i = 0;
					if(requestArgs.query){
						var filterKeywords = requestArgs.query.keywords;
						var ignoreCase = requestArgs.queryOptions ? requestArgs.queryOptions.ignoreCase : false; 
					
						var regexpList = {};
						var currFilter = null;
						for(i = 0; i < filterKeywords.length; i++){
							currFilter = filterKeywords[i];
							if(typeof currFilter === "string"){
								regexpList[currFilter] = dojo.data.util.filter.patternToRegExp("*"+currFilter+"*", ignoreCase);
							}
						}

						for(i = 0; i < arrayOfItems.length; ++i){
							var match = true;
							var candidateItem = arrayOfItems[i];
							if(candidateItem === null){
								match = false; 
							} else {
								for(var j = 0; j < filterKeywords.length; j++){
									currFilter = filterKeywords[j];
									if(!self._containsValue(candidateItem, 'label', currFilter, regexpList[currFilter]) && 
											!self._containsValue(candidateItem, 'description', currFilter, regexpList[currFilter])){
										match = false;
									}
								}
							}
							
							if(match){
								items.push(candidateItem);
							}
						}
						findCallback(items, requestArgs);
					} else {
						for(i = 0; i < arrayOfItems.length; ++i){
							var item = arrayOfItems[i];
							if(item !== null){
								items.push(item);
							}
						}
						findCallback(items, requestArgs);
					}
			};

			if(this._loadFinished){
				filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
			} else {

				if(this._jsonFileUrl){
					if(this._loadInProgress){
						this._queuedFetches.push({args: keywordArgs, filter: filter});
					} else {
						this._loadInProgress = true;
						var getArgs = {
								url: self._jsonFileUrl, 
								handleAs: "json-comment-optional"
						};
						var getHandler = dojo.xhrGet(getArgs);
						getHandler.addCallback(function(data){
							try{
								self._getItemsFromLoadedData(data);
								self._loadFinished = true;
								self._loadInProgress = false;
				
								filter(keywordArgs, self._getItemsArray(keywordArgs.queryOptions));
								self._handleQueuedFetches();
							}catch(e){
								self._loadFinished = true;
								self._loadInProgress = false;
								errorCallback(e, keywordArgs);
							}
						});
						getHandler.addErrback(function(error){
							self._loadInProgress = false;
							errorCallback(error, keywordArgs);
						});
					}
				} else if(this._jsonData){
					try{
						this._loadFinished = true;
						this._getItemsFromLoadedData(this._jsonData);
						this._jsonData = null;
						filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
					}catch(e){
						errorCallback(e, keywordArgs);
					}
				} else {
					errorCallback(new Error("dojo.data.ItemFileReadStore: No JSON source data was provided as either URL or a nested Javascript object."), keywordArgs);
				}
			}
		}
});

}

if(!dojo._hasResource["com.ibm.data.XpathHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.XpathHelper"] = true;
dojo.provide( "com.ibm.data.XpathHelper" );
// summary: This module is meant to be extended by modules that want to use XPath
//		to query DOM documents for information.  It provides read support and limited 
// 		write support.  Write support is limited in that if an xpath query is used to
//		set a node, property of a node, or DOM attribute of a node, then the write support
//		will only work if a node that matches that query already exists and is returned.
//		If no node matches the query and the Xpath query returns null, the all write operations
//		provided by this module will return false to indicate that the caller needs to handle
//		them in a different way like creating the node from scratch.
//
//		This module provides cross-browser methods for reading and writing to DOM nodes,
//		properties of DOM nodes, and DOM attributes of DOM nodes.  It also supports extensions
//		so that special constructs can be returned by a query instead of a DOM node or String value
//		of a property or attribute.  By default, no constructs are defined by this module.


dojo.require( "com.ibm.portal.xpath" );
dojo.require( "com.ibm.portal.xslt" );
dojo.require( "com.ibm.domUtilities" );

dojo.declare( "com.ibm.data.XpathHelper",
			null,
			{
				// namespaces: A map of prefix - namespace URI pairs used to specify what namespaces
				//		are used by this XpathHelper to query nodes in a DOM document
				namespaces: {},
				
				_getNodesByXpath: function(/*Object*/ context, /*String*/expr, /*String?*/prop) {
					// summary: Gets an array of DOM nodes or String node properties using the XPath expr relative 
					// 		to the context (not relative to the document root itself).  Returns an object allowing
					//		the caller to determine if the actual value requested is null or if there were no nodes
					//		found at all.
					// context: The XML element to use as the root context for the XPath query
					// expr: XPath expression used to find the nodes
					// prop: Optional prop to specify to get a property of the nodes instead of the nodes themselves.
					// returns: Object with two properties:
					//		-	{
					//		-		found: Boolean indicating if any nodes were found matching the XPath expression
					//		-		value: Array the actual nodes found by the expression, or the properties of the nodes
					// 		-			found by the expression as specified by the prop argument
					// 		-	} 
					var results = com.ibm.portal.xpath.evaluateXPath(expr, context, this.namespaces);
					var found = results && results.length > 0;
					if(prop) {
						var nodes = results;
						results = [];
						for(var i = 0; i < nodes.length; i++) {
							results[i] = this._getNodeProp(nodes[i], prop);
						}
					}
					return {found: found, value: results};
				},
				_getNodeByXpath: function(/*Object*/ context, /*String*/expr, /*String?*/ prop) {
					// summary: Gets a node using the XPath expr relative to 
					// 		the context	(not relative to the document root itself).  Returns an object allowing
					//		the caller to determine if the actual value requested is null or if there were no nodes
					//		found at all.
					// context: The XML element to use as the root context for the XPath query
					// expr: XPath expression used to find the node
					// prop: Optional prop to specify to get an property of the node instead of the node itself.
					// returns: Object with two properties:
					//		-	{
					//		-		found: Boolean indicating if any nodes were found matching the XPath expression
					//		-		value: DOMNode|Object the first node found by the expression, or the property of the first
					// 		-			node found by the expression as specified by the prop argument
					// 		-	}
					var result = this._getNodesByXpath(context, expr);
					var found = result.found;
					if(!found) return {found: found};
					var nodes = result.value;
					if(nodes.length > 0) {
						if(prop) return {found: true, value: this._getNodeProp(nodes[0], prop)};
						else return {found: true, value: nodes[0]};
					}
					return {found: false};
				},
				_getNodeProp: function(/*DOMNode*/ node, /*String*/ prop) {
					// summary:  Function to provide cross-browser support for properties of a queried
					//		node.  If the prop string is not special, control will be passed to the 
					//		handleConstructs function for this object.
					//		Currently only supports textContent as a special property since not
					//		all browsers implement the textContent property of DOM nodes.
					switch(prop) {
						case "textContent":
							return dojox.data.dom.textContent(node);
						default:
							return this.handleConstructs(node, prop);
					}
				},
				handleConstructs: function(/*DOMNode*/ node, /*String*/ prop) {
					// summary: Allows a data store to return complex javascript objects
					//		or alternative values when a specific non-standard property of a node 
					// 		is requested.  Example: An Atom Feed data store may create a special 
					// 		Person Construct when querying an <atom:author> element.  This construct
					//		may provide accessor functions to the name, email, and URI of the author.
					//		See http://tools.ietf.org/html/rfc4287#section-3.2
					//		Constructs that are created here should implement and handle write functionality
					//		themselves as this module will not handle writes to constructs natively.
					
					return node[prop];
				},
				_setNodeProp: function(/*DOMNode*/ node, /*String*/ prop, /*Object*/ value) {
					// summary: Allows a data store to set properties of a node in a cross-browser
					//		way for properties that may be non-standard.
					//		Currently only supports textContent as a special property since not
					//		all browsers implement the textContent property of DOM nodes.
					//		Note that for special constructs that are created using the handleConstructs
					// 		function, write support is handled by the returned construct object not by
					//		this module's functions.
					switch(prop) {
						case "textContent":
							dojox.data.dom.textContent(node, value);
							break;
						default:
							node[prop] = value;
					}
				},
				_getAttrsByXpath: function(/*Object*/ context, /*String*/ expr, /*String*/attrName) {
					// summary: Gets an array of attribute values for the nodes returned 
					// 		using the XPath expr relative to the context (not relative to the 
					// 		document root itself).  The attributes are retrieved using the 
					// 		getAttributeNS/getAttribute	functions of the nodes.  Returns an object allowing
					//		the caller to determine if the actual value requested is null or if there were no nodes
					//		found at all.
					// context: The XML element to use as the root context for the XPath query
					// expr: XPath expression used to find the nodes
					// attrName: The name of the attribute to get from each node returned by the XPath expr.
					//		If the attribute is namespaced, it will be split into its parts using the : delimiter
					//		and the namespace used during lookup will be the one keyed in this item or store's 
					// 		namespace map.
					// returns: Object with two properties:
					//		-	{
					//		-		found: Boolean indicating if any nodes were found matching the XPath expression
					//		-		value: Array the attribute values of the nodes found by the expression
					// 		-	}
					var result = this._getNodesByXpath(context, expr);
					var found = result.found;
					if(!found) return {found: found};
					var nodes = result.value;
					var attrs = [];
					for(var i = 0; i < nodes.length; i++) {
						attrs[i] = this._getAttribute(nodes[i], attrName);
					}
                    return {found: found, value: attrs};
				},
				_getAttrByXpath: function(/*Object*/ context, /*String*/expr, /*String*/ attrName) {
					// summary: Gets the attribute value of the first node returned 
					// 		using the XPath expr relative to the context (not relative to the 
					// 		document root itself).  The attribute is retrieved using the 
					// 		getAttributeNS/getAttribute	functions of the node.  Returns an object allowing
					//		the caller to determine if the actual value requested is null or if there were no nodes
					//		found at all.
					// context: The XML element to use as the root context for the XPath query
					// expr: XPath expression used to find the node
					// attrName: The name of the attribute to get from the node returned by the XPath expr.
					//		If the attribute is namespaced, it will be split into its parts using the : delimiter
					//		and the namespace used during lookup will be the one keyed in this item or store's 
					// 		namespace map.
					// returns: Object with two properties:
					//		-	{
					//		-		found: Boolean indicating if any nodes were found matching the XPath expression
					//		-		value: Array the attribute value of the first node found by the expression
					// 		-	}
					var result = this._getNodeByXpath(context, expr);
					var found = result.found;
					if(!found) return {found: found}; 
					var node = result.value;
                    return {found: found, value: this._getAttribute(node, attrName)};
				},
				_extractNSInfo: function(str) {
					// summary: Extracts the namespace URI, prefix, and local name of a QName according to this
					//		object's namespaces map that includes known prefixes and their namespace URIs.  If
					//		the string is not namespaced, then the namespaceURI and prefix are set to null, and
					//		the localName is set to the str as-is.
					// str: QName to extract namespace from
					// returns: Map - {namespaceURI: namespaceURI, prefix: prefix, localName: localName}
					return com.ibm.domUtilities.nsInfo(str, this.namespaces);
				},
				_getAttribute: function(/*DOMNode*/ node, /*String*/ attrName) {
					// summary: Gets an attribute of the given node, supporting namespaced and 
					// 		non-namespaced attributes.
					// node: Node to retrieve the attribute value from
					// attrName: The name of the attribute to get from the node.
					//		If the attribute is namespaced, it will be split into its parts using the : delimiter
					//		and the namespace used during lookup will be the one keyed in this item or store's 
					// 		namespace map.
					return com.ibm.domUtilities.getAttribute(node, attrName, this.namespaces);
				},
				_setAttribute: function(/*DOMNode*/ node, /*String*/ attrName, /*String*/ value) {
					// summary: Sets an attribute of the given node, supporting namespaced and 
					// 		non-namespaced attributes.
					// node: Node to set the attribute value on
					// attrName: The name of the attribute to set on the node.
					//		If the attribute is namespaced, it will be split into its parts using the : delimiter
					//		and the namespace used during lookup will be the one keyed in this item or store's 
					// 		namespace map.
					// value: The actual value to store in the node at the key attribute specified by attrName.
					if(value) com.ibm.domUtilities.setAttribute(node, attrName, value, this.namespaces);
					else com.ibm.domUtilities.removeAttribute(node, attrName, this.namespaces);
				},
				_setNodesByXpath: function(/*Object*/ context, /*String*/expr, /*String*/ prop, /*Array*/values) {
					// Empty for now since there's little use for this type of function, particularly due to 
					// mismatches between the number of values and number of returned nodes.
					//var nodes = this._getNodesByXpath(context, expr);
					
					return false;
				},
				_setNodeByXpath: function(/*Object*/ context, /*String*/expr, /*String*/ prop, /*Object*/value) {
					// summary: Will replace a node returned by the xpath expr query with the value object
					// 		if prop is null.  If prop exists, the specified property on the returned node
					// 		will be set to the value.
					// 		If no node is found that matches the xpath expr, this function returns false to indicate
					//		failure to the caller for further processing if necessary.
					// context: The DOM element to use as the root context for the XPath query
					// expr: XPath expression used to find the node
					// prop: Optional prop to specify to set a property of the node instead of the node itself.
					// returns: true if the set operation was successful, false otherwise
					var result = this._getNodeByXpath(context, expr);
					var found = result.found;
					if(!found) return false; 
					var node = result.value;
					if(prop) {
						this._setNodeProp(node, prop, value);
					}
					else {
						dojo.place(value, node, "before");
						node.parentNode.removeChild(node);
						delete node;
					}
					return true;
				},
				_setAttrsByXpath: function(/*Object*/ context, /*String*/expr, /*String*/ attrName, /*Array*/values) {
					// Empty for now since there's little use for this type of function, particularly due to 
					// mismatches between the number of values and number of returned nodes.
					//var nodes = this._getNodesByXpath(context, expr);
					
					return false;
				},
				_setAttrByXpath: function(/*Object*/ context, /*String*/expr, /*String*/ attrName, /*String*/value) {
					// summary: The specified DOM attribute on the returned node will be set to the value.
					// 		If no node is found that matches the xpath expr, this function returns false to indicate
					//		failure to the caller for further processing if necessary.
					// context: The DOM element to use as the root context for the XPath query
					// expr: XPath expression used to find the node
					// attrName: The name of the attribute to get from the node returned by the XPath expr.
					//		If the attribute is namespaced, it will be split into its parts using the : delimiter
					//		and the namespace used during lookup will be the one keyed in this item or store's 
					// 		namespace map.
					// returns: true if the set operation was successful, false otherwise
					var result = this._getNodeByXpath(context, expr);
					var found = result.found;
					if(!found) return false;
					var node = result.value;
					this._setAttribute(node, attrName, value);
					return true;
				},
				_createNSElement: function(doc, tagName, attrMap) {
					// summary: Cross-browser implementation for creating a namespaced node using the namespace
					//		map in this object to resolve prefixes to namespace URIs.
					//		If the tagName is not namespaced or a namespace URI cannot be resolved
					// 		using this object's namespaces map, a non-namespaced element will be created instead.
					// doc: DOMDoc document object to use to create the DOM node.
					// tagName: String name for the tag name of the element to create.  
					// 		Can be namespaced like prefix:localName or not like localName.
					// attrMap: Optional argument specifying a key-value map of attributes to be
					// 		set on the node before returning it. 
					// returns: DOMNode A new DOM node that was created by doc
					return com.ibm.domUtilities.createElement(doc, tagName, this.namespaces, attrMap);
				},
				__getClassName: function() {
					var cls = this.declaredClass;
					if(!cls) return "Object";
					var idx = cls.lastIndexOf(".");
					if(idx > -1 && (idx < cls.length - 1)) cls = cls.substring(idx + 1);
					return cls;
				}
			}
);

dojo.declare( "com.ibm.data._XmlConstruct",
			com.ibm.data.XpathHelper,
			{
				// summary: Provides a base construct for modules to extend when building constructs
				//		to be returned by queries using this module's APIs.
				isConstruct: true,
				toString: function() {
					// summary: Default toString method for this object so that it can be inserted
					//		in a string and be properly represented.  Overridable by extensions of this
					//		module.
					return "<"   + this.__getClassName() + ">";
				}
			}
);


}

if(!dojo._hasResource["com.ibm.data.AtomFeedStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.AtomFeedStore"] = true;
dojo.provide( "com.ibm.data.AtomFeedStore" );

dojo.require( "com.ibm.data.XpathHelper" );
dojo.require( "dojo.date.stamp" );

dojo.declare( "com.ibm.data.AtomFeedStore",
			com.ibm.data.XpathHelper,
			{
				// namespaces: Map
				//		Map of namespaces used by the ATOM feeds
				namespaces: {
		            "atom" : "http://www.w3.org/2005/Atom"
		        },
		        // _features: Map
				//		Map of features that this store supports.
				//		This generic AtomFeedStore only supports Read and Identity.
				_features: {
					'dojo.data.api.Read': true,
					'dojo.data.api.Write': false,
					'dojo.data.api.Identity': false,
					'dojo.data.api.Notification': false
				},
				attrMappings: {
					title: {xpath: "./atom:title", targetName: "textContent", type: "node", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:title");
							this._setNodeProp(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					id: {xpath: "./atom:id", targetName: "textContent", type: "node", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:id");
							this._setNodeProp(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					updated: {xpath: "./atom:updated", targetName: "AtomDateConstruct", type: "node"},
					category: {xpath: "./atom:category", targetName: "term", type: "attr", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:category");
							this._setAttribute(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					categoryScheme: {xpath: "./atom:category", targetName: "scheme", type: "attr", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:category");
							this._setAttribute(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					categoryLabel: {xpath: "./atom:category", targetName: "label", type: "attr", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:category");
							this._setAttribute(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					contributer: {xpath: "./atom:contributer", targetName: "AtomPersonConstruct", type: "node"},
					author: {xpath: "./atom:author", targetName: "AtomPersonConstruct", type: "node"},
					summary: {xpath: "./atom:summary", targetName: "textContent", type: "node", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:summary");
							this._setNodeProp(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					source: {xpath: "./atom:source", type: "node", setter: 
						function(item, rule, value){
							item.element.appendChild(value);
							return true;
						}
					},
					rights: {xpath: "./atom:rights", targetName: "textContent", type: "node", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:rights");
							this._setNodeProp(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					selfLink: {xpath: "./atom:link[@rel='self']", targetName: "href", type: "attr", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:link");
							this._setAttribute(elem, "rel", "self");
							this._setAttribute(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					link: {xpath: "./atom:link[@href]", targetName: "href", type: "attr", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:link");
							this._setAttribute(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					content: {xpath: "./atom:content", type: "node", setter: 
						function(item, rule, value){
							item.element.appendChild(value);
							return true;
						}
					}
				},
				
				// timeout: Number milliseconds to wait for a response for any request
				//		sent by this data store.
				timeout: null,
				
				// itemClass: String the class used to represent items of this store
				itemClass: "com.ibm.data.AtomEntryItem",
								
				constructor: function( /*Map*/ args ) {
					// summary: Creates a new instance of a com.ibm.data.AtomFeedStore
					// args: Map
					//		The args map will be used to replace any default values of this store with the
					//		values specified in the args map.
					dojo.mixin(this, args);
					this._dirtyItems = [];
					this._deletedItems = [];
					this._newItems = [];
				},
				toString: function() {
					return "<" + this.__getClassName() + ": " + this.url + ">";
				},
				handleConstructs: function(/*DOMNode*/ node, /*String*/ prop) {
					switch(prop) {
						case "AtomPersonConstruct": return new com.ibm.data.AtomPersonConstruct(node);
						case "AtomDateConstruct": return new com.ibm.data.AtomDateConstruct(node);
						default: return this.inherited(arguments);
					}
				},
				
				_assertIsItem: function(/*Object*/ something) {
					// summary: Throws an exception if the argument is not a valid item in this store.
					// something: Object to check to determine if it is an item in this store.
					if (!this.isItem(something)) {
						throw new Error(something + " is not an item in this store");
					}
					return true;
				},
				_assertIsString: function(/*Object*/ something) {
					// summary: Throws an exception if the argument is not a string.
					// something: Object to check to determine if it is a string.
					if (!dojo.isString(something)) {
						throw new Error(something + " is not a string");
					}
					return true;
				},
				_getItem: function(args) {
					// summary: Overridable function to create a new item in this store based on the args map.
					var cls = this.itemClass;
					if(dojo.isString(cls)) {
						cls = dojo.getObject(this.itemClass);
					}
					return new cls({store: this, element: args.element, loaded: true}); 
				},
				/********************************/
				/**                            **/
				/**                            **/
				/**    		 Public  		   **/
				/**                            **/
				/**                            **/
				/********************************/
								
				getFeedValue: function( /*String*/ attribute, /*Value?*/ defaultValue) {
					var ret = defaultValue;
					this._assertIsString(attribute);
					var mapRule = this.getMapRule(attribute, this.attrMappings);
					// if mapRule is null here, then we couldn't find one in the map, 
					// so we can't proceed successfully
					if(mapRule) {
						ret = this.getValueByMapping(null, mapRule, defaultValue);
					}
					
					return ret;
				},
				getFeedValues: function( /*String*/ attribute) {
					var ret = [];
					this._assertIsString(attribute);
					var mapRule = this.getMapRule(attribute, this.attrMappings);
					// if mapRule is null here, then we couldn't find one in the map, 
					// so we can't proceed successfully
					if(mapRule) {
						ret = this.getValuesByMapping(null, mapRule);
					}
					
					return ret;
				},
				extractEmptyFeed: function() {
					var toClone = this._getNodeByXpath(this._feed, "//atom:feed").value;
					var feedDOM = toClone.cloneNode(false);
					// get all non-entry nodes
					var result = this._getNodesByXpath(toClone, "/atom:feed/*");
					var nodes = result.value;
					for(var i = 0; i < nodes.length; i++) {
						if(nodes[i].nodeName != "atom:entry") {
							feedDOM.appendChild(nodes[i].cloneNode(true));
						}
					}
					return feedDOM;
				},
				extractSingleEntryFeed: function(/* com.ibm.data.AtomEntryItem */ item ) {
					var ret = this.extractEmptyFeed();
					ret.appendChild(item.element.cloneNode(true));
					return ret;
				},
				extractMultipleEntryFeed: function(/* Array */ items ) {
					var ret = this.extractEmptyFeed();
					dojo.forEach(items, function(item){
						ret.appendChild(item.element.cloneNode(true));
					});
					return ret;
				},
				
				getEPResolvedValue: function(	/* com.ibm.data.AtomEntryItem */ item, 
						/* String */ attribute, 
						/* Value? */ defaultValue){
					var ret = null;
					if(this.hasAttribute(item, attribute)) {
						var mapRule = this.getMapRule(attribute, item.attrMappings);
						// if mapRule is null here, then we couldn't find one in the map, 
						// so we return the defaultValue 
						if (!mapRule) {
							return ret;
						}
						ret = this.getValueByMapping(item, mapRule, defaultValue);
					}
					if (!ret) {
						ret = defaultValue;
					}
					ret = com.ibm.mashups.enabler.utils.EndpointHelper.resolve(ret);
					return ret;
				},
								
				/********************************/
				/**                            **/
				/**                            **/
				/**   dojo.data.api.Identity   **/
				/**                            **/
				/**                            **/
				/********************************/				
				getIdentity: function(/* item */ item) {
					// summary: See dojo.data.api.Identity.getIdentity()
					// returns: The entry's unique identifier
					if (item._id) {
						return item._id;
					}
					// this is a common operation... so cache this on the item itself
					var attrs = this.getIdentityAttributes(item);
					var id = "";
					dojo.forEach(attrs, function(attr){
						id += this.getEPResolvedValue(item, attr, "");
					}, this);
					item._id = id;
					return id;
				},
				
				getIdentityAttributes: function(/* item */ item){
					//	summary:
					//		Returns an array of attribute names that are used to generate the identity. 
					//		For most stores, this is a single attribute, but for some complex stores
					//		such as RDB backed stores that use compound (multi-attribute) identifiers
					//		it can be more than one.  If the identity is not composed of attributes
					//		on the item, it will return null.  This function is intended to identify
					//		the attributes that comprise the identity so that so that during a render
					//		of all attributes, the UI can hide the the identity information if it 
					//		chooses.
					//	item:
					//		The item from the store from which to obtain the array of public attributes that 
					//		compose the identifier, if any.
					return ['id']; // Array
				},
			
				fetchItemByIdentity: function(/* object */ keywordArgs){
					//	summary:
					//		Given the identity of an item, this method returns the item that has 
					//		that identity through the onItem callback.  Conforming implementations 
					//		should return null if there is no item with the given identity.  
					//		Implementations of fetchItemByIdentity() may sometimes return an item 
					//		from a local cache and may sometimes fetch an item from a remote server, 
					//
					// 	keywordArgs:
					//		An anonymous object that defines the item to locate and callbacks to invoke when the 
					//		item has been located and load has completed.  The format of the object is as follows:
					//		{
					//			identity: string|object,
					//			onItem: Function,
					//			onError: Function,
					//			scope: object
					//		}
					//	The *identity* parameter.
					//		The identity parameter is the identity of the item you wish to locate and load
					//		This attribute is required.  It should be a string or an object that toString() 
					//		can be called on.
					//		
					//	The *onItem* parameter.
					//		Function(item)
					//		The onItem parameter is the callback to invoke when the item has been loaded.  It takes only one
					//		parameter, the item located, or null if none found.
					//
					//	The *onError* parameter.
					//		Function(error)
					//		The onError parameter is the callback to invoke when the item load encountered an error.  It takes only one
					//		parameter, the error object
					//
					//	The *scope* parameter.
					//		If a scope object is provided, all of the callback functions (onItem, 
					//		onError, etc) will be invoked in the context of the scope object.
					//		In the body of the callback function, the value of the "this"
					//		keyword will be the scope object.   If no scope object is provided,
					//		the callback functions will be called in the context of dojo.global.
					//		For example, onItem.call(scope, item, request) vs. 
					//		onItem.call(dojo.global, item, request)
					
					
					// not implemented by the AtomFeedStore since this supports generic Atom Feeds and therefore
					// can't generate a URL to get a particular item by its identity without additional knowledge
					// of how the feed URLs are created... however, this can be overridden by a application consuming
					// an instance of this store that does have knowledge of the particular feed URLs
					return false;
				},
				/********************************/
				/**                            **/
				/**                            **/
				/**     dojo.data.api.Read     **/
				/**                            **/
				/**                            **/
				/********************************/
				getValue: function(	/* com.ibm.data.AtomEntryItem */ item, 
						/* String */ attribute, 
						/* Value? */ defaultValue){
					var ret = null;
					if(this.hasAttribute(item, attribute)) {
						var mapRule = this.getMapRule(attribute, item.attrMappings);
						// if mapRule is null here, then we couldn't find one in the map, 
						// so we return the defaultValue 
						if (!mapRule) {
							return ret;
						}
						ret = this.getValueByMapping(item, mapRule, defaultValue);
					}
					if (!ret) {
						ret = defaultValue;
					}
					
					return ret;
				},
				
				getValues: function(/* com.ibm.data.AtomEntryItem */ item,
						/* String */ attribute){
					if(this.hasAttribute(item, attribute)) {
						var mapRule = this.getMapRule(attribute, item.attrMappings);
						// if mapRule is null here, then we couldn't find one in the map, 
						// so we return an empty list
						if (!mapRule) {
							return [];
						}
						return this.getValuesByMapping(item, mapRule);
					}
					return [];
				},
				_extractParameterizedMapKey: function( /*String*/ mapKey ) {
					// summary: Expands a mapKey and determines if it is a key to a parameterizable map rule.
					//		If it is, extract the parameters from the argument and return the base key name
					//		used to look up the correct map rule in this object's attrMappings without the parameter
					// 		values, and also return the parameters as an array of strings.  
					// 		If the mapKey is not the key of a parameterizable map rule, just return the key and a 
					// 		null parameter array.
					// returns: Object {key: stringMapKey, parameters: arrayOfParameterValues}
					if(mapKey.charAt(mapKey.length - 1) == '$') {
						var start = mapKey.indexOf('[');
						var end = mapKey.lastIndexOf(']');
						if(start > -1 && end > start) {
							var baseName = mapKey.substring(0, start) + "$";
							var parms = mapKey.substring(start + 1, end).split(',');
							return {key: baseName, parameters: parms};
						}
					}
					return {key: mapKey, parameters: null};
				},
				
				getMapRule: function( /*String*/ mapKey, /*Object*/map, /*Boolean?*/ multiValued ) {
					// summary: Takes a mapKey string that ends in '$' to key into this object's 
					// 		attrMappings, and then parameterizes the map it finds according
					// 		to the parameters found within the '[...]' block of the mapKey.
					// 		Example: A mapKey of 'localizedName[edward, tj]$' would lookup
					// 			the key 'localizedName$' in this.attrMappings.
					// 			The resulting map rule would be used as a template for the returned
					//			map rule.  The parameters used to parameterize the template would
					//			be ['edward', 'tj'] and would match the ${0} and ${1} parameters
					//			in the map rule respectively.
					// returns: The map rule specified by this mapKey after all parameters have been
					//			bound to their values as specified by the caller.
					var obj = this._extractParameterizedMapKey(mapKey);
					var rule = map[obj.key];
					
					if (!rule) {
						return rule;
					}
					// if this is not a parameterized map rule, just return the map rule as-is
					if(!obj.parameters) {
						rule._ruleKey = obj.key;
						return rule;
					}
					
					// otherwise, parameterize the map by using the array of parameters passed into the attribute
					// like this ->  attribute[parm1, parm2, parm3]$
					// parameterized attributes always end with $
					var expRule = {};
					for(var prop in rule) {
						if(dojo.isString(rule[prop])) {
							expRule[prop] = dojo.string.substitute(rule[prop], obj.parameters, function(part){
								return dojo.string.trim(part);
							});
						}
						else {
							expRule[prop] = rule[prop];
						}
					}
					expRule._parameters = obj.parameters;
					expRule._ruleKey = obj.key;
					expRule.multiValued = !!multiValued;
					return expRule;
				},
				
				getValuesByMapping: function( /* com.ibm.data.AtomEntryItem */ item, 
						/* Object */ mapping){
					// summary: Executes the correct query API using the mapping argument to
					//		retrieve the desired attribute value for the item as specified by the 
					// 		mapping.
					// mapping: A mapping object from this object's attrMappings
					var ret = null;
					var xpath = mapping.xpath;
					var type = mapping.type;
					var targetName = mapping.targetName;
					var context = this._feed;
					if(item) {
						this._assertIsItem(item);
						context = item.element;
					}
					if(type == "attr") {
						ret = this._getAttrsByXpath(context, xpath, targetName);
					}
					else {
						ret = this._getNodesByXpath(context, xpath, targetName);
					}
					
					if(!ret.found && item._backup) {
						// we didn't find any nodes matching the xpath query, and
						// this item has been copied from its original for modification,
						// so perhaps we explicitly removed it in the modified version for
						// performance reasons... so lets look it up in the original unmodified version						
						context = item._backup;
						if(type == "attr") {
							ret = this._getAttrsByXpath(context, xpath, targetName);
						}
						else {
							ret = this._getNodesByXpath(context, xpath, targetName);
						}
						
						// if ret.found is still false, then just return the defaultValue
						if (!ret.found) {
							ret.value = [];
						}
					}
					
					return ret.value;
				},
				
				getValueByMapping: function( /* com.ibm.data.AtomEntryItem */ item, 
						/* Object */ mapping, 
						/* Value? */ defaultValue){
					// summary: Executes the correct query API using the mapping argument to
					//		retrieve the desired attribute value for the item as specified by the 
					// 		mapping.
					// mapping: A mapping object from this object's attrMappings
					// defaultValue: The default value to return if no value exists for the mapping.
					//		If null is returned by the query, the defaultValue will be returned
					//		instead.
					var ret = null;
					var xpath = mapping.xpath;
					var type = mapping.type;
					var targetName = mapping.targetName;
					var context = this._feed;
					if(item) {
						this._assertIsItem(item);
						context = item.element;
					}
					if(type == "attr") {
						ret = this._getAttrByXpath(context, xpath, targetName);
					}
					else {
						ret = this._getNodeByXpath(context, xpath, targetName);
					}
					
					if(!ret.found && item && item._backup) {
						// we didn't find any nodes matching the xpath query, and
						// this item has been copied from its original for modification,
						// so perhaps we explicitly removed it in the modified version for
						// performance reasons... so lets look it up in the original unmodified version						
						context = item._backup;
						if(type == "attr") {
							ret = this._getAttrByXpath(context, xpath, targetName);
						}
						else {
							ret = this._getNodeByXpath(context, xpath, targetName);
						}
						
						// if ret.found is still false, then just return the defaultValue
						if (!ret.found && defaultValue) {
							ret.value = defaultValue;
						}
					}
					
					if (type == "attr" && ret.value && ret.value.length === 0) {
						ret.value = null;
					}
					
					return ret.value;
				},
				
				setValueByMapping: function( /* com.ibm.data.AtomEntryItem */ item, 
						/* Object */ mapping, 
						/* Value */ value){
					// summary: Executes the correct query API using the mapping argument to
					//		retrieve the desired attribute value for the item as specified by the 
					// 		mapping.
					// mapping: A mapping object from this object's attrMappings
					// value: The value to set that matches the mapping rule on the item.
					var xpath = mapping.xpath;
					var type = mapping.type;
					var targetName = mapping.targetName;
					var context = this._feed;
					var ret = false;
					if(item) {
						this._assertIsItem(item);
						context = item.element;
					}
					if(type == "attr") {
						ret = this._setAttrByXpath(context, xpath, targetName, value);
					}
					else {
						ret = this._setNodeByXpath(context, xpath, targetName, value);
					}
					
					if(!ret && mapping.setter) {
						return mapping.setter.apply(this, arguments);
					}
					return ret;
				},
				
				setValuesByMapping: function( /* com.ibm.data.AtomEntryItem */ item, 
						/* Object */ mapping, 
						/* Array */ values){
					// summary: Executes the correct query API using the mapping argument to
					//		retrieve the desired attribute value for the item as specified by the 
					// 		mapping.
					// mapping: A mapping object from this object's attrMappings
					// values: The value to set that matches the mapping rule on the item.
					var xpath = mapping.xpath;
					var type = mapping.type;
					var targetName = mapping.targetName;
					var context = this._feed;
					var ret = false;
					if(item) {
						this._assertIsItem(item);
						context = item.element;
					}
					if(type == "attr") {
						ret = this._setAttrsByXpath(context, xpath, targetName, value);
					}
					else {
						ret = this._setNodesByXpath(context, xpath, targetName, value);
					}
					
					if(!ret && mapping.setter) {
						return mapping.setter.apply(this, arguments);
					}					
					return ret;
				},
				
				getAttributes: function(/* com.ibm.data.AtomEntryItem? */ item){
					// summary: Gets the valid attributes for the given item if one is provided and
					//		returns an array of attribute names in String format.  If the item is
					//		null, it returns an array of attributes on the current feed that can
					//		be retrieved via the getFeedValue function.
					// item: Optional argument to get the list of attributes from.
					var attrs = [];
					var x = null;
					if(item) {
						this._assertIsItem(item);
						for(x in item.attrMappings) {
							attrs.push(x);
						}
					}
					else {
						for(x in this.attrMappings) {
							attrs.push(x);
						}
					}
					
					return attrs;
				},
				
				hasAttribute: function(	/* com.ibm.data.AtomEntryItem */ item,
							/* String */ attribute){
					// summary: Returns true if the attribute is a valid attribute on the given item,
					//		false otherwise.  This does not check if the attribute value on the item
					//		is null or empty, but just checks if the item supports that attribute.
					// item: Item to check if the attribute is valid
					// attribute: Name of the attribute to check
					this._assertIsItem(item);
					this._assertIsString(attribute);
					if(attribute.charAt(attribute.length - 1) == '$') {
						// this looks like an attribute whose map rule is parameterizable
						// so lets defer checking if its valid until later so we don't
						// duplicate any string processing
						return true;
					}
					if (item.attrMappings[attribute]) {
						return true;
					}
					else {
						return false;
					}
				},
				
				containsValue: function(/* com.ibm.data.AtomEntryItem */ item,
							/* String */ attribute, 
							/* Object */ value){
					// summary: Returns true if the value of the attribute on the given item is
					//		equal to the value argument, false otherwise.  If the attribute is a
					//		multi-valued attribute, it will check all the values of the attribute.
					// item: Item to check if the attribute is valid
					// attribute: Name of the attribute to check
					// value: Value to compare the actual value(s) to.
					var values = this.getValues(item, attribute);
					for(var i = 0; i < values.length; i++) {
						if (values[i] == value) {
							return true;
						}
					}
					return false;
				},
				
				isItem: function(/* Object */ something){
					// summary: Returns true if the argument is a valid item in this store, false otherwise.
					// something: Object to check to determine if it is an item in this store.
					return something.store == this && something.element && !something._isDeleted;
				},
				
				isItemLoaded: function(/* Object */ something) {
					// summary: All valid items in this data store are loaded all of the time as there
					//		is no support currently for stubs.  Therefore, this is effectively the same
					//		as isItem.
					return this.isItem(something) && something.loaded;
				},
				
				loadItem: function(/* Object */ keywordArgs){
					// summary: All valid items in this data store are loaded all of the time as there
					//		is no support currently for stubs.  Therefore, this function is a no-op. 
				},
				getResultSize: function(/*DOMDoc*/ doc, /*Request*/ request, /*Array*/ items) {
					// summary: Gets the total number of items retrieved by the query, which is not
					//		necessarily the number of items returned to the callbacks to fetch.
					//		This can be used for paging purposes to get the total number of items
					//		on the server that match the query, where subsequent fetch requests can
					//		be made to retrieve additional pages.
					return items.length;
				},
				processFetchedItems: function(/*Array*/ items, /*Request*/ request) {
					// summary: Extension point for descendants or consumers of this store to preprocess
					// 		the list of items that the store returns from a fetch call.  Allows a store
					// 		to filter the list or implement caching techniques on the client, or sort
					// 		the items, or anything else.
					// items: Array items of this data store found in the response of a fetch call
					// returns: Array items of this data store after processing
					return items;
				},		
				processResponse: function(/*DOMDoc*/ xmlDoc, /*Request*/ request) {
					// summary: Extension point for descendants or consumers of this store to handle
					//		responses from server requests for data.  Most subclasses can leave this
					// 		function as-is, but if special handling is required that cannot be done
					// 		with other extension points, this can be overridden as well.
					// 		Must call onBegin, onItem, and onComplete callbacks correctly as 
					// 		specified by dojo.data.api.Read.  The onError callback is handled elsewhere.
					// xmlDoc: DOMDoc of the response from an IO request
					// request: Request used to fetch the result
					if (request.aborted) {
						return;
					}
					var composite = this._parseFeed(request, xmlDoc); 
					var items = composite.items;
					this._feed = composite.root;
					items = this.processFetchedItems(items, request);
					if (request.onBegin) {
						request.onBegin.call(request.scope, this.getResultSize(composite.root, request, items), request);
					}
					if(request.onItem) {
						for(var i = 0; i < items.length; i++) {
							if (request.aborted) {
								return;
							}
							request.onItem.call(request.scope, items[i], request);
						}
					}
					if(request.onComplete) {
						// call onComplete with items array if onItem wasn't specified, otherwise
						// call it with null
						if (request.aborted) {
							return;
						}
						if (!request.onItem) {
							request.onComplete.call(request.scope, items, request);
						}
						else {
							request.onComplete.call(request.scope, null, request);
						}
					}
				},
				fetch: function( keywordArgs ) {
					//    summary:
			        //        Given a query and set of defined options, such as a start and count of items to return,
			        //        this method executes the query and makes the results available as data items.
			        //        The format and expectations of stores is that they operate in a generally asynchronous 
			        //        manner, therefore callbacks are always used to return items located by the fetch parameters.
			        //
			        //    description:
			        //        A Request object will always be returned and is returned immediately.
			        //        The basic request is nothing more than the keyword args passed to fetch and 
			        //        an additional function attached, abort().  The returned request object may then be used 
			        //        to cancel a fetch.  All data items returns are passed through the callbacks defined in the 
			        //        fetch parameters and are not present on the 'request' object.
			        //
			        //        This does not mean that custom stores can not add methods and properties to the request object
			        //        returned, only that the API does not require it.  For more info about the Request API, 
			        //        see dojo.data.api.Request
			        //
			        //    keywordArgs:
			        //        The keywordArgs parameter may either be an instance of 
			        //        conforming to dojo.data.api.Request or may be a simple anonymous object
			        //        that may contain any of the following:
			        //        { 
			        //            query: query-string or query-object,
			        //            queryOptions: object,
			        //            onBegin: Function,
			        //            onItem: Function,
			        //            onComplete: Function,
			        //            onError: Function,
			        //            scope: object,
			        //            start: int
			        //            count: int
			        //            sort: array
			        //        }
			        //        All implementations should accept keywordArgs objects with any of
			        //        the 9 standard properties: query, onBegin, onItem, onComplete, onError 
			        //        scope, sort, start, and count.  Some implementations may accept additional 
			        //        properties in the keywordArgs object as valid parameters, such as 
			        //        {includeOutliers:true}.         
			        //
					//	The *query* parameter.
					//        The query may be optional in some data store implementations.
					//        The dojo.data.api.Read API does not specify the syntax or semantics
					//        of the query itself -- each different data store implementation
					//        may have its own notion of what a query should look like.
					//        However, as of dojo 0.9, 1.0, and 1.1, all the provided datastores in dojo.data
					//        and dojox.data support an object structure query, where the object is a set of 
					//        name/value parameters such as { attrFoo: valueBar, attrFoo1: valueBar1}.  Most of the
					//        dijit widgets, such as ComboBox assume this to be the case when working with a datastore 
					//        when they dynamically update the query.  Therefore, for maximum compatibility with dijit 
					//        widgets the recommended query parameter is a key/value object.  That does not mean that the
					//        the datastore may not take alternative query forms, such as a simple string, a Date, a number, 
					//        or a mix of such.  Ultimately, The dojo.data.api.Read API is agnostic about what the query 
					//        format.  
					//        Further note:  In general for query objects that accept strings as attribute 
					//        value matches, the store should also support basic filtering capability, such as * 
					//        (match any character) and ? (match single character).  An example query that is a query object
					//        would be like: { attrFoo: "value*"}.  Which generally means match all items where they have 
					//        an attribute named attrFoo, with a value that starts with 'value'.
			        //
			        //    The *queryOptions* parameter
			        //        The queryOptions parameter is an optional parameter used to specify optiosn that may modify
			        //        the query in some fashion, such as doing a case insensitive search, or doing a deep search
			        //        where all items in a hierarchical representation of data are scanned instead of just the root 
			        //        items.  It currently defines two options that all datastores should attempt to honor if possible:
			        //        {
			        //            ignoreCase: boolean, //Whether or not the query should match case sensitively or not.  Default behaviour is false.
			        //            deep: boolean     //Whether or not a fetch should do a deep search of items and all child 
			        //                            //items instead of just root-level items in a datastore.  Default is false.
			        //        }
			        //
			        //    The *onBegin* parameter.
			        //        function(size, request);
			        //        If an onBegin callback function is provided, the callback function
			        //        will be called just once, before the first onItem callback is called.
			        //        The onBegin callback function will be passed two arguments, the
			        //        the total number of items identified and the Request object.  If the total number is
			        //        unknown, then size will be -1.  Note that size is not necessarily the size of the 
			        //        collection of items returned from the query, as the request may have specified to return only a 
			        //        subset of the total set of items through the use of the start and count parameters.
			        //
			        //    The *onItem* parameter.
			        //        function(item, request);
			        //        If an onItem callback function is provided, the callback function
			        //        will be called as each item in the result is received. The callback 
			        //        function will be passed two arguments: the item itself, and the
			        //        Request object.
			        //
			        //    The *onComplete* parameter.
			        //        function(items, request);
			        //
			        //        If an onComplete callback function is provided, the callback function
			        //        will be called just once, after the last onItem callback is called.
			        //        Note that if the onItem callback is not present, then onComplete will be passed
			        //        an array containing all items which matched the query and the request object.  
			        //        If the onItem callback is present, then onComplete is called as: 
			        //        onComplete(null, request).
			        //
			        //    The *onError* parameter.
			        //        function(errorData, request); 
			        //        If an onError callback function is provided, the callback function
			        //        will be called if there is any sort of error while attempting to
			        //        execute the query.
			        //        The onError callback function will be passed two arguments:
			        //        an Error object and the Request object.
			        //
			        //    The *scope* parameter.
			        //        If a scope object is provided, all of the callback functions (onItem, 
			        //        onComplete, onError, etc) will be invoked in the context of the scope
			        //        object.  In the body of the callback function, the value of the "this"
			        //        keyword will be the scope object.   If no scope object is provided,
			        //        the callback functions will be called in the context of dojo.global().  
			        //        For example, onItem.call(scope, item, request) vs. 
			        //        onItem.call(dojo.global(), item, request)
			        //
			        //    The *start* parameter.
			        //        If a start parameter is specified, this is a indication to the datastore to 
			        //        only start returning items once the start number of items have been located and
			        //        skipped.  When this parameter is paired withh 'count', the store should be able
			        //        to page across queries with millions of hits by only returning subsets of the 
			        //        hits for each query
			        //
			        //    The *count* parameter.
			        //        If a count parameter is specified, this is a indication to the datastore to 
			        //        only return up to that many items.  This allows a fetch call that may have 
			        //        millions of item matches to be paired down to something reasonable.  
			        //
			        //    The *sort* parameter.
			        //        If a sort parameter is specified, this is a indication to the datastore to 
			        //        sort the items in some manner before returning the items.  The array is an array of 
			        //        javascript objects that must conform to the following format to be applied to the
			        //        fetching of items:
			        //        {
			        //            attribute: attribute || attribute-name-string,
			        //            descending: true|false;   // Optional.  Default is false.
			        //        }
			        //        Note that when comparing attributes, if an item contains no value for the attribute
			        //        (undefined), then it the default ascending sort logic should push it to the bottom 
			        //        of the list.  In the descending order case, it such items should appear at the top of the list.
			        // 
			        //    returns:
			        //        The fetch() method will return a javascript object conforming to the API
			        //        defined in dojo.data.api.Request.  In general, it will be the keywordArgs
			        //        object returned with the required functions in Request.js attached.
			        //        Its general purpose is to provide a convenient way for a caller to abort an
			        //        ongoing fetch.  
			        // 
			        //        The Request object may also have additional properties when it is returned
			        //        such as request.store property, which is a pointer to the datastore object that 
			        //        fetch() is a method of.
			        //
			        //    exceptions:
			        //        Throws an exception if the query is not valid, or if the query
			        //        is required but was not supplied.
					if (!keywordArgs) {
						keywordArgs = {};
					}
			        var me = this;
					keywordArgs.aborted = false;
					if (!keywordArgs.scope) {
						keywordArgs.scope = dojo.global;
					}
					var url = this._buildUrl(keywordArgs, this.url);
					keywordArgs.url = url;
					var loadHandler = function(response, ioArgs) {
						if (keywordArgs.aborted) {
							return;
						}
						if (dojo.isIE) {
							response = com.ibm.domUtilities.docFromString(response);
						}
						me.doc = response;
						me.processResponse(response, keywordArgs);
					};
					var errorHandler = function(error, ioArgs) {
						if (keywordArgs.onError) {
							keywordArgs.onError.call(keywordArgs.scope, error, keywordArgs);
						}
					};
					var preventCache = false;
					if(keywordArgs.queryOptions && keywordArgs.queryOptions.preventCache) {
						preventCache = true; 
					}
					var args = {
						preventCache: preventCache,
						content: {},
						url: url,
						handleAs: "xml"
					}
					if(keywordArgs.queryOptions && keywordArgs.queryOptions.timeout) {
						args.timeout = keywordArgs.queryOptions.timeout;
					}
					else if(this.timeout) {
						args.timeout = this.timeout;
					}
					if(dojo.isIE) {
						args.handleAs = "text";
						args.content["ibm.web2.contentType"] = "text/xml";
					}
					
					var handle = null;
					keywordArgs.abort = function() {
						keywordArgs.aborted = true;
						handle.cancel();
					};
					args = this.modifyRequestArgs(args, keywordArgs);
					
					handle = dojo.xhrGet(args);
					handle.addCallback(loadHandler);
					handle.addErrback(errorHandler);

					this._lastRequest = keywordArgs;
					return keywordArgs;
				},
				modifyRequestArgs: function(ioArgs, request) {
					// summary: Allows modification to request arguments before sending a request
					//		to the server.  Subclasses may override this to modify arguments
					//		like setting headers, manipulating the url, overriding a handleAs argument,
					//		or anything else.  Returns the modified request args.
					// ioArgs: dojo.__XhrArgs request args to be sent to the server
					// request: dojo.data.api.Request object
					// returns: dojo.__XhrArgs modified request arguments
					return ioArgs;
				},
				_queryUrlToObject: function(queryStr) {
					if (!queryStr || queryStr.length === 0) {
						return {};
					}
					var start = queryStr.indexOf("?");
					var str = queryStr;
					if(start > -1 && (start < str.length - 1)) { 
						// extract only the query part of the queryStr URL
						str = queryStr.substring(start + 1);
					}
					return dojo.queryToObject(str);
				},
				_prepareQuery: function(keywordArgs) {
					var query = keywordArgs.query;
					if (!query) {
						keywordArgs.query = {};
					}
					else 
						if (dojo.isString(query)) {
							keywordArgs.query = this._queryUrlToObject(query);
						}
					return this.prepareQuery(keywordArgs);
				},
				prepareQuery: function(keywordArgs) {
					// summary: User overridable function to modify the keywordArgs object
					// 		used to fetch results from the back end after the keywordArgs.query
					//		property has been normalized to an object map but before any other
					//		processing has taken place to build the URL.  This can be used to
					//		extract other parts of the keywordArgs map to add properties to the
					//		query object that are dependent on the back end implementation.  One
					//		example includes extracting the keywordArgs.sort parameters and appending
					//		appropriate properties to the keywordArgs.query property that will end
					//		up being built into the generated URL to the server.
					// returns: Object Modified keywordArgs object.
					return keywordArgs;
				},
				_buildUrl: function(keywordArgs, baseUrl) {
					// summary: Builds a url string from this store's base url and the
					// 		query argument.
					// keywordArgs: Object
					//		A request object map as sent to the fetch function
					// returns: String The constructed url.
					var ret = "";
					var keywordArgs = this._prepareQuery(keywordArgs);
					var queryPart = dojo.objectToQuery(keywordArgs.query);
					if (baseUrl.indexOf("?") > -1) {
						ret = baseUrl + "&" + queryPart; // String
					}
					else {
						ret = baseUrl + "?" + queryPart; // String
					}
					
					return (this.modifyUrl(keywordArgs, ret));
				},
				modifyUrl: function(keywordArgs, url) {
					return url;
				},
				_parseFeed: function(request, xmlDoc) {
					var items = [];
					var root = null;
					var result = this._getNodeByXpath(xmlDoc, "/atom:feed");
					if(!result.found) {
						result = this._getNodeByXpath(xmlDoc, "/atom:entry");
						if(!result.found) {
							// the root is not an atom:feed or atom:entry
							throw new Error("Illegal atom feed format: the root element is not an ATOM feed or entry element.");
						}
						else {
							// the root is an atom:entry element, so there's only one item
							root = result.value;
							var query = this._queryUrlToObject(request.url);
							items.push(this._getItem({element: root, query: query}));
						}
					}
					else {
						// the root is an atom:feed element
						root = result.value;
						var query = this._queryUrlToObject(request.url);
						result = this._getNodesByXpath(root ? root : xmlDoc, "./atom:entry");
						if(result.found) {
							var nodes = result.value;
							for(var i = 0; i < nodes.length; i++) {
								items.push(this._getItem({element: nodes[i], query: query}));
							}
						}
					}
					
					return {items: items, root: root};
				},
				
				getFeatures: function () {
					//	summary:
					//		Return supported data APIs
					//	returns:
					//		Map of supported data APIs with keys as API classes and values as true/false to indicate support
					//		by this store.
					return this._features; //returns Map
				},
				
				close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
					if (request && request.abort) {
						request.abort();
					}
				},
				
				getLabel: function(/* com.ibm.data.AtomEntryItem */ item){
					return this.getEPResolvedValue(item, "title");
				},
				
				getLabelAttributes: function(/* com.ibm.data.AtomEntryItem */ item){
					return ['title'];
				},
				
				/********************************/
				/**                            **/
				/**                            **/
				/**    dojo.data.api.Write     **/
				/**                            **/
				/**                            **/
				/********************************/
				newItem: function(/* Object? */ keywordArgs, /*Object?*/ parentInfo){
					// summary: Must be overridden by stores that extend this but support writes,
					//		like ATOM services that support APP.
					return null;
				},
				
				deleteItem: function(/* com.ibm.data.AtomEntryItem */ item){
					// summary: Deletes an item from the store.  Does not persist to the server until save is called.
					// item: The item to delete.
					// returns: Boolean true if successful, false otherwise.							
					if (!this._deletedItems) {
						this._deletedItems = [];
					}
					item.backup();
					this._deletedItems.push(item);
					// storing identity into item since we need it later and can't access it through store APIs
					// after it's been marked as deleted
					item.id = this.getIdentity(item);
					item._isDeleted = true;
					// we are assuming that order does not matter, so we don't have to save the item's
					// current position in the list of entries in this feed
					return true;
				},
				
				setValue: function(	/* com.ibm.data.AtomEntryItem */ item, 
						/* String */ attribute,
						/* almost anything */ value){
					var ret = false;
					var oldValue = null;
					if(this.hasAttribute(item, attribute)) {
						var mapRule = this.getMapRule(attribute, item.attrMappings);
						// if mapRule is null here, then we didn't find one
						// in the map, so we can't proceed successfully
						if(mapRule) {
							item.backup();
							oldValue = this.getValueByMapping(item, mapRule);
							ret = this.setValueByMapping(item, mapRule, value);
							if(ret) {
								this._addDirtyItem(item);
								item.logWrite({attribute: attribute, value: value});
							}
						}
					}
					else {
						throw new Error(attribute + " is not an attribute of any item in this store");
					}
					return {success: ret, oldValue: oldValue, newValue: value};
				},
				
				setValues: function(/* com.ibm.data.AtomEntryItem */ item,
						/* String */ attribute, 
						/* array */ values){
					var ret = false;
					var oldValue = null;
					if(this.hasAttribute(item, attribute)) {
						var mapRule = this.getMapRule(attribute, item.attrMappings, true);
						// if mapRule is null here, then we didn't find one
						// in the map, so we can't proceed successfully
						if(mapRule) {
							item.backup();
							oldValue = this.getValueByMapping(item, mapRule);
							ret = this.setValuesByMapping(item, mapRule, values);
							if(ret) {
								this._addDirtyItem(item);
								item.logWrite({attribute: attribute, values: values});
							}
						}
					}
					else {
						throw new Error(attribute + " is not an attribute of any item in this store");
					}
					return {success: ret, oldValue: oldValue, newValue: values};
				},
				
				unsetAttribute: function(/* com.ibm.data.AtomEntryItem */ item, 
								/* String */ attribute){
					var ret = false;
					var oldValue = null;
					var value = null;
					if(this.hasAttribute(item, attribute)) {
						var mapRule = this.getMapRule(attribute, item.attrMappings);
						// if mapRule is null here, then we didn't find one
						// in the map, so we can't proceed successfully
						if(mapRule) {
							item.backup();
							oldValue = this.getValueByMapping(item, mapRule);
							if(mapRule.type == "attr") {
								ret = this.setValueByMapping(item, mapRule, "");
								value = "";
							}
							else {
								// it's a node or node property
								if (mapRule.targetName) {
									ret = this.setValueByMapping(item, mapRule, null);
								}
								// if targetName, then it's a property
								else 
									if (oldValue) {
										// if no targetName, then it's a node... so remove the node				
										oldValue.parentNode.removeChild(oldValue);
										ret = true;
									}
							}
							
							if(ret) {
								this._addDirtyItem(item);
								item.logWrite({attribute: attribute, value: value});
							}
						}
					}
					return {success: ret, oldValue: oldValue, newValue: value};
				},
				
				save: function(/* Object */ keywordArgs){
					var it = null;
					for(var d = 0; d < this._deletedItems.length; d++) {
						it = this._deletedItems[d];
						it.persist();
						delete it.element;
					}
					for(var i = 0; i < this._dirtyItems.length; i++) {
						this._dirtyItems[i].persist();
					}
					for(var i = 0; i < this._newItems.length; i++) {
						this._newItems[i].persist();
					}
					
					// the persist operation of the above items set some values on their backups
					// which may have updated the dirtyItems list... by putting this below here
					// we're removing those immediately if they are here 
					
					// instead of just reinitializing these pointers,
					// splice these so that we can keep the same array references forever in case others have
					// pointers to these still (so that they won't break)
					this._newItems.splice(0, this._newItems.length);
					this._dirtyItems.splice(0, this._dirtyItems.length);
					this._deletedItems.splice(0, this._deletedItems.length);
				},
				_addDirtyItem: function(/* com.ibm.portal.data.ContentModelItem */ item) {
					if (item._isDirty) {
						return;
					}
					item._isDirty = true;
					
					// need to check this
					for(var i = 0; i < this._dirtyItems.length; i++) {
						if (this._dirtyItems[i] == item) {
							return;
						}
					}
					
					this._dirtyItems.push(item);
				},
				
				revert: function(){
					var it = null;
					for(var d = 0; d < this._deletedItems.length; d++) {
						it = this._deletedItems[d];
						it.restore();
						delete item._isDeleted;
					}
					for(var i = 0; i < this._dirtyItems.length; i++) {
						this._dirtyItems[i].restore();
					}
					this._dirtyItems = [];
					this._deletedItems = [];
				},
				
				isDirty: function(/* com.ibm.data.AtomEntryItem? */ item){
					// summary: Determines whether or not an item is dirty or if this store in general has dirty items.
					//		If item is specified, it returns true if that item is dirty, and false otherwise.
					//		If no item is specified, it returns an array of all the dirty items in this store (modified,
					//		deleted, and new), and false if there are none.
					// item: Optional argument to check if a particular item is dirty.  Leaving this null
					if (item && this._assertIsItem(item)) {
						return item._isDirty;
					}
					else {
						var array = this._dirtyItems.concat(this._deletedItems).concat(this._newItems);
						if (!array || array.length === 0) {
							return false;
						}
						else {
							return array;
						}
					}
				}
			}
);	


dojo.declare( "com.ibm.data.AtomEntryItem",
			com.ibm.data.XpathHelper, 
			{
				namespaces: dojo.mixin({}, com.ibm.data.AtomFeedStore.prototype.namespaces),
		        // attrMappings: This maps attribute names to a query object consisting of an 
		        // 		xpath expression used to find the relevant node(s), an attr name, and a type.
		        // 		The map contains key-value pairs in this format:
		        //		{
		        //			attributeName: {xpath: "./this/is/the/xpath/expr", targetName: "attrName"|null, type: "node"|"attr"}  
		        //		}
		        //		The xpath expression is resolved from the context of the atom:entry node 
		        // 		assigned to this atom feed entry item.  The attr property is an optional property
		        //		used to specify whether to get an object property of the returned DOM node object or 
		        // 		an attribute of the returned DOM node.  The type specifies what type the lookup
		        //		will perform in conjunction with the attr.  If the lookup is an attribute of the
		        // 		DOM node, it will attempt to resolve any namespacing it encounters when looking
		        // 		up the attribute.  The following explains the possible combinations of lookup 
		        // 		mechanisms.
		        // 		
		        // 		xpath: "./node", targetName: "textContent", type: "node" = node["textContent"]
		        //		xpath: "./node", targetName: "href", type: "attr" = node.getAttribute("href")
		        // 		xpath: "./node", targetName: "textContent", type: "node" = cross browser node["textContent"] property
		        // 		xpath: "./node", targetName: null, type: "node" = node
		        //  	xpath: "./node", targetName: null, type: "attr" = invalid query, requires the attribute name
		        //
		        //		Map rules can be parameterized by substituting parameters specified by the caller into the
		        //		rule template.  Parameterized map rules MUST end with a $ character to mark them as such.
		        // 		Example:
		        //	 		pAttribute$: {xpath: "./${0}['${1}']", targetName: "${2}", type: "node", customRuleProperty: "${1}"}
		        //
		        //		Callers supply a comma-separated list of parameter values to be bound to subsequent numerical
		        //		parameter keys in the template respectively surrounded by brackets immediately following the
		        //		name of the attribute key but before the $ character.
		        //		Example:
		        // 			To request the value of pAttribute$ with values set for the parameters, the caller would 
		        // 			use code like this -
		        //		store.getValue(item, "pAttribute[div, id, nodeValue]$");
		        //
		        attrMappings: dojo.mixin({}, com.ibm.data.AtomFeedStore.prototype.attrMappings, { 		
					subtitle: {xpath: "./atom:subtitle", targetName: "textContent", type: "node", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:subtitle");
							this._setNodeProp(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					generator: {xpath: "./atom:generator", targetName: "textContent", type: "node", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:generator");
							this._setNodeProp(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					icon: {xpath: "./atom:icon", targetName: "textContent", type: "node", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:icon");
							this._setNodeProp(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					logo: {xpath: "./atom:logo", targetName: "textContent", type: "node", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:logo");
							this._setNodeProp(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					}
				}), 
				
				constructor: function( /*Map*/ args ) {
					dojo.mixin(this, args);
					this._writeLog = [];
				},
				toString: function() {
					return "<" + this.__getClassName() + ">";
				},
				_onCreate: function() {
					// summary: Called internally to indicate an item has been created.  Should
					//		be called immediately on creation for stores that do not save to
					//		a backend server, but should be called on a valid server response for
					// 		an item that was created and saved to the server successfully.  This
					// 		calls onCreate to notify listeners of the event and can therefore be
					// 		overriden if custom logic needs to be applied to handle cases where
					// 		other conditions need to be satisfied before the onCreate event is called.
					this.onCreate();
				},
				onCreate: function() {
					// summary: Called when this item is created.  This callback is reserved
					// 		for sending notifications when a new item stub has been created
					// 		on the actual server.  This will be called when a new item has been
					//		created, saved to the server, and the server responds back with
					//		a verification of the created item.  Some stores may not have any
					// 		notion of creating items on the server and should call this immediately
					// 		when the item is created on the client, while others actually defer 
					// 		the full creation of a new item from the newItem call until after the 
					// 		save operation has persisted to the backend.  This allows interested 
					// 		parties to get notified when an item they've created is actually saved
					// 		on the server.  This is especially useful if the server has to generate
					//		something like a unique ID for the item which is not available on the
					//		item until the server response comes back.
				},
				_makeModifiable: function() {
					// summary: Modifies the element of this item to become modifiable
					//		for writing back to the server.  Subclasses can override this
					//		to do things like prune an item to only include necessary information
					//		for updates and nothing more.  This is called by backup once after
					//		backing up the original element first.  After this returns,
					//		this item's element may have been modified.  Any queries against
					//		this item should now include logic to check the backup copy as well
					//		if a query doesn't return anything as it could have been deleted by
					//		this function.  Care should be taken to ensure that the query result
					//		specifically detects if nothing was found or if the actual value was
					//		null or empty.
				},
				backup: function() {
					// summary: Creates a single backup copy of this item on the client for
					//		later restoration if necessary.  Calling this function multiple
					//		times before restore() or persist() is called on this item will do nothing
					//		after the initial backup, effectively maintaining only the first
					//		version of the item before any changes were made.
					if(!this._backup) {
						this._backup = this.element.cloneNode(true);
						this._makeModifiable();
						this._logEnabled = true;
						return true;
					}
					return false;
				},
				restore: function() {
					// summary: Restores the single backup copy of this item on the client,
					//		erasing all modifications made since backup() was called.
					if(!this._backup) {
						dojo.place(this._backup, this.element, "before");
						this.element.parentNode.removeChild(this.element);
						delete this.element;
						this.element = this._backup;
						this.backup = null;
						this._writeLog.splice(0, this._writeLog.length);
						return true;
					}
					return false;
				},
				persist: function() {
					// summary: Persists the changes made to this item since the backup() was
					// 		called.  After this is called, there is no way to restore the backup
					//		as it is erased completely.
					this._writeLogToBackup();
					this._isDirty = false;
					delete this._backup;
				},
				logWrite: function(map) {
					// summary: Logs write operations to this item so that they can later
					//		be repeated to the backup during persistence.
					if (this._logEnabled) {
						this._writeLog.push(map);
					}
				},
				_writeLogToBackup: function() {
					// summary: Repeates the write log for this item to the original backup
					// 		node.  Used mainly for persisting changes to support a 2-phase
					//		write operation (write, commit).
					if(this._backup) {
						this.element = this._backup;
						this._logEnabled = false;
						for(var i = 0; i < this._writeLog.length; i++) {
							var op = this._writeLog[i];
							if(op.values) {
								this.store.setValues(this, op.attribute, op.values);
							}
							else {
								this.store.setValue(this, op.attribute, op.value);
							}
						}
						this._writeLog.splice(0, this._writeLog.length);
					}
				}
			}
);


dojo.declare( "com.ibm.data.AtomPersonConstruct",
			com.ibm.data._XmlConstruct,
			{
				// summary: Construct for retrieving appropriate information
				// 		from an Atom Person Construct as defined in 
				// 		http://tools.ietf.org/html/rfc4287#section-3.2.
				
				namespaces: dojo.mixin({}, com.ibm.data.AtomFeedStore.prototype.namespaces),
		        
				constructor: function(/*DOMNode*/ root) {
					this.root = root;
				},
				getName: function() {
					// summary: Gets the name specified by this Atom person construct.
					// returns: String
					var result = this._getNodeByXpath(this.root, "./atom:name", "textContent");
					if (!result.found) {
						result.value = "";
					}
					return result.value;
				},
				getEmail: function() {
					// summary: Gets the email specified by this Atom person construct.
					// returns: String
					var result = this._getNodeByXpath(this.root, "./atom:email", "textContent");
					if (!result.found) {
						result.value = "";
					}
					return result.value;
				},
				getURI: function() {
					// summary: Gets the name specified by this Atom person construct.
					// returns: String
					var result = this._getNodeByXpath(this.root, "./atom:uri", "textContent");
					if (!result.found) {
						result.value = "";
					}
					return result.value;
				},
				toString: function() {
					// summary: Gets the name string specified by this Atom person construct.
					// returns: String
					return this.getName();
				}
			}
);

dojo.declare( "com.ibm.data.AtomDateConstruct",
			com.ibm.data._XmlConstruct,
			{
				// summary: Construct for retrieving javascript-appropriate information
				// 		from an Atom Date Construct as defined in 
				// 		http://tools.ietf.org/html/rfc4287#section-3.3.
				
				namespaces: dojo.mixin({}, com.ibm.data.AtomFeedStore.prototype.namespaces),
		        
				// Supports date strings in the following formats - 
				//		2003-12-13T18:30:02Z
				//		2003-12-13T18:30:02.25Z
				//		2003-12-13T18:30:02+01:00
				//		2003-12-13T18:30:02.25+01:00
				
				constructor: function(/*DOMNode*/ root) {
					this.root = root;
				},
				getDate: function() {
					// summary: Parses and retrieves a javascript Date object from a date string
					//		specified in an Atom feed according to the allowable string formats
					//		by the Atom spec itself (http://tools.ietf.org/html/rfc4287#section-3.3)
					// returns: Date
					if (this.date) {
						return this.date;
					}
					
					var d = this._getNodeProp(this.root, "textContent");
					if (!d) {
						return null;
					}
					
					this.date = dojo.date.stamp.fromISOString(d);
					return this.date;
				},
				getTime: function() {
					// summary: Gets the time specified by this Atom date construct.
					// returns: int
					this.getDate();
					if (this.date) {
						return this.date.getTime();
					}
					
					return null;
				},
				toString: function() {
					// summary: Gets a string representation of the date specified by this
					//		Atom data construct.
					this.getDate();
					if (this.date) {
						return this.date.toString();
					}
					return "";
				}
			}
);

}

if(!dojo._hasResource["com.ibm.data.OpenSearchFeedStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.OpenSearchFeedStore"] = true;
dojo.provide( "com.ibm.data.OpenSearchFeedStore" );

dojo.require( "com.ibm.data.AtomFeedStore" );
dojo.require( "dojox.data.dom" );

dojo.declare( "com.ibm.data.OpenSearchFeedStore",
			com.ibm.data.AtomFeedStore,
			{
				// namespaces: Map
				//		Map of namespaces used by the MashupHub feeds
				namespaces: dojo.mixin({}, com.ibm.data.AtomFeedStore.prototype.namespaces, {
		            "xhtml" : "http://www.w3.org/1999/xhtml",
		            "os" : "http://a9.com/-/spec/opensearch/1.1/"
		        }),
		        
		        attrMappings: dojo.mixin({}, com.ibm.data.AtomFeedStore.prototype.attrMappings, {
					nextLink: {xpath: "./atom:link[@rel='next']", targetName: "href", type: "attr"},
					startIndex: {xpath: "./os:startIndex", targetName: "textContent", type: "node"},
					itemsPerPage: {xpath: "./os:itemsPerPage", targetName: "textContent", type: "node"},
					totalResults: {xpath: "./os:totalResults", targetName: "textContent", type: "node"},
					query: {xpath: "./os:Query", type: "OpenSearchQueryConstruct"}
				}),
				
				// itemClass: String the class used to represent items of this store
				itemClass: "com.ibm.data.OpenSearchFeedItem",
				
				constructor: function( /*Map*/ args ) {
					// summary: Creates a new instance of a com.ibm.portal.data._PortalModelStore
					// args: Map
					//		The args map will be used to replace any default values of this store with the
					//		values specified in the args map.
					this._features['dojo.data.api.Identity'] = true;
				},
				
				handleConstructs: function(/*DOMNode*/ node, /*String*/ prop) {
					switch(prop) {
						case "OpenSearchQueryConstruct": return new com.ibm.data.OpenSearchQueryConstruct(node);
						default: return this.inherited(arguments);
					}
				},
				
				getResultSize: function(/*DOMDoc*/ doc, /*Request*/ request, /*Array*/ items) {
					// summary: Gets the total number of items retrieved by the query, which is not
					//		necessarily the number of items returned to the callbacks to fetch.
					//		This can be used for paging purposes to get the total number of items
					//		on the server that match the query, where subsequent fetch requests can
					//		be made to retrieve additional pages.
					var rule = this.attrMappings.totalResults;
					var ret = null;
					if(rule.type == "attr") ret = this._getAttrByXpath(doc, rule.xpath, rule.targetName);
					else ret = this._getNodeByXpath(doc, rule.xpath, rule.targetName);
					// if we found total results, return it as a number
					if(ret.found) return ret.value - 0;
					// otherwise, return -1 to indicate that we don't know
					else return -1;
				},
				
				/********************************/
				/**                            **/
				/**                            **/
				/**   dojo.data.api.Identity   **/
				/**                            **/
				/**                            **/
				/********************************/			
				fetchItemByIdentity: function(/* object */ keywordArgs){
					//	summary:
					//		Given the identity of an item, this method returns the item that has 
					//		that identity through the onItem callback.  Conforming implementations 
					//		should return null if there is no item with the given identity.  
					//		Implementations of fetchItemByIdentity() may sometimes return an item 
					//		from a local cache and may sometimes fetch an item from a remote server, 
					//
					// 	keywordArgs:
					//		An anonymous object that defines the item to locate and callbacks to invoke when the 
					//		item has been located and load has completed.  The format of the object is as follows:
					//		{
					//			identity: string|object,
					//			onItem: Function,
					//			onError: Function,
					//			scope: object
					//		}
					//	The *identity* parameter.
					//		The identity parameter is the identity of the item you wish to locate and load
					//		This attribute is required.  It should be a string or an object that toString() 
					//		can be called on.
					//		
					//	The *onItem* parameter.
					//		Function(item)
					//		The onItem parameter is the callback to invoke when the item has been loaded.  It takes only one
					//		parameter, the item located, or null if none found.
					//
					//	The *onError* parameter.
					//		Function(error)
					//		The onError parameter is the callback to invoke when the item load encountered an error.  It takes only one
					//		parameter, the error object
					//
					//	The *scope* parameter.
					//		If a scope object is provided, all of the callback functions (onItem, 
					//		onError, etc) will be invoked in the context of the scope object.
					//		In the body of the callback function, the value of the "this"
					//		keyword will be the scope object.   If no scope object is provided,
					//		the callback functions will be called in the context of dojo.global.
					//		For example, onItem.call(scope, item, request) vs. 
					//		onItem.call(dojo.global, item, request)
					return false;
				}
			}
);

dojo.declare( "com.ibm.data.OpenSearchFeedItem",
			com.ibm.data.AtomEntryItem, 
			{
				// namespaces: Map
				//		Map of namespaces used by the MashupHub feeds
				namespaces: dojo.mixin({}, com.ibm.data.OpenSearchFeedStore.prototype.namespaces)
			}
);


dojo.declare( "com.ibm.data.OpenSearchQueryConstruct",
			com.ibm.data._XmlConstruct,
			{
				namespaces: dojo.mixin({}, com.ibm.data.OpenSearchFeedStore.prototype.namespaces),
				
				constructor: function(/*DOMNode*/ root) {
					this.root = root;
				},
				getRole: function() {
					return this._getAttribute(this.root, "role");
				},
				getTitle: function() {
					return this._getAttribute(this.root, "title");
				},
				getTotalResults: function() {
					return this._getAttribute(this.root, "totalResults");
				},
				getSearchTerms: function() {
					return this._getAttribute(this.root, "searchTerms");
				},
				getCount: function() {
					return this._getAttribute(this.root, "count");
				},
				getStartIndex: function() {
					return this._getAttribute(this.root, "startIndex");
				},
				getStartPage: function() {
					return this._getAttribute(this.root, "startPage");
				},
				getLanguage: function() {
					return this._getAttribute(this.root, "language");
				},
				getInputEncoding: function() {
					return this._getAttribute(this.root, "inputEncoding");
				},
				getOutputEncoding: function() {
					return this._getAttribute(this.root, "outputEncoding");
				},
				toString: function() {
					return com.ibm.domUtilities.stringFromDoc(this.root);
				}
			}
);

}

if(!dojo._hasResource["com.ibm.data.MashupCatalogFeedStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.MashupCatalogFeedStore"] = true;
dojo.provide("com.ibm.data.MashupCatalogFeedStore");




dojo.declare("com.ibm.data.MashupCatalogFeedStore", 
		[com.ibm.data.OpenSearchFeedStore,com.ibm.data.CatalogMixin],
		{
			itemClass: "com.ibm.data.MashupCatalogFeedItem",
			searchable: false,
			
			namespaces: dojo.mixin({}, com.ibm.data.OpenSearchFeedStore.prototype.namespaces, {
		        "ext" : "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions",
				"model" : "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements",
				"base" : "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base",
				"catalog" : "http://www.ibm.com/xmlns/atom/opensearch/catalog/1.0/",
				"xsi" : "http://www.w3.org/2001/XMLSchema-datatypes",
				"openSearch" : "http://a9.com/-/spec/opensearch/1.1/"
		    }),
		       
			constructor: function( /*Map*/ args ) {
				this._features['dojo.data.api.Identity'] = true;
				var serviceDoc = com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_CATALOG;
				var serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(serviceDoc);
				this.isPortal = (serviceJson.idprefix == "id") ? false : true;
				
				this.url = serviceJson.url + "@"+ serviceJson.idprefix + ":" + this.category;
				var catalog = this.catalog ? this.catalog : "default";
				var mashupsIdPostfix = this.isPortal ? "" : "_c_" + catalog;
				this.url = this.url + mashupsIdPostfix;				
			},
				
			mapItem: function(/*item*/item){
				var map = {};
				var bigId = this.getEPResolvedValue(item, "id", null);
				if(this.isPortal) var ids = /^catalog:oid:(.+)@oid:.+/.exec(bigId);
				else var ids = /^catalog:id:(.+)@/.exec(bigId);
				map.id = ids.length > 1 ? ids[1] : "";
				map.label = this.getEPResolvedValue(item, "title", "untitled");
				map.description = this.getEPResolvedValue(item, "summary", "");
				map.thumbnail = this.getValue(item, "thumbnail", "");
				map.icon = this.getValue(item, "catalogIcon", "");
				map.onAutoWire = this.getEPResolvedValue(item, "catalogAutoWire", "");
				map.widgetParam = this.getEPResolvedValue(item, "catalogWidgetParam", "");
                if(map.widgetParam.length > 0) { map.prefsMapFcn = this.prefsMapFcn; }
                map.definitionURL = this.getValue(item, "definitionURL", "");
				map.type = "catalog";
				map.isTrusted = com.ibm.mm.builder.utils.iwidgetUtil.isWidgetTrusted(map.id);
				if(!map.thumbnail && builderConfig.builderPath) map["thumbnail"] = builderConfig.builderPath + "/images/gray_generic_widget_icon.png";
				return map;
			},
			
            prefsMapFcn: function(map){
				var pars = dojo.fromJson(map.widgetParam);
				var ret = {};
				for(var i in pars){
					ret[pars[i].itemName] = pars[i].itemValue;
				}
				return ret;
			},
            	
			prepareQuery: function(keywordArgs) {
				if(keywordArgs.count) {
					if(this.isPortal) keywordArgs.query["max-results"] = keywordArgs.count;
					else keywordArgs.query.num = keywordArgs.count;
					if(keywordArgs.start == null) {
						keywordArgs.start = 0;
					}
					if(this.isPortal) keywordArgs.query["start-index"] = keywordArgs.start;
					else keywordArgs.query.start = keywordArgs.start;
				}
				return keywordArgs;
			},
			
			validateSelf: function(){
				// summary: validates this datastore, if it does not validate the source will not be used
				// returns true for validation and false otherwise
				if(dojo.trim(ibmConfig["com.ibm.mashups.builder.contentShelf.mode"]) == "mashups") {
					var categories = com.ibm.mm.builder.utils.toolboxUtil.getCategories();
					var found = false;
					for (var i = 0; i < categories.length; i++) {
						var category = categories[i];
						var id = category.getID();
						if(id.indexOf(this.category) == 0) {
							found = true;
							break;
						}					
					}					
					return found;
					
				} else {
					return false;
				}
			},
			
			
			
			getSourceLabel: function(){
				var categories = com.ibm.mm.builder.utils.toolboxUtil.getCategories();
				for (var i = 0; i < categories.length; i++) {
					var category = categories[i];
					var id = category.getID();
                    if(id.indexOf(this.category) == 0
                        && (this.isPortal ? true : (id.indexOf("_c_") == this.category.length))) {
                        return com.ibm.mm.builder.utils.enablerUtil.getNodeTitle(category);
                    }
				}
				return false;
			}
});

dojo.declare( "com.ibm.data.MashupCatalogFeedItem",
		com.ibm.data.OpenSearchFeedItem, 
		{
			namespaces: dojo.mixin({}, com.ibm.data.MashupCatalogFeedStore.prototype.namespaces),
			// there is no 'widget.metadata' prefix for the metadata
	        attrMappings: dojo.mixin({}, com.ibm.data.OpenSearchFeedItem.prototype.attrMappings, {
				thumbnail: {xpath: "./atom:content/model:catalog-entry/atom:link[@ext:rel='icon']", targetName: "href", type: "attr"},
                catalogIcon: {xpath: "./atom:content/model:catalog-entry/atom:link[@ext:rel='icon']", targetName: "href", type: "attr"},
				catalogAutoWire: {xpath: "//model:metadata[@name='com.ibm.mashups.builder.autoWiringEnabled']/base:value", targetName: "value", type: "attr"},
                definitionURL: {xpath: "./atom:content/model:catalog-entry/atom:link[@ext:rel='definition']", targetName: "href", type: "attr"},
				catalogWidgetParam: {xpath: "//model:metadata[@name='widgetParam']/base:value", targetName: "value", type: "attr"}
			})
		}
);

}

if(!dojo._hasResource["com.ibm.data.MashupHubStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.MashupHubStore"] = true;
dojo.provide( "com.ibm.data.MashupHubStore" );


dojo.require( "com.ibm.data.ProxyHelper" );

dojo.declare( "com.ibm.data.MashupHubStore",
			[com.ibm.data.OpenSearchFeedStore, com.ibm.data.ProxyHelper],
			{
				// namespaces: Map
				//		Map of namespaces used by the MashupHub feeds
				namespaces: null,
		        
		        // itemClass: String the class used to represent items of this store
				itemClass: "com.ibm.data.MashupHubItem",
				
				constructor: function( /*Map*/ args ) {
					// summary: Creates a new instance of a this data store
					// args: Map
					//		The args map will be used to replace any default values of this store with the
					//		values specified in the args map.
					this._features['dojo.data.api.Identity'] = true;
					
					var newUrl = new com.ibm.mm.enabler.utils.HttpUrl(this.url);
					this.url = newUrl.toProxifiedString();
					
					this.bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf"); 
					
					if(args && args.namespace == 'widget') {
						this.namespaces = dojo.delegate(com.ibm.data.OpenSearchFeedStore.prototype.namespaces,{
							catalog: "http://www.ibm.com/xmlns/atom/opensearch/widget/1.0/"
							});
					} else if(args && args.namespace == 'feed') {
						this.namespaces = dojo.delegate(com.ibm.data.OpenSearchFeedStore.prototype.namespaces,{
							catalog: "http://www.ibm.com/xmlns/atom/opensearch/feed/1.0/"
							});
					} else {
						this.namespaces = dojo.delegate(com.ibm.data.OpenSearchFeedStore.prototype.namespaces,{
							catalog: "http://www.ibm.com/opensearch/1.0"
							});
					}	
				},
				
				handleConstructs: function(/*DOMNode*/ node, /*String*/ prop) {
					switch(prop) {
						case "OpenSearchQueryConstruct": return new com.ibm.data.MashupHubQueryConstruct(node);
						default: return this.inherited(arguments);
					}
				},
			
				fetchItemByIdentity: function(/* object */ keywordArgs){
					//	summary:
					//		Given the identity of an item, this method returns the item that has 
					//		that identity through the onItem callback.  Conforming implementations 
					//		should return null if there is no item with the given identity.  
					//		Implementations of fetchItemByIdentity() may sometimes return an item 
					//		from a local cache and may sometimes fetch an item from a remote server, 
					//
					// 	keywordArgs:
					//		An anonymous object that defines the item to locate and callbacks to invoke when the 
					//		item has been located and load has completed.  The format of the object is as follows:
					//		{
					//			identity: string|object,
					//			onItem: Function,
					//			onError: Function,
					//			scope: object
					//		}
					//	The *identity* parameter.
					//		The identity parameter is the identity of the item you wish to locate and load
					//		This attribute is required.  It should be a string or an object that toString() 
					//		can be called on.
					//		
					//	The *onItem* parameter.
					//		Function(item)
					//		The onItem parameter is the callback to invoke when the item has been loaded.  It takes only one
					//		parameter, the item located, or null if none found.
					//
					//	The *onError* parameter.
					//		Function(error)
					//		The onError parameter is the callback to invoke when the item load encountered an error.  It takes only one
					//		parameter, the error object
					//
					//	The *scope* parameter.
					//		If a scope object is provided, all of the callback functions (onItem, 
					//		onError, etc) will be invoked in the context of the scope object.
					//		In the body of the callback function, the value of the "this"
					//		keyword will be the scope object.   If no scope object is provided,
					//		the callback functions will be called in the context of dojo.global.
					//		For example, onItem.call(scope, item, request) vs. 
					//		onItem.call(dojo.global, item, request)
					var id = keywordArgs.identity;
					
					// our IDs may come in as urn:id:# and all we want is the # part
					if(id.indexOf(":") > -1) id = id.substring(id.lastIndexOf(":") + 1);
					var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
					var args = {
						query: {
							id: id,
							collection: "ui.all"
						},
						onComplete: dojo.hitch(this, 
							function(items) {
								if(items.length == 0 && keywordArgs.onError) {
									keywordArgs.onError.call(scope, "No items returned matching identity " + id);
								}
								else {
									keywordArgs.onItem.call(scope, items[0]);
								}
							}
						)
					};
					if(keywordArgs.onError) args.onError = dojo.hitch(scope, keywordArgs.onError);

					return this.fetch(args);
				}
			}
);

dojo.declare( "com.ibm.data.MashupHubItem",
		com.ibm.data.OpenSearchFeedItem, 
		{
			// namespaces: Map
			//		Map of namespaces used by the MashupHub feeds
			namespaces: dojo.mixin({}, com.ibm.data.MashupHubStore.prototype.namespaces),
			
	        attrMappings: dojo.mixin({}, com.ibm.data.OpenSearchFeedItem.prototype.attrMappings, {
				editLink: {xpath: "./atom:link[@rel='self']", targetName: "href", type: "attr"},
				category: {xpath: "./atom:category", targetName: "term", type: "attr"},
				catalogVersion: {xpath: "./atom:content/catalog:*/catalog:version", targetName: "textContent", type: "node"},
				catalogName: {xpath: "./atom:content/catalog:*/catalog:name", targetName: "textContent", type: "node"},
				catalogAuthor: {xpath: "./atom:content/catalog:*/catalog:author", targetName: "textContent", type: "node"},
				catalogDescription: {xpath: "./atom:content/catalog:*/catalog:description", targetName: "textContent", type: "node"},
				catalogRating: {xpath: "./atom:content/catalog:*/catalog:rating", targetName: "textContent", type: "node"},
				catalogUseCount: {xpath: "./atom:content/catalog:*/catalog:useCount", targetName: "textContent", type: "node"},
				catalogDateModified: {xpath: "./atom:content/catalog:*/catalog:dateModified", targetName: "textContent", type: "node"},
				catalogNumComments: {xpath: "./atom:content/catalog:*/catalog:numComments", targetName: "textContent", type: "node"},
				catalogTags: {xpath: "./atom:content/catalog:*/catalog:tags/catalog:tags", targetName: "textContent", type: "node"},
				catalogCategories: {xpath: "./atom:content/catalog:*/catalog:categories", targetName: "textContent", type: "node"},
				catalogDocumentationURL: {xpath: "./atom:content/catalog:*/catalog:documentationURL", targetName: "textContent", type: "node"},
				catalogDownloadURL: {xpath: "./atom:content/catalog:*/catalog:downloadURL", targetName: "textContent", type: "node"},
				catalogIcon: {xpath: "./atom:content/catalog:*/catalog:objectmeta/catalog:metadata[@name='icon']", targetName: "value", type: "attr"},
				catalogWidgetParam: {xpath: "./atom:content/catalog:*/catalog:objectmeta/catalog:metadata[@name='widget.metadata.widgetParam']", targetName: "value", type: "attr"},
				catalogOnNewWire: {xpath: "./atom:content/catalog:*/catalog:objectmeta/catalog:metadata[@name='widget.metadata.onNewWire']", targetName: "value", type: "attr"},
				catalogAutoWire: {xpath: "./atom:content/catalog:*/catalog:objectmeta/catalog:metadata[@name='widget.metadata.com.ibm.mashups.builder.autoWiringEnabled']", targetName: "value", type: "attr"},
				catalogPermission: {xpath: "./atom:content/catalog:*/catalog:permission", targetName: "textContent", type: "node"},
				catalogDefinitionURL: {xpath: "./atom:content/catalog:*/catalog:definitionURL", targetName: "textContent", type: "node"},
				catalogTitles: {xpath: "./atom:content/catalog:*/catalog:l10n/catalog:title", targetName: null, type: "node"},
				catalogSandboxUser: {xpath: "./atom:content/catalog:*/catalog:objectmeta/catalog:metadata[@name='sandbox_user']", targetName: "value", type: "attr"},
				catalogSandboxUserDefinition: {xpath: "./atom:content/catalog:*/catalog:objectmeta/catalog:metadata[@name='widget.metadata.sandbox_definition']", targetName: "value", type: "attr"},
				catalogWidgetType: {xpath: "./atom:content/catalog:*/catalog:objectmeta/catalog:metadata[@name='widget.metadata.widgetType']", targetName: "value", type: "attr"}				
			})
		}
);

dojo.declare( "com.ibm.data.MashupHubQueryConstruct",
			com.ibm.data.OpenSearchQueryConstruct,
			{
				namespaces: dojo.mixin({}, com.ibm.data.MashupHubStore.prototype.namespaces),
				
				getSearchType: function() {
					return this._getAttribute(this.root, "catalog:searchType");
				},
				getCollection: function() {
					return this._getAttribute(this.root, "catalog:collection");
				}
			}
);

}

if(!dojo._hasResource["com.ibm.data.MashupFileDataStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.MashupFileDataStore"] = true;
dojo.provide("com.ibm.data.MashupFileDataStore");

dojo.require( "com.ibm.data.CatalogMixin" );


dojo.declare("com.ibm.data.MashupFileDataStore", 
		[com.ibm.data.MashupHubStore,com.ibm.data.CatalogMixin],
		{	
				/********************************/
				/**                            **/
				/**                            **/
				/**  catalog extension point   **/
				/**                            **/
				/**                            **/
				/********************************/
				
			mapItem: function(/*item*/item){
				var map = {};
				map.id = this.getEPResolvedValue(item, "id", null);
				map.label = this.getEPResolvedValue(item, "catalogName", this.bundle["shelf_untitled"]);
				map.description = this.getEPResolvedValue(item, "catalogDescription", "");
				map.tags = this.getEPResolvedValue(item, "catalogTags", "");
				map.rating = this.getEPResolvedValue(item, "catalogRating", 0.0);
				map.url = this.getEPResolvedValue(item, "catalogDownloadURL", "");
				map.type = this.getEPResolvedValue(item, "category", "");
				map.thumbnail = this.getValue(item, "catalogIcon", "");
				map.widgetParam = this.getEPResolvedValue(item, "catalogWidgetParam", "");
				map.onNewWire = this.getEPResolvedValue(item, "catalogOnNewWire", "");
				map.onAutoWire = this.getEPResolvedValue(item, "catalogAutoWire", "");
				map.permission = this.getEPResolvedValue(item, "catalogPermission", "");
				map.definitionURL = this.getValue(item, "catalogDefinitionURL", "");
				map.sandbox_user = this.getEPResolvedValue(item, "catalogSandboxUser", "false");
				map.sandbox_definition = this.getEPResolvedValue(item, "catalogSandboxUserDefinition", "false");
				map.icon = this.getValue(item, "catalogIcon", "false");
				map.widgetType = this.getEPResolvedValue(item, "catalogWidgetType", "");
				if (map.widgetType === "mm_OSGadget") {
					map.titles = this.getValue(item, "catalogTitles", null);
				}
				map.hubURL = this.url;
				map.isTrusted = map.sandbox_user == "true" ? false : true;
				if(map.widgetParam.length > 0) { map.prefsMapFcn = this.prefsMapFcn; }
				if (!map.thumbnail && builderConfig.builderPath) {
					if (item.store.namespace == "widget"){
						map["thumbnail"] = builderConfig.builderPath + "/images/generic_widget_icon.png";
					} else {
						map["thumbnail"] = builderConfig.builderPath + "/images/gray_generic_feed_icon.png";
					}						
				}
				return map;
			},
			
			prefsMapFcn: function(map){
				var pars = dojo.fromJson(map.widgetParam);
				var ret = {};
				for(var i in pars){
					ret[pars[i].itemName] = pars[i].itemValue;
				}
				return ret;
			},
				
			prepareQuery: function(keywordArgs) {
				if(keywordArgs.query.keywords) {
					keywordArgs.query.searchTerms = keywordArgs.query.keywords.join(" ");
					keywordArgs.query.searchType = "";
					delete keywordArgs.query.keywords;
				}
				else if(keywordArgs.query.tags && keywordArgs.query.tags.length > 0) {
					keywordArgs.query.searchTerms = keywordArgs.query.tags[0];
					keywordArgs.query.searchType = "tag";
					delete keywordArgs.query.tags;
				}
				else {
					keywordArgs.query.searchTerms = "*";
					keywordArgs.query.searchType = "";
				}
				
				if(keywordArgs.sort && keywordArgs.sort.length > 0) {
					switch(keywordArgs.sort[0].attribute) {
						case "date": 
							keywordArgs.query.sortBy = "updated";
							break;
						default:
							keywordArgs.query.sortBy = keywordArgs.sort[0].attribute;
					}
					keywordArgs.query.sortOrder = "asc";
					if(keywordArgs.sort[0].descending == true) {
						keywordArgs.query.sortOrder = "desc";
					}
				}
				
				if(keywordArgs.count) {
					keywordArgs.query.count = keywordArgs.count;
					if(keywordArgs.start == null) {
						keywordArgs.start = 0;
					}
					keywordArgs.query.startIndex = keywordArgs.start;
				}
				
				// NEVER get the feed from the cache, defect 16700
				if(!keywordArgs.queryOptions){
					keywordArgs.queryOptions = {};
				}
				keywordArgs.queryOptions.preventCache = true;
				
				return keywordArgs;
			},
			
			validateSelf: function(){
				// summary: validates this datastore, if it does not validate the source will not be used
				// returns true for validation and false otherwise
				var configSvc = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
				var hubUrl = configSvc.getValue(com.ibm.mashups.enabler.services.ConfigConstants.HUB_URL);
				if(dojo.trim(ibmConfig["com.ibm.mashups.builder.contentShelf.mode"]) != "mashups" && hubUrl && this.url.indexOf("undefined") < 0) {
					return true;
				} else {
					return false;
				}
			}
});

}

if(!dojo._hasResource["com.ibm.widgets.InlineShelf"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.widgets.InlineShelf"] = true;
 // This is a dropdown inline shelf

dojo.provide("com.ibm.widgets.InlineShelf");



dojo.declare("com.ibm.widgets.InlineShelf",
	[dijit._Widget, dijit._Templated],
	{
		content: null,				// HTML content to display in the shelf
		link: null,					// an id or DOM element to which the shelf's toggle function should be attached
		shelfClass: "",				// css classes to put on the shelf
		displayState: "closed",		// state of the shelf, ie "open", "closed"
		everOpened: false,			// true when the shelf has been opened before
		wipeOut: null,				// animation to slide the shelf out of view
		wipeIn: null,				// animation to slide the shelf into view
		wipeTime: 300,				// the time the shelf takes to slide in/out in milliseconds
		templateString: '<div dojoAttachPoint="focusNode" style="display: none;" class="${shelfClass}"></div>',
		
		startup: function(){
			// create the hide/show animations
			this.wipeOut = dojo.fx.wipeOut({node:this.domNode, duration:this.wipeTime});
			this.wipeIn = dojo.fx.wipeIn({node:this.domNode, duration:this.wipeTime});
			
			// add the content
			if(this.content) this.domNode.appendChild(this.content);
			this.onCreate();
		},
		
		connectButton: function(){
			// set the onclick handler for the toggle link
			if (dojo.isString(this.link)) this.link = dojo.byId(this.link);
			if(this.link && !this.buttonConnection) {
				this.buttonConnection = dojo.connect(this.link, "onclick", this, this.toggleDisplay);
				dojo.removeClass(this.link,"lotusBtnDisabled");
			}
		},
		
		disconnectButton: function(){
			// set the onclick handler for the toggle link
			if(this.displayState == "open") this.toggleDisplay();
			if(this.buttonConnection) {
				dojo.disconnect(this.buttonConnection);
				this.buttonConnection = null;
			}
			if(this.link) dojo.addClass(this.link, "lotusBtnDisabled");
		},
		
		toggleDisplay: function(){
			// summary: changes the state of the display
			if(this.displayState == "open") {
				this.displayState = "closed";
				this.onClose();
				if(dojo.isIE) dojo.style(this.domNode,"display","none");
				else this.wipeOut.play();
			} else if (this.displayState == "closed"){
				this.displayState = "open";
				this.onOpen();
				if(!this.everOpened) { this.startup(); }
				if(dojo.isIE) dojo.style(this.domNode,"display","");
				else this.wipeIn.play();
				if(!this.everOpened) { this.onFirstOpen(); this.everOpened = true; }
			}
		},
		
		onCreate: function(){
			// summary: stub event for when the shelf is instantiated
		},
		
		onOpen: function(){
			// summary: stub event for when the shelf is displayed
		},
		
		onFirstOpen: function(){
			// summary: stub event for when the shelf is opened for the first time
		},
		
		onClose: function(){
			// summary: stub event for when the shelf is hidden
		}
});

}

if(!dojo._hasResource["com.ibm.data.CatalogFeedStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.CatalogFeedStore"] = true;
//C:\Documents and Settings\Administrator\workspace\enabler\lwp\enabler\eclipse\plugins\com.ibm.mm.enabler\resources\js\com\ibm\mm\enabler\model\catalog.js
//for category entry API line 923
dojo.provide("com.ibm.data.CatalogFeedStore");

dojo.declare("com.ibm.data.CatalogFeedStore", 
		[com.ibm.data.CatalogMixin],
		{
			searchable: false,
			items: [],
			attributes: ["id","label","description","thumbnail","definition"],
			features: {'dojo.data.api.Read': true},
			label: "",
			category: "",
			pageSize: 30,
			//defaultCatalog: "default",
			valid: true,
		       
			constructor: function(args) {
				this.items = [];
				this.category = args.category;
				var catalogModel = com.ibm.mashups.enabler.model.Factory.getCatalogCategoryModel();
				catalogModel.setStrategy(new com.ibm.mashups.enabler.strategy.ListLoadAheadStrategy(this.pageSize));
				/*var catalogStr = args.catalog || this.defaultCatalog;
				var categoryStr = ibmPortalConfig ? this.category : this.category + "_c_" + catalogStr;
				var category = catalogModel.find(categoryStr).start();*/
				var category = catalogModel.find(this.category).start();
				if(!category){
					this.valid = false;
					return;
				}
				var locale = com.ibm.mashups.enabler.context.Factory.getLocalizedContext().getLocale() || "en"; // RETURNS NULL
				this.label = category.getTitle(locale);
				var entryIter = catalogModel.getCatalogEntryModel(category).iterator();
				while (entryIter.hasNext()){
					var entry = entryIter.next();
					this.items.push({
						"id":entry.getID(),
						"label":entry.getTitle(locale),
						"description":entry.getDescription(locale),
						"thumbnail":entry.getIconURL(),
						"url":entry.getDefinitionURL()
					});
				}
			},
			
			getValue: function(/* item */ item, /* attribute-name-string */ attribute, /* value? */ defaultValue){
				return item[attribute] || defaultValue;
			},
			
			getValues: function(/* item */ item, /* attribute-name-string */ attribute){
				var value = this.getValue(item,attribute,null);
				return value ? [value] : [];
			},
			
			getAttributes: function(/* item */ item){
				return this.isItem(item) ? this.attributes : [];
			},
			
			hasAttribute: function(/* item */ item, /* attribute-name-string */ attribute){
				return item[attribute] ? true : false;
			},
			
			containsValue: function(/* item */ item, /* attribute-name-string */ attribute, /* anything */ value){
				var values = this.getValues(item,attribute);
				for(var i in values){
					if(values[i] == value) return true
				}
				return false;
			},
			
			isItem: function(/* anything */ something){
				for(var i in this.items){
					if(this.items[i].id == something.id) return true;
				}
				return false;
			},
			
			isItemLoaded: function(/* anything */ something){
				return this.isItem(something);
			},
			
			loadItem: function(/* object */ keywordArgs){
				// do nothing because all items are already loaded
			},
			
			fetch: function(/* Object */ keywordArgs){
				try {
					var scope = keywordArgs.scope || dojo.global,
						items = [],
						count = keywordArgs.count,
						start = keywordArgs.start;
					if(!start) start = 0;
					if(keywordArgs.onBegin) keywordArgs.onBegin.call(scope,this.items.length,keywordArgs);
					if(start < this.items.length){
						if(count && count + start <= this.items.length){
							items = this.items.slice(start,count);
						} else {
							items = this.items.slice(start);
						}
					}
					if(keywordArgs.onItem){
						dojo.forEach(items,function(item){keywordArgs.onItem.call(scope,item,keywordArgs)});
					}
					var itemsLeft = keywordArgs.onItem ? null : items;
					if(keywordArgs.onComplete) keywordArgs.onComplete.call(scope,itemsLeft,keywordArgs);
					return dojo.mixin(keywordArgs,{abort:function(){}});
				} catch (e) {
					if(keywordArgs.onError) keywordArgs.onError.call(scope,e,keywordArgs);
				}
			},
			
			getFeatures: function(){
				return this.features;
			},
			
			close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
				// nothing to do here
			},
			
			getLabel: function(/* item */ item){
				if(this.isItem(item)) return item["label"];
				else return null;
			},
			
			getLabelAttributes: function(/* item */ item){
				return ["label"];
			},
			
			getSourceLabel: function(){
				return this.label;
			},
			
			validateSelf: function(){
				// summary: validates this datastore, if it does not validate the source will not be used
				// returns true for validation and false otherwise
				return this.valid;
			}
});

}

if(!dojo._hasResource["com.ibm.customize.ResultsList"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.customize.ResultsList"] = true;
// This controls the list of results in the customization shelf.  It handles searching datastores and paging.  It usually works with a com.ibm.customize.Container as its parent.

dojo.provide("com.ibm.customize.ResultsList");





dojo.declare("com.ibm.customize.ResultsList",
	[dijit._Widget, dijit._Templated],
	{
		source: null,						// the source which is currently selected for searching
		parent: null, 						// the parent container for this widget
		searchTerms: [], 					// an array of search terms for fetch 
		currRequest: null,					// the current fetch request
		size: 9,							// the number of results displayed at once
		start: 0,							// the index of the first result displayed in the section
		totalItemCount: 0,					// the total number of items matched by the last search query
		draggable: false,					// true if items are draggable off the shelf
		dndType: "ibmPortalShelfItem",		// dnd type of items that are draggable
		cssClass: "",
		events: [],
		secondCssClass: "lotusPaletteWidgetContainer",
		listHtmlType: "div",
		templateString: '<div dojoAttachPoint="focusNode" class="${cssClass}">' +
							'<div class="${secondCssClass}">' +
								'<${listHtmlType} role="grid" dojoAttachPoint="table"></${listHtmlType}>' +
							'</div>' +
							'<div class="lotusPaging">' +
								'<div class="lotusLeft" dojoAttachPoint="pagingStatus"></div>' +
								'<div dojoAttachPoint="jumpToLink" class="ibmJumpToLink"></div>' +
								'<ul class="lotusRight lotusInlinelist">' +
									'<li class="lotusFirst" dojoAttachPoint="prevLink"></li>' +
									'<li dojoAttachPoint="nextLink"></li>' +
								'</ul>' +
							'</div>' +
						'</div>',
		
		constructor: function() {
			this.shelfBundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
			this.newCell = function(){};
		},
		
		postMixInProperties: function(){
			// summary: if this object was created without a parent, provide self with items usually provided by the parent object
			if(!this.parent){
				this.parent = {
					id:this.id,
					createLotusMsgNode: function(message,type){
						var err = dojo.create("div",{className:"lotusMessage lotus" + type, style:{ marginRight:"15px" }});
						var img = dojo.create("img",{src:this.imagePath + "/icon" + type + ".gif", alt:message});
						var msg = dojo.create("span",{innerHTML:message});
						err.appendChild(img);
						err.appendChild(msg);
						return err;
					},
					newCell: this.newCell
				}
			}
		},
		
		postCreate: function(){ 
			// summary: subscribes to events for when the search terms changes, the source changes, the user wants to add a saved search, or the size of the browser window adjusts
			this.events.push(dojo.subscribe("/epc/customize/search"+this.parent.id, this, this.changeSearchTerms));
			this.events.push(dojo.subscribe("/epc/customize/changeSource"+this.parent.id, this, this.changeSource));
			if(this.source) this.changeSource(this.source);
			if(this.draggable) this.setUpDndSource();
		},
		
		/**********************************/
		/*               DND              */
		/**********************************/
			
		setUpDndSource: function(){
			var cls = dojo.getObject(ibmPortalConfig.portletDndSource || "com.ibm.pb.dnd.layout.ShelfSource");  // LAURA
			this.dndSource = new cls(
				this.table,{
				creator: ibmPortalConfig.portletDndSource ? this.creator : cls.creator, 
				accept: [], 
				singular: true,
				skipForm: true,
				copyOnly: true,
				selfAccept: false,
				selfCopy: false,
				copyState: function(){	return true; }
			});
			this.events.push(dojo.subscribe("/dnd/drop", this, "onDrop"));
		},
		
		makeCellDraggable: function(cell){
			dojo.addClass(cell, "dojoDndItem dojoDndHandle");
			var control = dojo.query(".lotusInner", cell)[0];
			dojo.addClass(control,"ibmPortalControl");
			dojo.attr(cell, "dndType", this.dndType);
		},
		
		creator: function(item, hint){
			if(hint == "avatar"){
				var node = dojo.dnd.manager().nodes[0];
				var widget = dijit.byId(node.id);
				var title = (widget) ? widget.truncTitleText : "";
				var node = dojo.create("div",{className:"ibmPortalShelfAvatar",innerHTML:title});
				return {node: node, data: item, type: [this.dndType]};
			}
		},
		
		onDrop: function(source, nodes, copy){
			if(typeof com.ibm.portal.dnd != "undefined" && typeof com.ibm.portal.dnd.PORTLET_MEDIATOR != "undefined"){
				if(com.ibm.portal.dnd.PORTLET_MEDIATOR.currentType == this.dndType){
					com.ibm.portal.dnd.PORTLET_MEDIATOR.fromShelf = true;
					var widget = dijit.byId(nodes[0].id);
					widget.apply();
				}
			}
		},
		
		/**********************************/
		/*            SEARCH              */
		/**********************************/
		
		changeSearchTerms: function(/*String[]*/ searchTerms){
			// summary: does a new search when the filters change
			// searchTerms: an array of strings which are filters
			this.searchTerms = searchTerms;
			this.start = 0;
			this._search();
		},
		
		changeSource: function(/*Object*/ source){
			// summary: does a fresh search with no filters when the hub changes
			// source: the new source to search
			if(!source.hub.fetch) {
				this.clear();
				return;
			}
			if(this.currRequest) this.currRequest.abort();  // make sure any unfinished request is aborted
			this.source = source;
			this.searchTerms = [];
			this.start = 0;
			if(source.hub.noIcons) dojo.addClass(this.domNode.parentNode,"lotusPlain");
			else dojo.removeClass(this.domNode.parentNode,"lotusPlain");
			this._search();
		},
		
		_search: function(){
			// summary: does a fetch operation on the hub and fills the table with the results
			if(this.source.hub == null){
				// alert the user if the hub is null and clear out old info on the paging bar
				this.addMessage("Error",dojo.string.substitute(this.shelfBundle["shelf_badDatastoreError"], [this.source.label]));
				return;
			} else this.setLoading();
			
			// See dojo.data.api.Read.fetch()
			this.currRequest = this.source.hub.fetch({
				queryOptions: { ignoreCase: true },
				query: { keywords: this.searchTerms }, // query is the array of filters that the hub handles in its own way
				count: this.size,
				start: this.start,  
				
				onComplete: dojo.hitch(this, function(/*DatastoreItem[]*/items) {
					// summary: uses the items returned from the search to adjust what is displayed on the paging bars, and fill the table with results
					if(this.parent.filterAdd){
						// filter out any items that should not be displayed
						filterFcn = dojo.hitch(this,function(item){ return this.parent.filterAdd(this.source.hub.mapItem(item)); })
						items = dojo.filter(items,filterFcn);
					}
					if(items.length == 0) this.addMessage("Warning",this.shelfBundle["shelf_noResults"]);
					else this.fill(items);
				}),
				
				onError: dojo.hitch(this, function(/*String*/error, /*Object*/request) {
					console.error("error loading this category: " +error);
					// summary: displays an error message in the table and clears out old info on the paging bar
					if  (error.status == 401) {
						ibm.portal.debug.text( "Basic auth 401 found, trigger reload");
						var authObj = dojo.getObject("com.ibm.ajax.auth");
						if(authObj) com.ibm.ajax.auth.authenticationHandler();
						else window.location.reload();
						return;
					}
					this.clear();
					this.addMessage("Error", dojo.string.substitute(this.shelfBundle["shelf_feedError"], [this.source.label]));
				}),
				
				onBegin: dojo.hitch(this, function(totalNumber) {
					// summary: records the number of results returns in the current query
					this.totalItemCount = totalNumber;
				})
			});
		},
		
		/***********************************/
		/*              TABLE              */
		/***********************************/
		
		setLoading: function() {
			// summary: adds a loading indicator, for use before search results are appended
			this.clear();
			this.table.appendChild(document.createTextNode(this.shelfBundle["shelf_loading"]));
		},
		
		addMessage: function(/*String*/type,/*String*/msg){
			// summary: clears out the table and then builds it back up enough to enter a single message into the table
			// msg: the message to enter in the table
			this.clear();
			// create the cell
			var message = this.parent.createLotusMsgNode(msg,type);
			this.table.appendChild(message);
			// reset search numbers to 0 and adjust the info on the paging bar
			this.start = 0;
			this.totalItemCount = 0;
		},
		
		clear: function(){
			// summary: clears out the table
			var items = dojo.query(".lotusPaletteWidget",this.table);
			dojo.forEach(items, dojo.hitch(this, function(item){
				dijit.byId(item.id).destroy();
			}));
			this.table.innerHTML = "";
			this.pagingStatus.innerHTML = "";
			this.jumpToLink.innerHTML = "";
			this.prevLink.innerHTML = "";
			this.nextLink.innerHTML = "";
		},
		
		fill: function(/*DatastoreItem[]*/contents){
			// summary: fills the table with rows of catalog item results
			// contents: array holding DatastoreItems to be entered in the table
			this.clear();
			dojo.forEach(contents, dojo.hitch(this, function(item) {
				var newCell = this.parent.newCell(this.source,item);
				if(this.draggable) this.makeCellDraggable(newCell);
				this.table.appendChild(newCell);
			}));	
			this.adjustPagingBar();
			if(this.draggable) this.dndSource.sync();
		},
		
		/************************************/
		/*              PAGING              */
		/************************************/
		
		prev: function(){
			// summary: triggers when the user clicks the Previous link,
			this.start = this.start - this.size;
			this._search();
		},
		
		next: function(){
			// summary: triggers when the user clicks the Next link
			this.start = this.start + this.size;
			this._search();
		},
		
		jumpTo: function(/*Event*/e){
			// summary: triggers when a user jumps to a new page
			// event: the onkeydown event
			e = dojo.fixEvent(e, e.target);
			var key = e.keyCode;
		
			if(key == dojo.keys.ENTER){
				var input = this.jumpToInput.value;
				// if number is not an integer, return
				if(input.toString().search(/^[0-9]+$/) != 0)  return;
				// if number is less than 1, set it to 1
				if(input < 1) input = 1;
				
				// calculate the new starting index for result items and make sure it's within range
				var userInputStart = (input - 1) * this.size;
				var maxStart = (Math.ceil(this.totalItemCount/this.size) - 1) * this.size;
				this.start = (userInputStart < maxStart)?userInputStart:maxStart;
				// search with the new starting index
				this._search();
			}
		},
		
		adjustPagingBar: function(){
			// summary: triggers when a search is performed to make sure the numbers and contents of the bar match up
			var firstItem = (this.totalItemCount == 0)?0:this.start + 1;
			var lastItem = this.start + this.size;
			lastItem = (this.totalItemCount > lastItem)?lastItem:this.totalItemCount;
			
			// displays "Page X of Y" in the paging bar
			var currentPage = Math.ceil(firstItem/this.size);
			var totalPageCount = Math.ceil(this.totalItemCount/this.size);
			this.pagingStatus.innerHTML = dojo.string.substitute(this.shelfBundle["shelf_page"], [currentPage, totalPageCount]);
			
			// create Previous and Next links , and Jump to page ___  if appropriate
			this.createPrevNode(firstItem);
			this.createNextNode(lastItem);
			if(this.totalItemCount > this.size) this.createJumpTo(firstItem);
		},
		
		createPrevNode: function(/*Int*/firstItem){
			// summary: determines if a Previous link should be rendered and creates it if appropriate
			if(firstItem > 1){  // create a link
				var prev = dojo.create("a", {href:"javascript:void(0)",innerHTML:this.shelfBundle["shelf_previous"]});
				prev.onclick = this._createPrevNodeEventHandlers().onclick;
				this.prevLink.appendChild(prev);
			}
			else  // create unclickable text
				this.prevLink.appendChild(document.createTextNode(this.shelfBundle["shelf_previous"]));
		},
		
		_createPrevNodeEventHandlers: function () {
			return {
				onclick: dojo.hitch(this, function(){ this.prev(); return false; /* to prevent onBeforeUnload event from triggering in IE */ })
			};
		},
		
		createNextNode: function(/*Int*/lastItem){
			// summary: determines if a Next link should be rendered and creates it if appropriate
			if(this.totalItemCount > lastItem){  // create a link
				var next = dojo.create("a", {href:"javascript:void(0)",innerHTML:this.shelfBundle["shelf_next"]});
				next.onclick = this._createNextNodeEventHandlers().onclick;
				this.nextLink.appendChild(next);
			}
			else // create unclickable text
				this.nextLink.appendChild(document.createTextNode(this.shelfBundle["shelf_next"]));
		},
		
		_createNextNodeEventHandlers: function () {
			return {
				onclick: dojo.hitch(this, function(){ this.next(); return false; /* to prevent onBeforeUnload event from triggering in IE */ })
			};
		},
		
		createJumpTo: function(firstItem){
			// summary: if there is  more than one page available, this creates a "Jump to page ___" input
			this.jumpToInput = dojo.create("input",{type:"text",value:Math.ceil(firstItem/this.size),className:"lotusText","id":this.parent.id+"_jumpTo"});
			dojo.connect(this.jumpToInput, "onkeydown", this, this.jumpTo);
			this.jumpToLink.appendChild(dojo.create("label",{"innerHTML":this.shelfBundle["shelf_jumpTo"],"for":this.parent.id+"_jumpTo"}));
			this.jumpToLink.appendChild(this.jumpToInput);
		},
		
		destroy: function(){
			dojo.forEach(this.events, dojo.unsubscribe);
			this.events = null;
			this.inherited(arguments);
		}
});

}

if(!dojo._hasResource["com.ibm.customize.SearchBar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.customize.SearchBar"] = true;
// This is an input box which publishes all the words of the input's value as an array to a given event

dojo.provide("com.ibm.customize.SearchBar");





dojo.declare("com.ibm.customize.SearchBar",
	[dijit._Widget, dijit._Templated],
	{
		imagePath: ".",
		searchId: "",
		searchFieldText: "",
		searching: true,
		events: [],
		publishKeyword: "/epc/customize/search",	// the keyword to publish when a string is submitted
		templateString: '<div dojoAttachPoint="focusNode" class="lotusChunk lotusSearch">' +
							'<form role="search" action="" onsubmit="return false">' +
								'<label for="${searchBoxId}" class="ibmHiddenText" dojoAttachPoint="searchFieldLabel">${searchFieldText}</label>' +
								'<input id="${searchBoxId}" dojoAttachEvent="onkeydown:newSearch" class="lotusText" type="text" value="${searchFieldText}" onfocus="this.value=\'\';" dojoAttachPoint="searchField"/>' +
								'<span class="lotusBtnImg" title="${submitSearchText}">' +
									'<input dojoAttachEvent="onclick:searchButton" class="lotusSearchButton" type="image" alt="${submitSearchText}" src="${imagePath}/blank.gif"/>' +
									'<span class="lotusAltText">Search</span>' +
								'</span>' +
							'</form>' +
						'</div>',
		
		constructor: function() {
			this.bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
			this.submitSearchText = this.bundle["shelf_submitSearch"];
		},
		
		postMixInProperties: function(){
			this.searchBoxId = this.searchId + "_search";
		},
		
		postCreate: function(){
			// summary: subscribe to the event that changes the default text in the search box
			this.events.push(dojo.subscribe("/epc/customize/changeSource"+this.searchId, this, this.changeInputValue));
		},
		
		enableSearch: function(){
			if(!this.searching){
				this.searching = true;
				dojo.style(this.searchField, "backgroundColor", "#FFFFFF");
				dojo.style(this.searchField, "borderColor", "#FFFFFF");
				this.searchField.removeAttribute("disabled");
			}
		},
		
		disableSearch: function(){
			if(this.searching){
				this.searching = false;
				dojo.style(this.searchField, "backgroundColor", "#CCCCCC");
				dojo.style(this.searchField, "borderColor", "#CCCCCC");
				dojo.attr(this.searchField, "disabled", true);
			}
		},
		
		changeInputValue: function(/*Object*/source){
			// summary: changes the text in the field
			if(source.hub.searchable) this.enableSearch();
			else this.disableSearch();
			if(source.searchStr) this.searchFieldText = source.hub.searchable ? source.searchStr : "";
			else this.searchFieldText = source.hub.searchable ? dojo.string.substitute(this.bundle["shelf_search"], [source.label]) : "";
			this.searchField.value = this.searchFieldLabel.innerHTML = this.searchFieldText;
		},
		
		newSearch: function(/*Event*/e){
			// summary: triggers when a user is typing 
			// Publishes an event when the user presses the enter key
			e = dojo.fixEvent(e, e.target);
			var key = e.keyCode;
		
			if(dojo.isIE && key == dojo.keys.ENTER)
				this.searchButton();
		},
		
		searchButton: function(){
			// summary: creates an array of the space separated search terms and publishes them to an event
			if(this.searching){
				var searchWords = (this.searchField.value == this.searchFieldText) ? "" : this.searchField.value;  // no search criteria if field = "Search xxxx"
				var searchArray = searchWords.split(" ");
				// publish the event to a specific id so not all instances of the receiving class respond
				dojo.publish(this.publishKeyword+this.searchId, [searchArray]);
			}
		},
		
		destroy: function(){
			dojo.forEach(this.events, dojo.unsubscribe);
			this.events = null;
			this.inherited(arguments);
		}
});

}

if(!dojo._hasResource["dojo.fx.easing"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx.easing"] = true;
dojo.provide("dojo.fx.easing");

dojo.fx.easing = {
	// summary: 
	//		Collection of easing functions to use beyond the default 
	//		`dojo._defaultEasing` function.
	// 
	// description:
	//
	//		Easing functions are used to manipulate the iteration through
	//		an `dojo.Animation`s _Line. _Line being the properties of an Animation,
	//		and the easing function progresses through that Line determing
	//		how quickly (or slowly) it should go. Or more accurately: modify
	//		the value of the _Line based on the percentage of animation completed.
	//	
	//		All functions follow a simple naming convention of "ease type" + "when".
	//		If the name of the function ends in Out, the easing described appears
	//		towards the end of the animation. "In" means during the beginning,
	//		and InOut means both ranges of the Animation will applied, both
	//		beginning and end. 
	//
	//		One does not call the easing function directly, it must be passed to 
	//		the `easing` property of an animation.
	//
	//	example:
	//	|	
	//	|	var anim = dojo.fadeOut({
	//	|		node: 'node',	
	//	|		duration: 2000,
	//	|		//	note there is no () 
	//	|		easing: dojo.fx.easing.quadIn
	//	|	}).play();
	//
	
	linear: function(/* Decimal? */n){
		// summary: A linear easing function
		return n;
	},

	quadIn: function(/* Decimal? */n){
		return Math.pow(n, 2);
	},

	quadOut: function(/* Decimal? */n){
		return n * (n - 2) * -1;
	},

	quadInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 2) / 2; }
		return -1 * ((--n) * (n - 2) - 1) / 2;
	},

	cubicIn: function(/* Decimal? */n){
		return Math.pow(n, 3);
	},

	cubicOut: function(/* Decimal? */n){
		return Math.pow(n - 1, 3) + 1;
	},

	cubicInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 3) / 2; }
		n -= 2;
		return (Math.pow(n, 3) + 2) / 2;
	},

	quartIn: function(/* Decimal? */n){
		return Math.pow(n, 4);
	},

	quartOut: function(/* Decimal? */n){
		return -1 * (Math.pow(n - 1, 4) - 1);
	},

	quartInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 4) / 2; }
		n -= 2;
		return -1 / 2 * (Math.pow(n, 4) - 2);
	},

	quintIn: function(/* Decimal? */n){
		return Math.pow(n, 5);
	},

	quintOut: function(/* Decimal? */n){
		return Math.pow(n - 1, 5) + 1;
	},

	quintInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 5) / 2; };
		n -= 2;
		return (Math.pow(n, 5) + 2) / 2;
	},

	sineIn: function(/* Decimal? */n){
		return -1 * Math.cos(n * (Math.PI / 2)) + 1;
	},

	sineOut: function(/* Decimal? */n){
		return Math.sin(n * (Math.PI / 2));
	},

	sineInOut: function(/* Decimal? */n){
		return -1 * (Math.cos(Math.PI * n) - 1) / 2;
	},

	expoIn: function(/* Decimal? */n){
		return (n == 0) ? 0 : Math.pow(2, 10 * (n - 1));
	},

	expoOut: function(/* Decimal? */n){
		return (n == 1) ? 1 : (-1 * Math.pow(2, -10 * n) + 1);
	},

	expoInOut: function(/* Decimal? */n){
		if(n == 0){ return 0; }
		if(n == 1){ return 1; }
		n = n * 2;
		if(n < 1){ return Math.pow(2, 10 * (n - 1)) / 2; }
		--n;
		return (-1 * Math.pow(2, -10 * n) + 2) / 2;
	},

	circIn: function(/* Decimal? */n){
		return -1 * (Math.sqrt(1 - Math.pow(n, 2)) - 1);
	},

	circOut: function(/* Decimal? */n){
		n = n - 1;
		return Math.sqrt(1 - Math.pow(n, 2));
	},

	circInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return -1 / 2 * (Math.sqrt(1 - Math.pow(n, 2)) - 1); }
		n -= 2;
		return 1 / 2 * (Math.sqrt(1 - Math.pow(n, 2)) + 1);
	},

	backIn: function(/* Decimal? */n){
		// summary: 
		//		An easing function that starts away from the target, 
		//		and quickly accelerates towards the end value.
		// 
		//		Use caution when the easing will cause values to become 
		//		negative as some properties cannot be set to negative values.
		var s = 1.70158;
		return Math.pow(n, 2) * ((s + 1) * n - s);
	},

	backOut: function(/* Decimal? */n){
		// summary: 
		//		An easing function that pops past the range briefly, and slowly comes back. 
		//
		// description:
		//		An easing function that pops past the range briefly, and slowly comes back. 
		//
		//		Use caution when the easing will cause values to become negative as some 
		//		properties cannot be set to negative values.
		
		n = n - 1;
		var s = 1.70158;
		return Math.pow(n, 2) * ((s + 1) * n + s) + 1;
	},

	backInOut: function(/* Decimal? */n){
		// summary: 
		//		An easing function combining the effects of `backIn` and `backOut`
		//
		// description:
		//		An easing function combining the effects of `backIn` and `backOut`.
		//		Use caution when the easing will cause values to become negative 
		//		as some properties cannot be set to negative values.
		var s = 1.70158 * 1.525;
		n = n * 2;
		if(n < 1){ return (Math.pow(n, 2) * ((s + 1) * n - s)) / 2; }
		n-=2;
		return (Math.pow(n, 2) * ((s + 1) * n + s) + 2) / 2;
	},

	elasticIn: function(/* Decimal? */n){
		// summary: 
		//		An easing function the elastically snaps from the start value
		//
		// description:
		//		An easing function the elastically snaps from the start value
		//	
		//		Use caution when the elasticity will cause values to become negative 
		//		as some properties cannot be set to negative values.
		if(n == 0 || n == 1){ return n; }
		var p = .3;
		var s = p / 4;
		n = n - 1;
		return -1 * Math.pow(2, 10 * n) * Math.sin((n - s) * (2 * Math.PI) / p);
	},

	elasticOut: function(/* Decimal? */n){
		// summary: 
		//		An easing function that elasticly snaps around the target value,
		//		near the end of the Animation
		//
		// description:
		//		An easing function that elasticly snaps around the target value,
		//		near the end of the Animation
		//
		//		Use caution when the elasticity will cause values to become 
		//		negative as some properties cannot be set to negative values.
		if(n==0 || n == 1){ return n; }
		var p = .3;
		var s = p / 4;
		return Math.pow(2, -10 * n) * Math.sin((n - s) * (2 * Math.PI) / p) + 1;
	},

	elasticInOut: function(/* Decimal? */n){
		// summary: 
		//		An easing function that elasticly snaps around the value, near
		//		the beginning and end of the Animation.
		//
		// description:
		//		An easing function that elasticly snaps around the value, near
		//		the beginning and end of the Animation.
		//
		//		Use caution when the elasticity will cause values to become 
		//		negative as some properties cannot be set to negative values.
		if(n == 0) return 0;
		n = n * 2;
		if(n == 2) return 1;
		var p = .3 * 1.5;
		var s = p / 4;
		if(n < 1){
			n -= 1;
			return -.5 * (Math.pow(2, 10 * n) * Math.sin((n - s) * (2 * Math.PI) / p));
		}
		n -= 1;
		return .5 * (Math.pow(2, -10 * n) * Math.sin((n - s) * (2 * Math.PI) / p)) + 1;
	},

	bounceIn: function(/* Decimal? */n){
		// summary: 
		//		An easing function that 'bounces' near the beginning of an Animation
		return (1 - dojo.fx.easing.bounceOut(1 - n)); // Decimal
	},

	bounceOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that 'bounces' near the end of an Animation
		var s = 7.5625;
		var p = 2.75;
		var l; 
		if(n < (1 / p)){
			l = s * Math.pow(n, 2);
		}else if(n < (2 / p)){
			n -= (1.5 / p);
			l = s * Math.pow(n, 2) + .75;
		}else if(n < (2.5 / p)){
			n -= (2.25 / p);
			l = s * Math.pow(n, 2) + .9375;
		}else{
			n -= (2.625 / p);
			l = s * Math.pow(n, 2) + .984375;
		}
		return l;
	},

	bounceInOut: function(/* Decimal? */n){
		// summary: 
		//		An easing function that 'bounces' at the beginning and end of the Animation
		if(n < 0.5){ return dojo.fx.easing.bounceIn(n * 2) / 2; }
		return (dojo.fx.easing.bounceOut(n * 2 - 1) / 2) + 0.5; // Decimal
	}
};

}

if(!dojo._hasResource["dojox.fx.easing"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.fx.easing"] = true;
dojo.provide("dojox.fx.easing");
dojo.deprecated("dojox.fx.easing","Upgraded to Core, use dojo.fx.easing instead","2.0");

/*=====
	dojox.fx.easing = {
		// summary:
		//		An Alias to `dojo.fx.easing`. Moved to Core in Dojo 1.2.
	};
=====*/
dojox.fx.easing = dojo.fx.easing;

}

if(!dojo._hasResource["com.ibm.customize.Menu"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.customize.Menu"] = true;
// This is a menu.  It takes a list of sources that each have label attributes to use as display titles.  It publishes an event on a source each time one is clicked.
dojo.provide("com.ibm.customize.Menu");








dojo.declare("com.ibm.customize.Menu",
	[dijit._Widget, dijit._Templated],
	{
		sources: [],									// an array of objects containing labels and datastore hubs for each menu choice
		choices: {},									// a map of label : DOMNode pairs for each choice in the menu
		searchId: "",									// the id of the Container to search within
		menuTitle: "Menu",								// title for the menu
		current: null,									// the menu choice currently selected
		blankImg: "",									// path to a clear pixel
		selectedText: "currently selected category",
		unselectedText: "press enter to browse this category",
		publishKeyword: "/epc/customize/changeSource",	// the keyword to publish to add a filter
		templateString: '<div dojoAttachPoint="focusNode">' +
							'<h3>' +
								'<a dojoAttachPoint="back" title="back" class="lotusBtnImg lotusHidden" href="javascript:void(0)">' +
									'<img alt="${backText}" src="${blankImg}">' +
									'<span class="lotusAltText">${backText}</span>' +
								'</a>' +
								' ${menuTitle}' +
							'</h3>' +
							'<div role="menu" dojoAttachPoint="menuContainer" class="lotusMenu">' +
								'<ul dojoAttachPoint="list"></ul>' +
							'</div>' + 
						'</div>',
		
		constructor: function() {
			this.bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
			this.menuTitle = this.bundle["shelf_menu"];
			this.selectedText = this.bundle["menu_selected_text"] || this.selectedText;
			this.unselectedText = this.bundle["menu_unselected_text"] || this.unselectedText;
			this.backText = this.bundle["menu_back"] || "Back";
		},
	
		postCreate: function(){
			// summary: populates the menu with the given choices
			this.back.onclick = dojo.hitch(this,function(){this.backOneLevel();return false;});
			this.processSources(this.sources);
		},
		
		processSources: function(sources){
			dojo.empty(this.list);
			if(this.sources.length > 0) {
				// add the choices to the menu with the first choice selected
				dojo.forEach(sources, dojo.hitch(this, function(choice){ this.addLi(choice); }));
				if(!sources[0].categories) this.select(sources[0]);  // select if it's a datastore
				else dojo.publish(this.publishKeyword+this.searchId, [{hub:{searchable:false},label:sources[0].label}]);  // parent category
			}
		},
		
		addLi: function(/*Object*/choice){
			// summary: adds a new element to the list of choices in the menu
			// choice: a member of the choices array to add to the menu
			var li = dojo.create("li");
			if(choice.categories) dojo.addClass(li,"lotusMore");
			var evtHandlers = this._createLiEventHandlers(choice);
			var newLink = dojo.create("a",{href:"javascript:void(0)",innerHTML:choice.label});
			newLink.onclick = evtHandlers.anchorOnclick;
			li.appendChild(newLink);
			var selectedSpan = dojo.create("span",{innerHTML:this.unselectedText,className:"",className:"ibmHiddenText"});
			newLink.appendChild(selectedSpan);
			// when the menu item is clicked, call the select function on it
			dojo.connect(li, 'onclick', evtHandlers.liOnclick);
			// add the new item to DOM and the map of choices
			this.list.appendChild(li);
			this.choices[choice.url] = {li:li,span:selectedSpan};
		},
		
		_createLiEventHandlers: function (choice) {
			return {
				anchorOnclick: function(){ return false; /* to prevent onBeforeUnload event from triggering in IE */ },
				liOnclick: dojo.hitch( this, function(){ this.select(choice); })
			};
		},
		
		select: function(/*Object*/source){
			// summary: triggered by a menu click,
			// source: the menu choice that was clicked
			if(source.categories){
				this.processSources(source.categories);  // drill down
				dojo.removeClass(this.back,"lotusHidden");  // show back button
			} else {
				// publish the event to a specific id so not all instances of the receiving class respond
				dojo.publish(this.publishKeyword+this.searchId, [source]);  // search
				this.changeHighlighting(source);
			}
		},
		
		backOneLevel: function(){
			// summary: drills back up one level
			this.processSources(this.sources);
			dojo.addClass(this.back,"lotusHidden");
		},
		
		changeHighlighting: function(/*String*/source){
			// summary: switches which menu choice is selected
			// source: the menu choice that was chosen
			if(source.categories) return;  // parent categories are never highlighted
			if(this.current) {
				dojo.removeClass(this.choices[this.current].li, "lotusSelected");
				this.choices[this.current].span.innerHTML = this.unselectedText;
			}
			dojo.addClass(this.choices[source.url].li, "lotusSelected");
			this.choices[source.url].span.innerHTML = this.selectedText;
			this.current = source.url;
		}
});

}

if(!dojo._hasResource["com.ibm.pb.customize.LotusPaletteContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.customize.LotusPaletteContainer"] = true;
// This is a container which follows the one ui lotus palette spec
// must be provided with a processChange function
dojo.provide("com.ibm.pb.customize.LotusPaletteContainer");












dojo.declare("com.ibm.pb.customize.LotusPaletteContainer",
	[dijit._Widget, dijit._Templated],
	{
		jsonFile: null,				// a JSON configuration file
		dialog: null,				// a dijit dialog
		maxLabelLength: 16,			// the maximum length allowed for a label
		smallWidget: false,			// false if it follows the lotusLargeWidget pattern
		numRows: 4,					// the number of rows of results to be displayed
		label: "",					// a label for this palette section
		selectedClass: "lotusSelected",
		imagePath: ".",
		draggable: false,			// true if items can be dragged off the shelf, will only work with Portal dnd
		size: null,
		dndType: "ibmPortalShelfItem",		// dnd type of items that are draggable
		templateString:'<div class="lotusui">' +
							'<div class="lotusPaletteBody">' +
								'<div dojoAttachPoint="left" class="lotusPaletteNav"></div>' +
								'<div dojoAttachPoint="center" class="lotusPaletteContent lotusui"></div>' +
							'</div>' +
						'</div>',
						
		constructor: function() {
			this.nlsBundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
		},
		
		postMixInProperties: function() {
			if(this.smallWidget) {
				this.dialog = new com.ibm.pb.customize.LotusDialog({ blankImg:this.imagePath + "/blank.gif" });
				this.dialog.closeButtonNode.onclick = this.dialog.closeLinkNode.onclick = dojo.hitch(this,function(){this.dialog.cancel();return false;});
			}
		},
		
		processChange: function(/*Object*/args) {
		},
		
		getSize: function(){
			// summary: determine the width of a palette widget element on the page with all the current settings by adding one to the page, taking its width, then deleting it
			// use the palette widget to determine how many of them can fit in the ResultsList
			if(this.size) { return this.size; }
			var paletteWidget = dojo.create("div",{className:"lotusPaletteWidget"});
			document.body.appendChild(paletteWidget);
			if (paletteWidget.currentStyle){
				marginL = paletteWidget.currentStyle["marginLeft"].match(/^\d+/);
				marginR = paletteWidget.currentStyle["marginRight"].match(/^\d+/);
			} else if (window.getComputedStyle) {
				marginL = document.defaultView.getComputedStyle(paletteWidget,null).getPropertyValue("margin-left").match(/^\d+/);
				marginR = document.defaultView.getComputedStyle(paletteWidget,null).getPropertyValue("margin-right").match(/^\d+/);
			} 
			var width = paletteWidget.offsetWidth + marginL*1 + marginR*1;
			this.left.innerHTML = ".";
			var containerWidth = this.center.offsetWidth;
			this.left.innerHTML = "";
			var resultsInOneRow = Math.floor(containerWidth / width);
			dojo.destroy(paletteWidget);
			this.size = resultsInOneRow * this.numRows;
			return this.size;
		},
		
		destroyChildWidgets: function(){
			this.resultsList.destroy();
			this.resultsList = null;
			this.searchBar.destroy();
			this.searchBar = null;
			this.menu.destroy();
			this.menu = null;
			this.started = false;
		},
		
		setupUI: function(){
			// summary: starts the UI setup
			this.processJSON(dojo.hitch(this,this.setupUICallback));
		},
		
		setupUICallback: function(sources){
			// summary: populates the SearchBar, Menu and ResultsList widget with configuration data from the JSON file
			this.resultsList = new com.ibm.customize.ResultsList({parent:this,size:this.getSize(),draggable:this.draggable,dndType:this.dndType});
			this.center.appendChild(this.resultsList.domNode);
				
			this.searchBar = new com.ibm.customize.SearchBar({searchId:this.id,imagePath:this.imagePath});
			this.left.appendChild(this.searchBar.domNode);
				
			this.menu = new com.ibm.customize.Menu({sources:sources,searchId:this.id,menuTitle:this.label,blankImg:this.imagePath + "/blank.gif"});
			this.left.appendChild(this.menu.domNode);
		},
		
		processJSON: function(/*function*/callback){
			// summary: processes the JSON file associated with this controller
			// callback: function to call after the JSON is processed that uses the sourceObjects
			dojo.xhrGet ({
		        url: this.jsonFile,
		        handleAs: 'json',
		        load: dojo.hitch(this, function(data){
					// get the bundle if one is provided
					if(data.localizationPackageName && data.localizationBundleName){
						this.nlsJsonBundle = dojo.i18n.getLocalization(data.localizationPackageName, data.localizationBundleName);
					} else this.nlsJsonBundle = null;
					var sources = data.categories;
					var sourceObjects = [];
					for(var i = 0, l = sources.length; i < l; i++) {
						if(sources[i].categories){
							var childSourceObjects = [];
							var childSources = sources[i].categories;
							for(var j = 0, l2 = childSources.length; j < l2; j++){
								var source = this.processCategory(childSources[j]);
								if(source) childSourceObjects.push(source);
							}
							if(childSourceObjects.length > 0) {
								var label = this.getNlsString(sources[i].label,this.nlsBundle["add_content_untitled"]);
								sourceObjects.push({label:label,categories:childSourceObjects});
							}
						} else {
							var source = this.processCategory(sources[i]);							
							if(source) sourceObjects.push(source);
						}
					}
					if(callback) callback(sourceObjects);
				}),	
		        error: dojo.hitch(this, function (/*String*/error) {
					console.error("error loading categories: " +error);
		            var errorMsg = this.createLotusMsgNode(this.nlsBundle["shelf_configError"],"Error");
					dojo.style(errorMsg,"margin","20px");
					this.domNode.innerHTML = "";
					this.domNode.appendChild(errorMsg);
		        })
		    });
		},
		
		processCategory: function(source){
			// pass the exact same sources item to the class's constructor minus the datastore and renderers properties
			var map = dojo.mixin({}, source);
			if(map.url && this.createProxyURI) map.url = this.createProxyURI(map.url);
			delete map.datastore; delete map.renderers; delete map.label;
			// create a datastore hub and a new source object
			var hub = new (dojo.getObject(source.datastore))(map);
			
			if(!hub.validateSelf()) return null;  // do not use this source if it does not validate
			if(hub.getSourceLabel()) var label = hub.getSourceLabel();  // get dynamic label
			else var label = this.getNlsString(source.label,this.nlsBundle["add_content_untitled"]);
			var searchStr = this.getNlsString(source.searchStr,null);
			return {label:label,hub:hub,renderers:source.renderers,url:source.url,searchStr:searchStr};
		},
		
		getNlsString: function(key, defaultStr){
			var value = this.nlsJsonBundle && this.nlsJsonBundle[key] ? this.nlsJsonBundle[key] : key;
			if(value && value.length == 0) value = defaultStr;
			return value;
		},
		
		newCell: function(/*Object*/source,/*DataItem*/item){
			// summary:  override this function to fill innerCell with information about the given datastore item
			// source: a datastore
			// item: an item in source
			var map = source.hub.mapItem(item);
			
			var label = (map.label.length > this.maxLabelLength)?map.label.substring(0,this.maxLabelLength) + '...':map.label;
			if(this.smallWidget) {
				var applyToPage = dojo.hitch(this, function(cID,sID) { this.processIntermediateDialogs(map,source,item,cID,sID); });
				showDetails = dojo.hitch(this, function() { this.showDetailsDialog(map,applyToPage); return false; /* to prevent onBeforeUnload event from triggering in IE */ });
				
				var context = {hub:source.hub, item:item, map:map, truncTitleText:label, imagePath:this.imagePath, apply:applyToPage, showDetails:showDetails};
				if(source.hub.noIcons){
					var widget = new com.ibm.pb.customize.noIconSmallWidget(context);
				} else if(source.hub.namespace == "feed") {
					var widget = new com.ibm.pb.customize.feedSmallWidget(context);
				} else {
					if(ibmCfg.portalConfig.portalContext) {
						var widget = new com.ibm.pb.customize.smallWidget(context);
					} else {
						var widget = new com.ibm.pb.customize.smallMenuWidget(context);
					}
				}
			}
			else {
				var applyToPage = dojo.hitch(this, function() { this.processChange(dojo.mixin(map,{hub:source.hub, item:item})) });
				var widget = new com.ibm.pb.customize.largeWidget({map:map, truncTitleText:label, imagePath:this.imagePath, apply:applyToPage, current:map.id == this.current});
			}
			return widget.domNode;
		},
		
		createLotusMsgNode: function(/*string*/message,/*string*/type){
			// summary: returns a div node containing the given message with OneUI styles
			// msg: the error message to display
			// type: either "Warning", "Error", "Info" or "Confirm"
			var err = dojo.create("div",{className:"lotusMessage lotus" + type, style:{ marginRight:"15px"}, role: "alert"});
			var img = dojo.create("img",{src:this.imagePath + "/icon" + type + ".gif", alt:message});
			var msg = dojo.create("span",{innerHTML:message});
			err.appendChild(img);
			err.appendChild(msg);
			return err;
		},
		
		changeSelected: function(/*string*/selected){
			// reset the look of the previously selected style/layout
			var items = dojo.query(".lotusPaletteWidget",this.center), selectedWidget = null;
			dojo.forEach(items, dojo.hitch(this, function(item){
				selectedWidget = dijit.byId(item.id);
				if(selectedWidget) selectedWidget.deselect();
			}));
			selectedWidget = dijit.byId(selected);
			if(selectedWidget) selectedWidget.select();
		},
		
		/**********************************/
		/*        DIALOG HELPERS          */
		/**********************************/
		
		processIntermediateDialogs: function(/*MapItem*/map,/*Object*/source,/*DataItem*/item,/*string*/cID,/*string*/sID){
			// source: a datastore
			// item: an item in source's hub
			if(source.renderers.length == 0){
				// if there are no renderers defined, then this is a self renderer and we assume the item's id is the id of the piece of content to be added
				var takeoverObject = map.takeoverFcn ? {submit:dojo.hitch(this,this.processChange),getInfo:dojo.hitch(this,this.getValuesFromDialog),showInfo:dojo.hitch(this,this.showWaitMsgInDialog),submitObj:{hub:source.hub,item:item,sID:sID,cID:cID}} : {};
				var args = dojo.mixin(map,{takeoverObject:takeoverObject,sID:sID,cID:cID,hub:source.hub,item:item});
				this.processChange(args);
			} else if(source.renderers.length == 1){ 
				// if there is only one renderer available, add it
                var dataLabel = map.label;
				var args = dojo.mixin(dojo.mixin(map,source.renderers[0]),{label:dataLabel,prefsMapFcn:source.renderers[0].fcn,sID:sID,cID:cID,hub:source.hub,item:item});
				this.processChange(args);
			} else {
				// an intermediate dialog box is opened for the user to choose the desired renderer
				var content = dojo.create("div");
				var header = dojo.create("h4");
				var headerLabel = dojo.create("label",{"for":"widgetRendererSelect",innerHTML:this.nlsBundle["add_content_selectRenderer"]});
				header.appendChild(headerLabel);
				content.appendChild(header);
				var rendererSelect = dojo.create("select",{id:"widgetRendererSelect"});
				content.appendChild(rendererSelect);
				for(var i = 0; i < source.renderers.length; i++){
					var select = dojo.create("option",{innerHTML:this.nlsJsonBundle && this.nlsJsonBundle[source.renderers[i].label] ? this.nlsJsonBundle[source.renderers[i].label] : source.renderers[i].label,value:i});
					rendererSelect.appendChild(select);
				}
				var onSubmit = dojo.hitch(this, function(){ 
					this.dialog.cancel();
					var renderer = source.renderers[rendererSelect.value];
                    var dataLabel = map.label; 
					var args = dojo.mixin(dojo.mixin(map,renderer),{label:dataLabel,prefsMapFcn:renderer.fcn,sID:sID,cID:cID,hub:source.hub,item:item});
					this.processChange(args);
				});
				this.dialog.setContent(content, this.nlsBundle["add_content_addToPage"], this.nlsBundle["add_content_add"], onSubmit);
			}
		},
		
		showDetailsDialog: function(/*Object*/itemMap,/*Function*/submitFcn){
			// summary: prepares the markup for the popup dialog to display details.  
			// itemMap: a map of information on the content from the mapItem function of the CatalogMixin class
			// submitFcn: the function to call when the submit button on the dialog is clicked
			var content = dojo.create("div");
			var title = dojo.create("h4",{innerHTML:(itemMap.label)?itemMap.label:this.nlsBundle["add_content_untitled"]});
			content.appendChild(title);
			if(itemMap.updated) {
				var updated = dojo.create("p",{innerHTML:dojo.string.substitute(this.nlsBundle["add_content_lastMod"], [itemMap.updated])});
				content.appendChild(updated);
			}
			if(itemMap.description){ 
				var desc = dojo.create("p",{innerHTML:itemMap.description});
				content.appendChild(desc);
			}
			if(itemMap.tags && itemMap.tags.length > 0){ 
				var tags = dojo.create("p",{innerHTML:dojo.string.substitute(this.nlsBundle["add_content_tags"], [itemMap.tags.join(", ")])});
				content.appendChild(tags);
			}
			var submit = dojo.hitch(this, function(){ this.dialog.cancel(); submitFcn(); });
			this.dialog.setContent(content, this.nlsBundle["add_content_details"], this.nlsBundle["add_content_addToPage"], submit);
		},
		
		showWaitMsgInDialog: function(/*String*/message){
			// summary: abstracts the use of the popup dialog by displaying a message
			// message: message to display in the dialog
			if(!this.waitingDialog)
			this.waitingDialog = new com.ibm.pb.customize.LotusWaitingDialog({spinner:this.imagePath + "/loading.gif", message:message});
			try {
				this.waitingDialog.show();
			} catch(e) {}
			return dojo.hitch(this,function(){ this.waitingDialog.hide(); });
		},
		
		getValuesFromDialog: function(/*object*/formObjs,/*String*/titleText,/*String*/submitText,/*String*/errorText,/*function*/callback){ 
			// summary: abstracts the use of the popup dialog by dislaying a form of text fields to be filled out
			// formObjs: an array of objects in the form:	label: a label for a text field 
			//												id: an id for the input field
			//												value: a value for the same text field 
			//												regex: a regular expression that the value of the field must validate against
			//												error: an error to display if the value does not validate
			// titleText: text to user as the title of the dialog
			// submitText: text to user on the submit button in the dialog
			// errorText: an optional error message to preface the entire form in the dialog, ie a "try again" error
			// callback: a function to be called when all the fields validate, it is called with an object of key value pairs
			//			 with the labels from the formObjs as keys and their corresponding values from the form as values
			var content = document.createElement("div");
			for(var i = 0; i < formObjs.length; i++){ // add an input for each item in formObjs
				var title = dojo.create("h4");
				var titleLabel = dojo.create("label",{innerHTML:formObjs[i].label, "for":formObjs[i].id});
				var error = dojo.create("div",{className:"lotusFormError"}); formObjs[i].errorDiv = error;
				var additionalClasses = "";
				if (formObjs[i].ltr)
					additionalClasses =" lotusLTR";
				var input = dojo.create("input",{type:"text", id:formObjs[i].id, className:"lotusText" + additionalClasses, value:formObjs[i].value}); formObjs[i].input = input;
				title.appendChild(titleLabel);
				content.appendChild(error);
				content.appendChild(title);
				content.appendChild(input);
			}
			if(errorText) formObjs[0].errorDiv.innerHTML = errorText;
			var validateInputs = dojo.hitch(this, function(){
				var fail = false;
				var values = {};
				for(var i = 0; i < formObjs.length; i++){
					if(!formObjs[i].input.value.match(formObjs[i].regex)){ // if the input does not match the regular expression, display an error
						fail = true;
						formObjs[i].errorDiv.innerHTML = formObjs[i].error;
					} else { // if the input does match the regex, add the input value to the return object
						values[formObjs[i].id] = formObjs[i].input.value;
						formObjs[i].errorDiv.innerHTML = "";
					}
				}
				if(!fail){ // if all the regular expressions matched, run the callback function on the values and close the dialog
					this.dialog.cancel();
					callback(values);				
				}
			});
			this.dialog.setContent(content, titleText, submitText, dojo.hitch(this, function(){ validateInputs(); }));
		}
});

dojo.provide("com.ibm.pb.customize.smallWidget");
dojo.declare("com.ibm.pb.customize.smallWidget",
	[dijit._Widget, dijit._Templated],
	{
		truncTitleText: "",
		imagePath: ".",
		map: {},
		detailDialogInstruction: "press enter to see details dialog",
		templateString: '<div class="lotusPaletteWidget" role="gridcell" dojoAttachPoint="lotusPaletteWidget">' +
							'<div class="lotusInner">' +
								'<div class="lotusIcon">' +
									'<span>' +
										'<img alt="${truncTitleText}" src="${map.thumbnail}"/>' +
									'</span>' +
								'</div>' +
								'<div class="lotusTitle">' +
									'<h4>' +
										'<a href="javascript:void(0);" title="${map.label}" dojoAttachPoint="titleAnchor">' +
											'${truncTitleText}' +
											'<span class="ibmHiddenText">${detailDialogInstruction}</span>' +
										'</a>' +
									'</h4>' +
								'</div>' +
								'<a class="lotusIcon" href="javascript:void(0)" title="${addToPage}" dojoAttachPoint="addButton">' +
									'<img width="16" height="16" src="${blankPath}" alt="${addToPage}"/>' +
									'<span class="lotusAltText">+</span>' +
								'</a>' +
							'</div>' +
						'</div>',
						
		postMixInProperties: function(){
			var bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
			this.addToPage = bundle["add_content_addToPage"];
			this.detailDialogInstruction = bundle["detail_dialog_instruction_text"] || this.detailDialogInstruction;
			if (!this.map.thumbnail) {
				this.map.thumbnail = this.imagePath + "/portletIcon.gif";
			} else {
				this.map.thumbnail = com.ibm.mashups.enabler.utils.EndpointHelper.resolve(this.map.thumbnail);
			}
			this.blankPath = this.imagePath + "/blank.gif";
		},
			
		postCreate: function(){ 
			this.titleAnchor.onclick = dojo.hitch(this, function(){ this.showDetails(); return false; });
			this.addButton.onclick = dojo.hitch(this, function(){ this.apply(); return false; });
			
		},
		
		apply: function(){},
		
		showDetails: function(){},
		
		select: function(){},
		
		deselect: function(){},
		
		destroy: function() {
			this.titleAnchor.onclick = null;
			this.addButton.onclick = null;
			this.inherited(arguments);
		}
	});
	
dojo.provide("com.ibm.pb.customize.smallMenuWidget");
dojo.declare("com.ibm.pb.customize.smallMenuWidget",
	[dijit._Widget, dijit._Templated],
	{
		truncTitleText: "",
		imagePath: ".",
		hub: null,
		item: null,
		map: {},
		warningIcon: "",
		menu: null,
		isFeed: false,
		isOpenSocial: false,
		widgetActions: "Actions",
		detailDialogInstruction: "press enter to see details dialog",
		templateString: '<div class="lotusPaletteWidget" role="gridcell" dojoAttachPoint="lotusPaletteWidget">' +
							'<div class="lotusInner">' +
								'<div class="lotusIcon">' +
									'<span>' +
										'<img alt="${truncTitleText}" src="${map.thumbnail}"/>' +
									'</span>' +
								'</div>' +
								'<a class="lotusWarningIcon" href="javascript:void(0)" title="sandboxed" dojoAttachPoint="warningIconAnchor">' +
										'<img width="16" height="16" src="${blankPath}" alt=""/>' +
										'<span class="lotusAltText">sandboxed</span>' +
							    '</a>' +
								'<div class="lotusTitle">' +
									'<h4>' +
										'<a href="javascript:void(0);" title="${map.label}" dojoAttachPoint="titleAnchor">' +
											'${truncTitleText}' +
											'<span class="ibmHiddenText">${detailDialogInstruction}</span>' +
										'</a>' +
									'</h4>' +
								'</div>' +
								'<a class="lotusIcon" href="javascript:void(0)" title="${addToPage}" dojoAttachPoint="addButton">' +
									'<img width="16" height="16" src="${blankPath}" alt="${addToPage}"/>' +
									'<span class="lotusAltText">+</span>' +
								'</a>' +
								'<a class="lotusIcon iconContextMenu" href="javascript:void(0)" title="${widgetActions}" dojoAttachPoint="contextMenuButton">' +
								    '<img width="16" height="16" src="${menuIconPath}" alt="${widgetActions}"/>' +
								    '<span class="lotusAltText">${widgetActions}</span>' +
							    '</a>' +
							'</div>' +
						'</div>',
						
		postMixInProperties: function(){
			var bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
			this.addToPage = bundle["add_content_addToPage"];
			this.widgetActions = bundle["add_content_widgetActions"] || this.widgetActions; 
			this.detailDialogInstruction = bundle["detail_dialog_instruction_text"] || this.detailDialogInstruction;
			if (!this.map.thumbnail) {
				this.map.thumbnail = this.imagePath + "/portletIcon.gif";
			} else {
				this.map.thumbnail = com.ibm.mashups.enabler.utils.EndpointHelper.resolve(this.map.thumbnail);
			}
			if (!this.warningIcon) this.warningIcon = "/mum/images/oneui/iconWarning.gif";
			this.blankPath = this.imagePath + "/blank.gif";
			this.menuIconPath = "/mum/images/context_menu.png";
		},
			
		postCreate: function(){ 
			this.titleAnchor.onclick = dojo.hitch(this, function(){ this.showDetails(); return false; });
			this.addButton.onclick = dojo.hitch(this, function(){ this.apply(); return false; });
			if(!ibmCfg.portalConfig.portalContext) {  // Portal does not use the menus
				var applyToPage = dojo.hitch(this, function(){
					this.apply();
				});
				var updateShelfItemUI = dojo.hitch(this, function(){
					this._updateShelfItemUI(true);
				});

				if (this._isFromHub()) {
					this.isOpenSocial = this.map.widgetType == "mm_OSGadget";
				} else {
					this.isOpenSocial = this.hub.category == "OpenSocial";
				};
				this.isFeed = this.item.store.namespace == "feed";
				
				this.widgetContextMenu = new com.ibm.mm.builder.iwidget.SmallWidgetContextMenu({
					targetNodeIds: [this.contextMenuButton],
					entryId: this.map.id,
					category: this.category,
					widget: this,
					addWidget: applyToPage,
					updateWidgetItemUI: updateShelfItemUI
				});
				this._updateShelfItemUI(false);
			}
		},
		
		_isFromHub: function() {
			return dojo.trim(ibmConfig["com.ibm.mashups.builder.contentShelf.mode"]) == "hub";
		},
		
		_updateShelfItemUI: function(refresh){
			var cs = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
			var sandboxEnabled = cs.getValue(com.ibm.mashups.enabler.services.ConfigConstants.SANDBOXENABLED);
			if (!sandboxEnabled) return;
			
			if (refresh) {
				this.map.isTrusted = this.widgetContextMenu.widget.map.isTrusted;
            }
			
			if (this._isFromHub()) {	
				this.map.isTrusted = this.map.sandbox_definition == "true" ? false : true;
				if (this.map.isTrusted) {
					this.map.isTrusted = this.map.sandbox_user == "true" ? false : true;
				}	
			} else {		
				this.map.isTrusted = com.ibm.mm.builder.utils.iwidgetUtil.isWidgetTrusted(this.map.id);	
			}			
            
			if(this.map.isTrusted){
				dojo.removeClass(this.lotusPaletteWidget, "lotusPaletteUnTrustedWidget");
			} else {
				dojo.addClass(this.lotusPaletteWidget, "lotusPaletteUnTrustedWidget");
			}
		},
		
		apply: function(){},
		
		showDetails: function(){},
		
		select: function(){},
		
		deselect: function(){},
		
		destroy: function() {
			this.titleAnchor.onclick = null;
			this.addButton.onclick = null;
			if(this.widgetContextMenu) {
				this.widgetContextMenu._clearMenu();
				this.widgetContextMenu.destroy();
			}
			this.inherited(arguments);
		}
	});

dojo.provide("com.ibm.pb.customize.feedSmallWidget");
dojo.declare("com.ibm.pb.customize.feedSmallWidget",
	[dijit._Widget, dijit._Templated],
	{
		truncTitleText: "",
		imagePath: ".",
		hub: null,
		item: null,
		map: {},
		warningIcon: "",
		menu: null,
		detailDialogInstruction: "press enter to see details dialog",
		templateString: '<div class="lotusPaletteWidget" role="gridcell" dojoAttachPoint="lotusPaletteWidget">' +
							'<div class="lotusInner">' +
								'<div class="lotusIcon">' +
									'<span>' +
										'<img alt="${truncTitleText}" src="${map.thumbnail}"/>' +
									'</span>' +
								'</div>' +
								'<div class="lotusTitle">' +
									'<h4>' +
										'<a href="javascript:void(0);" title="${map.label}" dojoAttachPoint="titleAnchor">' +
											'${truncTitleText}' +
											'<span class="ibmHiddenText">${detailDialogInstruction}</span>' +
										'</a>' +
									'</h4>' +
								'</div>' +
								'<a class="lotusIcon" href="javascript:void(0)" title="${addToPage}" dojoAttachPoint="addButton">' +
									'<img width="16" height="16" src="${blankPath}" alt=""/>' +
									'<span class="lotusAltText">+</span>' +
								'</a>' +
							'</div>' +
						'</div>',
			
		postMixInProperties: function(){
			var bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
			this.addToPage = bundle["add_content_addToPage"];
			this.detailDialogInstruction = bundle["detail_dialog_instruction_text"] || this.detailDialogInstruction;
			if (!this.map.thumbnail) {
				this.map.thumbnail = this.imagePath + "/portletIcon.gif";
			} else {
				this.map.thumbnail = com.ibm.mashups.enabler.utils.EndpointHelper.resolve(this.map.thumbnail);
			}
			this.blankPath = this.imagePath + "/blank.gif";
		},
		
		postCreate: function(){	
			this.titleAnchor.onclick = dojo.hitch(this, function(){ this.showDetails(); return false; });
			this.addButton.onclick = dojo.hitch(this, function(){ this.apply(); return false; });
		},
		
		apply: function(){},
		
		showDetails: function(){},
		
		select: function(){},
		
		deselect: function(){},
		
		destroy: function() {
			this.titleAnchor.onclick = null;
			this.addButton.onclick = null;
			this.inherited(arguments);
		}
	});

dojo.provide("com.ibm.pb.customize.noIconSmallWidget");
dojo.declare("com.ibm.pb.customize.noIconSmallWidget",
	[com.ibm.pb.customize.smallWidget],
	{
		templateString: '<div class="lotusPaletteWidget" role="gridcell">' +
							'<div class="lotusInner">' +
								'<div class="lotusIcon"></div>' +
								'<div class="lotusTitle">' +
									'<h4>' +
										'<a href="javascript:void(0);" title="${map.label}" dojoAttachPoint="titleAnchor">${truncTitleText}</a>' +
									'</h4>' +
								'</div>' +
								'<a class="lotusIcon" href="javascript:void(0)" title="${addToPage}" dojoAttachPoint="addButton">' +
									'<img width="16" height="16" src="${blankPath}" alt=""/>' +
									'<span class="lotusAltText">+</span>' +
								'</a>' +
							'</div>' +
						'</div>'
	});
	
	dojo.provide("com.ibm.pb.customize.largeWidget");
	dojo.declare("com.ibm.pb.customize.largeWidget",
	[dijit._Widget, dijit._Templated],
	{
		truncTitleText: "",
		imagePath: ".",
		map:{},
		current: false,
		selectedClass: "lotusSelected",
		hiddenSelectedText: "currently selected for the page",
		hiddenUnselectedText: "press enter to select",
		templateString: '<div class="lotusPaletteWidget" dojoAttachPoint="addButton" role="gridcell" aria-selected="false">' +
							'<div class="lotusInner">' +
								'<div class="lotusIcon">' +
									'<span>' +
										'<img alt="${truncTitleText}" src="${map.thumbnail}"/>' +
									'</span>' +
								'</div>' +
								'<div class="lotusTitle">' +
									'<h4>' +
										'<a href="javascript:void(0);" title="${map.label}">' +
											'${truncTitleText}' +
											'<span class="ibmHiddenText" dojoAttachPoint="hiddenSelectedTextNode">${hiddenUnselectedText}</span>' +
										'</a>' +
									'</h4>' +
								'</div>' +
							'</div>' +
						'</div>',
						
		postMixInProperties: function(){
			if (!this.map.thumbnail) {
				this.map.thumbnail = this.imagePath + "/portletIcon.gif";
			} else {
				this.map.thumbnail = com.ibm.mashups.enabler.utils.EndpointHelper.resolve(this.map.thumbnail);
			}
			this.hiddenSelectedText = dojo.i18n.getLocalization("com.ibm.bundles", "Theme")["currently_selected"] || this.hiddenSelectedText;
			this.hiddenUnselectedText = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf")["selection_instructions"] || this.hiddenUnselectedText;
			this.id = this.map.id;
		},
		
		postCreate: function(){
			if(this.current) this.select();
			this.addButton.onclick = dojo.hitch(this, function(){ this.apply(); return false; });
		},
		
		apply: function(){},
		
		select: function(){
			this.current = true;
			dojo.addClass(this.domNode,this.selectedClass);
			dojo.attr(this.domNode,"aria-selected","true");
			this.hiddenSelectedTextNode.innerHTML = this.hiddenSelectedText;
		},
		
		deselect: function(){
			this.current = false;
			dojo.removeClass(this.domNode,this.selectedClass);
			dojo.attr(this.domNode,"aria-selected","false");
			this.hiddenSelectedTextNode.innerHTML = this.hiddenUnselectedText;
		},
		destroy: function () {
			this.addButton.onclick = null;
			this.inherited(arguments);
		}
	});
	




}

if(!dojo._hasResource["com.ibm.pb.customize.AddContentPaletteContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.customize.AddContentPaletteContainer"] = true;
// This is Add Content
dojo.provide("com.ibm.pb.customize.AddContentPaletteContainer");



dojo.declare("com.ibm.pb.customize.AddContentPaletteContainer",
	[com.ibm.pb.customize.LotusPaletteContainer],
	{
		createProxyURI: null,	// function to create a proxy url
		pendingItems: null,		// items pending creation/modification
		beforeSubmitDfds: null, // Array of Deferreds that must all get called back before submit processing can occur
		afterSubmitDfds: null, 	// Array of Deferreds that must all get called back before submit fires its onComplete/onError handlers
		
		constructor: function() {
			this.label = this.nlsBundle["add_content_browseContent"];
			this.tabLabel = this.nlsBundle["add_content_launchLabel"];
			this.pendingItems = {};
			this.beforeSubmitDfds = [];
			this.afterSubmitDfds = [];
			
			// create context menu loader
			/*var menuDefinitionsModule = "com.ibm.themes.PageBuilder2.menuDefinitions",
				ctxMenusObj = dojo.getObject("ibmCfg.themeConfig.contextMenus", true),
				addContentActionsMenu = new com.ibm.pb.contextMenu.JsonContextMenuLoader({url:dojo.moduleUrl(menuDefinitionsModule,"addContentActions.json"), data: ctxMenusObj.addContentActions});
			com.ibm.mashups.builder.model.Factory.getContextMenuModel().registerContextMenu("addContentActions", addContentActionsMenu);*/
		}
});

}

if(!dojo._hasResource["com.ibm.pb.customize.ChangeStylePaletteContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.customize.ChangeStylePaletteContainer"] = true;
// This is Change Style
dojo.provide("com.ibm.pb.customize.ChangeStylePaletteContainer");




dojo.declare("com.ibm.pb.customize.ChangeStylePaletteContainer",
	[com.ibm.pb.customize.LotusPaletteContainer],
	{
		numRows: 3,			// the number of rows of results to be displayed
		baseCssPath: "",
		
		constructor: function() {
			this.label = this.nlsBundle["change_style_browseStyles"];
			this.tabLabel = this.nlsBundle["change_style_launchLabel"];
			this.themeBundle = dojo.i18n.getLocalization("com.ibm.bundles", "Theme");
		},
		
		postCreate: function(){
			this.inherited("postCreate",arguments);
			
			// change the size of the item results by adding a css class
			dojo.addClass(this.domNode,"lotusLargeWidgets");
			
			// when the apply function from the Customize Controller API is executed, change the visually selected item
			this.connect(this, "apply", "changeSelection");
		},
		
		setupUICallback: function(sources){
			dojo.forEach(sources, dojo.hitch(this, function(source){ 
				source.hub.fetch({
					onComplete:dojo.hitch(this, function(items){
						dojo.forEach(items, dojo.hitch(this, function(item){
							if(item.id[0] != "initial" && dojo.query("[title~="+item.id[0]+"]").length == 0) {
								this._loadCss(this.baseCssPath + item.url[0],item.id[0],true,true);
							}
						}));
					})
				});
			}));
			this.inherited(arguments);
		},
		
		changeSelection: function(/*object*/args){
			// summary: called when apply is called
			// args: contains args.id which is the newly selected item
			this.changeSelected(args.id);
		},
		newCell: function(/*Object*/source,/*DataItem*/item){
			// summary:  override this function to fill innerCell with information about the given datastore item
			// source: a datastore
			// item: an item in source
			var node = this.inherited(arguments),
				widget = dijit.byNode(node),
				editStyleFn = dojo.getObject("ibmCfg.themeConfig.editStyle"); // check if its defined
			if(editStyleFn && widget && widget.textBlock){
				var anchor = dojo.create("a", {
					href: "javascript:void(0);",
					className: "editStyleLink",
					innerHTML: this.themeBundle.edit_title
				}, widget.textBlock);
				widget.connect(anchor, "onclick", function(){
					ibmCfg.themeConfig.editStyle({	// use the reference here in case it changes
						source: source, 
						item: item, 
						node: anchor, 
						apply: widget.apply	// this is the function to call when selecting a style
					});
				});
			}
			return node;
		}
});

}

if(!dojo._hasResource["com.ibm.pb.customize.ChangeLayoutPaletteContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.customize.ChangeLayoutPaletteContainer"] = true;
// This is Change Layout
dojo.provide("com.ibm.pb.customize.ChangeLayoutPaletteContainer");



dojo.declare("com.ibm.pb.customize.ChangeLayoutPaletteContainer",
	[com.ibm.pb.customize.LotusPaletteContainer],
	{
		numRows: 3,			// the number of rows of results to be displayed
		columnHash:{},		// an associative array of layout id to number of columns
		
		constructor: function() {
			this.label = this.nlsBundle["change_layout_browseLayouts"];
			this.tabLabel = this.nlsBundle["change_layout_launchLabel"];
		},
		
		postCreate: function(){
			this.inherited("postCreate",arguments);
			
			// change the size of the item results by adding a css class
			dojo.addClass(this.domNode,"lotusLargeWidgets hasLotusMessage");
			
			// add a message to the UI that says the layout will not change until the page is saved
			var msgNode = this.createLotusMsgNode(this.nlsBundle.change_layout_save_warning,"Warning");
			this.center.appendChild(msgNode);
			
			// when the apply function from the Customize Controller API is executed, change the visually selected item
			this.connect(this, "apply", "changeSelection");
		},
		
		changeSelection: function(/*object*/args){
			// summary: called when apply is called
			// args: contains args.id which is the newly selected item
			this.changeSelected(args.id);
		}
});

}

if(!dojo._hasResource["com.ibm.pb.customize.enabler.TabbedController"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.customize.enabler.TabbedController"] = true;
dojo.provide("com.ibm.pb.customize.enabler.TabbedController");







dojo.declare("com.ibm.pb.customize.enabler.TabbedController",
	[com.ibm.widgets.InlineShelf],
	{
		children: [],					// array of children to display in the tabs, each array item is an object {tabName:"",controller:object that implements the Customize Controller API}
		imagePath: '.',					// path to the css images
		hiddenClass: "lotusHidden",		// the css class name to hide content panes
		selectedClass: "lotusSelected",	// the css class to select tabs
		navModel: null,					// reference to the enabler navigation model
		widgetModel: null,				// reference to the enabler widget model
		selectedChild: null,			// the id of the currently selected child
		
		templateString: '<div class="lotusui ibmPortalCustomizeShelf lotusPalette" style="display: none;">' +
							'<div class="lotusPaletteHeader">' +							
							'<ul dojoAttachPoint="tabs" class="lotusTabs"></ul>' +
							'</div>' +
							'<div dojoAttachPoint="containerDivs"></div>' +
						'</div>',
		refreshRetryInterval: 300,		// if a refresh request is delayed because a save is pending, this is the amount of time to wait before retrying the refresh				
		
		constructor: function() {
			this.logger = com.ibm.mashups.enabler.logging.Logger.getLogger("com.ibm.pb.customize.enabler.TabbedController");
			this.logger.entering("contructor");
			this.bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
			this.messages = dojo.i18n.getLocalization("com.ibm.bundles", "SharedMessages");
			this.okText = this.bundle["shelf_saveChanges"];
			this.cancelText = this.bundle["add_content_cancel"];
			this.eventService = com.ibm.mashups.iwidget.services.ServiceManager.getService("eventService");
	        this.eventService.subscribeEvent("com.ibm.mashups.builder.changePageMode", this, this.onPageModeChanged);
            this.saveInProgress = false;
            this.changedToPageViewMode = false;
            this.logger.exiting("contructor");
		},

        /***********************************************/
        /*              Handle this event              */
        /***********************************************/
        
	    onPageModeChanged: function(iEvent){
	        var nsm = com.ibm.mashups.enabler.model.state.NavigationStateModelFactory.getNavigationStateModel();        
	        if(iEvent == "edit") {
	            nsm.startTransaction();
	        } else if(iEvent == "view") {
	            // if we are saving, then don't discard the transaction
	            // saveChildren() will commit the transaction if the page is actually saved
	            // or discard the transaction if the page is not saved, e.g. due to an error
	            // saveChildren() does need to know that a change to page view mode occurred
	            // also both saveInProgress and changedToPageViewMode will be reset in saveChildren()
	            if (this.saveInProgress)
	                this.changedToPageViewMode = true;
	            
	            // we are not saving, so just discard the transaction
	            else
                    nsm.discardTransaction();
	        }
	    },  
		
		/***********************************************/
		/*           Connect to these events           */
		/***********************************************/
		
		onFirstOpen: function(){
			// summary: stub event to be called the first time the shelf is opened
			this.logger.entering("onFirstOpen");
			this.fillTabs();
			this.logger.exiting("onFirstOpen");
		},
		
		onStartup: function(){
			// summary: stub event to be called when the TabbedController is started up
			this.logger.entering("onStartup");
			this.logger.exiting("onStartup");
		},
		
		onShowTab: function(/*Object*/selectedChild){
			// summary: stub event to be called whenever a tab is shown
			this.logger.entering("onShowTab",[selectedChild]);
			this.logger.exiting("onShowTab",[selectedChild]);
		},
		
		/***********************************************/
		/*         		  UI Functions    		       */
		/***********************************************/
		
		fillTabs: function(){
			// create the tabs and container divs for each child
			this.logger.entering("fillTabs");
			var count = 0, index = 0;
			dojo.forEach(
				this.children,
			    dojo.hitch(this, function(child){
					// create a node for the child to create its container
					var div = dojo.create("div",{className:this.hiddenClass},this.containerDivs);
					dojo.place(child.controller.domNode,div);
					child.containerDiv = div;
					// create a tab div to access the container
					var tab = dojo.create("li");
					var tabName = dojo.create("a",{innerHTML:child.controller.tabLabel,href:"javascript:void(0)"},tab);
					dojo.place(tab,this.tabs);
					child.tab = tab;
					// create an onclick handler for each tab that opens its container
					tabName.onclick = dojo.hitch(this,this.showTab,child);
					if	(child.controller.id == this.selectedChild) {
						index = count;
					}
					count++;
			}));
			if(this.children.length == 0) {
				this.disconnectButton();
			} else {
				this.showTab(this.children[index]);
			}
			this.onStartup();
			this.logger.exiting("fillTabs");
		},
		
		showTab: function(/*Object*/selectedChild){
			// summary: displays the content pane for the given controller
			// selectedChild: the child to show
			this.logger.entering("showTab",[selectedChild]);
			if(!selectedChild.controller.started){
				// if the child isn't started yet, start it up
				selectedChild.controller.startup(this);
			}
			// clear all the tabs and container divs
			dojo.forEach(
				this.children,
			    dojo.hitch(this, function(child){
					dojo.removeClass(child.tab,this.selectedClass);
					dojo.addClass(child.containerDiv,this.hiddenClass);
			}));
			// select and show the given child
			dojo.addClass(selectedChild.tab,this.selectedClass);
			dojo.removeClass(selectedChild.containerDiv,this.hiddenClass);
			this.onShowTab(selectedChild);
			this.selectedChild = selectedChild.controller.id;
			this.logger.exiting("showTab",[selectedChild]);
			return false; /* to prevent onBeforeUnload event from triggering in IE */
		},
		
		refreshTabs: function(/*Array*/newChildren){
			// summary: refreshes the tabs shown in the UI
			// newChildren: the new array of child controllers to show
			var refresh = false;
			if (this.children.length != newChildren.length) {
				refresh = true;
			} else {
				for (var i = 0,l=newChildren.length; i < l; i++) {
					var found = false;
					for (var j = 0,l2 = this.children.length; j < l2; j++) {
						if (newChildren[i].controller.id == this.children[j].controller.id) {
							found = true;
							break;
						}
					}
					if (!found) {
						refresh = true;
						break;
					}
				}
			}
			if (refresh) {
				this.children = newChildren;
				if(!this.everOpened) { return; }
				while(this.containerDivs.children.length > 0){
					this.containerDivs.removeChild(this.containerDivs.firstChild);
				}
				while(this.tabs.children.length > 0){
					this.tabs.removeChild(this.tabs.firstChild);
				}
				this.fillTabs();
			}
		},
		
		clearSelectedTab: function(){
			// summary: sets the currently selected child to null
			this.selectedChild = null;
		},
		
		/***********************************************/
		/*         		  Model Functions  		       */
		/***********************************************/
		
		undoChildren: function(){
			// summary: undoes all pending changes in the children
			this.logger.entering("undoChildren");
			dojo.forEach( this.children, dojo.hitch(this, function(child){ 
				child.controller.undo();	
			}));
			this.logger.entering("exiting");
		},
		
		beforeSave: function(callback){
		    
			// summary: calls the beforeSave function for each child
			this.logger.entering("beforeSave",[callback]);

		    if(!this.children || this.children.length==0){callback();return}//BSpace removed the the first panel.
			// this.children.length is 0 in their UI 
			
			// onPageModeChanged() handler needs to know if a save is in progress
			// this will be reset in saveChildren()
			this.saveInProgress = true;
			
			this.closeOverlay = this.children[0].controller.showWaitMsgInDialog(this.bundle["shelf_saving"]);
			var dfdl = new dojo.DeferredList(dojo.map( this.children, function(child){
				return child.controller.beforeSubmit();
			}, this));
			// add the callback in a timeout or else it occurs so fast the overlay won't show
			setTimeout( dojo.hitch(this, function(){ dfdl.addCallback(dojo.hitch(this, callback)); }), 500 );
			this.logger.exiting("beforeSave",[callback]);
		},
		_setDialogErr: function(dialog, msg, callback){
			dialog.containerNode.innerHTML = msg;
			var footer = dojo.create("div", {
				className: "lotusDialogFooter"
			}, dialog.containerNode, "after");
			if(callback){
				dojo.create("button", {
					className: "lotusBtn",
					innerHTML: this.bundle.add_content_ok,
					onclick: function(){
						callback();
					}
				}, footer);
			}
		},
		saveChildren: function(callback){ 
			// summary: calls submit on each child
			this.logger.entering("saveChildren",[callback]);
			var nsm = com.ibm.mashups.enabler.model.state.NavigationStateModelFactory.getNavigationStateModel();
			var waitingDialog = null;
			if (this.children && this.children.length) {
			    waitingDialog = this.children[0].controller.waitingDialog;
			}
			var saveErr = null;
		
			var refresh = ibmCfg.themeConfig.alwaysRefreshOnPageSave || 
						  com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().isDirty();
			com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(false);
			var count = this.children.length;
			var counter = dojo.hitch(this,function(){
				this.logger.entering("counter in saveChildren",[count]);
				if(--count === 0) {
					// commit widget and navigation models
					var nCommit = this.navModel.commit();
					nCommit.setFinishedCallback(dojo.hitch(this,function(x, status){
						if(status == 200 || status == 201 || status == 400) {
							if(status >= 300 && waitingDialog){
								saveErr = true;
								this._setDialogErr(waitingDialog, this.messages.shelf_customizeError_0, function(){
									window.location.reload();
								});
							}
							else {
								var wCommit = this.widgetModel.commit();
								wCommit.setFinishedCallback(dojo.hitch(this,function(x, status){
									callback();
									if(status == 200 || status == 201 || status == 400) {
										if(status >= 300 && waitingDialog && !saveErr){
											saveErr = true;
											this._setDialogErr(waitingDialog, this.messages.shelf_customizeError_0, function(){
												window.location.reload();
											});
										}
										else {
											// save was sucessful, so commit the nsm transaction
											nsm.commitTransaction();
		
											// if we have NOT changed to page view mode we are still in edit mode
											// so start a new transaction
			                                if (!this.changedToPageViewMode)
			                                    nsm.startTransaction();
		
			                                if(refresh && !ibmCfg.themeConfig.savePageCheckerUnloading && (dojo.isOpera || !ibmCfg.themeConfig.isPageRenderModeCSA)) { com.ibm.utilities.refreshPage(); }
											else if (refresh) {
												this.eventService.broadcastEvent("com.ibm.mashups.builder.layoutRefresh");
											} 
											this.closeOverlay();
										}
									} else {
	                                    // we have changed to page view mode
									    // and not successfully saved
	                                    // so discard the nsm transaction
	                                    if (this.changedToPageViewMode)
	                                        nsm.discardTransaction();
	                                    // if we are still in edit mode
	                                    // then keep the nsm transaction
	                                    // since subsequent saves might work
	
										this.logger.severe("saveChildren","Cannot save the widget model", arguments);
										this.closeOverlay();
									}
	
									// reset state members
									this.changedToPageViewMode= false;
						            this.saveInProgress = false;
								}));
								wCommit.start(false);
							}
						} else {
							this.logger.severe("saveChildren","Cannot save the navigation model", arguments);
							this.closeOverlay();
							callback();

                            // we have changed to page view mode
                            // and not successfully saved
                            // so discard the nsm transaction
                            if (this.changedToPageViewMode)
                                nsm.discardTransaction();
                            // if we are still in edit mode
                            // then keep the nsm transaction
                            // since subsequent saves might work

                            // reset state members
                            this.changedToPageViewMode = false;
                            this.saveInProgress = false;
						}
					}));
					nCommit.start(false);
				}
				this.logger.exiting("counter in saveChildren",[count]);
			});
			var handleError = dojo.hitch(this,function(errors){ 
			    // summary: publishes the given errors
				this.logger.entering("handleError in saveChildren",[errors]);
				counter();
				var message = this.bundle["add_content_addContentError"];
				var details = "";
				if(errors){
					message = errors[0].message;
					details = errors[0].responseText;
				}
				if(message.match(/\D401/)){ // Portal session not authenticated
					details = message;
					message = this.bundle["shelf_session_timeout"];
				}
				this.logger.severe("handleError in saveChildren",message + " : " + details,[errors]);
				this.logger.entering("handleError in saveChildren",[errors]);
			});
			dojo.forEach( this.children, function(child){ 
				if(child.controller.refresh && child.controller.dirty) { refresh = true; }
				child.controller.submit({onComplete:counter,onError:handleError});	
			}, this);
			this.logger.exiting("saveChildren",[callback]);
		}
});

}

if(!dojo._hasResource["com.ibm.pb.data.InstalledPortletStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.data.InstalledPortletStore"] = true;
dojo.provide( "com.ibm.pb.data.InstalledPortletStore" );


dojo.require( "com.ibm.data.AtomFeedStore" );
dojo.require( "com.ibm.data.CatalogMixin" );

dojo.declare( "com.ibm.pb.data.InstalledPortletStore",
			[com.ibm.data.AtomFeedStore,com.ibm.data.CatalogMixin],
			{
				// itemClass: String the class used to represent items of this store
				itemClass: "com.ibm.pb.data.InstalledPortletItem",
				_loadFinished: false,
				_items: [],
				noIcons: true,
				idPrefix: "",
				
				constructor: function( /*Map*/ args ) {
				},
				
				/********************************/
				/**                            **/
				/**                            **/
				/**  catalog extension point   **/
				/**                            **/
				/**                            **/
				/********************************/
				
				mapItem: function(/*DatstoreItem*/item){
					var map = {};
					var id = this.getEPResolvedValue(item, "id", "");
					map.id = id.indexOf(this.idPrefix) == 0 ? id.substring(this.idPrefix.length) : id;
					map.label = this.getEPResolvedValue(item, "title", "untitled");
					map.description = this.getEPResolvedValue(item, "summary", "");
					return map;
				},
			
				_fetchItems: function(/*Object*/ keywordArgs, /*Function*/ findCallback, /*Function*/ errorCallback){
					// summary: see dojo.data.util.simpleFetch
					//The _fetchItems() method should ignore any keywordArgs parameters for start, count, onBegin, onItem, onComplete, onError, sort, and scope, and only needs to handle queryOptions and query
					var me = this;
					var filter = function(requestArgs, arrayOfItems){ 
						
						var items = [];
						if(requestArgs.query){
							var keywordStrings = requestArgs.query.keywords;
							var ignoreCase = requestArgs.queryOptions ? requestArgs.queryOptions.ignoreCase : false; 
							
							// loop through the keywordStrings and push anything that isn't an empty string onto the keywords array
							var keywords = [];
							for(var i = 0; i < keywordStrings.length; i++){
								if(keywordStrings[i].length <= 0) { continue; }
								// make the keyword all lower case if we are doing a case insensitive search
								var keyword = (ignoreCase)?keywordStrings[i].toLowerCase():keywordStrings[i];
								keywords.push(keyword);
							}

							// loop through all the items in the store looking for matches
							for(var i = 0; i < arrayOfItems.length; ++i){
								var match = true;
								if(arrayOfItems[i] === null){
									match = false; 
								} else {
									// loop through each keyword and make sure the store item contains all of them to be a match
									for(var j = 0; j < keywords.length; j++){
										var title = me.getValue(arrayOfItems[i],"title","");
										if(ignoreCase) { title = title.toLowerCase(); } // make the title all lower case if we are doing a case insensitive search
										var summary = me.getValue(arrayOfItems[i],"summary","");
										if(ignoreCase) { summary = summary.toLowerCase(); } // make the summary all lower case if we are doing a case insensitive search
										// if the title AND summary both don't contain the keyword, the match is false
										if(title.indexOf(keywords[j]) == -1 && summary.indexOf(keywords[j]) == -1){
											match = false;
											break;
										}
									}
								}
								if(match) { items.push(arrayOfItems[i]); }
							}
							findCallback(items, requestArgs);
						} else {
							findCallback(arrayOfItems, requestArgs);
						}
					};

					if(this._loadFinished){ 
						filter(keywordArgs, this._items);
					} else {  // the items in the store must be fetched and stored for the first and last time
						var args = arguments;  // create a reference to arguments
						var savedOnComplete = arguments[0].onComplete;  // save a reference to arguments' onComplete function
						// create a new onComplete function to pass to the inherited fetch function
						args[0].onComplete = function(items){  	args[0].onComplete = savedOnComplete; // restore the onComplete function for _fetchItems
																me._loadFinished = true; 
																me._items = items;	// store the items in the store
																filter(keywordArgs,items);
															};
						// retrieve all the store items using the inherited fetch function
						com.ibm.data.AtomFeedStore.prototype.fetch.apply(this, args);
					}
				}
});
dojo.extend(com.ibm.pb.data.InstalledPortletStore,dojo.data.util.simpleFetch);

dojo.declare( "com.ibm.pb.data.InstalledPortletItem",
			com.ibm.data.AtomEntryItem, 
			{
			}
);

}

if(!dojo._hasResource["com.ibm.portal.fx.Transitions"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.fx.Transitions"] = true;
dojo.provide("com.ibm.portal.fx.Transitions");




com.ibm.portal.fx.Transitions = {
	collapsePortlets: function(/*String*/ portletQuery, /*Object*/args){
		dojo.query(portletQuery || ".wpsPortletBody").wipeOut(dojo.mixin({duration: 1000}, args)).play();
	},
	uncollapsePortlets: function(/*String*/ portletQuery, /*Object*/args){
		dojo.query(portletQuery || ".wpsPortletBody").wipeIn(dojo.mixin({duration: 1000}, args)).play();
	},
	combineMapAnimations: function(/*Object*/ map, /*String?*/ prop) {
		prop = prop || "animation";
		return dojo.fx.combine(dojo.map(map, function(obj){
			return obj.animation;
		}));
	},
	chainMapAnimations: function(/*Object*/ map, /*String?*/ prop) {
		prop = prop || "animation";
		return dojo.fx.chain(dojo.map(map, function(obj){
			return obj.animation;
		}));
	},
	fadeToOverlay: function(/*String*/ query, /*Object*/ args, /*Node?*/ context) {
		// summary: Find all nodes matched by the query and cross-fade them out with an overlay
		//		over top of them which has a class called fxOverlay applied to it to visualize
		//		the old position of the node that was just faded.
		// query: String query used in conjunction with dojo.query to retrieve nodes to animate
		// args: Object animation arguments
		// context: Node optional to use as context from which to execute the dojo.query
		// returns: Object map of matching nodes in this format ->
		//		{
		//			overlay: Node overlay node which is faded in by animation
		//			node: Node matched by query that is faded out by animation
		//			animation: dojo._Animation cross-fading animation used to animate the transition
		// 				of the overlay and node.
		// 		} 
		//			
		return dojo.query(query, context).map(function(node) {
			var overlay = dojo.doc.createElement("div");
			var cds = dojo.coords(node);
			dojo.style(overlay, "position", "absolute");
			dojo.style(overlay, "top", (cds.y - dojo.style(node, "marginTop")) + "px");
			dojo.style(overlay, "left", (cds.x - dojo.style(node, "marginLeft")) + "px");
			dojo.style(overlay, "height", cds.h + "px");
			dojo.style(overlay, "width", cds.w + "px");
			dojo.addClass(overlay, "fxOverlay");
			dojo.style(overlay, "border", "solid 2px orange");
			dojo.style(overlay, "opacity", 0);
			dojo.body().appendChild(overlay);
			var anim = dojo.fx.combine([
				dojo.fadeIn(dojo.mixin({node: overlay}, args)),
				dojo.fadeOut(dojo.mixin({node: node}, args))
			]);
			
			return {overlay: overlay, node: node, animation: anim};
		});
	},
	fadeOutOverlay: function(/*Object*/ map, /*Object*/ args) {
		// summary: Cross-fades overlays with the nodes they cover by fading out the overlay
		// 		then fading in the covered node.  Can be used in conjunction with fadeToOverlay.
		//		Note that at the end of each animation, the overlay node will be destroyed.
		// map: Map of overlay nodes and their corresponding nodes that they cover in this format -> 
		// 		{
		//			overlay: Node overlay node
		//			node: Node being covered by overlay
		//		}
		// args: Object animation arguments
		// returns: Object map of matching nodes in this format ->
		//		{
		//			overlay: Node overlay node which is faded out by animation
		//			node: Node matched by query that is faded in by animation
		//			animation: dojo._Animation used to cross-fade the overlays with the
		//				nodes they cover.
		// 		}
		return dojo.map(map, function(obj){
			var anim = dojo.fx.combine([
				dojo.fadeOut(dojo.mixin({node: obj.overlay}, args)),
				dojo.fadeIn(dojo.mixin({node: obj.node}, args))
			]);
			return {overlay: obj.overlay, node: obj.node, animation: anim};
		});
	},
	positionOverlay: function(/*Object*/ map, /*Object*/ args) {
		// summary: Slides overlays to the position and size of the nodes they cover.  Can be
		//		used in conjunction with fadeToOverlay.
		// map: Map of overlay nodes and their corresponding nodes that they cover in this format -> 
		// 		{
		//			overlay: Node overlay node to reposition/resize
		//			node: Node being covered by overlay
		//		}
		// args: Object animation arguments
		// returns: Object map of matching nodes in this format ->
		//		{
		//			overlay: Node overlay node which is faded in by animation
		//			node: Node matched by query that is faded out by animation
		//			animation: dojo._Animation used to slide/resize the overlays to the
		//				nodes they cover.
		// 		}
		return dojo.map(map, function(obj){
			var newCds = dojo.coords(obj.node);
			var oldCds = dojo.coords(obj.overlay);
			
			var sizingAnim = dojo.animateProperty(dojo.mixin({ 
				node: obj.overlay,
				properties: {
					width: { start: oldCds.w, end: newCds.w, unit:"px" },
					height: { start: oldCds.h, end: newCds.h, unit:"px" }
				}
			}, args));
			
			var anim = dojo.fx.combine([
				dojo.fx.slideTo(dojo.mixin({node: obj.overlay, unit:"px", left: newCds.l, top: newCds.t}, args)), 
				sizingAnim
			]);
			return {overlay: obj.overlay, node: obj.node, animation: anim};
		});
	},
	showOverlayDisable: function(node,loading){
		// summary: displays an overlay on top of the given node, disabling its contents
		// node: a string for dojo.query that produces nodes to overlay
		// loading: if true, a loading icon will display on the overlay
		var nodes = dojo.query(node); // TODO: a foreach for this array
		var overlays = [];
		for(var i = 0; i < nodes.length; i++){
			nodes[i].style.position = "relative";  // the position of the div must be relative or absolute so the overlay will stay within it
			// create the overlay div
			var overlay = document.createElement("div");
			dojo.addClass(overlay,"ibmPortalOverlayShow");
			if(loading) {
				dojo.addClass(overlay,"ibmPortalLoading");
			}
			// make the overlay the same size as the main content
			dojo.style(overlay,"height",nodes[i].offsetHeight+"px");
			dojo.style(overlay,"width",nodes[i].offsetWidth+"px");
			// save the overlay
			overlays.push(overlay);
			// show the overlay
			nodes[i].appendChild(overlay);
		} 
		return overlays;
	},
	hideOverlayDisable: function(/*DOMNode[]*/overlays){
		// summary: removes an array of overlays by deleting the nodes
		for(var i = 0; i < overlays.length; i++) {
			overlays[i].parentNode.removeChild(overlays[i]);
		}
	}
};

}

if(!dojo._hasResource["com.ibm.pb.customize.enabler.AddContentController"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.customize.enabler.AddContentController"] = true;
dojo.provide("com.ibm.pb.customize.enabler.AddContentController");










com.ibm.pb.customize.enabler.AddContentController = {
	dirty: false,			// true if there are changes pending
	started: false,			// true if the widget is started up
	refresh: true,			// true if a page refresh is required to show saved changes
	navModel: null,			// reference to enabler navigation model
	imagePath: "",			// path to CSS images
	pendingItems: {},		// map of controls on the page
	placeholders: [],		// array of SSA portlet placeholders
	ibmMainContainer: "ibmMainContainer",
    layoutParser: com.ibm.pb.layout.LayoutParser,

	startup: function(){
		this.logger.entering("startup");
		this.started = true;	// widget is atarted
		this.eventService = com.ibm.mashups.iwidget.services.ServiceManager.getService("eventService");
		this.setupUI();
		this.logger.exiting("startup");
	},
		
	makeDirty: function(/*bool*/change){
		// summary: changes the dirty attribute
		// change: value to assign to the dirty attribute
		this.logger.entering("makeDirty",[change]);
		this.dirty = change;
		this.logger.exiting("makeDirty",[change]);
	},
		
	processChange: function(/*Object*/args){
		// summary: used as an event handler for when a user clicks on an item
		//			hub: the datastore to which the object belongs
		//			item: the item in the datastore being processed
		//			prefsMapFcn (optional): a function or the name of a function in dot notation which retrieves a map of the preferences for the piece of content
		//			takeoverFcn (optional 2): a function or the name of a function in dot notation which will be run instead of the body of this submit function
		//			takeoverObject (optional 2): an object of parameters to be passed to takeoverFcn, must have both takeoverFcn and takeoverObject
		//			cID (optional 3): id of the container in which to insert the control
		//			sID (optional 3): goes with cID, it is the sibling id before which to insert the control
		if(!this.dirty) { this.makeDirty(true); }
		if(args.takeoverFcn) {
			// if there is a takeover function, pass control on to it
			args.takeoverObject = dojo.mixin(args.takeoverObject, {controller:this});
			if(dojo.isString(args.takeoverFcn)) { return dojo.getObject(args.takeoverFcn)(args.takeoverObject); }
			else { return args.takeoverFcn(args.takeoverObject); }
		} else { return this.apply(args); }
	},
		
	apply: function(/*Object*/args){
		// summary: applies a change associated with a selected item to the page
		// args: same as processChange
		this.logger.entering("apply",[args]);
		
		// get layout model
		var navId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		var layoutModel = this.navModel.getLayoutModel(navId);
		
		// get container into which the widget/portlet will go
		var container = null,
			containerDef = null,
			containerId = "";
		if(args.cID) { 
			//get container id from dnd
			containerId = args.cID; 
			containerDef = layoutModel.find(containerId);
			if(containerDef) {
				containerDef.setFinishedCallback(dojo.hitch(this, "_apply"), {layoutModel:layoutModel, args:args});
				containerDef.start(false);
			} else { this.logger.severe("apply","Cannot find dnd container in the layout model",[args]); }
		} else {
			// get the container from the layout model
			var mainContainer = dojo.query("[name~="+this.ibmMainContainer+"]");
			if(mainContainer.length > 0){
				// get the container which has the ibmMainContainer name
				containerId = this.layoutParser.extractID(mainContainer[0]);
				containerDef = layoutModel.find(containerId);
				if(containerDef) {
					containerDef.setFinishedCallback(dojo.hitch(this, "_apply"), {layoutModel:layoutModel, args:args});
					containerDef.start(false);
				} else { this.logger.severe("apply","Cannot find main container in the layout model",[args]); }
			} else {
				// get first container on the page
				var rootDef = layoutModel.getRoot();
				if(rootDef) { var root = rootDef.start(); }
				else { this.logger.severe("apply","Cannot find root layout container in the model",[args]); return; }
				var iter = layoutModel.getChildren(root);
				while(iter.hasNext()){
					container = iter.next();
					if(dojo.byId(container.getID())) { break; }
				}
				if(container) { this._apply(container, "200", {layoutModel:layoutModel, args:args});  }
				else { this.logger.severe("apply","Cannot find any containers in the layout model",[args]); }
			}
		}
		this.logger.exiting("apply",[args]);
	},
	
	_apply: function(container, status, x){
		// summary: inserts a control onto the page
		// container: layout container into which to put the new control
		// status: HTTP status returned from request to get container
		// x: {layoutModel, args from apply()}
		this.logger.entering("_apply",[container, status, x]);
		if(status != 200) {
			this.logger.severe("_apply","Could not find container in the layout model.  Cannot Add Content",[container, status, x]);
			return;
		}
		
		if(container instanceof com.ibm.mashups.enabler.layout.LayoutContainer){
			// get the next sibling
			var nextSibling = null;
			if(x.args.cID && !x.args.sID){
				nextSibling = null;
			} else if(x.args.sID){
				// get the next sibling from dnd
				var controlId = x.args.sID;
				if(controlId) { nextSiblingDef = x.layoutModel.find(controlId); }
				if(nextSiblingDef) { nextSibling = nextSiblingDef.start(); }
				if(!nextSibling) { this.logger.warning("_apply","Cannot find dnd sibling control in the layout model.  Setting it to null.",[container, status, x]); }
			} else {
				// get the first control in the container
				var iter = x.layoutModel.getChildren(container);
				if(iter.hasNext()) { nextSibling = iter.next(); }
			}
			if(nextSibling && !(nextSibling instanceof com.ibm.mashups.enabler.layout.LayoutControl)) { 
				// make sure nextSiblig is a Layout Control object
				this.logger.warning("_apply","Next sibling control is not an instance of com.ibm.mashups.enabler.layout.LayoutControl.  Setting it to null.",[container, status, x]);
				nextSibling = null;
			}
			
			// create the new control instance
			var constants = com.ibm.mashups.enabler.layout.Constants;
			var context = {};
			context[constants.CONTEXT_TYPE] = constants.LAYOUT_CONTROL;
            if (x.args.definitionURL) {
                context[constants.CONTEXT_WIDGET_DEFINITION_URL] = x.args.definitionURL;
            } else {
                context[constants.CONTEXT_WIDGET_DEFINITION_ID] = new com.ibm.pb.customize.Identifiable(x.args.id);
            }
            context[constants.DISPLAY_TITLE] = x.args.label;
			var control = x.layoutModel.create(context);
			if(!ibmCfg.portalConfig.portalContext){
				// temp fix for 18222 - Portal only
				control.setSkin(this._getDefaultSkin());
			}
			
			if (x.args.widgetType === "mm_OSGadget") {
				// gadgets do not have translated titles in their definition,
				// so the titles are forced in from the catalog for gadgets
				var titles = {};
				if (x.args.titles) {
					for (var i = 0; i < x.args.titles.childNodes.length; i++) {
						if (x.args.titles.childNodes[i].nodeType === 1) {
							titles[x.args.titles.childNodes[i].attributes[0].textContent] = x.args.titles.childNodes[i].textContent;
						}
					}
				}
				
				for (var loc in titles) {
					if (titles[loc] !== "{$data}") {
						control.setTitle(titles[loc],loc);
					}
				}
			} else {
				var defaultTitle=this._getWidgetDefinitionTitle(x.args.definitionURL);
				if(defaultTitle !=x.args.label){
					control.setTitle(x.args.label,dojo.locale);
				}
			}
			
			this._addLayoutControlMetaData(control, x.args);
			x.layoutModel.insert(control, container, nextSibling);
			this.pendingItems[x.args.label] = control;
			
			// get and assign prefs
			if(x.args.prefsMapFcn) { this._addPreferences(control, x.args.prefsMapFcn, x.args); }
			
			//set sandbox mark
			this._setSandboxMark(control, !x.args.isTrusted);
			
			// refresh the layout to show the new widget
			this._addPreviewToPage(control, container, nextSibling, x.args.label);
			
		} else { this.logger.severe("_apply","Container is not an instance of com.ibm.mashups.enabler.layout.LayoutContainer",[container, status, x]); }
		com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(true);
		this.logger.exiting("_apply",[container, status, x]);
	},
	_getWidgetDefinitionTitle:function(definitionURL){
		if(!definitionURL) { return null; }  // Portlets do not have definition URLs, so this functions breaks in Portal
		var globalModel = com.ibm.mashups.iwidget.model.Factory.getGlobalWidgetModel();
		var widgetDef = globalModel.getWidgetDefinitionByURL(definitionURL).start(true);
		try {
			return widgetDef.getItemSet("idescriptor").items.title.values[dojo.locale];
		}catch(e){
			return null;
		}
	},
	
	_addPreviewToPage: function(control, container, nextSibling, title) {
		// summary: adds the markup to the page for live preview of the new widget
		// control: the control instance added to the page
		// container: the container instance to which the control was added
		// nextSibling: if present, the control was inserted before this other control
		// title: title of the control
		var getPreviewHTML = control.getRuntimeContent().getRuntimeMarkup();
		var me = this;
		getPreviewHTML.setFinishedCallback(function(markup){
			// put the runtime markup on the page
			if(ibmCfg.portalConfig.portalContext){
				// temp fix for 18222 and Portal blog&wiki UAT breakage - Portal only!
				var i = markup.match(/id-([^"]+)/);
				i = i && i[1];
				markup = '<div class="component-control id-'+i+'"><div class="lotusWidget2"><h2 class="dojoDndHandle"><span>' + title + '</span></h2><div class="lotusWidgetBody"><div class="iw-iWidget" id="'+i+'"><a class="iw-Definition" href="/mccenabler/js/com/ibm/mm/enabler/layout/widget/NonPreviewableContentWidget.xml"></a></div></div></div></div>';
			}
			var controlId = control.getID(), containerId = container.getID();
			if (nextSibling) {
				var sibling = me.layoutParser.findById(nextSibling.getID());
				var newWidget = dojo.place(markup, sibling, "before");
			} else {
				var parent = me.layoutParser.findById(containerId);
				var newWidget = dojo.place(markup, parent, "last");
			}
			// add dnd attributes
			// if you run into problems here with newWidget not a correct DOM Node, this is most likely
			// due to LayoutContainerFragment.html and/or LayoutControlFragment.html containing leading or trailing whitespaces.
			// If so, remove them before building, because dojo.place behaves strange otherwise and returns a DOM fragment instead of a real DOM Node.
			dojo.addClass(newWidget, "dojoDndItem"); 
			dojo.attr(newWidget, "dndType", "lm");
			// extend the sandboxed widget height to 100%
			dojo.forEach(
				dojo.query(".iw-iWidget[sandbox='true']",newWidget),
				function(sandboxed){
					dojo.addClass(sandboxed,"extendediwiWidget");
				}
			);				
			// publish events to parse, add, and wire the widget
			dojo.publish("/com/ibm/mashups/livetext/livetextchanged", [newWidget, true]);
			me.eventService.broadcastEvent("com.ibm.mashups.builder.addNewWidget", controlId); 
			com.ibm.mm.builder.utils.wireUtil.triggerAutowire(controlId);	
			ibmCfg.controllers.dnd.dndSources[containerId].sync();
		});
		getPreviewHTML.start(false); 
	},
	
	_getDefaultSkin: function(){
		// summary: returns the default skin for the theme assigned to the current page
		var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		var page = this.navModel.find(pageId).start();
		var themeId = page.getTheme();
		var theme = com.ibm.mashups.enabler.model.Factory.getThemeModel().find(themeId).start();
		return theme.getDefaultSkin();
	},
	
	_setSandboxMark: function (control, markSandBox) {
		try {
			var windowDef = this.widgetModel.getWidgetWindow(control);
			windowDef.setFinishedCallback(dojo.hitch(this,function(window, status){
				if(status == 200){
					instanceDef = this.widgetModel.getWidgetInstance(window);
					instanceDef.setFinishedCallback(dojo.hitch(this,function(instance, status){
						if(status == 200){
							instance.setSandboxed(markSandBox);
						}
					}));
					instanceDef.start(true);
				}
			}));
			windowDef.start(true);	
		} catch(e) {
			console.log(e);			
		}
	},
	
	_addLayoutControlMetaData: function(control, map) {
		// summary: add meta data to layout control
		// key refers to results from mapItem()
		control.setMetaData("thumbnail", map["thumbnail"]);
		control.setMetaData("onNewWire", map["onNewWire"]);
		control.setMetaData("onAutoWire", map["onAutoWire"]);
		control.setMetaData("icon", map["icon"]);
		control.setMetaData("widgetParam", map["widgetParam"]);
	},
	
	_addPreferences: function(control, prefsMapFcn, map){
		// summary: adds preferences to a control
		// control: layout control to which to add preferences
		// prefsMapFcn: function which gets the preferences
		// map: arguments for the prefsMapFcn
		this.logger.entering("_addPreferences",[control, prefsMapFcn, map]);
		var windowDef = this.widgetModel.getWidgetWindow(control);
		windowDef.setFinishedCallback(dojo.hitch(this,function(window, status){
			if(status == 200){
				instanceDef = this.widgetModel.getWidgetInstance(window);
				instanceDef.setFinishedCallback(dojo.hitch(this,function(instance, status){
					if(status == 200){
						var prefsDef = this.widgetModel.getHierarchicalPreferences(instance);
						prefsDef.setFinishedCallback(dojo.hitch(this,function(preferences, status){
							if(status == 200) {
								var prefsMap = {};
								if(dojo.isString(prefsMapFcn)) { prefsMap = dojo.getObject(prefsMapFcn)(map); }
								else { prefsMap = prefsMapFcn(map); }
								for(var i in prefsMap){ 
									preferences.setValue(i, prefsMap[i]);
								}
							} else { this.logger.warning("_addPreferences","Cannot preferences for widget instance",[control, prefsMapFcn, map]); }
						}));
						prefsDef.start(true);
					} else { this.logger.warning("_addPreferences","Cannot find instance for widget window",[control, prefsMapFcn, map]); }
				}));
				instanceDef.start(true);
			} else { this.logger.warning("_addPreferences","Cannot find widget window based on newly created control",[control, prefsMapFcn, map]); }
		}));
		windowDef.start(true);
		this.logger.exiting("_addPreferences",[control, prefsMapFcn, map]);
	},
		
	undo: function(){
		// summary: reverts any changes made
		this.logger.entering("undo");
		dojo.forEach(this.beforeSubmitDfds, function(dfd){dfd.cancel();});
		this.beforeSubmitDfds = [];
		/*var isSSA = true;
		if(isSSA){
			for(var p in this.placeholders) {
				this.placeholders[p].destroyRecursive();
			}
		} else {
			var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
			this.eventService.broadcastEvent("com.ibm.mashups.builder.layoutRefresh");
		}*/
		this.logger.exiting("undo");
	},
		
	beforeSubmit: function(/*Object*/args){
		// summary: Called before submit and returns a Deferred object which should be used
		// 		to get notified when the controller is ready to be submitted.  Attach a callback
		//		to this Deferred to call the controller's submit function instead of calling
		//		it directly.
		this.logger.entering("beforeSubmit",[args]);
		var dfd = (new dojo.Deferred()).addCallback(dojo.hitch(this, function(){
			// add all the controls to the containers
			// chain up all the functions added with addBeforeSubmit
			this.logger.entering("beforeSubmit dfd callback");
			var dfdl = new dojo.DeferredList(this.beforeSubmitDfds, false, false);
			dojo.forEach(this.beforeSubmitDfds, function(df){
				if(df.fired < 0) {
					df.callback();
				}
			});
			this.beforeSubmitDfds = [];
			this.logger.exiting("beforeSubmit dfd callback");
			return dfdl;	// chain up another deferred after dfd calls back so subsequent callbacks now wait on dfdl
		}));
		// add the error callack to the deferred object
		if(args && args.onError) { dfd.addErrback(args.onError); }
		dfd.callback();
		this.logger.exiting("beforeSubmit",[args]);
		return dfd;
	},
		
	submit: function(/*Object*/args){
		// summary: saves any pending changes
		// args: args may contain an onComplete handler
		this.logger.entering("submit",[args]);
		if(this.dirty) { this.makeDirty(false); }
		this.pendingItems = {};
		if(args.onComplete) { args.onComplete(); }
		this.logger.exiting("submit",[args]);
	},
	
	addBeforeSubmit: function(/*Deferred*/ dfd) {
		// summary: Adds a Deferred object to the list of deferreds that must all be fired before
		//		the Deferred object returned by beforeSubmit is fired. 
		this.logger.entering("addBeforeSubmit",[dfd]);
		if(dfd) { this.beforeSubmitDfds.push(dfd); }
		this.logger.exiting("addBeforeSubmit",[dfd]);
	},

/********************************************************/
/*     Functions to create "new" pieces of content      */
/********************************************************/	

	newFeed: function(/*object*/args){
	    // summary: creates a new Feedspace portlet with user provided feed url
		// args: contains one item, getInfo, which is a function that takes the form data and runs a callback function on the validated user inputs
		var bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
		var form = [{label:bundle["new_feed_title"], id:"new_feed_title", value:"", regex:/\S+/, error:bundle["new_feed_enterTitle"]}, 
					{label:bundle["new_feed_url"], id:"new_feed_url", value:"http://", regex:/http:\/\/\S+/, error:bundle["new_feed_enterUrl"], ltr:true}];
		var submit = dojo.hitch(args.controller, function(inputs){ 
			var label = inputs["new_feed_title"].replace(/<(?:\/?)script>/g,""), url = inputs["new_feed_url"].replace(/<(?:\/?)script>/g,"");
			args.submit(dojo.mixin(args.submitObj,{id:ibmPortalConfig.contentRenderers.feed,label:label})); // add the feed reader to the page
			this.addBeforeSubmit((new dojo.Deferred()).addCallback(dojo.hitch(this, function(){      
				// set a beforeSubmit function to add the feed preferences
				this._addPreferences(this.pendingItems[label], this.getFeedspacePreferences, {url:url,label:label});
			})));
		});
		args.getInfo(form, bundle["new_feed_addFeed"], bundle["add_content_add"], null, submit);
	},
		
	newWCM: function(/*object*/args){
	    // summary: creates a new WCM item like a blog, wiki or solo blog
		// args: an associative array that contains:	getInfo: function that takes the form data and runs a callback function on the validated user inputs
		//												showInfo: function that takes an HTML payload and displays it to the user, then runs a callback function
		//												label (optional): a human readable label for the WCM item, such as "Wiki"
		//												type (optional): the type of WCM content, such as "WIKI_LIBRARY_TYPE"
		//												submitObj (optional): {
		//													hub (optional): a datastore
		//													item (optional): an item in the datastore
		//												}
		// args MUST contain EITHER label, type OR hub and an item containing the "label", and "type" attributes
		var label = args.label ? args.label : args.submitObj.hub.getValue(args.submitObj.item,"label","");
		label = (args.submitObj.hub && args.submitObj.hub.nlsBundle && args.submitObj.hub.nlsBundle[label]) ? args.submitObj.hub.nlsBundle[label] : label;
		var waitingMsg = args.submitObj.hub.getValue(args.submitObj.item,"waitingMsg","");
		waitingMsg = (args.submitObj.hub && args.submitObj.hub.nlsBundle && args.submitObj.hub.nlsBundle[waitingMsg]) ? args.submitObj.hub.nlsBundle[waitingMsg] : waitingMsg;
		var type = args.type ? args.type : args.submitObj.hub.getValue(args.submitObj.item,"type","");
		var bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
		var genericAddStr = dojo.string.substitute(bundle["new_generic_add"], [label]);
		var processInputs = dojo.hitch(args.controller, function(inputs){ 
			// context "this" object points to controller, args.controller 
			var name = dojo.string.trim(inputs["new_wcm_name"]).replace(/<(?:\/?)script>/g,"");
			if(name && name.length > 0){
				var contr = com.ibm.pb.customize.enabler.AddContentController;
				var closeOverlay = args.showInfo(waitingMsg);
				var validateNameCallback = dojo.hitch(this, function(response){
					closeOverlay();
					if(response == "false"){
						// set up the portlet to get created now
						args.submit(dojo.mixin(args.submitObj,{id:ibmPortalConfig.contentRenderers.wcm,label:name}));
						prefsCallback = dojo.hitch(this, function(data){
							this._addPreferences(this.pendingItems[name], this.getJSRWcmRenderingPreferences, {contextid:data, title:name});
						});
						this.addBeforeSubmit((new dojo.Deferred()).addCallback(dojo.hitch(this, "_processCreateLibrary", name, type, prefsCallback, null)));
					} else {
						// alert the user that the name is already in use
						args.getInfo(form,genericAddStr,bundle["add_content_add"],bundle["new_wcm_wcmNameTaken"],processInputs);
					}
				});
				contr._validateNameField(name, type, validateNameCallback, closeOverlay);
			} else {
				// alert the user that the name is invalid
				args.getInfo(form,genericAddStr,bundle["add_content_add"],bundle["new_wcm_wcmNameInvalid"],processInputs);
			}
		});
		// get a name for the WCM object from the user
	    var form = [{label:bundle["new_wcm_name"],id:"new_wcm_name",value:"",regex:/^[\s\S]/,error:bundle["new_wcm_wcmNameInvalid"]}];
		args.getInfo(form,genericAddStr,bundle["add_content_add"],null,processInputs);
	},

/*******************************************************/
/*       functions to create control preferences   	   */
/*******************************************************/	

	getFeedReaderPreferences: function(/*Object*/itemMap){
		var map = {};
		map.feedURL = itemMap.url;
		map.title = itemMap.label;
		return map;
	},
	
	getChartPreferences: function(/*Object*/itemMap){
		var map = {};
		map.dataSourceURL = itemMap.url;
		map.caption = itemMap.label;
		return map;
	},
	
	getDataViewerPreferences: function(/*Object*/itemMap){
		var map = {};
		map.feedURL = itemMap.url;
		map.tableTitle = itemMap.label;
		return map;
	},
    
	getDrilldownPreferences: function(/*Object*/itemMap){
		var map = {};
        var url={"baseURL":itemMap.url,"parameters":[]};
        
		map.__vUrl = dojo.toJson(url);
		map.tableTitle = itemMap.label;
		return map;
	},    

	getFeedspacePreferences: function(/*Object*/itemMap){
		// summary: get feedspace portlet preferences
	    var url = itemMap.url.replace(/&/g,"&amp;").replace(/>/g,"&gt;").replace(/</g,"&lt;").replace(/"/g,"&quot;").replace(/'/,"&#39;");
		var label = itemMap.label.replace(/&/g,"&amp;").replace(/>/g,"&gt;").replace(/</g,"&lt;").replace(/"/g,"&quot;").replace(/'/,"&#39;");
		var map = {};          
		map.selection = "/0";            
	    var subscriptions = '<?xml version="1.0" encoding="utf-8"?><opml version="2.0"><head><title><![CDATA[My Subscription List]]></title><dateModified><![CDATA[' + Date() + ']]></dateModified></head><body><outline xml:id="Subscription_1';
	    subscriptions = subscriptions + '" text="' + label + '" type="rss" isComment="false" isBreakpoint="false" created="' + Date() + '" category="" title="' + label + '" xmlUrl="' + url + '" /></body></opml>';
	    map.subscriptions = subscriptions;
	    return map;
	},
	
	getJSRWcmRenderingPreferences: function(/*Object*/args){  
		// summary: get WCM rendering portlet preferences
		var displayTitle = args.title.replace(/&/g,"&amp;").replace(/>/g,"&gt;").replace(/</g,"&lt;").replace(/"/g,"&quot;").replace(/'/,"&#39;");
		var map = {};
	    map.WCM_CONTENT_CONTEXT_IDR = args.contextid; 
	    map.WCM_BROADCASTS_TO = "WCM_LINKING_NONE";
	    map.WCM_CONTENT_TYPE = "CONTENT";
	    map.WCM_CONTENT_CONTEXT_TYPE = "sitearea";
	    map.WCM_LISTENS_TO = "WCM_LINKING_OTHER";
	    map.WCM_PORTLET_TITLE = displayTitle;
		map.WCM_PORTLET_TITLE_TYPE = "WCM_PORTLET_TITLE_TYPE_GENERAL";      
	    return map;
	},

/*******************************************************/
/*       Helper functions for blogs/wikis  	           */
/*******************************************************/

	_addResourceListFeed: function(/*string*/new_name, /*string*/wcmLibraryType, /*string*/operation_id, /*function*/callback, /*function*/ errorFcn){
		// summary: verifies the name or copies a library with the feed
		var baseUrl = ibmPortalConfig.contentHandlerURI;
		var feedUrl = baseUrl + "?uri=wcmbw:id:" + operation_id;
		var message = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf")["new_wcm_wcmNotCopied"];
		var getResourceHandler = dojo.xhrPost({
			url: feedUrl,
			content: {
				blogname: new_name,
				resourcetype: wcmLibraryType,
				pageid: com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID()
			},
			handleAs: "json",
			preventCache: true,
			load: function(data){
				var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID(),
					navModel = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel(),
					page = navModel.find(pageId).start(),
					hasScopeMetadata = page.getMetaData("param.sharing.scope") != null,
					canSet = page.confirmSetMetaData("param.sharing.scope");
				if(operation_id == "copylibrary" && canSet && !hasScopeMetadata){
					page.setMetaData("param.sharing.scope","ibm.wcm.0." + new Date().getTime());
				}
				if(callback) { callback(data.items[0]); }
			},
			error: function(error,ioArgs){
				if(errorFcn) { errorFcn(); }
			}
		 });
		return getResourceHandler;
	},

	_validateNameField: function(/*String*/newName, /*String*/wcmLibraryType, /*function*/callback, /*function*/ errorFcn){
		// summary: makes sure that a name is not already taken in WCM
		var rv = false;
		if (newName && newName.length > 0) {
			rv = true;
			return com.ibm.pb.customize.enabler.AddContentController._addResourceListFeed(newName, wcmLibraryType, "validinput", callback, errorFcn); 
		}
	},

	_processCreateLibrary: function(/*String*/name, /*String*/wcmLibraryType, /*function*/callback, /*function*/ errorFcn){
		// summary: creates a blog/wiki
		var newLibrId = null;
		var templateName = "Wiki Template v70";
		if (wcmLibraryType == "BLOG_LIBRARY_TYPE") {
			templateName = "Blog Template v70";
		}else if (wcmLibraryType == "BLOG_SOLO_TYPE") {
			templateName = "Blog Solo Template v70";
		}    
		return com.ibm.pb.customize.enabler.AddContentController._addResourceListFeed(name, templateName, "copylibrary", callback, errorFcn);
	}
};

dojo.provide("com.ibm.pb.customize.Identifiable");
dojo.declare("com.ibm.pb.customize.Identifiable",com.ibm.mashups.enabler.Identifiable,{
	_id: null,
	
	constructor: function(id) {
		this._id = id;
	},
	
	getID: function() {
		return this._id;
	}
});

}

if(!dojo._hasResource["com.ibm.pb.data.LayoutTemplateModelStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.data.LayoutTemplateModelStore"] = true;
dojo.provide( "com.ibm.pb.data.LayoutTemplateModelStore" );

dojo.require( "com.ibm.data.CatalogMixin" );

// summary: Wrappers Enabler's Layout Template Model API into a dojo DataStore for use with dojo widgets.
// description: This data store in particular is designed to work with the Customize shelf widgets. 
dojo.declare( "com.ibm.pb.data.LayoutTemplateModelStore", [com.ibm.data.CatalogMixin], {
	constructor: function () {
		this.model = com.ibm.mashups.enabler.layouttemplate.Factory.getTemplateModel();
		this.localizedContext = com.ibm.mashups.enabler.context.Factory.getLocalizedContext();
	},
	searchable: true,
	mapItem: function(/*DatstoreItem*/item){
		// summary: Returns a map containing parameters for the given datastore item
		//		The keys must be: label, description, tags, rating, url, id, thumbnail
		//			- label, description, url, id and thumbnail are strings
		//			- tags is an array of strings
		//			- rating is a string that contains a number, note that decimal values are allowed
		//		Having all content search datastores create item maps with the same keys allows the widget to access information uniformly
		// 		If a datastore item does not have the same attributes for getValue as seen below, this function must be overridden
		//		If a datastore item does not have an appropriate parameter to  map to a key, that map entry can be omitted
		//		An example for a datastore item with a title, summary, rating and identifier:
		//			mapItem: function(/*DatastoreItem*/item){
		//				// does not include the "tags" and "url" keys
		//				var map = {};
		//				map["label"] = this.getValue(item, "title", "untitled");
		//				map["description"] = this.getValue(item, "summary", null);
		//				map["rating"] = this.getValue(item, "rating", "0.0");
		//				map["id"] = this.getValue(item, "identifier", null);
		//				map["thumbnail"] = this.getValue(item, "thumbnail", null);
		//				return map;
		//			}
		var map = {};
		map.label = "untitled3";
		var titleFar = this.getLabel(item);
		if (titleFar) {
			if ( typeof titleFar == "string" ) {
				map.label = titleFar;
			} else {
				titleFar.setFinishedCallback(function (title) {
					map.label = title ? title : "untitled2";
				});
				titleFar.start();
			}
		}
		
		map["description"] = null;

		var url = this.getValue(item, "URL", "");
		map["url"] = "dav:"+url.substring(url.indexOf("dav")+4,url.lastIndexOf("/"));  // dav url
		
		map["id"] = this.getValue(item, "ID", null);
		
		map.thumbnail = "";
		var webDavBasePath = com.ibm.mm.enabler.remote.WebDavUrlFactory.createUrl();
		var thumbnailPath = item.getMetaData( "thumbnailPath" );
		if ( thumbnailPath ) {
		   if( typeof thumbnailPath === "string" ){
			   map.thumbnail = webDavBasePath + thumbnailPath;
		   } else {
			   thumbnailPath.setFinishedCallback(function (path) {
				   map.thumbnail = webDavBasePath + path;
			   });
			   thumbnailPath.start();
		   }
		}
		
		return map;
	},
	/********************************/
	/**                            **/
	/**                            **/
	/**    dojo.data.api.Identity  **/
	/**                            **/
	/**                            **/
	/********************************/
   getIdentity: function(/* item */ item) {
		//    summary:
		//        Returns a unique identifier for an item. The return value will be
		//        either a string or something that has a toString() method.
		//    item:
		//        The item from the store from which to obtain its identifier.
		//    exceptions:
		//        Conforming implementations may throw an exception or return null if
		//        item is not an item.
		return this.getValue( "ID" );
    },
    getIdentityAttributes: function(/* item */ item) {
		//    summary:
		//        Returns an array of attribute names that are used to generate the identity.
		//        For most stores, this is a single attribute, but for some complex stores
		//        such as RDB backed stores that use compound (multi-attribute) identifiers
		//        it can be more than one. If the identity is not composed of attributes
		//        on the item, it will return null. This function is intended to identify
		//        the attributes that comprise the identity so that so that during a render
		//        of all attributes, the UI can hide the the identity information if it
		//        chooses.
		//    item:
		//        The item from the store from which to obtain the array of public attributes that
		//        compose the identifier, if any.
    	return [ "ID" ];
    },
    fetchItemByIdentity: function(/* object */ keywordArgs) {
		//    summary:
		//        Given the identity of an item, this method returns the item that has
		//        that identity through the onItem callback. Conforming implementations
		//        should return null if there is no item with the given identity.
		//        Implementations of fetchItemByIdentity() may sometimes return an item
		//        from a local cache and may sometimes fetch an item from a remote server,
		//
		//    keywordArgs:
		//        An anonymous object that defines the item to locate and callbacks to invoke when the
		//        item has been located and load has completed. The format of the object is as follows:
		//        {
		//            identity: string|object,
		//            onItem: Function,
		//            onError: Function,
		//            scope: object
		//        }
		//    The *identity* parameter.
		//        The identity parameter is the identity of the item you wish to locate and load
		//        This attribute is required. It should be a string or an object that toString()
		//        can be called on.
		//
		//    The *onItem* parameter.
		//        Function(item)
		//        The onItem parameter is the callback to invoke when the item has been loaded. It takes only one
		//        parameter, the item located, or null if none found.
		//
		//    The *onError* parameter.
		//        Function(error)
		//        The onError parameter is the callback to invoke when the item load encountered an error. It takes only one
		//        parameter, the error object
		//
		//    The *scope* parameter.
		//        If a scope object is provided, all of the callback functions (onItem,
		//        onError, etc) will be invoked in the context of the scope object.
		//        In the body of the callback function, the value of the "this"
		//        keyword will be the scope object. If no scope object is provided,
		//        the callback functions will be called in the context of dojo.global.
		//        For example, onItem.call(scope, item, request) vs.
		//        onItem.call(dojo.global, item, request)
		keywordArgs.path = keywordArgs.identity.toString();
		return this.fetch( keywordArgs );
    },
    /********************************/
	/**                            **/
	/**                            **/
	/**    dojo.data.api.Read      **/
	/**                            **/
	/**                            **/
	/********************************/
	getFeatures: function() {
		//    summary:
		//        The getFeatures() method returns an simple keyword values object
		//        that specifies what interface features the datastore implements.
		//        A simple CsvStore may be read-only, and the only feature it
		//        implements will be the 'dojo.data.api.Read' interface, so the
		//        getFeatures() method will return an object like this one:
		//        {'dojo.data.api.Read': true}.
		//        A more sophisticated datastore might implement a variety of
		//        interface features, like 'dojo.data.api.Read', 'dojo.data.api.Write',
		//        'dojo.data.api.Identity', and 'dojo.data.api.Attribution'.
		return {
			'dojo.data.api.Read': true,
			'dojo.data.api.Identity': true
		};
	},
    getValue: function( /* item */ item, /* attribute-name-string */ attribute, 
                        /* value? */ defaultValue){
		var result = defaultValue;
		var attribute_lc = attribute.toLowerCase();
	
		switch ( attribute_lc ) {
			case "title": 
				result = this.localizedContext.getTitle( item );
				break;
			case "description": 
				result = this.localizedContext.getDescription( item );
				break;
			case "id": 
				result = item.remoteFile.getName();
				break;
			default: 
				if ( this.hasAttribute( item, attribute ) ) {
					result = item[ "get" + attribute ]();
				}
		}
   		
   		return result;
    },
   getValues: function(/* item */ item, /* attribute-name-string */ attribute) {
		//    summary:
		//         This getValues() method works just like the getValue() method, but getValues()
		//        always returns an array rather than a single attribute value.  The array
		//        may be empty, may contain a single attribute value, or may contain many
		//        attribute values.
		//        If the item does not have a value for the given attribute, then getValues()
		//        will return an empty array: [].  (So, if store.hasAttribute(item, attribute)
		//        returns false, then store.getValues(item, attribute) will return [].)
		//
		//    item:
		//        The item to access values on.
		//    attribute:
		//        The attribute to access represented as a string.
		//
		//    exceptions:
		//        Throws an exception if *item* is not an item, or *attribute* is not a string
		var result = [];
		var value = this.getValue( item, attribute, undefined );
		if ( value !== undefined ) {
			result.push( value );
		}
		return result;
    },
    getAttributes: function(/* item */ item) {
		//    summary:
		//        Returns an array with all the attributes that this item has.  This
		//        method will always return an array; if the item has no attributes
		//        at all, getAttributes() will return an empty array: [].
		//
		//    item:
		//        The item to access attributes on.
		//
		//    exceptions:
		//        Throws an exception if *item* is not an item, or *attribute* is not a string
		var attr = [];
		
		for (var prop in item ) {
			if (Object.prototype.hasOwnProperty.call( item, prop ) && prop.indexOf( "get" ) === 0 ) {
				attr.push( prop.substring( 3 ) );
			}
		}
		return attr;
    },
    hasAttribute: function(/* item */ item, /* attribute-name-string */ attribute) {
		//    summary:
		//        Returns true if the given *item* has a value for the given *attribute*.
		//
		//    item:
		//        The item to access attributes on.
		//    attribute:
		//        The attribute to access represented as a string.
		//
		//    exceptions:
		//        Throws an exception if *item* is not an item, or *attribute* is not a string
    	var result = false;
    	if ( item[ "get" + attribute ] !== undefined ) {
    		result = true;
    	}
    	
    	return result;
    },
    containsValue: function(/* item */ item, /* attribute-name-string */ attribute, /* anything */ value) {
		//    summary:
		//        Returns true if the given *value* is one of the values that getValues()
		//        would return.
		//
		//    item:
		//        The item to access values on.
		//    attribute:
		//        The attribute to access represented as a string.
		//    value:
		//        The value to match as a value for the attribute.
		//
		//    exceptions:
		//        Throws an exception if *item* is not an item, or *attribute* is not a string
		var result = false;
		var values = this.getValues( item, attribute );

		if ( values.length > 0 ) {
			var l = values.length;
			var i = 0;
			var found = false;
			while ( !found && i < l ) {
				found = ( values[i] === value );
				i++;
			}
		}
		
		return result;
    }, 
    isItem: function(/* anything */ something) {
		//    summary:
		//        Returns true if *something* is an item and came from the store instance.
		//        Returns false if *something* is a literal, an item from another store instance,
		//        or is any object other than an item.
		//
		//    something:
		//        Can be anything.
		//
   		return something !== undefined && something.declaredClass === "com.ibm.mm.enabler.layouttemplate.LayoutTemplate";
    },
    isItemLoaded: function(/* anything */ something) {
		//    summary:
		//        Returns false if isItem(something) is false.  Returns false if
		//        if isItem(something) is true but the the item is not yet loaded
		//        in local memory (for example, if the item has not yet been read
		//        from the server).
		//
		//    something:
		//        Can be anything.
		//
		return this.isItem(something) && something.remoteFile !== undefined;
	},
	loadItem: function(/* object */ keywordArgs) {
		//    summary:
		//        Given an item, this method loads the item so that a subsequent call
		//        to store.isItemLoaded(item) will return true.  If a call to
		//        isItemLoaded() returns true before loadItem() is even called,
		//        then loadItem() need not do any work at all and will not even invoke
		//        the callback handlers.  So, before invoking this method, check that
		//        the item has not already been loaded.
		//     keywordArgs:
		//        An anonymous object that defines the item to load and callbacks to invoke when the
		//        load has completed.  The format of the object is as follows:
		//        {
		//            item: object,
		//            onItem: Function,
		//            onError: Function,
		//            scope: object
		//        }
		//    The *item* parameter.
		//        The item parameter is an object that represents the item in question that should be
		//        contained by the store.  This attribute is required.
		//    The *onItem* parameter.
		//        Function(item)
		//        The onItem parameter is the callback to invoke when the item has been loaded.  It takes only one
		//        parameter, the fully loaded item.
		//
		//    The *onError* parameter.
		//        Function(error)
		//        The onError parameter is the callback to invoke when the item load encountered an error.  It takes only one
		//        parameter, the error object
		//
		//    The *scope* parameter.
		//        If a scope object is provided, all of the callback functions (onItem,
		//        onError, etc) will be invoked in the context of the scope object.
		//        In the body of the callback function, the value of the "this"
		//        keyword will be the scope object.   If no scope object is provided,
		//        the callback functions will be called in the context of dojo.global().
		//        For example, onItem.call(scope, item, request) vs.
		//        onItem.call(dojo.global(), item, request) 
	},
    getLabel: function(/* item */ item) {
		//    summary:
		//        Method to inspect the item and return a user-readable 'label' for the item
		//        that provides a general/adequate description of what the item is.
		//
		//    description:
		//        Method to inspect the item and return a user-readable 'label' for the item
		//        that provides a general/adequate description of what the item is.  In general
		//        most labels will be a specific attribute value or collection of the attribute
		//        values that combine to label the item in some manner.  For example for an item
		//        that represents a person it may return the label as:  "firstname lastlame" where
		//        the firstname and lastname are attributes on the item.  If the store is unable
		//        to determine an adequate human readable label, it should return undefined.  Users that wish
		//        to customize how a store instance labels items should replace the getLabel() function on
		//        their instance of the store, or extend the store and replace the function in
		//        the extension class.
		//
		//    item:
		//        The item to return the label for.
		//
		//    returns:
		//        A user-readable string representing the item or undefined if no user-readable label can
		//        be generated.
		return this.localizedContext.getTitle( item );   	
    },
    getLabelAttributes: function(/* item */ item) {
		//    summary:
		//        Method to inspect the item and return an array of what attributes of the item were used
		//        to generate its label, if any.
		//
		//    description:
		//        Method to inspect the item and return an array of what attributes of the item were used
		//        to generate its label, if any.  This function is to assist UI developers in knowing what
		//        attributes can be ignored out of the attributes an item has when displaying it, in cases
		//        where the UI is using the label as an overall identifer should they wish to hide
		//        redundant information.
		//
		//    item:
		//        The item to return the list of label attributes for.
		//
		//    returns:
		//        An array of attribute names that were used to generate the label, or null if public attributes
		//        were not used to generate the label.
		return [ "Title", "Description" ];
    },
    fetch: function(/* Object */ keywordArgs) {
		//    summary:
		//        Given a query and set of defined options, such as a start and count of items to return,
		//        this method executes the query and makes the results available as data items.
		//        The format and expectations of stores is that they operate in a generally asynchronous
		//        manner, therefore callbacks are always used to return items located by the fetch parameters.
		//
		//    description:
		//        A Request object will always be returned and is returned immediately.
		//        The basic request is nothing more than the keyword args passed to fetch and
		//        an additional function attached, abort().  The returned request object may then be used
		//        to cancel a fetch.  All data items returns are passed through the callbacks defined in the
		//        fetch parameters and are not present on the 'request' object.
		//
		//        This does not mean that custom stores can not add methods and properties to the request object
		//        returned, only that the API does not require it.  For more info about the Request API,
		//        see dojo.data.api.Request
		//
		//    keywordArgs:
		//        The keywordArgs parameter may either be an instance of
		//        conforming to dojo.data.api.Request or may be a simple anonymous object
		//        that may contain any of the following:
		//        {
		//            query: query-string or query-object,
		//            queryOptions: object,
		//            onBegin: Function,
		//            onItem: Function,
		//            onComplete: Function,
		//            onError: Function,
		//            scope: object,
		//            start: int
		//            count: int
		//            sort: array
		//        }
		//        All implementations should accept keywordArgs objects with any of
		//        the 9 standard properties: query, onBegin, onItem, onComplete, onError
		//        scope, sort, start, and count.  Some implementations may accept additional
		//        properties in the keywordArgs object as valid parameters, such as
		//        {includeOutliers:true}.
		//
		//        The *query* parameter.
		//        The query may be optional in some data store implementations.
		//        The dojo.data.api.Read API does not specify the syntax or semantics
		//        of the query itself -- each different data store implementation
		//        may have its own notion of what a query should look like.
		//        However, as of dojo 0.9, 1.0, and 1.1, all the provided datastores in dojo.data
		//        and dojox.data support an object structure query, where the object is a set of
		//        name/value parameters such as { attrFoo: valueBar, attrFoo1: valueBar1}.  Most of the
		//        dijit widgets, such as ComboBox assume this to be the case when working with a datastore
		//        when they dynamically update the query.  Therefore, for maximum compatibility with dijit
		//        widgets the recommended query parameter is a key/value object.  That does not mean that th
		//        the datastore may not take alternative query forms, such as a simple string, a Date, a number,
		//        or a mix of such.  Ultimately, The dojo.data.api.Read API is agnostic about what the query
		//        format.
		//        Further note:  In general for query objects that accept strings as attribute
		//        value matches, the store should also support basic filtering capability, such as *
		//        (match any character) and ? (match single character).  An example query that is a query object
		//        would be like: { attrFoo: "value*"}.  Which generally means match all items where they have
		//        an attribute named attrFoo, with a value that starts with 'value'.
		//
		//    The *queryOptions* parameter
		//        The queryOptions parameter is an optional parameter used to specify options that may modify
		//        the query in some fashion, such as doing a case insensitive search, or doing a deep search
		//        where all items in a hierarchical representation of data are scanned instead of just the root
		//        items.  It currently defines two options that all datastores should attempt to honor if possible:
		//        {
		//            ignoreCase: boolean, //Whether or not the query should match case sensitively or not.  Default behaviour is false.
		//            deep: boolean     //Whether or not a fetch should do a deep search of items and all child
		//                            //items instead of just root-level items in a datastore.  Default is false.
		//        }
		//
		//    The *onBegin* parameter.
		//        function(size, request);
		//        If an onBegin callback function is provided, the callback function
		//        will be called just once, before the first onItem callback is called.
		//        The onBegin callback function will be passed two arguments, the
		//        the total number of items identified and the Request object.  If the total number is
		//        unknown, then size will be -1.  Note that size is not necessarily the size of the
		//        collection of items returned from the query, as the request may have specified to return only a
		//        subset of the total set of items through the use of the start and count parameters.
		//
		//    The *onItem* parameter.
		//        function(item, request);
		//        If an onItem callback function is provided, the callback function
		//        will be called as each item in the result is received. The callback
		//        function will be passed two arguments: the item itself, and the
		//        Request object.
		//
		//    The *onComplete* parameter.
		//        function(items, request);
		//
		//        If an onComplete callback function is provided, the callback function
		//        will be called just once, after the last onItem callback is called.
		//        Note that if the onItem callback is not present, then onComplete will be passed
		//        an array containing all items which matched the query and the request object.
		//        If the onItem callback is present, then onComplete is called as:
		//        onComplete(null, request).
		//
		//    The *onError* parameter.
		//        function(errorData, request);
		//        If an onError callback function is provided, the callback function
		//        will be called if there is any sort of error while attempting to
		//        execute the query.
		//        The onError callback function will be passed two arguments:
		//        an Error object and the Request object.
		//
		//    The *scope* parameter.
		//        If a scope object is provided, all of the callback functions (onItem,
		//        onComplete, onError, etc) will be invoked in the context of the scope
		//        object.  In the body of the callback function, the value of the "this"
		//        keyword will be the scope object.   If no scope object is provided,
		//        the callback functions will be called in the context of dojo.global().
		//        For example, onItem.call(scope, item, request) vs.
		//        onItem.call(dojo.global(), item, request)
		//
		//    The *start* parameter.
		//        If a start parameter is specified, this is a indication to the datastore to
		//        only start returning items once the start number of items have been located and
		//        skipped.  When this parameter is paired with 'count', the store should be able
		//        to page across queries with millions of hits by only returning subsets of the
		//        hits for each query
		//
		//    The *count* parameter.
		//        If a count parameter is specified, this is a indication to the datastore to
		//        only return up to that many items.  This allows a fetch call that may have
		//        millions of item matches to be paired down to something reasonable.
		//
		//    The *sort* parameter.
		//        If a sort parameter is specified, this is a indication to the datastore to
		//        sort the items in some manner before returning the items.  The array is an array of
		//        javascript objects that must conform to the following format to be applied to the
		//        fetching of items:
		//        {
		//            attribute: attribute || attribute-name-string,
		//            descending: true|false;   // Optional.  Default is false.
		//        }
		//        Note that when comparing attributes, if an item contains no value for the attribute
		//        (undefined), then it the default ascending sort logic should push it to the bottom
		//        of the list.  In the descending order case, it such items should appear at the top of the list.
		//
		//    returns:
		//        The fetch() method will return a javascript object conforming to the API
		//        defined in dojo.data.api.Request.  In general, it will be the keywordArgs
		//        object returned with the required functions in Request.js attached.
		//        Its general purpose is to provide a convenient way for a caller to abort an
		//        ongoing fetch.
		//
		//        The Request object may also have additional properties when it is returned
		//        such as request.store property, which is a pointer to the datastore object that
		//        fetch() is a method of.
		//
		//    exceptions:
		//        Throws an exception if the query is not valid, or if the query
		//        is required but was not supplied.
		if ( keywordArgs === undefined || keywordArgs === null ) {
			keywordArgs = {};
		}
		
		var query = keywordArgs.query;
		var keywordSearch = query ? query.keywords : undefined;
		var onBegin = keywordArgs.onBegin;
		var onItem = keywordArgs.onItem;
		var onComplete = keywordArgs.onComplete;
		var onError = keywordArgs.onError;
		var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
		var start = keywordArgs.start ? keywordArgs.start : 0;
		var count = keywordArgs.count ? keywordArgs.count : Number.POSITIVE_INFINITY;
		var sort = keywordArgs.sort ? keywordArgs.sort : undefined;
		var caseInsensitive = keywordArgs.queryOptions ? keywordArgs.queryOptions.ignoreCase : undefined;
		var sortAttribute;
		var isSortDescending;
		
		if ( sort ) {
			sortAttribute = sort.attribute;
			isSortDescending = sort.descending;
		} else {
			sortAttribute = "id";
		}
		
		var request = keywordArgs;
		request.abort = function () {
			//any way to abort an Enabler Deferred?
			this.aborted = true;
		};
		
		//These odd looking function definitions simply ensure that all of the possible events are defined to a valid
		//function - even if it's an empty function. This simplifies the iteration code a bit, saving a few conditional
		//checks later on.
		var makeOnBeginCall = onBegin ? function (size) {
			if ( !request.aborted ) {
				onBegin.call( scope, size, request );
			}
		} : function () {};
		
		//Determines whether or not an item should be accepted into the result set.
		var me = this;
		var itemAcceptFunction = function ( item ) {
			var shouldCall = !request.aborted;
			if ( keywordSearch && keywordSearch.length > 0) {
				var id = me.mapItem(item).label;
				if ( caseInsensitive ) {
					id = id.toUpperCase();
				}
				
				var len = keywordSearch.length;
				var keyword;
				for ( var i = 0; i < len; i++ ) {
					keyword = keywordSearch[i];
					
					if ( caseInsensitive ) {
						keyword = keyword.toUpperCase();
					}
					if ( id.indexOf( keyword ) < 0 ) {
						return false;
					}
				}	
			}
			return shouldCall;
		};
		
		var makeOnItemCall = onItem ? function ( item ) {
			onItem.call( scope, item, request );
		} : function () {};
		
		var me = this;
		var makeOnCompleteCall = onComplete ? function ( items ) {
			if ( !request.aborted ) {
				if ( onItem ) {
					onComplete.call( scope, null, request );
				} else {
					//Sorts based on the given attribute. The given attribute must be one of the attributes provided by
					//this.mapItem().
					if ( sortAttribute ) {
						var lc_sortAttribute = sortAttribute.toLowerCase();
						//Control sort order based on the provided options.
						var lessThan = isSortDescending ? 1 : -1;
						var greaterThan = isSortDescending ? -1 : 1;
						var equal = 0;
						
						var sortedItems = items.sort(function (a,b) {
							var retVal = undefined;
							var item1 = me.mapItem(a);
							var item2 = me.mapItem(b);
							
							var value1 = item1[ lc_sortAttribute ];
							var value2 = item2[ lc_sortAttribute ];
							
							//Sorting logic:
							//According to the dojo data API definition, any undefined values go to the end for 
							//ascending sort order and the beginning for descending sort order. We return a -1 if value1
							//is less than value2, a 0 if value1 is equal to value2, or a +1 if value1 is greater than 
							//value2. The return value is reversed (via the local variables declared outside of this
							//function) if the sort order is descending (ascending is the default).
							
							if ( !value1 && value2 ) {
								retVal = greaterThan;
							} else if ( !value1 && !value2 ) {
								retVal = equal;
							} else if ( value1 && !value2 ) {
								retVal = lessThan;
							} else if ( value1 === value2 ) {
								retVal = equal;
							} else if ( value1 > value2 ) {
								retVal = greaterThan;
							} else if ( value1 < value2 ) {
								retVal = lessThan;
							} else {
								retVal = equal;
							}
							
							return retVal;
						});
						
						//Make sure we didn't have a failure or lose an item along the way - probably better to return
						//out of order items instead of nothing or an incomplete result set.
						if ( sortedItems !== null && sortedItems.length === items.length ) {
							items = sortedItems.splice( start, count );
						}
					}	
					
					onComplete.call( scope, items, request );
				}
			}
		} : function () {};
		
		var makeOnBeginCall = onBegin ? function ( size ) {
			if ( !request.aborted ) {
				onBegin.call(scope, size, request );
			}
		} : function () {};
		
		var path = query ? query.path : query;
		this._loadItems( path, start, count, itemAcceptFunction, makeOnBeginCall, makeOnItemCall, makeOnCompleteCall );
		
		return request;		
	},
	_loadItems: function ( path, start, count, itemAcceptFunction, makeOnBeginCall, makeOnItemCall, makeOnCompleteCall ) {
		//No specific path is specified or all items are requested. 
		if ( path === undefined || path === null || path === "*" ) {
			var me = this;
			var it = this.model.iterator();
			//The iterator has to load before it can be used. Can switch to setForEachCallback once that is fully implemented.
			it.setFinishedCallback(function () {
				var size = it.size();
				size.setFinishedCallback( function ( totalCount ) {
					var results = [];
					var pending = 0;
					returnedCount = 0;
					var delayedStarts = [];
					var threshold = 2;
					while ( it.hasNext() ) {
						var item = it.next();
						item.setFinishedCallback(function (object) { 
							if ( object && itemAcceptFunction( object ) ) {
								results.push( object );
								//Before an item is fully "loaded", we need to ensure that all remote artifacts are
								//loaded since the widgets make no allowance for a title or description being an 
								//asynchronous call away. Requesting the title here forces the item to load the remote
								//properties file for the current locale that defines the title and description.
								var locale = me.localizedContext.getLocale( object );
								//The available locales are local to the item once the initial PROPFIND occurs. If a 
								//title exists for the current configured locale, then this item isn't finished until 
								//that remote properties file is loaded. Otherwise, just be done.
								if ( locale ) {
									var d = object.getTitle( locale );
									d.setFinishedCallback(function () {
										var d2 = object.getMetaData( "" );
										d2.setFinishedCallback(function () {
											pending--;
											makeOnItemCall( object );
											returnedCount++;
											if ( pending < threshold && delayedStarts.length > 0 ) {
												pending++;
												delayedStarts.pop().start();
											}
											if ( returnedCount == totalCount ) {
												// make callbacks
												makeOnBeginCall( results.length );
												makeOnCompleteCall( results );
											}
										});
										d2.start();
									});
									d.start();
								} else {
									pending--;
									makeOnItemCall( object );
								}
							} else {
								returnedCount++;
								pending--;
								if ( pending < threshold && delayedStarts.length > 0 ) {
									pending++;
									delayedStarts.pop().start();
								}
								if ( returnedCount == totalCount ) {
									// make callbacks
									makeOnBeginCall( results.length );
									makeOnCompleteCall( results );
								}
							}	 
						});
						if ( pending >= threshold ) {
							delayedStarts.push( item );
						} else {
							pending++;
							item.start();
						}
					}
				});
				size.start();
			});
			it.start();
		} else {
			var deferred = this.model.find( path );
			deferred.setFinishedCallback(function (object) {
				makeOnBeginCall( 1 );
				makeOnItemCall( object );
				makeOnCompleteCall( object );
			});
			deferred.start();
		}
	}
    /********************************/
	/**                            **/
	/**                            **/
	/**    dojo.data.api.Write     **/
	/**                            **/
	/**                            **/
	/********************************/
	// deleteItem: function(/* item */ item){
// 		//	summary:
// 		//		Deletes an item from the store.
// 		//
// 		//	item: 
// 		//		The item to delete.
// 		//
// 		//	exceptions:
// 		//		Throws an exception if the argument *item* is not an item 
// 		//		(if store.isItem(item) returns false).
// 		//	example:
// 		//	|	var success = store.deleteItem(kermit);
// 		var success = false;
// 		
// 		if ( this.isItem( item ) ) {
// 			this.model.remove( item );
// 			success = true;
// 		}
// 		
// 		return success;
// 	},
// 	setValue: function(	/* item */ item, 
// 						/* string */ attribute,
// 						/* almost anything */ value){
// 		//	summary:
// 		//		Sets the value of an attribute on an item.
// 		//		Replaces any previous value or values.
// 		//
// 		//	item:
// 		//		The item to modify.
// 		//	attribute:
// 		//		The attribute of the item to change represented as a string name.
// 		//	value:
// 		//		The value to assign to the item.
// 		var attribute = attribute.toLowerCase();
// 		var success = false;
// 		switch ( attribute ) {
// 			case "title": 
// 				item.setTitle( value.locale, value.string );
// 				success = true;
// 			break;
// 			case "description": 
// 				item.setDescription( value.locale, value.string );
// 				success = true;
// 			break;
// 			case "metadata": 
// 				item.setMetaData( value.name, value.value );
// 				success = true;
// 			break;
// 		}
// 		
// 		return success;
// 	},
// 	setValues: function(/* item */ item, /* string */ attribute, /* array */ values) {
// 		//    summary:
// 		//        Adds each value in the *values* array as a value of the given
// 		//        attribute on the given item.
// 		//        Replaces any previous value or values.
// 		//        Calling store.setValues(x, y, []) (with *values* as an empty array) has
// 		//        the same effect as calling store.unsetAttribute(x, y).
// 		//
// 		//    item:
// 		//        The item to modify.
// 		//    attribute:
// 		//        The attribute of the item to change represented as a string name.
// 		//    values:
// 		//        An array of values to assign to the attribute..
// 		//
// 		//    exceptions:
// 		//        Throws an exception if *values* is not an array, if *item* is not an
// 		//        item, or if *attribute* is neither an attribute object or a string.
// 		//    examples:
// 		//        var success = store.setValues(kermit, "color", ["green", "aqua"]);
// 		//        success = store.setValues(kermit, "color", []);
// 		//        if (success) {assert(!store.hasAttribute(kermit, "color"));}
// 		var l = values.length;
// 		var success = true;
// 		for ( var i = 0; i < l; i++ ) {
// 			success = success && this.setValue( item, attribute, values[i] );
// 		}
// 		
// 		return success;
//     },
// 	newItem: function(/* Object */ keywordArgs, /*Object?*/ parentInfo) {
// 		//   summary:
// 		//        Returns a newly created item.  Sets the attributes of the new
// 		//        item based on the *keywordArgs* provided.  In general, the attribute
// 		//        names in the keywords become the attributes in the new item and as for
// 		//        the attribute values in keywordArgs, they become the values of the attributes
// 		//        in the new item.  In addition, for stores that support hierarchical item
// 		//        creation, an optional second parameter is accepted that defines what item is the parent
// 		//        of the new item and what attribute of that item should the new item be assigned to.
// 		//        In general, this will assume that the attribute targetted is multi-valued and a new item
// 		//        is appended onto the list of values for that attribute.
// 		//
// 		//    keywordArgs:
// 		//        A javascript object defining the initial content of the item as a set of JavaScript 'property name: value' pairs.
// 		//    parentInfo:
// 		//        An optional javascript object defining what item is the parent of this item (in a hierarchical store.  Not all stores do hierarchical items),
// 		//        and what attribute of that parent to assign the new item to.  If this is present, and the attribute specified
// 		//        is a multi-valued attribute, it will append this item into the array of values for that attribute.  The structure
// 		//        of the object is as follows:
// 		//        {
// 		//            parent: someItem,
// 		//            attribute: "attribute-name-string"
// 		//        }
// 		//
// 		//    exceptions:
// 		//        Throws an exception if *keywordArgs* is a string or a number or
// 		//        anything other than a simple anonymous object.
// 		//        Throws an exception if the item in parentInfo is not an item from the store
// 		//        or if the attribute isn't an attribute name string.
// 		//    examples:
// 		//        var kermit = store.newItem({name: "Kermit", color:[blue, green]});
// 		var path = keywordArgs.path;
// 		var metadata = keywordArgs.metadata;
// 		var titles = keywordArgs.titles;
// 		var descriptions = keywordArgs.descriptions;
// 		
// 		var item = this.model.create({ name: path });
// 		if ( metadata ) {
// 			for (var prop in metadata ) {
// 				if (Object.prototype.hasOwnProperty.call( metadata, prop ) ) {
// 					item.setMetaData( prop, metadata[prop] );
// 				}
// 			}
// 		}
// 		if ( titles ) {
// 			for (var locale in titles ) {
// 				if (Object.prototype.hasOwnProperty.call( titles, locale ) ) {
// 					item.setMetaData( locale, titles[locale] );
// 				}
// 			}
// 		}
// 		if ( descriptions ) {
// 			for (var locale in descriptions ) {
// 				if (Object.prototype.hasOwnProperty.call( descriptions, locale ) ) {
// 					item.setMetaData( locale, descriptions[locale] );
// 				}
// 			}
// 		}
// 		return item;
// 	},
// 	save: function(/* object */ keywordArgs) {
// 		//    summary:
// 		//        Saves to the server all the changes that have been made locally.
// 		//        The save operation may take some time and is generally performed
// 		//        in an asynchronous fashion.  The outcome of the save action is
// 		//        is passed into the set of supported callbacks for the save.
// 		//
// 		//    keywordArgs:
// 		//        {
// 		//            onComplete: function
// 		//            onError: function
// 		//            scope: object
// 		//        }
// 		//
// 		//    The *onComplete* parameter.
// 		//        function();
// 		//
// 		//        If an onComplete callback function is provided, the callback function
// 		//        will be called just once, after the save has completed.  No parameters
// 		//        are generally passed to the onComplete.
// 		//
// 		//    The *onError* parameter.
// 		//        function(errorData);
// 		//
// 		//        If an onError callback function is provided, the callback function
// 		//        will be called if there is any sort of error while attempting to
// 		//        execute the save.  The onError function will be based one parameter, the
// 		//        error.
// 		//
// 		//    The *scope* parameter.
// 		//        If a scope object is provided, all of the callback function (
// 		//        onComplete, onError, etc) will be invoked in the context of the scope
// 		//        object.  In the body of the callback function, the value of the "this"
// 		//        keyword will be the scope object.   If no scope object is provided,
// 		//        the callback functions will be called in the context of dojo.global.
// 		//        For example, onComplete.call(scope) vs.
// 		//        onComplete.call(dojo.global)
// 		//
// 		//    returns:
// 		//        Nothing.  Since the saves are generally asynchronous, there is
// 		//        no need to return anything.  All results are passed via callbacks.
// 		//    examples:
// 		//        store.save({onComplete: onSave});
// 		//        store.save({scope: fooObj, onComplete: onSave, onError: saveFailed});
// 		var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
// 		var d = this.model.commit();
// 		d.setFinishedCallback(function () {
// 			onComplete.call( scope );
// 		});
// 	},
// 	revert: function() {
// 		//    summary:
// 		//        Discards any unsaved changes.
// 		//    description:
// 		//        Discards any unsaved changes.
// 		//
// 		//    examples:
// 		//        var success = store.revert();
// 		this.model._revert();
// 		return true;
// 	},
// 	isDirty: function(/* item? */ item) {
// 		//    summary:
// 		//        Given an item, isDirty() returns true if the item has been modified
// 		//        since the last save().  If isDirty() is called with no *item* argument,
// 		//        then this method returns true if any item has been modified since
// 		//        the last save().
// 		//
// 		//    item:
// 		//        The item to check.
// 		//
// 		//    exceptions:
// 		//        Throws an exception if isDirty() is passed an argument and the
// 		//        argument is not an item.
// 		//    examples:
// 		//        var trueOrFalse = store.isDirty(kermit); // true if kermit is dirty
// 		//        var trueOrFalse = store.isDirty();       // true if any item is dirty
// 		var result = false;
// 		
// 		if ( item ) {
// 			result = item._isDirty();
// 		} else {
// 			result = this.model.isDirty();
// 		}
// 		
// 		return result;
// 	}
});
			 

}

if(!dojo._hasResource["com.ibm.pb.customize.enabler.ChangeLayoutController"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.customize.enabler.ChangeLayoutController"] = true;
dojo.provide("com.ibm.pb.customize.enabler.ChangeLayoutController");

com.ibm.pb.customize.enabler.ChangeLayoutController = {
	current: "",		// the layout currently selected on the shelf
	set: "",			// the layout currently set on the page
	url: "",			// url to the current layout
	navModel: null,		// enabler navigation model
	dirty: false,		// true if there are changes pending
	started: false,		// true if the widget is started up
	refresh: true,		// true if a page refresh is required to show saved changes
	layoutMarkupMetadataKey: "com.ibm.portal.layout.template.markup",
	layoutNameMetadataKey: "com.ibm.portal.layout.template.file.name.",
	layoutFileName: "layout.html",
	ibmMainContainer: "ibmMainContainer",
	ibmHiddenWidgets: "ibmHiddenWidgets",
	containerNameMetadataKey: "com.ibm.portal.layoutnode.localname",
		
	startup: function(){
		// summary: sets up variables and UI
		this.logger.entering("startup");
		this.current = this.set;		// the currently selected layout is one set for the page
		this.started = true;			// this widget is now started up
		this.setupUI();
		if(!this.navModel) { this.navModel = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel(); }
		this.logger.exiting("startup");
	},
		
	makeDirty: function(/*bool*/change){
		// summary: changes the dirty attribute
		// change: value to assign to the dirty attribute
		this.logger.entering("makeDirty",[change]);
		this.dirty = change;
		this.logger.exiting("makeDirty",[change]);
	},
		
	processChange: function(/*Object*/args){
		// summary: used as an event handler for when a user clicks on an item
		// args: {id:id of the item clicked}
		this.logger.entering("processChange",[args]);
		this.apply(args); 
		this.logger.exiting("processChange",[args]);
	},
		
	apply: function(/*Object*/args){
		// summary: applies a change associated with a selected item to the page
		// args: {id:id of the item clicked}
		this.logger.entering("apply",[args]);
		if(args.id != this.current) {								// return if no change has been made from the current selection
			if(args.id == this.set) { this.makeDirty(false); }		// Change Layout is no longer dirty if the selection is the same as on the server
			else if(this.dirty === false) { this.makeDirty(true); }	// else make Change Layout dirty if it is not already
			this.current = args.id;	
			this.url = args.url;
		}
		com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(true);
		this.logger.exiting("apply",[args]);
	},
		
	undo: function(){
		// summary: reverts any changes made
		this.logger.entering("undo");
		if(this.started) { this.apply({id:this.set}); }
		this.logger.exiting("undo");
	},
		
	beforeSubmit: function(/*Object*/args){
		// summary: Called before submit and returns a Deferred object which should be used
		// 		to get notified when the controller is ready to be submitted.  Attach a callback
		//		to this Deferred to call the controller's submit function instead of calling
		//		it directly.
		this.logger.entering("beforeSubmit",[args]);
		var dfd = new dojo.Deferred();
		dfd.callback();
		this.logger.exiting("beforeSubmit",[args]);
		return dfd;
	},
		
	submit: function(/*Object*/args){
		// summary: saves any pending changes
		// args: {onComplete:function to call when submission is done, onError:function to call if submission fails}
		this.logger.entering("submit",[args]);
		if(!this.dirty) { 
			// if Change Layout is not dirty, call the onComplete handler and return
			if(args.onComplete) { args.onComplete(); }
			this.logger.exiting("submit",[args]);
			return; 
		}
		var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		if(pageId){
			var pageRef = this.navModel.find(pageId);
			if(pageRef){
				// get the current page and then call setLayout
				pageRef.setFinishedCallback(dojo.hitch(this, "_setLayout"), args);
				pageRef.start(false);
			} else {
				this.logger.severe("submit","Cannot get page node object from Navigation Model",[args]);
				if(args.onError) { args.onError(); }
			}
		} else {
			this.logger.severe("submit","Cannot get current page id",[args]);
			if(args.onError) { args.onError(); }
		}
		this.logger.exiting("submit",[args]);
	},
	
	/**********************************************/
	/*                                            */
	/*                 	helpers                   */
	/*                                            */
	/**********************************************/
	
	_setLayout: function(page, status, args){
		// summary: receives the current page and from that gets the page content
		// page: current page object
		// status: HTTP status returned from request to retrieve page
		// args: from submit()
		this.logger.entering("_setLayout",[page,status,args]);
		if(status == "200"){
			var contentDef = page.getContent();
			if(contentDef){
				contentDef.setFinishedCallback(dojo.hitch(this,"_setMetadata"), args);
				contentDef.start(false);
			} else {
				this.logger.severe("submit","Cannot get page content object",[args]);
				if(args.onError) { args.onError(); }
			}
		} else {
			this.logger.severe("submit","Cannot get page object for current node",[args]);
			if(args.onError) { args.onError(); }
		}
		this.logger.exiting("_setLayout",[page,status,args]);
	},
	
	_setMetadata: function(content, status, args){
		// summary: receives the current page content and from that sets the layout metadata
		// page: current page content object
		// status: HTTP status returned from request to retrieve page content
		// args: from submit()
		this.logger.entering("_setMetadata",[content,status,args]);
		if(status == 200){
			var markup = content.getMetaData(this.layoutMarkupMetadataKey) || "html";
			if(content.confirmSetStaticContentURL(this.url) && content.confirmSetMetaData(this.layoutNameMetadataKey+markup,this.layoutFileName) && content.confirmRefreshStaticMarkup()) {
					content.setStaticContentURL(this.url);
					content.setMetaData(this.layoutNameMetadataKey+markup,this.layoutFileName);
					content.refreshStaticMarkup();
					this.makeDirty(false);		// make Change Layout not dirty
					this.set = this.current; 	// the set layout is now the same as the current
					this._reparentControls(args.onComplete);
			} else {
				this.logger.severe("_setMetadata","User does not have permission to set new static layout template",[content,status,args]);
				if(args.onError) { args.onError(); }
			}
		} else {
			this.logger.severe("_setMetadata","Cannot get page content object",[content,status,args]);
			if(args.onError) { args.onError(); }
		}
		this.logger.exiting("_setMetadata",[content,status,args]);
	},
	
	_reparentControls: function(onComplete){
		// summary: finds the root layout node
		// onComplete: function to call when finished
		this.logger.entering("_reparentControls",[onComplete]);
		var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID(),
			layoutModel = this.navModel.getLayoutModel(pageId),
			rootDef = layoutModel.getRoot();
		if(rootDef) {
			rootDef.setFinishedCallback(dojo.hitch(this, "_processControls"), {layoutModel:layoutModel, onComplete:onComplete});
			rootDef.start(false);
		} else { 
			this.logger.severe("_reparentControls","Cannot find root layout container in the Layout Model",[onComplete]);  
			if(onComplete) { onComplete(); }
		}
		this.logger.exiting("_reparentControls",[onComplete]);
	},
	
	_processControls: function(root, status, x){
		// summary: gets the children of the root layout node and reparents those in containers to be deleted
		// root: root layout node object
		// status: HTTP status from request to retrieve root
		// x.layoutModel: layout model for the page
		// x.onComplete: function to call when finished
		this.logger.entering("_processControls",[root,status,x]);
		if(status == 200 || status == "200"){
			var children = x.layoutModel.getChildren(root);
			var newParent = null,
				orphans = [];
			// find the named main layout container
			while(children.hasNext()){
				var child = children.next();
				// found main container
				if(child.getMetaData(this.containerNameMetadataKey) == this.ibmMainContainer) { newParent = child; }
				// skip hidden widgets container
				else if (child.getMetadata(this.containerNameMetadataKey) == this.ibmHiddenWidgets) { continue; }
				else { // reparent controls from this container into the main container
					var grandChildren = x.layoutModel.getChildren(child);
					while(grandChildren.hasNext()) { orphans.push(grandChildren.next()); }
				}
			}
			if(newParent) {
				// reparent the orphans into newParent
				for(var i = 0; i < orphans.length; i++){
					if(!(orphans[i] instanceof com.ibm.mashups.enabler.layout.LayoutControl)) { this.logger.warning("_reparentControls","Orphan is not an instance of a layout control.  Skipping...");  continue; }
					/*if(x.layoutModel.confirmInsert(orphans[i], newParent)) { */try{x.layoutModel.insert(orphans[i], newParent); }catch(e){console.error(e);}//}
					//else { this.logger.warning("_reparentControls","Cannot insert orphaned control into new container.  It will be lost.");  continue; }
				}
			} else { this.logger.warning("_reparentControls","Cannot find the container named '" + this.ibmMainContainer + "' to hold orphaned controls.  They will be lost."); }
			if(x.onComplete) { x.onComplete(); }
		} else {
			this.logger.severe("_processControls","Cannot find root layout container in the Layout Model",[root,status,x]); 
			if(x.onComplete) { x.onComplete(); }
		}
		this.logger.exiting("_processControls",[root,status,x]);
	}
};

}

if(!dojo._hasResource["com.ibm.pb.dnd.layout.LayoutColumnSource"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.dnd.layout.LayoutColumnSource"] = true;
dojo.provide("com.ibm.pb.dnd.layout.LayoutColumnSource");



dojo.declare("com.ibm.pb.dnd.layout.LayoutColumnSource", [com.ibm.pb.dnd.layout.LayoutSource], {
	ghostClass: "ibmDndGhost",
	onDndCancel: function(){
		if (this.isDragging) {
			this.inherited(arguments);			
			this.removeGhost();	
			ibmCfg.controllers.dnd.ghostHeight = 0;
		}
	},
	onOverEvent: function(){
		// create ghost in empty container
		this.inherited(arguments);
		this.active = true;
		
		if(this.isDragging && dojo.dnd.manager().target){
			var currentContainer = dojo.dnd.manager().target.node;	
			var controls = dojo.query('.'+ibmCfg.controllers.dnd.CONTROL_CLASS, currentContainer);	
			// check to see if layout container is empty
			if(controls.length == 0){			
				this.emptyContainer = currentContainer;
				if(dojo.query("."+this.ghostClass).length == 0 ){
					// creating ghosts in empty layout container
					this.emptyContainer.appendChild(this.createGhostMarkup(ibmCfg.controllers.dnd.ghostHeight));
				}
			}else{
			// clear the empty layout container object
				this.emptyContainer = null;			
			}
		}
	},
	onOutEvent: function(){
		// remove ghost from empty container
		this.inherited(arguments);
		this.active = false;
		
		if(dojo.query("."+this.ghostClass).length > 0){
			this.removeGhost();
		}
	},
	_addItemClass: function(node, type){
		// create ghost
		if(this.current == node && this.before == type){ return; }
		if(this.isDragging){
			this.createGhost(node, type);
		}
	},
	_removeItemClass: function(node, type){
		// remove ghost
		if(this.isDragging){
			this.removeGhost();
		}
	},
	createGhost: function(node, type){        
		// determines whether to create ghost before or after node
		if(type == "Before"){
			node.insertBefore(this.createGhostMarkup(ibmCfg.controllers.dnd.ghostHeight),
							  node.childNodes[0]);
		}else if (type == "After"){
			node.appendChild(this.createGhostMarkup(ibmCfg.controllers.dnd.ghostHeight));
			if (this.hasHeight(node)) {
			// add height to the node to make room for the ghost ONLY if there is a height set on the node
			// otherwise the node will become too long
				var nodeHeight = dojo.style(node, "height");
				if (nodeHeight > 0) {
				// make sure the node height isn't 0 as it always is the first time this function is called
				// add the ghost height to the height of the node (which is the widget skin)
					ibmCfg.controllers.dnd.ghostAddedHeight = true;
					dojo.style(node, "height", nodeHeight + ibmCfg.controllers.dnd.ghostHeight + "px");
				}
			}
		}
	},
	removeGhost: function(){
		//removes ghost from page
		var ghost = dojo.byId(this.ghostClass);
		if(ghost){
			var parent = ghost.parentNode;
			parent.removeChild(ghost);
			var nodeHeight = dojo.style(parent, "height");
			if (ibmCfg.controllers.dnd.ghostAddedHeight) {
				// if the createGhost() function added some height to the dnd node, 
				// it must now be removed since the ghost was removed
				ibmCfg.controllers.dnd.ghostAddedHeight = false;
				dojo.style(parent, "height", nodeHeight - ibmCfg.controllers.dnd.ghostHeight + "px");
	
			}
		}
	},
	createGhostMarkup: function(height){
		// creates ghost markup for a column container
		var elem = document.createElement("div");
		elem.className = this.ghostClass;
		elem.id = this.ghostClass;
		dojo.style(elem, { height: height+"px"});
		return elem;
	},
	hasHeight: function(node){
		//returns true if the node has a height set on it
		var style = dojo.attr(node, "style");
		if (!style) {
			return false;
		} else if (style.toString().toLowerCase().indexOf("height") >=0) {
			return true;
		} else if (style.height) {
			return true;
		} else {
			return false;
		}
	}
});

com.ibm.pb.dnd.layout.LayoutColumnSource.creator = function(item, hint){
	if(hint=="avatar"){
		var node = dojo.doc.createElement("div");
		if(dojo.dnd.manager().source.current){
			// use the portlet markup as avatar
			node.innerHTML = String(item);
			var elem = dojo.byId(dojo.dnd.manager().source.current.id);
			dojo.style(node, { width: elem.offsetWidth+"px", opacity: 0.5});
		}
		return {node: node, data: item, type: [ibmCfg.controllers.dnd.DND_TYPE]};
	} else {  // this works the same way as the _normalizedCreator/onDropExternal functions in dojo.dnd.Source when there is no given creator function
		if(dojo.isOpera) {
			var node = dojo.byId(ibmCfg.controllers.dnd.currentNode.id).parentNode;  // opera returned nothing using dojo.query below
		} else {
			var node = dojo.query("[id~="+ibmCfg.controllers.dnd.currentNode.id+"]")[0];
		}
		if(node){
			var source = ibmCfg.controllers.dnd.dndSources[com.ibm.pb.layout.LayoutParser.extractID(node.parentNode)];
			if(source){
				var t = source.getItem(node.id);
				source.delItem(node.id);
				source.selection = {};  // prevents node from being deleted in dojo.dnd.Selector.deleteSelectedNodes called by onDropExternal()
				return {node: node, data: t.data, type: t.type};
			}
		} 
		return {node:dojo.create("div"),data:item,type:ibmCfg.controllers.dnd.DND_TYPE}
	}
};

com.ibm.pb.dnd.layout.LayoutColumnSource.withHandles = true;

}

if(!dojo._hasResource["com.ibm.pb.dnd.layout.LayoutRowSource"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.dnd.layout.LayoutRowSource"] = true;
dojo.provide("com.ibm.pb.dnd.layout.LayoutRowSource");



dojo.declare("com.ibm.pb.dnd.layout.LayoutRowSource", [com.ibm.pb.dnd.layout.LayoutSource], {
	ghostClass: "ibmDndHorizontalGhost",
	currentHolder: null, // this holds the current target when the ghost is inserted before a source
	onDrop: function(source, nodes, copy){
		this.LOGGER.entering("onDrop",[source, nodes, copy]);
		
		//this.firstInsert is used to flag whether the widget will be inserted in the first place of the un-empty row 
		/*
		if(!this.current && !this.emptyContainer && !this.firstInsert){
			this.onDndCancel();
			return;
		}
		*/
						
		// A row source can have an active target without a current target
		// active in the dojo framework. currentHolder will hold the target 
		// to determine where to drop when this scenario occurs
		if(!this.current && this.currentHolder && !this.emptyContainer){
			this.current = this.currentHolder;		
			this.before = true; 
		}
		
		var currentNode = dojo.byId(nodes[0].id);
		var sourceItem = source.getItem(currentNode.id);
		var type = sourceItem ? sourceItem.type : null;
		if(sourceItem && (type == ibmCfg.controllers.dnd.SHELF_TYPE)){
			this.onDropFromShelf(source, nodes, copy);
		}else{
			if(this != source){
				this.onDropExternal(source, nodes, copy);
			}else{
				this.onDropInternal(nodes, copy);
			}
		}
		var currentContainer = null;
		if(this.current){
			currentContainer = this.current.parentNode;
		}else{
			currentContainer = this.emptyContainer;
		}
		
		if (!currentContainer) {
			currentContainer = dojo.dnd.manager().target.node;
		}
		
		if (!currentContainer) {
			currentContainer = currentNode.parentNode;
		}
		
		this._calculateDropTarget(currentNode, currentContainer, type);
		
		this._processDrop(currentNode, currentContainer, source, nodes, copy);
		
		dojo.style(currentNode,{cursor:"default",opacity: 1,display: ""});
		
		this.LOGGER.exiting("onDrop",[source, nodes, copy]);
	},
	onDndCancel: function(){
		if (this.isDragging) {
			this.inherited(arguments);			
			this.removeGhost();	
		}
	},
	onOverEvent: function(){
		// create ghost in empty container
		this.inherited(arguments);
		this.active = true;
		
		if(this.isDragging && dojo.dnd.manager().target){
			var currentContainer = dojo.dnd.manager().target.node;	
			var controls = dojo.query('.'+ibmCfg.controllers.dnd.CONTROL_CLASS, currentContainer);	
			
			this.emptyContainer = currentContainer;
			if(dojo.query("."+this.ghostClass).length == 0 ){
					// creating ghosts in empty layout container
					this.emptyContainer.appendChild(this.createEmptyContainerGhostMarkup());
				}
			if(controls.length != 0){	
				this.emptyContainer = null;
			}
			
			/*
			// check to see if layout container is empty
			if(controls.length == 0){			
				this.emptyContainer = currentContainer;
				if(dojo.query("."+this.ghostClass).length == 0 ){
					// creating ghosts in empty layout container
					this.emptyContainer.appendChild(this.createEmptyContainerGhostMarkup());
				}
			}else{
			// clear the empty layout container object
				this.emptyContainer = null;			
			}
			*/
		}
	},
	onOutEvent: function(){
		// remove ghost from empty container
		this.inherited(arguments);
		this.active = false;
		
		if((dojo.query("."+this.ghostClass).length > 0) && !this.current){
			this.removeGhost();
		}
	},	
	_markTargetAnchor: function(before){
		// determines when to create the ghost, before or after the target
		if(this.current == this.targetAnchor && this.before == before){ return; }
		if((this.targetAnchor != this.current) && !this.targetSelected){
			this.currentHolder = null;
			this.removeGhost(this.targetAnchor);
		}
		if (this.current == null && this.targetAnchor != null) {
			this.current = this.targetAnchor;
		}else if (this.targetAnchor == null && this.current != null){
			this.targetAnchor = this.current;
		}else{
			this.targetAnchor = this.current;
		}
		this.targetBox = null;
		this.before = before;
		if(this.targetAnchor){
			this.currentHolder = this.current;
			this.createGhost(this.targetAnchor, this.before ? "Before" : "After");
			if(this.before){
				this.firstInsert = true;
			}
		}else{
			this.firstInsert = false;
		}
	},
	_unmarkTargetAnchor: function(){
		// determines when to remove the ghost
		if(!this.targetAnchor && !this.targetSelected){ return; }
		this.removeGhost(this.targetAnchor);
		this.currentHolder = null;
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;
	},
	createGhost: function(node, type){	
		// creates ghost on page
		if(type == "Before"){
			var container = node.parentNode;
			container.insertBefore(this.createGhostMarkup(), node);
		}else if (type == "After"){
			var container = node.parentNode;
			container.insertBefore(this.createGhostMarkup(), node.nextSibling);
		}
	},
	removeGhost: function(){
		// removes ghost from page
		var ghost = dojo.byId(this.ghostClass);
		if(ghost){
			var parent = ghost.parentNode;
			parent.removeChild(ghost);
		}
	},
	createGhostMarkup: function(){
		// check to see if a ghost is already on the page
		// and remove before creating another
		this.removeGhost();
		var elem = document.createElement("div");
		dojo.attr(elem, {
				className: this.ghostClass,
				id: this.ghostClass,
		        onmouseover: dojo.hitch(this, function(e){	if (!e) var e = window.event; 
															if(dojo.isIE){ this.targetSelected=e.srcElement; }
															else{ this.targetSelected = e.currentTarget; }
											  }
										),
				onmouseout: dojo.hitch(this, function(){ this.targetSelected = null })
			});
		return elem;
	},
	createEmptyContainerGhostMarkup: function(){
		var elem = document.createElement("div");
		elem.className = this.ghostClass;
		elem.id = this.ghostClass;
		return elem;
	}
});

com.ibm.pb.dnd.layout.LayoutRowSource.creator = function(item, hint){
		if(hint=="avatar"){
			var node = dojo.doc.createElement("div");
			if(dojo.dnd.manager().source.current){
				// use the portlet markup as avatar
				node.innerHTML = String(item);
				var elem = dojo.byId(dojo.dnd.manager().source.current.id);
				dojo.style(node, { width: elem.offsetWidth+"px", opacity: 0.5});
			}
			return {node: node, data: item, type: [ibmCfg.controllers.dnd.DND_TYPE]};
		} else {  // this works the same way as the _normalizedCreator/onDropExternal functions in dojo.dnd.Source when there is no given creator function
			if(dojo.isOpera) {
				var node = dojo.byId(ibmCfg.controllers.dnd.currentNode.id).parentNode;  // opera returned nothing using dojo.query below
			} else {
				var node = dojo.query("[id~="+ibmCfg.controllers.dnd.currentNode.id+"]")[0];
			}
			if(node){
				var source = ibmCfg.controllers.dnd.dndSources[com.ibm.pb.layout.LayoutParser.extractID(node.parentNode)];
				if(source){
					var t = source.getItem(node.id);
					source.delItem(node.id);
					source.selection = {};  // prevents node from being deleted in dojo.dnd.Selector.deleteSelectedNodes called by onDropExternal()
					return {node: node, data: t.data, type: t.type};
				}
			} 
			return {node:dojo.create("div"),data:item,type:ibmCfg.controllers.dnd.DND_TYPE}
		}
};

com.ibm.pb.dnd.layout.LayoutRowSource.withHandles = true;

}

if(!dojo._hasResource["com.ibm.mashups.builder.IEvent"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.builder.IEvent"] = true;
dojo.provide("com.ibm.mashups.builder.IEvent");

/**
 * com.ibm.mashups.builder.IEvent defines all the public builder iEvents.<br/>
 * Usage: call com.ibm.mashups.builder.iEvents.SWITCH_PAGE to retreive the ievent name "com.ibm.mashups.builder.switchPage".<br/>
 * <br/>
 * 1. IEvent naming convention.<br/>
 * IEvent name such as <code>SWITCH_PAGE</code> and <code>PAGE_SWITCHED</code> followed the basic rules:<br/> 
 * (1) Verb + Noun:  the iEvent is broadcast before the action happen.<br/>
 * (2) Noun + Past Tense Verb:  the iEvent is broadcast after the action happen.<br/>
 * <br/>
 * 2. Terminology<br/>
 * (1) System iWidget: iWidget defined in theme.<br/>
 * (2) User iWidget: iWidget which user can add to a mashup page.<br/>
 * (3) Layout iWidget: a kind of system iWidget that manages user iWidget instances.<br/>
 * (4) None visible area: a kind of system iWidget that contains user iWidget instances in a hidden area.<br/>
 * (5) iWidget definition id: unique id of the iWidget metadata in enabler catalog API.<br/>
 * (6) iWidget instance id: unique id of the iWidget instance.<br/>
 * @ibm-api
 * @ibm-module Builder
 */

dojo.declare("com.ibm.mashups.builder.IEvent", null, {

	/**
	* <b>Description</b>: System iWidget should broadcast SWITCH_PAGE iEvent to switch to a given page.<br/>
	* Navigation iWidget must handle SWITCH_PAGE iEvent to switch page; 
	* after page is switched, iEvent <code>"com.ibm.mashups.builder.pageSwitched"</code> must be broadcast. <br/>
	* <b>Value</b>: "com.ibm.mashups.builder.switchPage"<br/>
	* <b>Payload</b>: id of page which will be switched to. Could be <code>null</code>. If null, Navigation iWidget will load current user's first page.<br/>
	* <b>Payload type</b>: text <br/> 
	* @type String
	*/
	SWITCH_PAGE: "com.ibm.mashups.builder.switchPage",
	
	/**
	* <b>Description</b>: Navigation iWidget must broadcast PAGE_SWITCHED iEvent after page is switched. <br/>
	* System iWidget can handle PAGE_SWITCHED iEvent to do follow-up work.<br/>
	* <b>Value</b>: "com.ibm.mashups.builder.pageSwitched"<br/>
	* <b>Payload</b>: id of page switched to. Could be <code>null</code> which means no page available for current user.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/	
	PAGE_SWITCHED: "com.ibm.mashups.builder.pageSwitched",
	
	/**
	* <b>Description</b>: System iWidget should broadcast SWITCH_SPACE iEvent to switch to a given space.<br/>
	* SpaceSelector iWidget must handle SWITCH_PAGE iEvent to switch space.<br/>
	* <b>Value</b>: "com.ibm.mashups.builder.switchSpace"<br/>
	* <b>Payload</b>: id of space which will be switched to. Could be <code>null</code>. If null, SpaceSelector iWidget will switch to personal space.<br/>
	* <b>Payload type</b>: text <br/> 
	* @type String
	*/
	SWITCH_SPACE: "com.ibm.mashups.builder.switchSpace",
	
	/**
	* <b>Description</b>: System iWidget broadcasts REFRESH_SPACE_SELECTOR iEvent to announce the space model has been changed.<br/>
	* SpaceSelector iWidget must handle REFRESH_SPACE_SELECTOR iEvent to retrieve the latest space model and re-render the UI.<br/>
	* For example, when user creates space within space manager, REFRESH_SPACE_SELECTOR iEvent must be broadcast to refresh Space Selector iWidget UI.<br/>
	* <b>Value</b>: "com.ibm.mashups.builder.refreshSpaceSelector"<br/>
	* <b>Payload</b>: id of space which title will be shown up in space selector. Could be <code>null</code>. If null, current space title will be used.<br/>
	* <b>Payload type</b>: text <br/> 
	* @type String
	*/
	REFRESH_SPACE_SELECTOR: "com.ibm.mashups.builder.refreshSpaceSelector",
	
	/**
	* <b>Description</b>: REFRESH_SPACE_LIST_IN_SPACE_MANAGER iEvent is broadcasted to refresh the space list in the space manager when space manager is opened<br/>
	* Space Manager iWidget must handle this iEvent to retrieve the latest space model and re-refresh the space list<br/>
	* <b>Value</b>: "com.ibm.mashups.builder.spacemanager.refreshSpaceList"<br/>
	* <b>Payload</b>: N/A<br/> 
	* @type String
	*/
	REFRESH_SPACE_LIST_IN_SPACE_MANAGER: "com.ibm.mashups.builder.spacemanager.refreshSpaceList",
	
	/**
	* <b>Description</b>: System iWidget broadcasts REFRESH_NAVIGATION iEvent to announce the navigation model has been changed.<br/>
	* Navigation iWidget must handle REFRESH_NAVIGATION iEvent to retrieve the latest navigation model and re-render the UI.<br/>
	* For example, when user changes the page title, REFRESH_NAVIGATION iEvent must be broadcast to refresh Navigation iWidget UI.<br/>
	* <b>Value</b>: "com.ibm.mashups.builder.refreshNavigation"<br/>
	* <b>Payload</b>: N/A<br/> 
	* @type String
	*/
	REFRESH_NAVIGATION: "com.ibm.mashups.builder.refreshNavigation",
	
	/**
	* <b>Description</b>: System iWidget broadcasts REFRESH_PALETTE iEvent to announce the catalog model has been changed.<br/>
	* Palette iWidget must handle this iEvent to retrieve the latest catalog model and re-render the UI.<br/>
	* For example, when a new iWidget is added to Palette, REFRESH_PALETTE iEvent must be broadcast to refresh Palette.<br/>
	* <b>Value</b>: "com.ibm.mashups.builder.refreshPalette"<br/>
	* <b>Payload</b>: N/A<br/> 
	* @type String
	*/
	REFRESH_PALETTE: "com.ibm.mashups.builder.refreshPalette",
	
	/**
	* <b>Description</b>: System iWidget can broadcast LAYOUT_REFRESH iEvent to announce the layout model has been changed.<br/>
	* TemplateLayout iWidget handles LAYOUT_REFRESH iEvent to retrieve the latest layout model and re-render the UI.<br/>
	* <b>Value</b>: "com.ibm.mashups.builder.layoutRefresh"<br/>
	* <b>Payload</b>: id of layout container or control whose markup will be refreshed. Can be <code>null</code>. If null, entire layout (all containers) will be refreshed.<br/>
	* <b>Payload type</b>: text <br/> 
	* @type String
	*/
	LAYOUT_REFRESH: "com.ibm.mashups.builder.layoutRefresh",
	
	/**
	* <b>Description</b>: TemplateLayout iWidget broadcasts LAYOUT_REFRESHED iEvent after layout is done refreshing. <br/>
	* <b>Value</b>: "com.ibm.mashups.builder.layoutRefreshed"<br/>
	* <b>Payload</b>: <code>null</code> if entire layout was just refreshed. Or, id of individual layout container or control that was just refreshed.<br/>
	* <b>Payload type</b>: text <br/> 
	* @type String
	*/
	LAYOUT_REFRESHED: "com.ibm.mashups.builder.layoutRefreshed",
	
	/**
	* <b>Description</b>: System iWidget can broadcast REFRESH_CONTEXT_MENU iEvent to notify a context menu's listeners that it was updated.<br/>
	* <b>Value</b>: "com.ibm.mashups.builder.refreshContextMenu"<br/>
	* <b>Payload</b>: id of context menu whose listeners will be fired. Must not be <code>null</code>.
	* <b>Payload type</b>: text <br/> 
	* @type String
	*/
	REFRESH_CONTEXT_MENU: "com.ibm.mashups.builder.refreshContextMenu",
	
	/**
	* <b>Description</b>: System iWidget can broadcast VIEW_HIDDEN_WIDGETS iEvent to show any hidden-widget containers on the page.<br/>
	* TemplateLayout iWidget handles VIEW_HIDDEN_WIDGETS iEvent to show any hidden-widget containers on the page.<br/>
	* <b>Value</b>: "com.ibm.mashups.builder.viewHiddenWidgets"<br/>
	* <b>Payload</b>: none; can be <code>null</code>.<br/>
	* <b>Payload type</b>: text <br/> 
	* @type String
	*/
	VIEW_HIDDEN_WIDGETS: "com.ibm.mashups.builder.viewHiddenWidgets",
	
	/**
	* <b>Description</b>: System iWidget can broadcast HIDE_HIDDEN_WIDGETS iEvent to hide any hidden-widget containers on the page.<br/>
	* TemplateLayout iWidget handles HIDE_HIDDEN_WIDGETS iEvent to hide any hidden-widget containers on the page.<br/>
	* <b>Value</b>: "com.ibm.mashups.builder.hideHiddenWidgets"<br/>
	* <b>Payload</b>: none; can be <code>null</code>.<br/>
	* <b>Payload type</b>: text <br/> 
	* @type String
	*/
	HIDE_HIDDEN_WIDGETS: "com.ibm.mashups.builder.hideHiddenWidgets",
	
	/**
	* <b>Description</b>: CHANGE_PAGE_MODE iEvent will be broadcast when page mode is changed. <br/>
	* System iWidgets can define CHANGE_PAGE_MODE iEvent as handled iEvent to update their UI accordingly. In current Lotus Mashups it only has 2 modes
	* ("view"/"edit"), but it's not limited to. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.changePageMode"<br/>
	* <b>Payload</b>: new page mode. Either "view" or "edit". Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/		
	CHANGE_PAGE_MODE: "com.ibm.mashups.builder.changePageMode",

	/**
	* <b>Deprecated</b>: ADD_WIDGET_FROM_TOOLBOX iEvent is replaced witch ADD_WIDGET.<br/>
	* <b>Description</b>: System iWidget can broadcast ADD_WIDGET_FROM_TOOLBOX iEvent to create a user iWidget instance.<br/>
	* Layout iWidget must define ADD_WIDGET_FROM_TOOLBOX iEvent as handled iEvent to create the user iWidget instance according to the payload<br/>
	* Base on the enabler CatalogModel API, system iWidget developer can get the user iWidget metadata according to user iWidget definition id.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.addWidgetFromToolbox"<br/>
	* <b>Payload</b>: this event could accept three kinds of payload.<br/>
	* &nbsp;-&nbsp;<b>Text</b>: It is the iWidget definition id. The definition id must a valid value in enabler CatalogModel API. Must not be <code>null</code>.<br/>. 
	* &nbsp;&nbsp;&nbsp;For example: "feedReader".<br/>
	* &nbsp;-&nbsp;<b>Json Object</b>: It includes widget definition id and itemset as preference. The itemset enable developer set/change preference of widget instance during widget rendering.<br/>
	* &nbsp;&nbsp;&nbsp;For example:<br/>
	* &nbsp;&nbsp;&nbsp;{id: "feedReader", itemset: {feedUrl:"http://www.ibm.com/feed.rss"} }.<br/>
	* &nbsp;-&nbsp;<b>Json Object</b>: It includes widget definition metadata and itemset as preference. The keys in widgetMetadata should be same as enabler API.<br/>
	* &nbsp;&nbsp;&nbsp;For example:<br/>
	* &nbsp;&nbsp;&nbsp;{<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;widgetMetadata: <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;{<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;id:"feedReader", <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;definition:"/feedReader/feedReader.xml", <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;icon: "/feedReader/feedreader.png", <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;title:{en:"FeedReader", zh_CN:"yueduqi"},<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;onNewWire: "",<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;widgetParam: "[]"<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;}, <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;itemset: {feedURL:"http://www.ibm.com/feed.rss"} <br/>
	* &nbsp;&nbsp;&nbsp;}<br/>
	* <b>Payload type</b>: text or json<br/> 
	* @type String
	*/
	ADD_WIDGET_FROM_TOOLBOX: "com.ibm.mashups.builder.addWidgetFromToolbox",
	
	/**
	* <b>Description</b>: System iWidget can broadcast ADD_WIDGET iEvent to create a user iWidget instance.<br/>
	* Layout iWidget must define ADD_WIDGET iEvent as handled iEvent to create the user iWidget instance according to the payload<br/>
	* Base on the enabler CatalogModel API, system iWidget developer can get the user iWidget metadata according to user iWidget definition id.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.addWidget"<br/>
	* <b>Payload</b>: this event could accept three kinds of payload.<br/>
	* &nbsp;-&nbsp;<b>Text</b>: It is the iWidget definition id. The definition id must a valid value in enabler CatalogModel API. Must not be <code>null</code>.<br/>. 
	* &nbsp;&nbsp;&nbsp;For example: "feedReader".<br/>
	* &nbsp;-&nbsp;<b>Json Object</b>: It includes widget definition id and itemset as preference. The itemset enable developer set/change preference of widget instance during widget rendering.<br/>
	* &nbsp;&nbsp;&nbsp;For example:<br/>
	* &nbsp;&nbsp;&nbsp;{id: "feedReader", itemset: {feedUrl:"http://www.ibm.com/feed.rss"} }.<br/>
	* &nbsp;-&nbsp;<b>Json Object</b>: It includes widget definition metadata and itemset as preference. The keys in widgetMetadata should be same as enabler API.<br/>
	* &nbsp;&nbsp;&nbsp;For example:<br/>
	* &nbsp;&nbsp;&nbsp;{<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;widgetMetadata: <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;{<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;id:"feedReader", <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;definition:"/feedReader/feedReader.xml", <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;icon: "/feedReader/feedreader.png", <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;title:{en:"FeedReader", zh_CN:"yueduqi"},<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;onNewWire: "",<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;widgetParam: "[]"<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;}, <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;itemset: {feedURL:"http://www.ibm.com/feed.rss"} <br/>
	* &nbsp;&nbsp;&nbsp;}<br/>
	* <b>Payload type</b>: text or json<br/> 
	* @type String
	*/
	ADD_WIDGET: "com.ibm.mashups.builder.addWidget",
	
	/**
	* <b>Description</b>: RESET_STYLE iEvent is broadcast to recreate a empty style of layout iWidget.<br/> 
	* Layout iWidget can define RESET_STYLE iEvent as handled iEvent to recreate style of itself.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.resetStyle"<br/>
	* <b>Payload</b>: N/A<br/>
	* @type String
	*/	
	RESET_STYLE: "com.ibm.mashups.builder.resetStyle",

	/**
	* <b>Description</b>: DISPLAY_WIDGET iEvent is broadcast to move a user iWidget instance from none visible area to layout iWidgets.<br/> 
	* Layout iWidget must define DISPLAY_WIDGET iEvent as handled iEvent to show a user iWidget instance in itself.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.displayWidget"<br/>
	* <b>Payload</b>: user iWidget instance id. Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/	
	DISPLAY_WIDGET: "com.ibm.mashups.builder.displayWidget",

	/**
	* <b>Description</b>: HIDE_WIDGET iEvent is broadcast to move a user iWidget instance from layout iWidget to none visible area. <br/>
	* None visible area must define HIDE_WIDGET iEvent as handled iEvent to hide a user iWidget instance into itself.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.hideWidget"<br/>
	* <b>Payload</b>: user iWidget instance id. Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	HIDE_WIDGET: "com.ibm.mashups.builder.hideWidget",

	/**
	* <b>Description</b>: RENAME_WIDGET iEvent is broadcast to rename a user iWidget instance. <br/>
	* Layout iWidget must define RENAME_WIDGET iEvent as handled iEvent to update the user iWidget instance name.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.renameWidget"<br/>
	* <b>Payload</b>: data of the updated user iWidget name. Must not be <code>null</code>. <br/>
	* {<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;id: user iWidget instance id,<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;name: new name of the user iWidget instance<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;locale(optional): locale of the name<br/>
	* }<br/>
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/
	RENAME_WIDGET: "com.ibm.mashups.builder.renameWidget",

	/**
	* <b>Description</b>: RENAME_ACTION iEvent is broadcast from iWidget context menu to inform another system iWidget to handle rename iWidget action. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.renameAction"<br/>
	* <b>Payload</b>: user iWidget instance id. Must not be <code>null</code>. <br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	RENAME_ACTION: "com.ibm.mashups.builder.renameAction",

	/**
	* <b>Description</b>: MOVE_ACTION iEvent is broadcast from iWidget context menu to inform another system iWidget to handle move iWidget action. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.moveAction"<br/>
	* <b>Payload</b>: user iWidget instance id. Must not be <code>null</code>. <br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	MOVE_ACTION: "com.ibm.mashups.builder.moveAction",
	
	/**
	* <b>Description</b>: RESIZE_ACTION iEvent is broadcast from iWidget context menu to inform another system iWidget to handle resize iWidget action. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.resizeAction"<br/>
	* <b>Payload</b>: user iWidget instance id. Must not be <code>null</code>. <br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	RESIZE_ACTION: "com.ibm.mashups.builder.resizeAction",
	
	/**
	* <b>Description</b>: CHANGE_SKIN iEvent is broadcast to change the skin of a user iWidget instance. <br/>
	* Layout iWidget must define CHANGE_SKIN iEvent as handled iEvent to update the skin of user iWidget instance. <br/>
	* Enabler ThemeModel will provide all the Skin available in current theme. Builder runtime model API will provide current theme Id. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.changeSkin"<br/>
	* <b>Payload</b>: user iWidget instance and the new skin id. The new skin id must be a valid skin id in enabler ThemeModel api.
	* Must not be <code>null</code>.<br/>
	* {<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;id: user iWidget instance id<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;skin: new skin name<br/>
	* }<br/>
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/
	CHANGE_SKIN: "com.ibm.mashups.builder.changeSkin",

	/**
	* <b>Description</b>: Mashup skin service broadcasts SKIN_LOADED iEvent when skin is loaded. <br/>
	* Layout iWidget could define SKIN_LOADED iEvent as handled iEvent if it needs to
	* initialize some additional handlers in user iWidget skin, such as drag and drop handler, resize handler. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.skinLoaded"<br/>
	* <b>Payload</b>: skin parameters. Must not be <code>null</code>.<br/>
	* {<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;widgetId: user iWidget instance id,<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;skinNodeId: user iWidget skin node,<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;skinParam: skin parameters defined in skin.js in the skin resource directory, json<br/>
	* }<br/>
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/
	SKIN_LOADED: "com.ibm.mashups.builder.skinLoaded",

	/**
	* <b>Description</b>: RESIZE_WIDGET iEvent should be broadcast to adjust the width or height of user iWidget instance. <br/>
	* Layout iWidget can define RESIZE_WIDGET iEvent as handled iEvent to update the size of user iWidget instance. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.resizeWidget"<br/>
	* <b>Payload</b>: size of user iWidget instance. Must not be <code>null</code>.<br/>
	* {<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;widgetId: id of user iWidget instance,<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;width: new width of user iWidget instance,<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;height: new height of user iWidget instance<br/>
	* }<br/>
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/	
	RESIZE_WIDGET: "com.ibm.mashups.builder.resizeWidget",

	/**
	* <b>Description</b>: WIDGET_RESIZED iEvent should be broadcast after the size of user iWidget instance is changed. <br/>
	* User iWidget can define WIDGET_RESIZED iEvent as handled iEvent to do some follow up work if necessary. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.widgetResized"<br/>
	* <b>Payload</b>: size of user iWidget instance. Must not be <code>null</code>.<br/>
	* {<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;widgetId: id of user iWidget instance,<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;width: new width of user iWidget instance,<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;height: new height of user iWidget instance<br/>
	* }<br/>
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/	
	WIDGET_RESIZED: "com.ibm.mashups.builder.widgetResized",
	
	/**
	* <b>Description</b>: WIDGET_MOVED iEvent should be broadcast after a user iWidget instance is moved in the layout. <br/>
	* User iWidget can define WIDGET_MOVED iEvent as handled iEvent to do some follow up work if necessary. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.widgetMoved"<br/>
	* <b>Payload</b>: user iWidget instance id. Must not be <code>null</code>. <br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/	
	WIDGET_MOVED: "com.ibm.mashups.builder.widgetMoved",
	
	/**
	* <b>Description</b>: WIDGET_DELETED iEvent should be broadcast after a user iWidget instance has been deleted from the layout. <br/>
	* User iWidget can define WIDGET_DELETED iEvent as handled iEvent to do some follow up work if necessary. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.widgetDeleted"<br/>
	* <b>Payload</b>: user iWidget instance id. Must not be <code>null</code>. <br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/	
	WIDGET_DELETED: "com.ibm.mashups.builder.widgetDeleted",

	/**
	* <b>Description</b>: System iWidget should broadcast EMBED_PAGE iEvent to embed a mashup page. <br/>
	* System iWidget (e.g EmbedPage) can define EMBED_PAGE iEvent as handled iEvent to accept to create the embedding UI for the page. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.embedPage"<br/>
	* <b>Payload</b>: page id. Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	EMBED_PAGE: "com.ibm.mashups.builder.embedPage",

	/**
	* <b>Description</b>: System iWidget broadcasts EMBED_WIDGET iEvent to embed a user iWidget. <br/>
	* System iWidget (e.g EmbedWidget) can define EMBED_WIDGET iEvent as handled iEvent to create the embedding UI for the user iWidget. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.embedWidget"<br/>
	* <b>Payload</b>: user iWidget definition id. Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	EMBED_WIDGET: "com.ibm.mashups.builder.embedWidget",
	
	/**
	* <b>Description</b>: System iWidget broadcasts LOAD_PAGESOURCE iEvent to load the page source. <br/>
	* System iWidget (e.g PageSource) can define LOAD_PAGESOURCE iEvent as handled iEvent to load the mashup page user iWidget markups into its source container. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.loadPageSource"<br/>
	* <b>Payload</b>: N/A<br/> 
	* @type String
	*/
	LOAD_PAGESOURCE: "com.ibm.mashups.builder.loadPageSource",

	/**
	* <b>Description</b>: System iWidget broadcasts SAVE_PAGE iEvent to save page source. <br/>
	* System iWidget (e.g PageSource) must define SAVE_PAGE iEvent as handled iEvent to persist the user iWidget markups in current page.<br/>
	* System iWidget developer can call enabler Navigation Model API to persist current page. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.savePage"<br/>
	* <b>Payload</b>: page id. Must not be <code>null</code>. <br/> 
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	SAVE_PAGE: "com.ibm.mashups.builder.savePage",

	/**
	* <b>Description</b>: System iWidget broadcasts EDIT_PAGE iEvent to load page property. <br/>
	* System iWidget (e.g PagePropertyEditor) can define EDIT_PAGE iEvent as handled iEvent to load the page properties, for example, page theme and page title. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.editPage"<br/>
	* <b>Payload</b>: page id. Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	EDIT_PAGE: "com.ibm.mashups.builder.editPage",

	/**
	* <b>Description</b>: System iWidget broadcasts UPDATE_PAGE_TITLE iEvent to update page title. <br/>
	* System iWidget (e.g PageTitle) can define UPDATE_PAGE_TITLE iEvent as handled iEvent to update the title in its displaying area. <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.updatePageTitle"<br/>
	* <b>Payload</b>: page title. Must not be <code>null</code>.<br/>
	* {<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;title: page title,<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;locale(optional): locale of the title<br/>
	* }<br/>
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/
	UPDATE_PAGE_TITLE: "com.ibm.mashups.builder.updatePageTitle",
	
	/**
	* <b>Description</b>: System iWidgets broadcasts HIGHLIGHT_WIDGET iEvent to make Layout iWidget highlight a specific user iWidget.<br/>
	* Layout iWidget should define HIGHLIGHT_WIDGET iEvent as handled iEvent in case it wants to highlight a user iWidget instance.  <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.highlightWidget"<br/>
	* <b>Payload</b>: id of user iWidget instance. Must not be <code>null</code>. <br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	HIGHLIGHT_WIDGET: "com.ibm.mashups.builder.highlightWidget",
	
	/**
	* <b>Description</b>: System iWidgets broadcasts REMOVE_WIDGET_HIGHTLIGHT iEvent to make Layout iWidget remove highlight of a specific user iWidget.<br/>
	* Layout iWidget should define REMOVE_WIDGET_HIGHTLIGHT iEvent as handled iEvent in case it wants to remove the highlight on a user iWidget instance.  <br/>
    * <b>Value</b>: "com.ibm.mashups.builder.removeWidgetHighlight"<br/>
	* <b>Payload</b>: id of user iWidget instance. Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	REMOVE_WIDGET_HIGHTLIGHT: "com.ibm.mashups.builder.removeWidgetHighlight",
	
	/**
	* <b>Description</b>: System iWidget broadcasts ADD_NEW_WIRE iEvent to inform layout iWidget to add a new wire. <br/>
	* Layout iWidget must define ADD_NEW_WIRE iEvent as handled iEvent, update the iWidget markup to add the new wire.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.addNewWire"<br/>
	* <b>Payload</b>: context information used to create new wire. Must not be <code>null</code>.<br/>
	* { <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;sourceWidgetId: id of the wire's source iWidget<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;sourceEvent: published event name of the source iWidget<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;targetWidgetId: id of the wire's target iWidget<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;targetEvent: handled event name of the target iWidget<br/>
	* } <br/>
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/	
	ADD_NEW_WIRE: "com.ibm.mashups.builder.addNewWire",
	
	/**
	* <b>Description</b>: System iWidget broadcasts REMOVE_NEW_WIRE iEvent to inform layout iWidget to remove an existing wire.<br/>
	* Layout iWidget must define REMOVE_NEW_WIRE iEvent as handled iEvent, update the iWidget markup to remove the wire.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.removeWire"<br/>
	* <b>Payload</b>: context information used to remove the wire. Must not be <code>null</code>.<br/>
	* { <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;sourceWidgetId: id of the wire's source iWidget<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;sourceEvent: published event name of the source iWidget<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;targetWidgetId: id of the wire's target iWidget<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;targetEvent: handled event name of the target iWidget<br/>
	* }<br/>
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/	
	REMOVE_WIRE: "com.ibm.mashups.builder.removeWire",
	
	/**
	* <b>Description</b>: CREATE_AUTO_WIRE iEvent can be broadcast to add auto wire between the specified widget with other iWidget instances on the page. <br/>
	* Auto wire: Auto wire happens when<br/>
	* 1. Auto-wiring is enabled for the page;<br/>
	* 2. Specified iWidget is auto-wiring enabled;<br/>
	* 3. The wired-to iWidget is auto-wiring enabled;<br/>
	* 4. Wire the specified iWidget's iEvent to other iWidget's iEvent when the two iEvent have exactly same event name and payload type.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.createAutoWire"<br/>
	* <b>Payload</b>: iWidget instance id. Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	CREATE_AUTO_WIRE: "com.ibm.mashups.builder.createAutoWire",
	
	/**
	* <b>Description</b>: AUTO_WIRE_CREATED iEvent will be broadcast to after a auto wire is added between two iWidget instances. <br/>
	* Auto wire: refer to the iEvent CREATE_AUTO_WIRE.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.autoWireCreated"<br/>
	* <b>Payload</b>: Created auto-wires information. Must not be <code>null</code>.<br/>
	* { <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;widgetId: instance id of the new added iWidget<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;publishTo: Array, contains all instance id of the widgets which the new added iWidget is wired with as source iWidget<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;receiveFrom: Array, contains all instance id of the widgets which the new added iWidget is wired with as target iWidget<br/>
	* }<br/>
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/
	AUTO_WIRE_CREATED: "com.ibm.mashups.builder.autoWireCreated",
	
	/**
	* <b>Description</b>: CREATE_QUICK_WIRE iEvent can be broadcast to add a wire between two iWidget instances automatically. <br/>
	* Quick wire: Check the two iWidget's first iEvent, if they can be wired together(payload type is compatible), wire them. 
	* QUICK_WIRE_CREATED iEvent must be broadcast after the wire is created<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.createQuickWire"<br/>
	* <b>Payload</b>: context information used to add the wire. Must not be <code>null</code>.<br/>
	* { <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;sourceWidgetId: id of the wire's source iWidget<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;targetWidgetId: id of the wire's target iWidget<br/>
	* }<br/>
	* 
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/
	CREATE_QUICK_WIRE: "com.ibm.mashups.builder.createquickWire",
	
	/**
	* <b>Description</b>: QUICK_WIRE_CREATED iEvent will be broadcast to after a quick wire is added between two iWidget instances. <br/>
	* Quick wire: refer to the iEvent CREATE_QUICK_WIRE.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.quickWireCreated"<br/>
	* <b>Payload</b>: context information of the added wire. Must not be <code>null</code>.<br/>
	* { <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;sourceWidgetId: id of the wire's source iWidget<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;targetWidgetId: id of the wire's target iWidget<br/>
	* }<br/>
	* 
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/
	QUICK_WIRE_CREATED: "com.ibm.mashups.builder.quickWireCreated",
	
	/**
	* <b>Description</b>: EDIT_WIRE iEvent can be broadcast to launch the wiring interface of a iWidget instance.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.editWire"<br/>
	* <b>Payload</b>: id of the user iWidget instance. Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/	
	EDIT_WIRE: "com.ibm.mashups.builder.editWire",
	
	/**
	* <b>Description</b>: DISPLAY_WIRE_GRAPH iEvent can be broadcast to display the wire graph of a iWidget instance.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.displayWireGraph"<br/>
	* <b>Payload</b>: id of the user iWidget instance. Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/	
	DISPLAY_WIRE_GRAPH: "com.ibm.mashups.builder.displayWireGraph",
	
	/**
	* <b>Description</b>: System iWidget broadcasts SHARE_PAGE iEvent to notice other iWidgets to start the sharing work.
	* System iWidget (e.g SharePage iWidget) which implements the sharing function should define this iEvent as handled iEvent<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.sharePage"<br/>
	* <b>Payload</b>: id of the shared page. Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	SHARE_PAGE: "com.ibm.mashups.builder.sharePage",

	/**
	* <b>Description</b>: System iWidget broadcasts SHARE_SPACE iEvent to notice other iWidgets to start the sharing work.
	* System iWidget (e.g SharePage iWidget) which implements the sharing function should define this iEvent as handled iEvent<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.shareSpace"<br/>
	* <b>Payload</b>: id of the shared space. Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	SHARE_SPACE: "com.ibm.mashups.builder.shareSpace",

	/**
	* <b>Description</b>: System iWidget broadcasts PUBLISH_PAGE iEvent to notice other iWidgets to start the publishing page work.
	* System iWidget (e.g PublishPage iWidget) which implements the publish function must define this iEvent as handled iEvent<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.publishPage"<br/>
	* <b>Payload</b>: id of the published page. Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	PUBLISH_PAGE: "com.ibm.mashups.builder.publishPage",
	
	/**
	* <b>Description</b>: System iWidget broadcasts VIEW_MORE_PAGE iEvent to notice other iWidgets it wants to accept pages from other users.
	* System iWidget (e.g viewMorePage iWidget) which implements that function should define this iEvent as handled iEvent.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.viewMorePage"<br/>
	* <b>Payload</b>: N/A<br/>
	* @type String
	*/
	VIEW_MORE_PAGE: "com.ibm.mashups.builder.viewMorePage",
	
	/**
	* <b>Description</b>: DISPLAY_IN_MASHUP_DIALOG iEvent can be broadcast to display system iWidget in a configurable mashup dialog.<br/>
	* The system iWidget need to be linked to a type id in "dialogDisplayer" iWidget's itemset. <br/>
	* Current supported type id: 
	* <ul>
	* 	<li>sharePage</li>
	*   <li>publishPage</li>
	*   <li>viewMorePage</li>
	*   <li>renameWidget</li>
	*   <li>wireInteface</li>
	*   <li>pageSource</li>
	*   <li>embedPage</li>
	*   <li>embedWidget</li>
	*   <li>editPage</li>
	*   <li>hiddenWidgetCorral</li>
	*   <li>editSpace</li>
	*   <li>spaceManager</li>
	*   <li>createPage</li>
	*   <li>createSpace</li>
	*   <li>copySpace</li>
	*   <li>changeSpaceOwnerWidget</li>
	*   
	* </ul>
    * <b>Value</b>: "com.ibm.mashups.builder.displayInMashupDialog"<br/>
	* <b>Payload</b>: <br/>
	* {<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;widgetTypeId: type id. Must not be <code>null</code>.<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;eventName(optional): name of the iEvent of target iWidget which will be invoked <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;eventPayloa(optional): payload which will be passed to the handled iEvent<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;dialogParam(optional): parameters used to configure the dialog<br/>
	* }<br/>
	* Example: show wire interface iWidget in a dialog:<br/>
	* We have below definition in theme.html. <br/>
	* &lt;div class="mm_iWidget" id="dialogDisplayer"> <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&lt;a class="mm_Definition" href="/mum/widget-catalog/displayHelper.xml">&lt;/a> <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&lt;span class="mm_ItemSet" title="attributes" style="visibility: hidden; display: none;"> <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;a class="mm_Item" style="visibility: hidden; display: none;" href="#wireInterface">wireInterface&lt;/a> <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;....... <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;&lt;/span><br/>
	* &lt;/div> <br/>
	* Call following javascript code to display wire iWidget in dialog and invoke <code>EDIT_WIRE<code> iEvent.<br/>
	* var dialogParam = <br/>
	* { <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;nodeWidth: "486px", <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;...... <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;displayCloseButton: false, <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;hasRightBottomResizer: false <br/>
	* } <br/>
	* var payload = <br/>
	* {<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;widgetTypeId: "pageSoure", <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;eventName:  "com.ibm.mashups.builder.loadPageSource", <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;eventPayload:  ..., <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;dialogParam:  dialogParam <br/>
	* }<br/>
	* var eventService = com.ibm.mashups.services.ServiceManager.getService("eventService"); <br/>
	* eventService.broadcastEvent("com.ibm.mashups.builder.displayInMashupDialog", payload);<br/>
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/
	DISPLAY_IN_MASHUP_DIALOG: "com.ibm.mashups.builder.displayInMashupDialog",
	
	/**
	* <b>Description</b>: CLOSE_MASHUP_DIALOG iEvent can be broadcast with a system iWidget id. 
	* If that system iWidget is displayed in a mashup dialog, then the dialog will be closed.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.closeMashupDialog"<br/>
	* <b>Payload</b>: id of the system iWidget instance which is displayed in the mashup style dialog. Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
	CLOSE_MASHUP_DIALOG: "com.ibm.mashups.builder.closeMashupDialog",
		
	/**
	* <b>Description</b>: STATE_CHANGED iEvent can be broadcast to change the widget's state, restore/max/min the current widget by layout widget.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.stateChanged"<br/>
	* <b>Payload</b>: id of the iWidget instance, and widget's state. Must not be <code>null</code>.<br/>
	* {<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;widgetId: iwidgetId, <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;state:  "max"/"min"/"normal" <br/>
	* }<br/>
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/
	STATE_CHANGED: "com.ibm.mashups.builder.stateChanged",
	
	/**
	* <b>Description</b>: Broadcast OPEN_PAGE iEvent to open a given page and optionally broadcast widget events to it.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.openPage"<br/>
	* <b>Payload</b>: This event can accept two kinds of payload, id of the page to be opened or a json object in the follow format. Must not be <code>null</code>.<br/>
	* { <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;pid: id of the page to be opened<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;eventsArray: an array of events to be broadcast to the widgets on the page<br/>
	* }<br/>
	* The "eventsArray" should include one or more event objects in this format <br/>
	* { <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;targetEvent: targetEvent<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;payload: payload<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;payloadType: payloadType<br/>
	* }<br/>
	* <b>Payload type</b>: text or json<br/>
	* @type String
	*/
	OPEN_PAGE:"com.ibm.mashups.builder.openPage",
	
	/**
	* <b>Description</b>: Broadcast CLOSE_PAGE iEvent to close a given page that was opened using the OPEN_PAGE iEvent. If CLOSE_PAGE is broadcast multiple times, 
	* the actual close page action only happen once.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.closePage"<br/>
	* <b>Payload</b>: id of the page to be closed. <br/>
	* <b>Payload type</b>: text<br/> 
	* @type String
	*/
    CLOSE_PAGE:"com.ibm.mashups.builder.closePage",
	
	/**
	* <b>Description</b>: System iWidget broadcasts CREATE_PAGE iEvent to initiate Create Page widget, which allows to create a new page.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.createPage"<br/>
	* <b>Payload</b>: <code>spaceId</code> refers to under which space new page will be created, <code>parentPageId</code> refers to new page's parent page. Must not be <code>null</code>.<br/>
	* {<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;spaceId: space id, <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;parentPageId: parent page id<br/>
	* }<br/>	
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/
    CREATE_PAGE:"com.ibm.mashups.builder.createPage",

	/**
	* <b>Description</b>: System iWidget broadcasts CREATE_SPACE iEvent to initiate Create Space widget, which allows to create a new space.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.createSpace"<br/>
	* <b>Payload</b>: <code>createSpaceFromSpaceManager</code> should be set as <code>true</code> if CREATE_SPACE is broadcast inside Space Manager, otherwise <code>false</code>. Must not be <code>null</code>.<br/>
	* {<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;createSpaceFromSpaceManager: a boolean value to indicate whether the create space action is launched from space manager<br/>
	* }<br/>
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/
    CREATE_SPACE:"com.ibm.mashups.builder.createSpace",

	/**
	* <b>Description</b>: System iWidget broadcasts EDIT_SPACE iEvent to initiate Edit Space widget, which allows to edit existing space.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.editSpace"<br/>
	* <b>Payload</b>: id of space to be edited. Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/>
	* @type String	
	*/
    EDIT_SPACE:"com.ibm.mashups.builder.editSpace",

	/**
	* <b>Description</b>: System iWidget broadcasts COPY_SPACE iEvent to initiate Copy Space widget, which allows to clone given space.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.copySpace"<br/>
	* <b>Payload</b>: <code>spaceId</code> is the id of space to be cloned, and <code>copySpaceInSpaceManager</code> should be set as <code>true</code> if COPY_SPACE is broadcast inside Space Manager, otherwise <code>false</code>. Must not be <code>null</code>.<br/>
	* {<br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;spaceId: id of the space, <br/>
	* &nbsp;&nbsp;&nbsp;&nbsp;copySpaceInSpaceManager: a boolean value to indicate whether the copy space action is launched from space manager <br/>
	* }<br/>
	* <b>Payload type</b>: json<br/> 
	* @type String
	*/
    COPY_SPACE:"com.ibm.mashups.builder.copySpace",		
	
	/**
	* <b>Description</b>: System iWidget broadcasts CHANGE_SPACE_OWNER iEvent to initiate Change Space Owner widget, which allows to change space owner. After space owner is changed, two dojo events will be published:<br/>
	* <code>dojo.publish("com.ibm.mm.builder.spaceEditorWidget.OwnerName", [username]);</code><br/>
	* <code>dojo.publish("com.ibm.mm.builder.spaceEditorWidget.OwnerId", [userid]);</code><br/>
	* These two dojo events should be subscribed to get new owner id and owner name.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.changeSpaceOwner"<br/>
	* <b>Payload</b>: original space owner id, Must not be <code>null</code>.<br/>
	* <b>Payload type</b>: text<br/>
	* @type String
	*/
    CHANGE_SPACE_OWNER:"com.ibm.mashups.builder.changeSpaceOwner",
	
	/**
	* <b>Description</b>: System iWidget broadcasts MANAGE_SPACES iEvent to initiate Space Manager widget, which allows to manage spaces.<br/>
    * <b>Value</b>: "com.ibm.mashups.builder.manageSpaces"<br/>
	* <b>Payload</b>: N/A<br/>
	* @type String
	*/
    MANAGE_SPACES:"com.ibm.mashups.builder.manageSpaces"	
});

com.ibm.mashups.builder.iEvents = new com.ibm.mashups.builder.IEvent();

}

if(!dojo._hasResource["com.ibm.mm.builder.utils.PageWidthChanger"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.builder.utils.PageWidthChanger"] = true;
dojo.provide("com.ibm.mm.builder.utils.PageWidthChanger");

dojo.declare("com.ibm.mm.builder.utils.PageWidthChanger", null, {

	constructor : function() {
		this._changer = new com.ibm.mm.builder.utils.PageWidthChanger.Changer();
		this._pageWidthMode = {};
		if (dojo.isIE == 6) {
			this._resizeFn = dojo.hitch(this, this._resizeHandler);
			window.attachEvent("onresize", this._resizeFn);
		}
	},	
	
	setWidthRangeProvider : function(provider) {
		this._wrp = provider;
	},
	
	setWidgetListProvider : function(provider) {
	    this._wlp = provider;
	},
	
	setPageWidthModeProvider : function(provider) {
		this._pwmp = provider;
	},
	
	init : function() {
		this._pageWidthMode = this.getPageWidthMode();
	},
	
	addPageWidthModeChangedListener : function(listener) {
		this._pwmcListener = listener;
	},
	
	getPageWidthRange : function() {
		return {
			min : this._wrp.getMinimum(),
			max : this._wrp.getMaximum()
		};
	},
	
	_getBodyWidthToFitViewport : function() {
		var d = this._getViewportDimension();
		var delta = dojo.marginBox(dojo.body()).w - dojo.contentBox(dojo.body()).w;
		return d.width - delta - 23;
	},
	
	changeWidthToFitViewport : function(minWidth) {
		var d = this._getViewportDimension();
		var delta = dojo.marginBox(dojo.body()).w - dojo.contentBox(dojo.body()).w;
		/*var fluidWidth = d.width - delta - 60;
		dojo.style(dojo.query("html")[0], {
			"width" : (d.width - delta) + "px"
		});
		this._changer.setWidth(fluidWidth);*/
		this.useFluidLayout(minWidth);
		//this._changer.setOverflow();
	},
	
	useFluidLayout : function(minWidth) {
		this._changer.restoreToFluidLayout();
		if (minWidth) {
			var min = Math.max(parseInt(minWidth,10), this._wrp.getMinimum());
			this.setMinimumWidth(min);
		}
	},
	
	_getViewportDimension : function() {
	  var w = window.innerWidth || (document.documentElement && document.documentElement.clientWidth) || document.getElementsByTagName('body')[0].clientWidth;
	  var h = window.innerHeight || (document.documentElement && document.documentElement.clientHeight) || document.getElementsByTagName('body')[0].clientHeight;
	  return {width : w, height : h};
	},
	
	changeWidth : function(width) {
	    var minWidth = this._pageWidthMode.mode == "dynamic" ? this._getBodyWidthToFitViewport() : 0;
		var viewportWidth = this._getBodyWidthToFitViewport();
		//var min = Math.max(this._wrp.getMinimum(), minWidth), max = this._wrp.getMaximum();
		var min = Math.min(this._wrp.getMinimum(), this._wrp.getMaximum()) , max = this._wrp.getMaximum();
		width = Math.min(Math.max(width, min), max);
		if (this._pageWidthMode.mode == "dynamic") {
			width = width < viewportWidth ? "auto" : width;
		}
		this._changer.changeTo(width, min, max);
	},
	
	changeWidthDelta : function(delta) {
	    var oldX = this._changer.getWidth();
		this.changeWidth(oldX + delta);
	},
	
	setMinimumWidth : function(width) {
		this._changer.setMinimum(width);
	},

	
	_normalizeWidgetId : function(widgetId) {
	   return  widgetId.indexOf(com.ibm.mm.builder.constant.skinIdPrefix) === 0 ? widgetId : com.ibm.mm.builder.constant.skinIdPrefix + widgetId;
	},
	
	_getMinimumWidthForWidgets : function(widgetIds) {
	  widgetIds = widgetIds || this._wlp.getWidgetIds() || [];
	  var rightPos = dojo.map(widgetIds, function(widgetId) {
	   var box = dojo.marginBox(dojo.byId(this._normalizeWidgetId(widgetId)));
			return box.l + box.w;
	  }, this);
	  var min = 0;
	  dojo.forEach(rightPos, function(value) {
		  min = Math.max(min, value);
	  });
	  return min;
	},
	
	_getWidgetMarginBox : function(widgetId) {
	    widgetId = this._normalizeWidgetId(widgetId);
	    return dojo.marginBox(dojo.byId(widgetId));
	},
	
	_isWidthInViewport : function(width) {
		var d = this._getViewportDimension();
		var delta = dojo.marginBox(dojo.body()).w - dojo.contentBox(dojo.body()).w;
		var fluidWidth = d.width - delta - 60;
		return width < fluidWidth;
	},
	
	getPageWidthMode : function() {
	    return (this._pwmp && this._pwmp.getPageWidthMode && this._pwmp.getPageWidthMode()) || {};
	},
	
	setPageWidthMode : function(pageWidthMode) {
		if (pageWidthMode.mode !== this._pageWidthMode.mode || 
			parseInt(pageWidthMode.width,10) !== parseInt(this._pageWidthMode.width,10)) {
			this._pageWidthMode = pageWidthMode;
			
			var payload = {
				"type" : "change",
				"summary" : {
				   "old" : dojo.clone(this._pageWidthMode),
				   "new" : dojo.clone(pageWidthMode)
				}
			};
			if (this._pwmcListener) {
				if (dojo.isFunction(this._pwmcListener)) {
				   this._pwmcListener(payload);
				}
				else if (dojo.isObject(this._pwmcListener)) {
					this._pwmcListener.fn.apply(this._pwmcListener.scope || dojo.global, [payload]);
				}
			}
		}
	},
	
	hasReactionToWidget : function() {
	   return this._pageWidthMode.mode == "dynamic";
	},
	
	_needsToAdjustWidth : function() {
		return this._pageWidthMode.mode == "dynamic";
	},
	
	_changeWidthAndScroll : function(width, left) {
		this.changeWidth(width);
		this._changer.scrollToView(left);
	},

	widgetToBeResized : function(widgetId, width, deltaW) {
	    if (!this.hasReactionToWidget()) {
			return;
		}
		//this._changer.cancelOverflow();
		/*if (deltaW < 0) {
			var box = this._getWidgetMarginBox(widgetId);
			if (box.l + width > this._changer.getWidth()) {
			  this.changeWidth(box.l + width);
			  this._changer.scrollToView(box.l);
			}
		}*/
		var box = this._getWidgetMarginBox(widgetId);
		this.changeWidth(box.l + width);
		this._changer.scrollToView(box.l);
	},
	
	widgetToBeMoved : function(widgetId, left, width) {
	   if (!this.hasReactionToWidget()) {
			return;
		}
		if (typeof left == "undefined" || left === null) {
			throw new Error("Please provide the left coordinate.");
		}
		if (typeof width == "undefined" || width === null) {
			if (widgetId) {
				var box = this._getWidgetMarginBox(widgetId);
				width = box.w;
			}
			else {
				throw new Error("Please provide the widget id or width.");
			}
		}
		//this._changer.cancelOverflow();
		
		this.changeWidth(left + width);
		this._changer.scrollToView(left);
	},
	
	
	widgetToBeRemoved : function() {
	   if (!this.hasReactionToWidget()) {
			return;
	   }
	   var widgetIds = this._wlp.getWidgetIds();
	   var min = this._getMinimumWidthForWidgets(widgetIds);
	   this.changeWidth(min);
	},
	
	_resizeHandler : function() {
		if (!this._needsToAdjustWidth()) {
			return;
		}
		window.detachEvent("onresize", this._resizeFn);
		this._clearWidthChangerTimeout();
		this._widthChangerTimeout = window.setTimeout(dojo.hitch(this, this._changeMinMaxWidth), 300);
		window.attachEvent("onresize", this._resizeFn);
	},
	
	_changeMinMaxWidth : function() {
		var min = this._wrp.getMinimum();
		var max = this._wrp.getMaximum();
		this._changer._changeMinMaxWidth(min, max);
		this._clearWidthChangerTimeout();
	},
	
	_clearWidthChangerTimeout : function() {
		if (this._widthChangerTimeout) {
			window.clearTimeout(this._widthChangerTimeout);
		}
	}
	
});


dojo.declare("com.ibm.mm.builder.utils.PageWidthChanger.Changer", null, {
	WIDTH_CHANGE_THROTTLE_VALUE : 50,
	
	constructor : function() {
		this._node = dojo.body();
		this._toChangeWidthTimer = null;
	},
	
	_getNode : function() {
		return this._node;
	},
	
	_changeMinMaxWidth : function(min, max) {
		var currentWidth = document.documentElement.clientWidth;
		var newWidth = currentWidth < min ? min : (currentWidth > max ? max : currentWidth);
		if (newWidth == min || newWidth == max) {
			this.setWidth(newWidth);
		}
		else {
			this.setWidth("auto");
		}
	},
	
	changeTo : function(/* Number */ width, min, max) {
		//this._clearChangeToTimer();
		//this._toChangeWidthTimer = window.setTimeout(dojo.hitch(this, this._doChangeTo, width, min, max),this.WIDTH_CHANGE_THROTTLE_VALUE);
		this._doChangeTo(width, min, max);
	},
	
	_clearChangeToTimer : function() {
		if (this._toChangeWidthTimer) {
			window.clearTimeout(this._toChangeWidthTimer);
		}
	},
	
	_normalizeNumericValues : function(value) {
		if (value || value === 0) {
			var v = /^\d+$/.test(value + "") ? (value + "px") : value;
			return v;
		}
	},
	
	_hackForShadowMenu : function() {
		dojo.query(".mashupsShadowNode").style("left", 0);
	},
	
	_doChangeTo : function(width, min, max) {
		var obj = {};
		var w = this._normalizeNumericValues(width);
		if (w) {
			obj.width = w;
		}
		var minW = this._normalizeNumericValues(min);
		if (minW) {
			obj.minWidth = minW;
		}
		var maxW = this._normalizeNumericValues(max);
		if (maxW) { 
			obj.maxWidth = maxW;
		}
		dojo.style(this._getNode(), obj);
		this._hackForShadowMenu();
	},
	
	setMinimum : function(width) {
	    this._doChangeTo(null, width);
	},
	
	setWidth : function(width) {
		this._doChangeTo(width);
	},
	
	getWidth : function() {
	  return dojo.style(this._getNode(), "width");
	},
	
	setOverflow : function() {
		dojo.style(dojo.query("html")[0], {
			"overflowX" : "hidden"
		});
		window.scrollTo(0, window.pageYOffset);
	},
	
	cancelOverflow : function() {
		dojo.style(dojo.query("html")[0], {
			"overflowX" : "auto"
		});
	},
	
	scrollToView : function(xPos) {
		//window.scrollTo(xPos, window.pageYOffset);
	},
	
	restoreToFluidLayout : function() {
		var maxWidth = dojo.isIE ? "100%" : "auto";
		this._doChangeTo("auto", "auto", "100%");
	}
});

if (!dojo.isObject("com.ibm.mm.builder.utils.pageWidthChanger")) {
	dojo.setObject("com.ibm.mm.builder.utils.pageWidthChanger", new com.ibm.mm.builder.utils.PageWidthChanger());
}

}

if(!dojo._hasResource["com.ibm.mm.builder.dnd.QuickWireTarget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.builder.dnd.QuickWireTarget"] = true;
/**
 * @author Yi Ming Huang(huangyim@cn.ibm.com)
 */
dojo.provide("com.ibm.mm.builder.dnd.QuickWireTarget");

/**
 * FreeformSource is the drop target of one column in free form
 */
dojo.declare("com.ibm.mm.builder.dnd.QuickWireTarget", dojo.dnd.Target, {
	
	//accept: ["iWidgetDefinition", "iWidgetInstance"],
	accept: ["lm"],
	
	handler: null,
	
	onDndDrop: function(source, nodes, copy, target){
		try {
			this.inherited(arguments);
		} catch (e) {
			//the source node will be null for unknown reason.
		}
	},
	/**
	onDropExternal: function(source, nodes, copy){
		var sid = nodes[0].getAttribute("dnddata");
		if(nodes[0].getAttribute("dndtype") === "iWidgetDefinition"){
			sid = this.layout.createWidget(nodes[0].getAttribute("dnddata"));
		}
		//hide before do autowire
		com.ibm.mm.builder.utils.iwidgetUtil.hideWidget(sid);
		var self = this;
		this.handler = com.ibm.mm.builder.utils.iwidgetUtil.invokeCallbackByWidgetInstanceId(sid, function(){
			var payload = {
				sourceWidgetId: sid,
				targetWidgetId: self.targetId,
				callBackFuncContext: self,
				callbackFunc: "moveEffect"
			};
			var es = com.ibm.mashups.services.ServiceManager.getService("eventService");
			es.broadcastEvent("com.ibm.mashups.builder.createQuickWire", payload);
			com.ibm.mm.builder.utils.iwidgetUtil.showWidget(sid);
			self.disconnectHandler();
		});
	},
	*/
	
	onDropExternal: function(source, nodes, copy) {
		var type = dojo.attr(nodes[0], "dndtype");
		if(this._acceptType(type)) {
			var sourceLcId = com.ibm.pb.layout.LayoutParser.extractID(nodes[0]);
			var targetLcId = this.node.title;
			var payload = {
				sourceWidgetId: sourceLcId,
				targetWidgetId: targetLcId
			};
			var es = com.ibm.mashups.services.ServiceManager.getService("eventService");
			es.broadcastEvent("com.ibm.mashups.builder.createQuickWire", payload);
		}
	},
	
	disconnectHandler: function(){
		if (this.handler) {
			com.ibm.mashups.services.ServiceManager.getService("eventService").unsubscribeEvent(this.handler, null);
		}
	},
	
	_acceptType: function(type) {
		if(this.accept[type])
			return true;
		return false;
	}
});

com.ibm.mm.builder.dnd.QuickWireTargetMap = {};

}

if(!dojo._hasResource["com.ibm.mm.builder.dnd.QuickWireHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.builder.dnd.QuickWireHelper"] = true;
dojo.provide("com.ibm.mm.builder.dnd.QuickWireHelper");



dojo.declare("com.ibm.mm.builder.dnd.QuickWireHelper", null, {
	
	quickWireHandlers: {},
	
	quickWireHandlerHeight: 34,
	
	createQuickWireTarget: function(nodeId) {
		var lcDiv = com.ibm.pb.layout.LayoutParser.findById(nodeId);
		var handler = dojo.query(".decoration-titlebar", lcDiv)[0];
		var coords = dojo.coords(handler);
		var div = dojo.create("div", {zIndex: 999, title: nodeId});
		
		var left = coords.x + (window.scrollX || document.documentElement.scrollLeft);
		var top = coords.y + (window.scrollY || document.documentElement.scrollTop);
		div.style.cssText = "position: absolute; "
						+ "height: " + coords.h + "px; "
						+ "width: " + coords.w + "px; "
						+ "left: " + left + "px; "
						+ "top: " + top + "px;"
		//div.style.backgroundColor = "blue";
		dojo.body().appendChild(div);
		var target = new com.ibm.mm.builder.dnd.QuickWireTarget(div);
		
		this.quickWireHandlers[nodeId] = target;
	},
	
	destroyQuickWireTarget: function(nodeId) {
		var target = this.quickWireHandlers[nodeId];
		if(target) {
			var div = target.node;
			dojo.body().removeChild(div);
			delete target;
			this.quickWireHandlers[nodeId] = null;
		}
	},
	
	destroyAllQuickWireTarget: function() {
		for(var id in this.quickWireHandlers) {
			this.destroyQuickWireTarget(id);
		}
	}
});

}

if(!dojo._hasResource["com.ibm.pb.dnd.layout.LayoutFreeformSource"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.dnd.layout.LayoutFreeformSource"] = true;
/**
 * @author Qing Guo
 */

 dojo.provide("com.ibm.pb.dnd.layout.LayoutFreeformSource");
 
 
 
 
 
 
 
 dojo.declare("com.ibm.pb.dnd.layout.LayoutFreeformSource", 
 	[com.ibm.dnd.TunedLayoutSource, com.ibm.mm.builder.dnd.QuickWireHelper, com.ibm.pb.logging.Loggable], {
 	mouseDown:false,
	topPos: null,
	height: 700,
	managerOverrides: {
		OFFSET_X: 1,
		OFFSET_Y: 1,
		makeAvatar: function(){
			return new com.ibm.pb.dnd.layout.LayoutControlAvatar(this);
		}
	},
	layoutParser: com.ibm.pb.layout.LayoutParser,
	constructor: function() {
		this.evtSvc = com.ibm.mashups.services.ServiceManager.getService("eventService");
		this.resizeWidgetEvt = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.resizeWidget", this, "__widgetResized");
		this.addNewWidget = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.addNewWidget", this, "_addNewWidget");
		var currentPageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		this.lm = ibmCfg.controllers.dnd.nm.getLayoutModel(currentPageId);
		if(this.node.style){
			this.height = dojo.style(this.node,"height");
			if(this.node.style.width){
				if (dojo.isIE) {
					dojo.style(this.node, "width", "100%");
				}
				else {
					this.node.style.removeProperty("width");
				}
			}
		}
		this.count = dojo.query(".component-control", this.node).length;
		//following codes are used for MoveWidget Dialog
		var cst = dojo.coords(this.node, true);
		var pageWidthChanger = com.ibm.mm.builder.utils.pageWidthChanger;
		pageWidthChanger.setWidthRangeProvider(this.createFixedWidthRangeProvider(cst.w));
		//end
   		this.onOverEvent();
	},
		
	onDrop: function(source, nodes, copy,draggedOut){	
		this.LOGGER.entering("onDrop",[source, nodes, copy]);
		this.inherited(arguments);
		var currentNode = nodes[0];
		
		//dojo.style(dojo.body(),{cursor:"default"});
		var sourceItem = source.getItem(currentNode.id);
		if (sourceItem && (sourceItem.type == ibmCfg.controllers.dnd.SHELF_TYPE)) {
			this.layoutContainer = dojo.dnd.manager().target.node;
		}else{
			if(source.anchor){
				this.layoutContainer = source.anchor.parentNode;
			}else{
				this.layoutContainer = dojo.dnd.manager().target.node;
			}
		}
		var _containerId = this.layoutParser.extractID(this.node);
		var _currentNodeId = this.layoutParser.extractID(currentNode);
		var lcCoords = dojo.coords(this.node, true);
		
		if (sourceItem && (sourceItem.type == ibmCfg.controllers.dnd.SHELF_TYPE)) {
			com.ibm.pb.dnd.layout.LayoutFreeformSource.addWidgetByDrop = true;
			var widget = dijit.byId(nodes[0].id);
			widget.apply(_containerId, null);
		} else {
			widgetNode = currentNode;
			var lctrl = this.lm.find(_currentNodeId).start();
			var _position = {
				x: Math.max(this._last_drop_X - lcCoords.x - this._last_layerX, 0),
				y: Math.max(this._last_drop_Y - lcCoords.y, 0)
			};
			if(draggedOut){//user's mouse has moved out of the layout.
				var avatarPosition = dojo.position(dojo.dnd.manager().avatar.node);
				_position = {
				x: Math.max(avatarPosition.x- dojo.dnd.manager().OFFSET_X- lcCoords.x, 0),
				y: Math.max(avatarPosition.y - dojo.dnd.manager().OFFSET_Y- lcCoords.y, 0)
				};
			}
			
			if(source != this){
				_position = {
				x: Math.max(this._last_drop_X - lcCoords.x, 0),
				y: Math.max(this._last_drop_Y - lcCoords.y, 0)
				};
				if (lctrl) {
					var styleTxt = lctrl.getMetaData("css-style-backup");
					if (styleTxt) 
						widgetNode.style.cssText = styleTxt;
				}
			}
			
			widgetNode.style.cssText += "; position: absolute; "
									+ "left: " + _position.x + "px; "
									+ "top: " + _position.y + "px;"
			if (lctrl) {
				lctrl.setMetaData("css-style", widgetNode.style.cssText);
			}
			this._saveWidgetStyles();//to update the zindex for all widgets.
			
			if(dojo.hasClass(source.node, "hiddenWidgetsContainer")){
				var containerDef = this.lm.find(_containerId);
				if (containerDef) {
					containerDef.setFinishedCallback(dojo.hitch(this, "_insert"), {
					layoutModel: this.lm,
					controls: { target: null, current: currentNode }});
					containerDef.start(false);
				}
				else {
					this.LOGGER.severe("onDrop", "Cannot find container in the layout model");
				}
			}			
		}
		
		if (this.updateContainer){
			var ctrol = this.lm.find(_containerId).start();
			this._saveContainerSize(ctrol);
			this.updateContainer = false;
		}
		this.evtSvc.broadcastEvent("com.ibm.mashups.builder.widgetMoved", _currentNodeId);
		com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(true);
		
		this.isDragging = false;
				
		this.LOGGER.exiting("onDrop",[source, nodes, copy]);
	},
	
	_addNewWidget: function(iEvent){
		var newPlaceHolderID = iEvent;
		//judget whether widget is added to freeform layout rather than hidden area
		var _nws = dojo.query(".id-"+newPlaceHolderID, this.node);
		if (!_nws || !_nws[0]){
			return;
		}
		var lcCoords = dojo.coords(this.node, true);
		var lc = com.ibm.pb.dnd.layout.LayoutFreeformSource.getWidgetLocation(this.count, lcCoords);
		var _position;
		if (com.ibm.pb.dnd.layout.LayoutFreeformSource.addWidgetByDrop) {
			_position = {
				x: Math.max(this._last_drop_X - lcCoords.x, 0),
				y: Math.max(this._last_drop_Y - lcCoords.y, 0),
				zIndex: this.count
			};
		}else{
			_position = {x:lc.x, y:lc.y, zIndex: lc.zIndex};
		}
		if(lcCoords.h < _position.y+300){
			this._saveContainerHeight(_position.y+300);		
		}
		com.ibm.pb.dnd.layout.LayoutFreeformSource.setWidgetPosition(newPlaceHolderID, _position);
		com.ibm.pb.dnd.layout.LayoutFreeformSource.setWidgetInitHeight(newPlaceHolderID);
		this.refreshCallback = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.layoutRefreshed", this, dojo.hitch(this, "_loadDefaultSize", newPlaceHolderID));
		
		com.ibm.pb.dnd.layout.LayoutFreeformSource.addWidgetByDrop = false;
		this.count++;
	},
	_saveContainerHeight:function(newHeight){
		dojo.style(this.node, "height", newHeight + "px");
		var _containerId =this.layoutParser.extractID(this.node);
		var ctrol = this.lm.find(_containerId).start();
		this._saveContainerSize(ctrol);	
	},
	
	_saveWidgetStyles:function(){
		dojo.query(".component-control", this.node).forEach(function(node, index, arr){
			var id =this.layoutParser.extractID(node);
			if(id){
				var lctrl = this.lm.find(id).start();
				if (lctrl) {
					lctrl.setMetaData("css-style", node.style.cssText);
				}
			}
		},this);
	},
	_syncZindex:function(ctrlnode){
		var currentZ = ctrlnode.style.zIndex;
		if (currentZ) {
			dojo.query(".component-control", this.node).forEach(function(node, index, arr){
				var tempZ = node.style.zIndex;
				if (tempZ && tempZ >= currentZ) {
					tempZ--;
					node.style.zIndex=tempZ;
				}
			});
		}
		ctrlnode.style.zIndex = this.count;
	},
	_loadDefaultSize: function(id){
		try{
			var newPlaceHolderDiv = this.layoutParser.findById(id);
			var widgetBody = dojo.query(".lotusWidgetBody", newPlaceHolderDiv)[0];
			var wb = dojo.query(".iw-iWidget",widgetBody)[0];
			if (wb && wb.id) {
				var wrapper = com.ibm.mashups.iwidget.model.Factory.getGlobalWidgetModel().find(wb.id);
				if (wrapper) {
					var iitems = wrapper.getIDescriptorItems();
					var defaultHeight = iitems.getItemValue("defaultHeight");
					var defaultWidth = iitems.getItemValue("defaultWidth");
					var payload = null;
					if (defaultHeight || defaultWidth) {
						payload = {};
						if (defaultHeight) {
							payload.newHeight = defaultHeight;
						}
						if (defaultWidth) {
							payload.newWidth = defaultWidth;
						}
					}
					if (payload) {
						this.evtSvc.publishEvent(com.ibm.mashups.iwidget.Constants.RESIZE_WIDGET + "." + wb.id, payload);
					}
				}
			}
			if(this.refreshCallback){
				this.evtSvc.unsubscribeEvent(this.refreshCallback);
				this.refreshCallback = null;
				this.newWidgetId = null;
			}
		}catch(er){
			this.LOGGER.severe("_loadDefaultSize", er.message);
		}
	},
	_insert: function(container, status, x){
		// summary: inserts a control to the new location
		// container: layout container into which to put the new control
		// status: HTTP status returned from request to get container
		// x: {layoutModel, controls {target, current dragging node}}
		this.LOGGER.entering("_insert",[container, status, x]);
		if(status != 200) {
			this.LOGGER.severe("_insert","Could not find container in the layout model.  Cannot move control.",[container, status, x]);
			return;
		}

		if(container instanceof com.ibm.mashups.enabler.layout.LayoutContainer){
			var current = x.layoutModel.find(this.layoutParser.extractID(x.controls.current)).start();
			
			if(!current){
				this.LOGGER.severe("_insert","Cannot find current dragging node in the layout model", [x.controls.current]);
			}
				
			var target = null;
			if (x.controls.target) { 
				target = x.layoutModel.find(this.layoutParser.extractID(x.controls.target)).start();
				if(!target){
					this.LOGGER.severe("_insert","Cannot find target node in the layout model", [x.controls.target]);
				}
			}
	
			if( x.layoutModel.confirmInsert(current,container,target) ){
				x.layoutModel.insert(current,container,target);
			}
			
			com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(true);
			
			if(!this.delaySave){
				ibmCfg.controllers.dnd.nm.commit().start();				
			}
			
		} else { 
			this.LOGGER.severe("_insert","Container is not an instance of com.ibm.mashups.enabler.layout.LayoutContainer",[container, status, x]); 
		}
		
		this.LOGGER.exiting("_insert",[container, status, x]);
	},
	onDndCancel: function(){
		if (dojo.dnd.manager().source !== this) { 
			return;
		}
		
		if (this.isDragging) {
			//mouse move out of the layout, ondrop will not be called by dnd manager, manuall call it.
			this.onDrop(this,this.getSelectedNodes(),false,true);
			
			this.inherited(arguments);			
			ibmCfg.controllers.dnd.ghostHeight = 0;
			this.isDragging = false;
		}
		
		if (ibmCfg.controllers.dnd.currentNode && !ibmCfg.controllers.dnd.currentNode._isCanceled) {
					
			if (!dojo.isIE) {
				dojo.style(ibmCfg.controllers.dnd.currentNode, {display: ""});
			}
			
			ibmCfg.controllers.dnd.currentNode._isCanceled = true;
		}
		
		try{
			if(dojo.dnd.manager().target && dojo.dnd.manager().target.declaredClass != this.declaredClass && dojo.hasClass(dojo.dnd.manager().source.node, "ibmFreeform")){
				var currentId = this.layoutParser.extractID(dojo.dnd.manager().nodes[0]);	
				this.evtSvc.broadcastEvent("com.ibm.mashups.builder.leaveFreeformContainer"+"."+currentId);
			}
		}catch(e){}
		
		try{
			if(dojo.dnd.manager().target && dojo.dnd.manager().target.declaredClass == this.declaredClass && !dojo.hasClass(dojo.dnd.manager().source.node, "ibmFreeform")){
				var currentId = this.layoutParser.extractID(dojo.dnd.manager().nodes[0]);
				this.evtSvc.broadcastEvent("com.ibm.mashups.builder.enterFreeformContainer"+"."+currentId);
			}
		}catch(e){}
		
		this.current = null;
		this.destroyAllQuickWireTarget();
	},
	destroy: function(){
		this.evtSvc.unsubscribeEvent(this.resizeWidgetEvt);
		this.evtSvc.unsubscribeEvent(this.addNewWidget);
		this.count = 0;
		this.onOutEvent();
		this.inherited(arguments);
	},
	onDndStart: function(source, nodes, copy){
		this.inherited(arguments);
		this.isDragging = true;
		this._syncZindex(nodes[0]);
		
		var currentNode = dojo.byId(nodes[0].id);
		// current node stored in controller for use in creator
		ibmCfg.controllers.dnd.currentNode = currentNode;
		ibmCfg.controllers.dnd.currentNode._isCanceled = false;
		
		var sourceItem = source.getItem(nodes[0].id);
		if(sourceItem && (sourceItem.type != ibmCfg.controllers.dnd.SHELF_TYPE)){
			if (!dojo.isIE) {
				dojo.style(currentNode, {display: "none"});
			}
		}
	},
	onOverEvent: function(){
		if(!this.onmousemoveEvent){
			this.onmousemoveEvent = dojo.connect(this.node, "onmousemove", this, "onMouseMove");
		}
		
		dojo.dnd.manager().overSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOver();
		}
		this.active = true;
	},
	
	onOutEvent: function(){
		if(this.onmousemoveEvent){
			dojo.disconnect(this.onmousemoveEvent);
			delete this.onmousemoveEvent;
		}
		
		dojo.dnd.manager().outSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOut();
		}
		this.active = false;
	},
	

	onMouseUp: function(e){
		try {
			this._last_drop_X = e.pageX;
			this._last_drop_Y = e.pageY;
			com.ibm.pb.dnd.layout.LayoutFreeformSource.superclass.onMouseUp.call(this, e);
		} catch (e) {
			this.LOGGER.severe("onMouseUp", e.message);
		}
	},
	
	onMouseDown: function(e){
		com.ibm.pb.dnd.layout.LayoutFreeformSource.superclass.onMouseDown.call(this, e);
		this._last_layerX =  dojo.isOpera ? e.offsetX: e.layerX;
		this._last_layerY =  dojo.isOpera ? e.offsetY: e.layerY;
	},
	
	onMouseMove: function(e){
			if(this.isDragging && this.targetState == "Disabled"){ return; }
			dojo.dnd.Source.superclass.onMouseMove.call(this, e);
			var m = dojo.dnd.manager();
			if(!this.isDragging){
				if(this.mouseDown && this.isSource &&
						(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay)){
					var nodes = this.getSelectedNodes();
					if(nodes.length){
						m.startDrag(this, nodes, this.copyState(dojo.isCopyKey(e), true));
					}
				}
			}
		
			//com.ibm.pb.dnd.layout.LayoutFreeformSource.superclass.onMouseMove.call(this, e); //don't call this for performance concern
			if (this.isDragging) {
				var m = dojo.dnd.manager();
				if (this === m.source && m.avatar && m.avatar.node && m.avatar.node.style.marginLeft === "0px" && (dojo.isOpera || e.layerX > 0)) 
				{
//					var ml = builderConfig.isBidi ? (parseInt(m.nodes[0].style.width, 10) - this._last_layerX) : this._last_layerX;
					var ml = this._last_layerX;
					dojo.style(m.avatar.node, {
						'marginLeft': -ml + "px",
						'marginTop' : "0px"
						//'marginTop': 34-this._last_layerY + "px",
					});
					//dojo.query(".dojoDndItem", m.avatar.node).style({	display: ""});
				}				
				
				if(!this.targetBox){
					this.targetBox = dojo.position(m.avatar.node, true);
				}
				this.topPos = e.pageY - this.targetBox.y;
				
				m.canDrop(true);
			}
	},
	
	_addItemClass: function(node, type){
		if(this.isDragging){
			if(type!="Anchor") {
				var _id = this.layoutParser.extractID(node);
				if(this.topPos > this.quickWireHandlerHeight) {
					this.destroyQuickWireTarget(_id);
				} else {
					if(this.quickWireHandlers[_id]) {
						// do nothing
					} else {
						this.createQuickWireTarget(_id);
					}
				}
			}
		}
	},
	
	_removeItemClass: function(node, type){
		if(this.isDragging){
		}
	},
	
	moving: function(e){
		try {
			var m = dojo.dnd.manager();
			var avatar = m.avatar;
			if(!avatar){
				return false;
			}
			
			dojo.dnd.autoScroll(e);
			
			var a = dojo.coords(avatar.node);
			var ml = parseInt(avatar.node.style.marginLeft, 10) || 0;
			var mt = parseInt(avatar.node.style.marginTop, 10) || 0;
			//avatar container coords
			var ac = {
				l: (e.pageX + m.OFFSET_X),
				t: (e.pageY + m.OFFSET_Y),
				w: a.w - ml,
				h: a.h
			};
			
			var cst = dojo.coords(this.node, true);
			//source constrained area
			var c = {
				l:cst.x + 1 - ml,
				r:cst.x + cst.w - ml - ac.w, 
				t:cst.y - mt,
				b:cst.y + cst.h - ac.h
			};
			//limit four direction.
			avatar.node.style.left = (ac.l < c.l ? c.l : c.r < ac.l ? c.r : ac.l) + "px";
			avatar.node.style.top = (ac.t < c.t ? c.t : c.b < ac.t ? c.b : ac.t) + "px";
			//limit only top-left direction.
			//var al = ac.l < c.l ? c.l : ac.l;
			//var at = ac.t < c.t ? c.t : ac.t;
			//dojo.marginBox(avatar.node, {l: al, t: at});
			//adjust the width
			com.ibm.mm.builder.utils.pageWidthChanger.widgetToBeMoved(null, ac.l, ac.w);
			//adjust the height
			if (e.pageY > c.b) {
				this.updateContainer = true;
			   var newHeight = (e.pageY + ac.h);
			   dojo.style(this.node, "height", newHeight + "px");
			}
		} catch (err) {
			this.LOGGER.severe("moving", err.message);
		}
	},

	_saveContainerSize: function(ctrl){
		var cheight = dojo.style(this.node,"height");
		var sstyle = "height:"+cheight+"px;";
		ctrl.setMetaData("css-style", sstyle);
	},
	
	_saveWidgetSize: function(widgetId, layoutControlId, control){
		var node = dojo.byId(widgetId).parentNode;
		var layoutCtl = this.lm.find(layoutControlId).start();
		var tp = control.style.overflow;
		if(tp){
			control.style.removeProperty("overflow");
		};
		layoutCtl.setMetaData("css-style", control.style.cssText);
		layoutCtl.setMetaData("wrapper-css-style", node.style.cssText);
		if(tp){
			control.style.overflow = tp;
		};
	},
	
	__widgetResized: function(iEvent){
		var widgetId = iEvent.widgetId;
		var newH = iEvent.newHeight + 50;
		var control = this.layoutParser.findById(widgetId);
		var top = 0;
		if (control.style) {
			var top = dojo.style(control, "top");
			if (!top) 
				top = 0;
		}
		
		if (top + newH > this.height) {
			this.updateContainer = true;
			var newheight = top + newH + 50;
			this.height = newheight;
			this.node.style.height = newheight + "px";
		}
		if (this.updateContainer) {
			var ctrol = this.lm.find(this.layoutParser.extractID(this.node)).start();
			this._saveContainerSize(ctrol);
			this.updateContainer = false;
		}
	},
	
	createFixedWidthRangeProvider : function(width) {
	   return {
	      getMinimum : function() {
		     return width;
		  },
		  getMaximum : function() {
		     return width;
		  }
	   };
	}
 });
 
com.ibm.pb.dnd.layout.LayoutFreeformSource.creator = function(item, hint){
	if(hint=="avatar"){
		var _node = dojo.doc.createElement("div");
		if(dojo.dnd.manager().source.current){
			// use the portlet markup as avatar
			_node.innerHTML = String(item);
			var elem = dojo.byId(dojo.dnd.manager().source.current.id);
			var isPlaceHolder = false;
			//if(!dojo.hasClass(elem, "ibmPlaceholder")){
			var _id = this.layoutParser.extractID(dojo.dnd.manager().source.current);
			var widget = dojo.byId(_id);
			if(!widget){
				widget = dojo.query(".iw-iWidget", elem)[0];
				isPlaceHolder = true;
			}
			var m = dojo.contentBox(widget.parentNode);
			var ava_widget = dojo.query("[id~="+_id+"]",_node)[0]; 
			if(!ava_widget && isPlaceHolder){
				ava_widget = dojo.query(".iw-iWidget", _node)[0];
			}
			if(dojo.isFF){
				dojo._setMarginBox(ava_widget.parentNode, 0, 0, m.w + 17, m.h);
			}else{
				dojo._setMarginBox(ava_widget.parentNode, 0, 0, m.w, m.h);
			}
			if(isPlaceHolder){
				var size = dojo.contentBox(elem);
				dojo.style(_node, { width: size.w+"px", height: size.h+"px", opacity: 0.5});
			}else{
				dojo.style(_node, { width: elem.style.width, height: elem.style.height, opacity: 0.5});			
			}				
		}
		return {node: _node, data: item, type: [ibmCfg.controllers.dnd.DND_TYPE]};
	} else {  // this works the same way as the _normalizedCreator/onDropExternal functions in dojo.dnd.Source when there is no given creator function
		var node;
		if(dojo.isOpera) {
			node = dojo.byId(ibmCfg.controllers.dnd.currentNode.id).parentNode;  // opera returned nothing using dojo.query below
		} else {
			node = dojo.query("[id~="+ibmCfg.controllers.dnd.currentNode.id+"]")[0];
		}
		if(node){
			var source = ibmCfg.controllers.dnd.dndSources[this.layoutParser.extractID(node.parentNode)];
			if(source){
				var t = source.getItem(node.id);
				source.delItem(node.id);
				source.selection = {};  // prevents node from being deleted in dojo.dnd.Selector.deleteSelectedNodes called by onDropExternal()
				return {node: node, data: t.data, type: t.type};
			}
		} 
		return {node:dojo.create("div"),data:item,type:ibmCfg.controllers.dnd.SHELF_TYPE}
	}
};
com.ibm.pb.dnd.layout.LayoutFreeformSource.withHandles = true;
com.ibm.pb.dnd.layout.LayoutFreeformSource.addWidgetByDrop = false;

com.ibm.mashups.services.ServiceManager.getService("eventService").subscribeEvent("com.ibm.mashups.builder.initWidgets", null, function(iEvent){
	try {
		var container = com.ibm.pb.layout.LayoutParser.findById(iEvent.containerId);
		var lcCoords = dojo.coords(container, true);
		for(var i = 0; i < iEvent.data.length; i++){
			var item = iEvent.data[i];
			var widgetId = item.widgetId;
			var idx = item.index;
			var lc = com.ibm.pb.dnd.layout.LayoutFreeformSource.getWidgetLocation(idx, lcCoords);
			com.ibm.pb.dnd.layout.LayoutFreeformSource.setWidgetPosition(widgetId, lc);
			com.ibm.pb.dnd.layout.LayoutFreeformSource.setWidgetInitHeight(widgetId);
		}
		com.ibm.mashups.enabler.navigation.Factory.getNavigationModel().commit().start();
		com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(false);
	}catch(e){
		this.LOGGER.severe("com.ibm.mashups.builder.initWidgets", e.message);
	}
});

com.ibm.pb.dnd.layout.LayoutFreeformSource.getWidgetLocation = function(index, coords){
	try{
		var maxWidth = coords.w;
		if(!index) index = 0;
		if (index * 30 < maxWidth-400) {
			return {
				x: 30 * index,
				y: 30 * index,
				zIndex: index+1
			}
		}else{
			return {
				x: (30 * index) % (maxWidth-400),
				y: 30 * index,
				zIndex: index+1
			}
		}
	}catch(e){
		this.LOGGER.severe("com.ibm.pb.dnd.layout.LayoutFreeformSource.getWidgetLocation", e.message);
	}
};
com.ibm.pb.dnd.layout.LayoutFreeformSource.setWidgetPosition = function(widgetId, position){
	try {
		var _style = "position:absolute;left:" + position.x + "px;top:" + position.y + "px;width:400px;z-index:" + position.zIndex + ";";
		var currentPageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		var lm = ibmCfg.controllers.dnd.nm.getLayoutModel(currentPageId);
		var ctl = lm.find(widgetId).start();
		ctl.setMetaData("css-style", _style);
		var control = com.ibm.pb.layout.LayoutParser.findById(widgetId);
		if (control) {
			control.style.position = "absolute";
			control.style.left = position.x + "px";
			control.style.top = position.y + "px";
			control.style.width = "400px";
			control.style.zIndex = position.zIndex;
		}
	}catch(e){
		this.LOGGER.severe("com.ibm.pb.dnd.layout.LayoutFreeformSource.setWidgetPosition", e.message);
	};
};
com.ibm.pb.dnd.layout.LayoutFreeformSource.setWidgetInitHeight = function(widgetId){
	var currentPageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
	var lm = ibmCfg.controllers.dnd.nm.getLayoutModel(currentPageId);
	var control = com.ibm.pb.layout.LayoutParser.findById(widgetId);
	var outheight = 300;
	if (control.style.height){
		outheight = parseInt(control.style.height);
	}else{
		control.style.height = outheight + "px";
	}
	if(control){
		var wb = dojo.query(".lotusWidgetBody", control)[0];
		var innerheight = outheight - 50;
		if(wb) wb.style.height = innerheight+"px";
		var layoutCtl = lm.find(widgetId).start();
		layoutCtl.setMetaData("widget-body-height", innerheight+"px");
		layoutCtl.setMetaData("css-style", control.style.cssText);
	}
};

}

if(!dojo._hasResource["com.ibm.pb.dnd.layout.LayoutMashupColumnSource"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.dnd.layout.LayoutMashupColumnSource"] = true;
dojo.provide("com.ibm.pb.dnd.layout.LayoutMashupColumnSource");




dojo.declare("com.ibm.pb.dnd.layout.LayoutMashupColumnSource", 
		[com.ibm.pb.dnd.layout.LayoutColumnSource, com.ibm.mm.builder.dnd.QuickWireHelper], {

	layoutParser: com.ibm.pb.layout.LayoutParser,

	onDndCancel: function() {
		this.inherited(arguments);
		this.destroyAllQuickWireTarget();
	},
	
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		// Customization: add topPos
		if(this.isDragging && this.targetState == "Disabled"){ return; }
		dojo.dnd.Source.superclass.onMouseMove.call(this, e);
		var m = dojo.dnd.manager();
		if(!this.isDragging){
			if(this.mouseDown && this.isSource &&
					(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay)){
				var nodes = this.getSelectedNodes();
				if(nodes.length){
					m.startDrag(this, nodes, this.copyState(dojo.isCopyKey(e), true));
				}
			}
		}
		if(this.isDragging){
			// calculate before/after
			var before = false;
			var topPos = -1;
			if(this.current){
				if(!this.targetBox || this.targetAnchor != this.current){
					this.targetBox = dojo.position(this.current, true);
				}
				if(this.horizontal){
					topPos = e.pageX - this.targetBox.x;
					before = topPos < (this.targetBox.w / 2);
				}else{
					topPos = e.pageY - this.targetBox.y;
					before = topPos < (this.targetBox.h / 2);
				}
			}
			if(this.current != this.targetAnchor || before != this.before){
				this._markTargetAnchor(before, topPos);
				m.canDrop(!this.current || m.source != this || !(this.current.id in this.selection));
			}
		}
	},
	
	_markTargetAnchor: function(before, topPos){
		// summary:
		//		assigns a class to the current target anchor based on "before" status
		// before: Boolean
		//		insert before, if true, after otherwise
		// Customization: add topPos
		if(this.current == this.targetAnchor && this.before == before){ return; }
		if(this.targetAnchor){
			this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
		this.targetAnchor = this.current;
		this.targetBox = null;
		this.before = before;
		if(this.targetAnchor){
			this._addItemClass(this.targetAnchor, this.before ? "Before" : "After", topPos);
		}
	},
	
	_addItemClass: function(node, type, topPos){
		// summary: adds a class with prefix "dojoDndItem" and create ghost
		// node: Node: a node
		// type: String: a variable suffix for a class name
		if(this.current == node && this.before == type){ return; }
		if(this.isDragging){
			var _id = this.layoutParser.extractID(node);
			if(topPos == undefined) {
				// we are dragging from the content shelf
				this.createGhost(node, type);
			} else if(topPos > this.quickWireHandlerHeight) {
				// we are not dragging on the handler
				this.createGhost(node, type);
				this.destroyQuickWireTarget(_id);
			} else {
				// we are dragging on the handler
				if(this.quickWireHandlers[_id]) {
					// do nothing
				} else {
					this.createQuickWireTarget(_id);
				}
			}
		}
	}
});

com.ibm.pb.dnd.layout.LayoutMashupColumnSource.creator = 
	dojo.hitch(com.ibm.pb.dnd.layout.LayoutMashupColumnSource, 
			com.ibm.pb.dnd.layout.LayoutColumnSource.creator);

com.ibm.pb.dnd.layout.LayoutMashupColumnSource.withHandles = true;

}

if(!dojo._hasResource["com.ibm.pb.dnd.layout.LayoutMashupRowSource"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.dnd.layout.LayoutMashupRowSource"] = true;
dojo.provide("com.ibm.pb.dnd.layout.LayoutMashupRowSource");




dojo.declare("com.ibm.pb.dnd.layout.LayoutMashupRowSource", 
		[com.ibm.pb.dnd.layout.LayoutRowSource, com.ibm.mm.builder.dnd.QuickWireHelper], {
	
	layoutParser: com.ibm.pb.layout.LayoutParser,
	
	onDndCancel: function() {
		this.inherited(arguments);
		this.destroyAllQuickWireTarget();
	},
	
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		// Customization: add topPos
		if(this.isDragging && this.targetState == "Disabled"){ return; }
		dojo.dnd.Source.superclass.onMouseMove.call(this, e);
		var m = dojo.dnd.manager();
		if(!this.isDragging){
			if(this.mouseDown && this.isSource &&
					(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay)){
				var nodes = this.getSelectedNodes();
				if(nodes.length){
					m.startDrag(this, nodes, this.copyState(dojo.isCopyKey(e), true));
				}
			}
		}
		if(this.isDragging){
			// calculate before/after
			var before = false;
			var topPos = -1;
			if(this.current){
				if(!this.targetBox || this.targetAnchor != this.current){
					this.targetBox = dojo.position(this.current, true);
				}
				if(this.horizontal){
					before = (e.pageX - this.targetBox.x) < (this.targetBox.w / 2);
					topPos = e.pageY - this.targetBox.y;
					
					// create QuickWireTarget when mouse over the handler
					if(this.targetAnchor) {
						var node = this.targetAnchor;
						var _id = this.layoutParser.extractID(node);
						if(topPos >= this.quickWireHandlerHeight) {
							this.destroyQuickWireTarget(_id);
						} else {
							if(this.quickWireHandlers[_id]) {
								// do nothing
							} else {
								this.createQuickWireTarget(_id);
							}
						}
					}
					// end of customization	
				}else{
					topPos = e.pageY - this.targetBox.y;
					before = topPos < (this.targetBox.h / 2);
				}
			}
			if(this.current != this.targetAnchor || before != this.before){
				this._markTargetAnchor(before);
				m.canDrop(!this.current || m.source != this || !(this.current.id in this.selection));
			}
		}
	}
	
});

com.ibm.pb.dnd.layout.LayoutMashupRowSource.creator = 
	dojo.hitch(com.ibm.pb.dnd.layout.LayoutMashupRowSource, 
		com.ibm.pb.dnd.layout.LayoutRowSource.creator);

com.ibm.pb.dnd.layout.LayoutMashupRowSource.withHandles = true;

}

if(!dojo._hasResource["com.ibm.pb.dnd.layout.LayoutMashupCustomRowSource"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.dnd.layout.LayoutMashupCustomRowSource"] = true;
dojo.provide("com.ibm.pb.dnd.layout.LayoutMashupCustomRowSource");



dojo.declare("com.ibm.pb.dnd.layout.LayoutMashupCustomRowSource", 
		com.ibm.pb.dnd.layout.LayoutMashupRowSource, {
	
	layoutParser: com.ibm.pb.layout.LayoutParser,
	clearDiv: null,
	constructor: function(){
		this.containerID = this.layoutParser.extractID(this.node);
		this.clearDivID = "clear-" + this.containerID;
		this.initClearDiv(this.clearDivID);
		this.node.appendChild(this.clearDiv);
		
//		this.moveWidget = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.widgetMoved", this, "_widgetMoved");
	},
	initClearDiv:function(id){
		this.clearDiv = dojo.query(".ibmClearRow", this.node)[0];
		if(!this.clearDiv){
			this.clearDiv = dojo.doc.createElement("div");
		}
		dojo.attr(this.clearDiv, {
			"id": id,
			"class": "ibmClearRow"
		});
	},
	removeClearDiv: function(){
		var temp = dojo.byId(this.clearDivID);
		if (temp) {
			this.node.removeChild(temp);
			this.clearDiv = null;
		}
		temp = null;
	},
	addClearDiv:function(){
		this.initClearDiv(this.clearDivID);
		this.node.appendChild(this.clearDiv);
	},
	updateClearDiv:function(){
		this.clearDiv = null;
		this.clearDiv = dojo.byId(this.clearDivID);
		if(!this.clearDiv){
			this.initClearDiv(this.clearDivID);
		}
		this.node.appendChild(this.clearDiv);
	},
	onDndCancel: function() {
		this.inherited(arguments);
		var clearDiv = dojo.query(".ibmClearRow", this.node)[0];
		if (!clearDiv) {
			this.addClearDiv();
		}
	},
	
	onDrop: function(source, nodes, copy){
		this.removeClearDiv();
		this.inherited(arguments);
	},
	createGhost: function(node, type){	
		// creates ghost on page
		if(type == "Before"){
			var container = node.parentNode;
			container.insertBefore(this.createGhostMarkup(), node);
		}else if (type == "After"){
			var container = node.parentNode;
			container.insertBefore(this.createGhostMarkup(), node.nextSibling);
		}
	},
	removeGhost: function(){
		// removes ghost from page
		var ghost = dojo.byId(this.ghostClass);
		if(ghost){
			var parent = ghost.parentNode;
			parent.removeChild(ghost);
		}
	},
	onOverEvent: function(){
		// create ghost in empty container
		this.inherited(arguments);
		this.active = true;
		if(this.isDragging && dojo.dnd.manager().target){
			if(dojo.query("."+this.ghostClass).length > 0 ){
				this.updateClearDiv();
			}
		}
	},

	destroy: function(){
		this.inherited(arguments);
		this.clearDiv = null;
		this.inherited(arguments);
	},
	
	_widgetMoved: function(iEvent){
		this.inherited(arguments);
		var widgetId = iEvent;
		var nodes = dojo.query(".id-"+widgetId ,this.node);
		if (nodes.length >= 1) {
			var last = this.node.lastChild;
			if(dojo.hasClass(last, "ibmClearRow")){
				//do nothing
			}else{
				this.updateClearDiv();
			}
		}
	}
});

com.ibm.pb.dnd.layout.LayoutMashupCustomRowSource.creator = 
	dojo.hitch(com.ibm.pb.dnd.layout.LayoutMashupCustomRowSource, 
		com.ibm.pb.dnd.layout.LayoutRowSource.creator);

com.ibm.pb.dnd.layout.LayoutMashupCustomRowSource.withHandles = true;

}

if(!dojo._hasResource["com.ibm.pb.dijit.WidgetResizer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.dijit.WidgetResizer"] = true;
dojo.provide("com.ibm.pb.dijit.WidgetResizer");




dojo.declare("com.ibm.pb.dijit.WidgetResizer", [dijit._Widget, dijit._Templated],{
	templateString: 
		'<div dojoAttachEvent="onmousedown: _beginSizing" class="lotusWidgetResizer">'+
			'<div></div>'+
		'</div>',
		
	widgetId: null,
	//component-control node of the widget
	control: null,
	// contentElm: widget content dom node id
	contentElm: "",	

	widgetBodyElmId: "",
	
	minWidth:20,
	
	minHeight:20,
	
	oldHeight:0,
	
	// layout-container
	parentNode:null,
	
	intLeftRTLOrig:0,
	
	intOriginalWidth:0,
	
	isRtl: builderConfig.isBidi,
	
	postCreate: function(){
		if(this.isRtl){
			dojo.addClass(this.domNode, "rtl");
			// #16766 we shall set the left/right inline instead of using css class
			// because in rtl, the right=0 will also take effect
			dojo.style(this.domNode, "left", 0);
		} else {
			dojo.style(this.domNode, "right", 0);
		}
		this.runtimeModel = com.ibm.mashups.builder.model.Factory.getRuntimeModel();
		this.evtSvc = com.ibm.mashups.services.ServiceManager.getService("eventService");
		this.bEvents = com.ibm.mm.builder.eventTopics;
		//com.ibm.mashups.iwidget.services.ServiceManager.getService("eventService");
		this.changePageModeEvt = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.changePageMode", this, "_onPageModeChanged");
		this.leaveFreeformEvt = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.leaveFreeformContainer"+"."+this.widgetId, this, "_onleaveFreeform");
		this.enterFreeformEvt = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.enterFreeformContainer"+"."+this.widgetId, this, "_onenterFreeform");
		this._checkMode();
	},
	_checkMode: function(){
		if (this.runtimeModel.getCurrentPage().getPageMode() == "edit" && dojo.hasClass(this.parentNode,"ibmFreeform")){
			this.domNode.style.display="block";
		}else{
			this.domNode.style.display="none";
		}
	},
	_onPageModeChanged: function(iEvent){
		if(iEvent == "edit" && dojo.hasClass(this.parentNode,"ibmFreeform")){
			this.domNode.style.display="block";
		}else{
			this.domNode.style.display="none";
		}
	},
	_onleaveFreeform: function(iEvent){
		this.domNode.style.display="none";
	},
	_onenterFreeform: function(iEvent){
		this.domNode.style.display="block";		
	},
	_beginSizing: function(/*Event*/ e){
		this.maxSize = this.createMaxSize();
		if (this._isSizing){ 
			return false; 
		}
		
		this.targetDomNode = this.control;
		this.intLeftRTLOrig = parseInt(this.targetDomNode.style.left,10);
		this.intOriginalWidth = parseInt(this.targetDomNode.style.width,10);
		
		if (!this.targetDomNode) {
			return;
		}
		this._isSizing = true;
		
		if (dojo.isIE){
			this.startPoint  = {'x':event.clientX, 'y':event.clientY};
		}else{
			this.startPoint  = {'x':e.clientX, 'y':e.clientY};
		}

		var mb = dojo._getContentBox(this.targetDomNode);
		var widgetbody = dojo.byId(this.widgetId).parentNode;
		var wb = dojo._getContentBox(widgetbody);
		this.startSize  = {'w':mb.w, 'h':mb.h};
		this.startWidgetSize = {'w':wb.w, 'h':wb.h};
		this.eventHandles = []; 
		this.eventHandles.push(dojo.connect(document, "onmousemove", this, "_changeSizing")); 
		this.eventHandles.push(dojo.connect(document, "onmouseup", this, "_endSizing")); 
		this.eventHandles.push(dojo.connect(document, "onmouseleave", this, "_endSizing")); 

		// Publish to the start sizing topic
		dojo.publish(this.bEvents.Resize_StartSizing, [this.widgetId]);
		e.preventDefault();	
	},

	_changeSizing: function(/*Event*/ e){
		// On IE, if you move the mouse above/to the left of the object being resized,
		// sometimes clientX/Y aren't set, apparently.  Just ignore the event.
		try{
			if (dojo.isIE){
				if(!event.clientX  || !event.clientY){
					return; 
				}				
			}else{
				if(!e.clientX  || !e.clientY){
					return; 
				}				
			}
		} catch (ex){
			// sometimes you get an exception accessing above fields...
			return;
		}

		var dx,dy;
		if (dojo.isIE){
			dx = this.startPoint.x - event.clientX;
			dy = this.startPoint.y - event.clientY;
		}else{
			dx = this.startPoint.x - e.clientX;
			dy = this.startPoint.y - e.clientY;
		}
		var newW;
		if(!this.isRtl){
			newW = this.startSize.w - dx;
		}else{
			newW = this.startSize.w + dx;	//for rtl
		}
		
		var newH = this.startSize.h - dy;
		var newWidgetH = this.startWidgetSize.h - dy;
		
		com.ibm.mm.builder.utils.pageWidthChanger.widgetToBeResized(this.control.id, newW, dx);
		// minimum size check
		var minWidthExceeded = newW < this.minWidth;
		if (minWidthExceeded) {
			newW = this.minWidth;
		}
		if (newH < this.minHeight) {
			newH = this.minHeight;
		}
		// maximum size check
		var maxWidthExceeded = newW > this.maxSize.w;
		if (maxWidthExceeded) {
			newW =this.maxSize.w;
		}
		if (newH > this.maxSize.h) {
			var _delta = this.startSize.h - this.startWidgetSize.h;
			newH = this.maxSize.h;
			newWidgetH = this.maxSize.h - _delta;
		}

		//In Right to Left, move the widget to the left by the same amount that the width is increased.
		var intLeft;
		if (this.isRtl) {
			if(minWidthExceeded){				
				intLeft = this.intLeftRTLOrig + (this.intOriginalWidth - this.minWidth);
			}else{				
				intLeft = maxWidthExceeded ? 0 : this.intLeftRTLOrig - dx;
			}
		}else{			
			intLeft = parseInt(this.targetDomNode.style.left,10);
		}
		
		var intTop = parseInt(this.targetDomNode.style.top,10);
		
		dojo._setBox(this.targetDomNode,intLeft,intTop,newW,newH);
		
		tempWidgetBody = dojo.byId(this.widgetId);
		if(!this.widgetBodyPadTop && tempWidgetBody){
			
			this.widgetBodyPadTop = com.ibm.mm.builder.utils.htmlUtil.getCurrentStyle(tempWidgetBody ,"paddingTop");
			
			if (this.widgetBodyPadTop) {
				this.widgetBodyPadTop = parseInt(this.widgetBodyPadTop,10);
			} else {
				this.widgetBodyPadTop = 0;
			}
		}

		var contentDiv = tempWidgetBody.parentNode;
		
		var hei = newWidgetH - this.widgetBodyPadTop;
		//hei + 10 for temp solve
		dojo._setMarginBox(contentDiv,0,0,newW,hei+10);
		
		if(this.skin){
			if(this.skin == "Thin"){
				var nht = hei - 20 ;
				tempWidgetBody.style.height = nht + "px";
			}else if(this.skin == "Standard"){
				var nhs = hei - 10;
				tempWidgetBody.style.height = nhs + "px";
			}
		}
		if (dojo.isSafari && builderConfig.isBidi) {
			contentDiv.style.width = (parseInt(this.targetDomNode.style.width,10) - 51) + "px";
		}
		this._updateContainer();
		// Publish to the change sizing topic
		var sizeDelta = {x: dx, y: dy};
		dojo.publish(this.bEvents.Resize_ChangeSizing, [sizeDelta]);
		e.preventDefault();
	},

	_endSizing: function(/*Event*/ e){
		this.runtimeModel.getCurrentPage().setDirty(true);
		while (this.eventHandles.length > 0) {
			dojo.disconnect(this.eventHandles.pop());
		}
		dojo.publish(this.widgetId + this.bEvents.Resize_EndSizing, [e]);
		
		payload = {
			widgetId: this.widgetId,
			layoutControlId: this.layoutControlId,
			width: parseInt(this.targetDomNode.style.width,10),
			height: parseInt(this.targetDomNode.style.height,10),
			action: "only_save"
		}
		this.evtSvc.broadcastEvent("com.ibm.mashups.builder.resizeWidget", payload);
		
		this._isSizing = false;
		this.widgetBodyPadTop = null;
		dojo.style(dojo.body(),{cursor:"default"});
	},
	
	_updateContainer: function(){
		if(this.parentNode){
			var lcoords = dojo.coords(this.parentNode);
			var widgetBox = dojo.coords(this.control.id);
			if(widgetBox.t + widgetBox.h > lcoords.h){
				this.parentNode.style.height = (widgetBox.t + widgetBox.h) + "px";
			}
		}
	},
	
	createMaxSize:function(){
		var curWidget = this.control;
		if(curWidget && curWidget.parentNode){
			var curLeft = parseInt(curWidget.style.left,10);
			var curWidth = parseInt(curWidget.style.width,10);
			var curTop = parseInt(curWidget.style.top,10);	
			var wholeWidth = dojo._getContentBox(curWidget.parentNode).w;
			var wholeHeight = dojo._getContentBox(curWidget.parentNode).h;
			
			var maxWidth = this.isRtl ? curLeft + curWidth : wholeWidth - curLeft;
			return {
				w: maxWidth,
				h: wholeHeight - curTop
			};
		}
		return null;
	},
	

	destroy : function(){
		this.widgetId = null;
		this.targetDomNode = null;
		this.evtSvc.unsubscribeEvent(this.changePageModeEvt);
		this.evtSvc.unsubscribeEvent(this.leaveFreeformEvt);
		this.evtSvc.unsubscribeEvent(this.enterFreeformEvt);
		this.inherited(arguments);
	},
	
	onKeyPress: function(evt){
		if (evt.keyCode == dojo.keys.ENTER) {
			this.show();
		}
	},
	
	show: function(){
		if (com.ibm.mm.builder.utils.pageUtil.getCurrentPageMode() != "edit") {
			return;
		}
		clearTimeout(this.showTimer);
		this.showTimer = setTimeout(dojo.hitch(this, function(){
			if (this.domNode) {
				this.domNode.style.display = "";
			}
		}), 100);
	},
	
	hide: function(){
		if (com.ibm.mm.builder.widgetMenu && com.ibm.mm.builder.widgetMenu.pMenu && com.ibm.mm.builder.widgetMenu.pMenu.isShowingNow) {
			return;
		}
		clearTimeout(this.hideTimer);
		clearTimeout(this.showTimer);
		this.hideTimer = setTimeout(dojo.hitch(this, function(){
			if (this.domNode) {
				this.domNode.style.display = "none";
			}
		}), 100);
	}
});

}

if(!dojo._hasResource["com.ibm.pb.layout.resize.WidgetResizer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.layout.resize.WidgetResizer"] = true;
// @author Laura
// This module is a handle that can be attached to a widget skin and used to drag the widge to a different size

dojo.provide("com.ibm.pb.layout.resize.WidgetResizer");




dojo.declare("com.ibm.pb.layout.resize.WidgetResizer", [dijit._Widget, dijit._Templated],{


	
	widgetId: null, 		// id of the widget
	controlId: "",			// the id of the given layout control
	containerId: "",		// the id of the container in which the control resides
	skinNode: null,			// a reference to the control's modifiable runtime skin node which corresponds to the given control
	alwaysShow: false,		// true if the handle should always show, false to show only in edit mode, PLEASE NOTE: you must provide a mechanism to save the resizing in view mode, it is only automatic in edit mode
	controlStyle: {},		// the current style on the skin object from the DOM in dojo.style format
	widgetStyle: {},		// the current style on the widget node from the DOM in dojo.style format
	point: {x:0,y:0},		// the {x,y} coordinates of the dragging handle point
	positionAttrs: {"width":null,"height":null,"top":null,"left":null,"position":"static"},	// list of style attribute names that have to do with position
	templateString: '<div>' +
						'<div dojoAttachPoint="handle" dojoAttachEvent="onmousedown:_beginSizing" class="ibmWidgetResizer lotusRight" style="display:none;"></div>' +
						'<div style="clear: both;"></div>' +
					'<div>',
	
	postCreate: function() {
		// summary: intialize variables and connect events
		this.evtSvc = com.ibm.mashups.services.ServiceManager.getService("eventService");
		this.widgetMovedHandler = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.widgetMoved", this, "_clearStyle");
		
		//This is used to avoid losing position when switching to freeform layout and then dnd a widget
		this.initWidgetHandler = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.initWidgets", this, "_initWidgetsInFreeformLayout");
		if (this.alwaysShow) {
			// if the handle should always be shown, make it visible now
			this._showHandle();
		} else {
			// if the handle should only be shown in page edit mode, attach to the changePageMode event to we know when to show it
			this.changeModeHandler = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.changePageMode", this, "_checkMode");
            if (this.forceShowHandle) {
                this._checkMode(com.ibm.mm.iwidget.Constants.mode_edit);
            }
            else {
                this._checkMode(com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getPageMode());
            }
		}
	},
	
	destroy: function() {
		// summary: destoys the nodes and connections associated with this dijit
		this.evtSvc.unsubscribeEvent(this.widgetMovedHandler, null);
		this.evtSvc.unsubscribeEvent(this.initWidgetHandler, null);
		this.evtSvc.unsubscribeEvent(this.changeModeHandler, null);
		this.widgetId = null;
		this.inherited(arguments);
	},
	
	/** GET/SET STYLE FROM/ON THE SERVER **/
	
	setStyle: function(/*Object*/ style) {
		// summary: called to save a new skin style to the server
		// style: a style object
		//		  if it exists, height is found at style.height and has a string format of "XXXXpx"
		//		  if it exists, width is found at style.width and has a string format of "XXXXpx"
		// a widget instance can be retrieved from the Widget Model API using the ID: this.widgetId
		style = style || {};
		this.skinNode.setStyle(style);
	},
	
	getStyle: function() {
		// returns the style object for the current node from wherever it is stored server-side
		return this.skinNode.getStyle();
	},
	
	_setWidgetStyle: function(/*String*/ style) {
		// summary: along with _setWidgetStyleContinue() sets the wrapper-css-style metadata on the control that will get applied to its widget DOM node
		// style: a string that represents the inline style for the widget DOM node
		var navModel = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel();
		var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		var layoutModel = navModel.getLayoutModel(pageId);
		var control = layoutModel.find(this.controlId);
		var me = this;
		control.setFinishedCallback( function(control, status) {
			me._setWidgetStyleContinue(control, status, style);
		}, style);
		control.start(false);
	},
	
	_setWidgetStyleContinue: function(control, status, style) {
		// summary: along with _setWidgetStyle() sets the wrapper-css-style metadata on the control that will get applied to its widget DOM node
		// control: a layout control on which to set the metadata
		// status: the HTTP status result from retrieving the control
		// style: a string that represents the inline style for the widget DOM node
		if (status == 200) {
			control.setMetaData("widget-body-height", style);
		} else {
			console.error("Could not find control when setting wrapper-css-style metadata");
		}
	},
	
	/** RESIZE DOM EVENT FUNCTIONS **/
	
	_beginSizing: function(/*Event*/ e) {
		// summary: called when the user selects the resize handle with the mouse
		// e: DOM event
		if (this._isSizing) {   // return if the handle is already dragging
			return false; 
		} else {
			this._isSizing = true;
		}
		
		// set the drag cursor on the body
		dojo.style(dojo.body(), {cursor:dojo.style(this.handle).cursor});
		
		// store these values so they can be dynamically updated as the handle is dragged: 	
		//				x and y coordinates of the drag handle
		//				the style on the skin/control node
		//				the style on the widget body node
		this.point = {'x':e.clientX, 'y':e.clientY};
		// results of _getDomPositionStyle are mixed together in case the left/height/width attributes are not included in 
		//		the runtime skin style from getStyle() but are present in the DOM and stored elsewhere
		var controlNode = com.ibm.pb.layout.LayoutParser.findById(this.controlId);	
		this.controlStyle  = dojo.mixin(this._getDomPositionStyle(controlNode), this.getStyle()); 
		var widgetNode = dojo.byId(this.widgetId).parentNode;
		this.widgetStyle = this._getDomPositionStyle(widgetNode);
		this.oldSize= this._getWidgetSize();
		
		// connect to events for dragging the mouse
		this.eventHandles = []; 
		this.eventHandles.push(dojo.connect(document, "onmousemove", this, "_changeSizing")); 
		this.eventHandles.push(dojo.connect(document, "onmouseup", this, "_endSizing")); 
		this.eventHandles.push(dojo.connect(document, "onmouseleave", this, "_endSizing")); 

		e.preventDefault();	
	},
	
	_getWidgetSize:function(){
		var size={};
		var control = com.ibm.pb.layout.LayoutParser.findById(this.controlId);
		var div = dojo.query(".lotusWidgetBody",control)[0]; 
		var contentBox =dojo.contentBox(div);
		var marginBox =dojo.marginBox(div);
		size.newContentWidth = contentBox.w;
		size.newContentHeight = contentBox.h;
		size.newMarginWidth = marginBox.w;
		size.newMarginHeight = marginBox.h;
		return size;
	},

	_changeSizing: function(/*Event*/ e) {
		// summary: called when the user moves the resize handle
		// e: DOM event

		// calculate the dy/dx of the drag handle => calculate the new height and width of the control
		var dx = this.point.x - e.clientX;  this.point.x -= dx;
		var dy = this.point.y - e.clientY; 	this.point.y -= dy;
		var oldHeight = this.controlStyle.height;
		this._updatePosition(this.controlStyle, dx, dy);  // changes the height, width and left attributes to incorporate dy/dx
		
		// filter the style and set the resulting style on the skin in the DOM
		// need to create a new style object instead of using this.controlStyle directly to avoid errors in IE
		// the IE errors happen because sometimes this.controlStyle has other attributes in it that are out of our control
		this.skinNode.checkStyle(this.controlStyle);  // NOTE: checkStyle will update the this.controlStyle object
		var controlNode = com.ibm.pb.layout.LayoutParser.findById(this.controlId);	
		var style = {};
		style.height = this.controlStyle.height ? this.controlStyle.height : null;
		style.width = this.controlStyle.width ? this.controlStyle.width : null;
		style.left = this.controlStyle.left ? this.controlStyle.left : null;
		dojo.style(controlNode, style);
		
		// change height on the widget body to reflect changes in the skin height
		// this is to make sure the widget body takes up the full height available to it in the skin
		if (this.controlStyle.height) {
			var dy = parseInt(oldHeight) - parseInt(this.controlStyle.height);
			this._updatePosition(this.widgetStyle, dx, dy);  // changes the height attribute to incorporate dy/dx
			var widgetNode = dojo.byId(this.widgetId).parentNode;
			dojo.style(widgetNode, "height", this.widgetStyle.height);
		}

		e.preventDefault();
	},

	_endSizing: function(/*Event*/ e) {
        // summary: called when the user stops moving the resize handle, does clean up and then sets the style on the skin
		// e: DOM event

	    var currentPage = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage();

		this._isSizing = false;
		
		// cleanup: reset cursor, set the page as dirty, and disconnect drag handlers
		dojo.style(dojo.body(),{cursor:"default"});
		currentPage.setDirty(true);
		while (this.eventHandles.length > 0) {
			dojo.disconnect(this.eventHandles.pop());
		}

		// save the change in skin/control size and widget height
		this.setStyle(this.controlStyle);
		if (this.controlStyle.height) {
			// a height is set on the widget body as well as the skin/control to make sure the widget fills up the entire available height inside the skin
			this._setWidgetStyle(this.widgetStyle.height);
		}

        // if there is no template url
        // associated with this layout navstate
        // then get it from the page instance
        // and set it here
        var navStateModel = com.ibm.mashups.enabler.model.state.NavigationStateModelFactory.getNavigationStateModel();
        var layoutAccessor =  com.ibm.mashups.enabler.model.state.AccessorFactory.getLayoutAccessor(navStateModel, currentPage.getID());

        if (!layoutAccessor.getTemplateURL()) {
            var navNode = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel().find(currentPage.getID()).start(true);
            var content = navNode.getContent().start(true);
            var templateUrl = content.getStaticContentURL();

            layoutAccessor.setTemplateURL(templateUrl)
        }
        
		// save the new control size in the NavState
		var navStateModel = com.ibm.mashups.enabler.model.state.NavigationStateModelFactory.getNavigationStateModel();
		var accessorFactory = com.ibm.mashups.enabler.model.state.AccessorFactory;
		var widgetAccessor = accessorFactory.getWidgetAccessor(navStateModel, this.widgetId);
		widgetAccessor.setSize(this.controlStyle.width, this.controlStyle.height);
		navStateModel.commit().start();

		// broadcast the widget change size event to tell the widget it can reflow since its size has changed
		// create a new changes object instead of just using {height:this.controlStyle.height,width:this.controlStyle.width} 
		//			because if either height or width is null, errors are thrown
		var box = this._getBoxWithScroll();
		var changes = {"widgetId":this.widgetId};
		if (this.controlStyle.width) {
			changes.newWidth = parseInt(this.controlStyle.width);
		}else {
			changes.newWidth=box.w;
		}
		if (this.controlStyle.height) {
			changes.newHeight = parseInt(this.widgetStyle.height);
		}else {
			changes.newHeight = box.h;
		}
		this.evtSvc.publishEvent(com.ibm.mashups.iwidget.Constants.RESIZE_WIDGET + "." + this.widgetId, changes);
		this.evtSvc.publishEvent("com.ibm.mashups.builder.resizeWidget", changes);  // needed for freeform layout
	},
	
	_getSizeWithScroll:function(oldSize,newSize,dimension){
		var gap = (newSize["newMargin"+dimension] - newSize["newContent"+dimension]) -(oldSize["newMargin"+dimension] - oldSize["newContent"+dimension]);
		if(gap >0){//While resizing, we do not change the widget style, the gap between margin & content should be fixed, 
						//gap means there may be a scroll bar show up during resize, but this scrollbar may not be needed if the widget
						//can handle properly for the size change.
			if(newSize["newContent"+dimension] +gap < newSize["newMargin"+dimension] ){//I don't know any situation that this can be false, just in case.
				return newSize["newContent"+dimension] +gap;
			}
		}
		return newSize["newContent"+dimension];
	},
	
	_getBoxWithScroll:function(){
		var newSize = this._getWidgetSize();
		var result = {h:newSize.newContentHeight,w:newSize.newContentWidth};
		if(!this.oldSize) return result;
		result.h= this._getSizeWithScroll(this.oldSize,newSize,"Height");
		result.w= this._getSizeWithScroll(this.oldSize,newSize,"Width");
		return result;
	},
	
	/** HANDLE HELPERS **/
	
	_hideHandle: function() {
		// summary: hides the resize handle
		if(this.handle) {
			dojo.style(this.handle,'display',"none");
		}
	},
	
	_showHandle: function() {
		// summary: shows the resize handle
		// #18844 runtimeModel is not ready in view mode
		if (!this.skinNode) {
			var runtimeModel = com.ibm.mashups.enabler.runtime.skin.Factory.getRuntimeSkinModel();
			this.skinNode = runtimeModel.find(this.controlId);
			//#19026 lazy find when first time show the resize handle
			var controlNode = com.ibm.pb.layout.LayoutParser.findById(this.controlId);	
			this.containerId = com.ibm.pb.layout.LayoutParser.extractID(controlNode.parentNode);
		}
		var canResize = this.skinNode.checkStyle(this.getStyle());
		if (canResize && this.handle) {
			dojo.style(this.handle,'display',"block");
		}
	},
	
	/** FUNCTIONS FOR ATTACHING TO ENABLER EVENTS **/
	
	_checkMode: function(/*String*/ mode) {
		// summary: shows the resize handle if we are in page edit mode
		// mode: the page's current mode, the handle will show if mode = "edit"
		if (mode == com.ibm.mm.iwidget.Constants.mode_edit) {
			this._showHandle();
		} else {
			this._hideHandle();
		}
	},
	_initWidgetsInFreeformLayout: function(iEvent){
		this.containerId = iEvent.containerId;
	},
	_clearStyle: function(/*string*/ id) {
		// summary: clears out styles that have been set on the skin and widget nodes on the server and in the DOM
		//			ONLY triggered if the control has been moved to another container
		// id: id for which the styles should be cleared
		if (this.controlId != id) {
			// not me
			return;
		}
		
		var controlNode = com.ibm.pb.layout.LayoutParser.findById(this.controlId);	
		var newContainerId = com.ibm.pb.layout.LayoutParser.extractID(controlNode.parentNode);
		if (newContainerId != this.containerId) {
			this.containerId = newContainerId;
			// clear the position attributes from the current style
			var noPositionStyle = this._removePositionFromStyle(this.getStyle());
			// save the control and widget style to the server
			this.setStyle(noPositionStyle);
			this._setWidgetStyle("");
			// clear the styles in the DOM too
			var w = dojo.byId(this.widgetId);
			if(w) { 
				if(dojo.isSafari) {
					dojo.style(w.parentNode, "height", null);
				} else {
					w.parentNode.removeAttribute("style");
				}
			}
			dojo.style(controlNode, this.positionAttrs);
			// clear the runtime model cache for this node
			var rsModel = com.ibm.mashups.enabler.runtime.skin.Factory.getRuntimeSkinModel();
			rsModel.invalidate(this.skinNode);
		}
	},
	
	/** HELPERS TO MANIPULATE STYLE OBJECTS **/
	
	_removePositionFromStyle: function(/*Object*/ style) {
		// summary: removes all the DOM attributes having to do with position from a given style object
		//			for example, this is done when a widget is moved into a different container and needs to be reset
		// style: the style object to manipulate
		for (var i in this.positionAttrs) {
			if (style[i]) {
				delete style[i];
			}
		}
	},
	
	_getDomPositionStyle: function(/*DOMNode*/ node) {
		// summary: returns a style object with only height/width/left?/top? attributes
		//			style attributes are retrieved from the DOM
		// node: the DOM node for which to get the style
		var dBox = dojo.contentBox(node);
		var dStyle = dojo.style(node);
		var ret = {};
		ret.height = dStyle.height;
		if (ret.height == "auto") {
			ret.height = dBox.h + "px";
		}
		ret.width = dStyle.width;
		if (ret.width == "auto") {
			ret.width = dBox.w + "px";
		}
		if (dStyle.left != "auto") {
			ret.left = dStyle.left;
		}
		if (dStyle.right != "auto") {
			ret.right = dStyle.right;
		}
		return ret;
	},
	
	_updatePosition: function(/*Object*/ style, /*int*/ dx, /*int*/ dy) {
		// summary: updates a size object using dx/dy, the style object pass in will be updated
		// style: CSS style object
		// dx: change in x direction in pixels
		// dy: change in y direction in pixels
		var width = parseInt(style.width);
		var height = parseInt(style.height);
		if (width && dx) {
			if (!ibmCfg.themeConfig.isRTL) {
				style.width = width - dx + "px";
			} else {
				style.width = width + dx + "px";
				// adjust the left parameter in RTL mode 
				// so the widget doesn't jump to the right
				var left = parseInt(style.left);
				if (left) {
					style.left = left - dx + "px";
				}
			}
		}
		if (height && dy) {
			style.height = height - dy + "px";
		}
	}
});

}

if(!dojo._hasResource["com.ibm.pb.layout.filters.ContainerStyleFilter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.layout.filters.ContainerStyleFilter"] = true;
// @author Laura
// This module implements the com.ibm.mashups.enabler.runtime.skin.StyleFilter interface
// It is meant to be applied to a layout container
// Other modules should inherit from this and overwrite the _isAgree and _filterStyle functions to make any changes
// known to inherit : 	com.ibm.pb.layout.filters.ColumnContainerStyleFilter
//						com.ibm.pb.layout.filters.RowContainerStyleFilter
//						com.ibm.pb.layout.filters.FreeFormContainerStyleFilter

dojo.provide("com.ibm.pb.layout.filters.ContainerStyleFilter");

dojo.declare("com.ibm.pb.layout.filters.ContainerStyleFilter", [com.ibm.mashups.enabler.runtime.skin.StyleFilter], {

	widgetMaxHeight: 10000,	// widget maximum height in pixels
	widgetMaxWidth: 10000,	// widget maximum width in pixels
	widgetMinWidth: 100,	// widget minimum width in pixels
	widgetMinHeight: 80,	// widget minimum height in pixels
	containerMinWidth: 5, 	// container minimum width in a percentage
	
	onChangingStyle: function(layoutNode, oldStyle, newStyle, filterControl, filterChain) {
		// summary: set disagree if _isAgree returns false, then trigger the next filter
		// 			gets called as part of the filter chain triggered when checkStyle() is called on a runtime skin node
		var node = com.ibm.pb.layout.LayoutParser.findById(layoutNode.getID());
        if (!this._isAgree(layoutNode, node, oldStyle, newStyle)) {
			filterControl.setDisagree();   
        }
		this._filterStyle(layoutNode, node, oldStyle, newStyle);
		filterChain.doFilter(layoutNode, oldStyle, newStyle, filterControl);
    },
    
    onChangeStyle: function(layoutNode, oldStyle, newStyle, filterChain) {
		// summary: filter the new style and call the next filter if _isAgree returns true
		// 			gets called as part of the filter chain triggered when setStyle() is called on a modifiable runtime skin node
		var node = com.ibm.pb.layout.LayoutParser.findById(layoutNode.getID());
        if (this._isAgree(layoutNode, node, oldStyle, newStyle)) {
			this._filterStyle(layoutNode, node, oldStyle, newStyle);
            filterChain.doFilter(layoutNode, oldStyle, newStyle);
        }
    },
	
	_isContainer: function(layoutNode) {
		// summary: returns true if the given node is a layout container
		return (layoutNode instanceof com.ibm.mashups.enabler.layout.LayoutContainer);
	},
	
	_isControl: function(layoutNode) {
		// summary: returns true if the given node is a layout control
		return (layoutNode instanceof com.ibm.mashups.enabler.layout.LayoutControl);
	},
	
	_isAgree: function(layoutNode, node, oldStyle, newStyle) {
		// summary: return true if the new style is allowed, false otherwise
		return true;
	},
	
	_filterStyle: function(layoutNode, node, oldStyle, newStyle) {
		// summary: makes any needed changes to the newStyle object
	}
});

}

if(!dojo._hasResource["com.ibm.pb.layout.filters.ColumnContainerStyleFilter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.layout.filters.ColumnContainerStyleFilter"] = true;
dojo.provide("com.ibm.pb.layout.filters.ColumnContainerStyleFilter");

dojo.declare("com.ibm.pb.layout.filters.ColumnContainerStyleFilter", [com.ibm.pb.layout.filters.ContainerStyleFilter], {
	// corresponds to a column container type
	
    _filterStyle: function(layoutNode, node, oldStyle, newStyle) {
		if (this._isControl(layoutNode)) {
			// resize widget
			// widgets in columns cannot change width, it should be fluid
			if (newStyle.width){
				delete newStyle.width;
			}
			// make sure the widget is not too tall
			if (parseInt(newStyle.height) > this.widgetMaxHeight) {
				newStyle.height = this.widgetMaxHeight + "px";
			}
			// make sure the widget is not too short
			if (parseInt(newStyle.height) < this.widgetMinHeight) {
				newStyle.height = this.widgetMinHeight + "px";
			}
		} else if (this._isContainer(layoutNode)) {
			// resizer container
			if(parseInt(newStyle.width) < this.containerMinWidth){
				newStyle.width = this.containerMinWidth+"%";
			}
		}
	},
	_isAgree: function(layoutNode, node, oldStyle, newStyle) {
		// summary: return true if the new style is allowed, false otherwise
		if (this._isContainer(layoutNode)) {
			if(parseInt(newStyle.width) < this.containerMinWidth){
				return false;
			}
		}
		return true;
	}
});

}

if(!dojo._hasResource["com.ibm.pb.layout.filters.RowContainerStyleFilter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.layout.filters.RowContainerStyleFilter"] = true;
dojo.provide("com.ibm.pb.layout.filters.RowContainerStyleFilter");

dojo.declare("com.ibm.pb.layout.filters.RowContainerStyleFilter", [com.ibm.pb.layout.filters.ContainerStyleFilter], {
	// corresponds to a row container type
	
    _filterStyle: function(layoutNode, node, oldStyle, newStyle) {
		if (this._isControl(layoutNode)) {
			// resize widget
			// widgets in rows cannot change height, it should be fluid
			if (newStyle.height){
				delete newStyle.height;
			}
			// make sure widget is not too wide (ie: extends beyond its container)
			var maxWidth = dojo.contentBox(node.parentNode).w;
			if (parseInt(newStyle.width) > maxWidth) {
				newStyle.width = maxWidth + "px";
			}
			// make sure the widget is not too thin
			if (parseInt(newStyle.width) < this.widgetMinWidth) {
				newStyle.width = this.widgetMinWidth + "px";
			}
		} else if (this._isContainer(layoutNode)) {
			// resizer container
			if(parseInt(newStyle.width) < this.containerMinWidth){
				newStyle.width = this.containerMinWidth+"%";
			}
		}
	},
	_isAgree: function(layoutNode, node, oldStyle, newStyle) {
		// summary: return true if the new style is allowed, false otherwise
		if (this._isContainer(layoutNode)) {
			if(parseInt(newStyle.width) < this.containerMinWidth){
				return false;
			}
		}
		return true;
	}
});

}

if(!dojo._hasResource["com.ibm.pb.layout.filters.FreeFormContainerStyleFilter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.layout.filters.FreeFormContainerStyleFilter"] = true;
dojo.provide("com.ibm.pb.layout.filters.FreeFormContainerStyleFilter");

dojo.declare("com.ibm.pb.layout.filters.FreeFormContainerStyleFilter", [com.ibm.pb.layout.filters.ContainerStyleFilter], {
	// corresponds to a free form container type
	
    _filterStyle: function(layoutNode, node, oldStyle, newStyle) {
		if (this._isControl(layoutNode)) {
			// resize widget
			var maxWidth = dojo.contentBox(node.parentNode).w - dojo.style(node, "left");
			// make sure the widget is not too wide (ie extends beyond its container)
			if (parseInt(newStyle.width) > maxWidth) {
				newStyle.width = maxWidth + "px";
			}
			// make sure the widget is not too tall
			if (parseInt(newStyle.height) > this.widgetMaxHeight) {
				newStyle.height = this.widgetMaxHeight + "px";
			}
			// make sure the widget is not too thin
			if (parseInt(newStyle.width) < this.widgetMinWidth) {
				newStyle.width = this.widgetMinWidth + "px";
			}
			// make sure the widget is not too short
			if (parseInt(newStyle.height) < this.widgetMinHeight) {
				newStyle.height = this.widgetMinHeight + "px";
			}
		} else if (this._isContainer(layoutNode)) {
			// resizer container
			if(parseInt(newStyle.width) < this.containerMinWidth){
				newStyle.width = this.containerMinWidth+"%";
			}
		}
	},
	_isAgree: function(layoutNode, node, oldStyle, newStyle) {
		// summary: return true if the new style is allowed, false otherwise
		if (this._isContainer(layoutNode)) {
			if(parseInt(newStyle.width) < this.containerMinWidth){
				return false;
			}
		}
		return true;
	}
});

}

if(!dojo._hasResource["com.ibm.pb.PB2EditModeLoadedFlag"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.pb.PB2EditModeLoadedFlag"] = true;
dojo.provide( "com.ibm.pb.PB2EditModeLoadedFlag" );

/* Included at the end of the edit mode layer to provide a flag for dojo.io.script to check for it's existence. */
com.ibm.pb.PB2EditModeLoadedFlag = "true";

}


dojo.i18n._preloadLocalizations("com.ibm.themes.PageBuilder2.nls.theme", ["ROOT","ar","ca","cs","da","de","de-de","el","en","en-gb","en-us","es","es-es","fi","fi-fi","fr","fr-fr","he","he-il","hr","hu","it","it-it","ja","ja-jp","ko","ko-kr","nb","nl","nl-be","nl-nl","no","pl","pt","pt-br","pt-pt","ro","ro-ro","ru","sk","sl","sv","th","tr","uk","xx","zh","zh-cn","zh-tw"]);
